@@ -34,6 +34,7 @@ if fout.close() != None:
 version = re_version.match(output)
 
 # Imagemagick by default
+im = False
 gm = False
 
 if version != None:
@@ -41,6 +42,7 @@ if version != None:
     minor = int(version.group(2))
     patch = int(version.group(3))
     version = hex(major * 65536 + minor * 256 + patch)
+    im = True
 else:
     # Try GraphicsMagick
     re_version = re.compile(r'^GraphicsMagick.*http:..www.GraphicsMagick.org.*$')
@@ -48,17 +50,25 @@ else:
     if version != None:
         gm = True
 
-opts = "-depth 8"
+if im or gm:
+    opts = "-depth 8"
+elif sys.platform == 'darwin':
+    command = 'lyxconvert'
 
 # If supported, add the -define option for pdf source formats
 if sys.argv[1] == 'pdf' and (version >= 0x060206 or gm):
     opts = '-define pdf:use-cropbox=true ' + opts
 
 # If supported, add the -flatten option for ppm target formats (see bug 4749)
-if sys.argv[3] == 'ppm' and (version >= 0x060305 or gm):
+if sys.argv[3] == 'ppm' and (im and version >= 0x060305 or gm):
     opts = opts + ' -flatten'
 
-if os.system(r'%s %s "%s" "%s"' % (command, opts, sys.argv[2], sys.argv[3] + ':' + sys.argv[4])) != 0:
+# print >> sys.stdout, command, sys.argv[2], sys.argv[4]
+if (im or gm) and os.system(r'%s %s "%s" "%s"' % (command, opts, sys.argv[2], sys.argv[3] + ':' + sys.argv[4])) != 0:
+    print >> sys.stderr, sys.argv[0], 'ERROR'
+    print >> sys.stderr, ('Execution of "%s" failed.' % command)
+    sys.exit(1)
+elif not im and not gm and sys.platform == 'darwin' and os.system(r'%s "%s" "%s"' % (command, sys.argv[2], sys.argv[4])) != 0:
     print >> sys.stderr, sys.argv[0], 'ERROR'
     print >> sys.stderr, ('Execution of "%s" failed.' % command)
     sys.exit(1)
