@@ -13,10 +13,11 @@
 #define BUFFER_H
 
 #include "OutputEnums.h"
-#include "OutputParams.h"
 
+#include "support/unique_ptr.h"
 #include "support/strfwd.h"
 #include "support/types.h"
+#include "support/FileNameList.h"
 
 #include <map>
 #include <list>
@@ -50,6 +51,7 @@ class MacroData;
 class MacroNameSet;
 class MacroSet;
 class OutputParams;
+class otexstream;
 class Paragraph;
 class ParConstIterator;
 class ParIterator;
@@ -63,17 +65,18 @@ class WordLangTuple;
 namespace frontend {
 class GuiBufferDelegate;
 class WorkAreaManager;
-}
+} // namespace frontend
 
 namespace support {
 class DocFileName;
 class FileName;
-class FileNameList;
-}
+class FileNamePairList;
+} // namespace support
 
 namespace graphics {
 class PreviewLoader;
-}
+class Cache;
+} // namespace graphics
 
 
 class Buffer;
@@ -142,24 +145,6 @@ public:
 		PreviewError
 	};
 
-	/// Method to check if a file is externally modified, used by
-	/// isExternallyModified()
-	/**
-	 * timestamp is fast but inaccurate. For example, the granularity
-	 * of timestamp on a FAT filesystem is 2 seconds. Also, various operations
-	 * may touch the timestamp of a file even when its content is unchanged.
-	 *
-	 * checksum is accurate but slow, which can be a problem when it is
-	 * frequently used, or used for a large file on a slow (network) file
-	 * system.
-	 *
-	 * FIXME: replace this method with support/FileMonitor.
-	 */
-	enum CheckMethod {
-		checksum_method, ///< Use file checksum
-		timestamp_method ///< Use timestamp, and checksum if timestamp has changed
-	};
-
 	///
 	enum UpdateScope {
 		UpdateMaster,
@@ -247,7 +232,7 @@ private:
 	typedef std::map<Buffer const *, Buffer *> BufferMap;
 	///
 	void cloneWithChildren(BufferMap &, CloneList *) const;
-	/// save timestamp and checksum of the given file.
+	/// save checksum of the given file.
 	void saveCheckSum() const;
 	/// read a new file
 	ReadStatus readFile(support::FileName const & fn);
@@ -324,13 +309,13 @@ public:
 	    method with a string stream if the output is supposed to go to a
 	    file. \code
 	    ofdocstream ofs;
-	    otexstream os(ofs, texrow);
+	    otexstream os(ofs);
 	    ofs.open("test.tex");
 	    writeLaTeXSource(os, ...);
 	    ofs.close();
 	    \endcode is NOT equivalent to \code
 	    odocstringstream oss;
-	    otexstream os(oss, texrow);
+	    otexstream os(oss);
 	    writeLaTeXSource(os, ...);
 	    ofdocstream ofs;
 	    ofs.open("test.tex");
@@ -369,8 +354,15 @@ public:
 	///
 	bool isDepClean(std::string const & name) const;
 
-	/// whether or not disk file has been externally modified
-	bool isExternallyModified(CheckMethod method) const;
+	/// Whether or not disk file has been externally modified. Uses a checksum
+	/// which is accurate but slow, which can be a problem when it is frequently
+	/// used, or used for a large file on a slow (network) file system.
+	bool isChecksumModified() const;
+
+	/// Flag set by the FileSystemWatcher.
+	/// Fast but (not so) inaccurate, can be cleared by the user.
+	bool notifiesExternalModification() const;
+	void clearExternalModification() const;
 
 	/// mark the main lyx file as not needing saving
 	void markClean() const;
@@ -415,6 +407,18 @@ public:
 	 */
 	support::DocFileName getReferencedFileName(std::string const & fn) const;
 
+	/// Format a file name for LaTeX output (absolute or relative or filename only,
+	/// depending on file and context)
+	std::string const prepareFileNameForLaTeX(std::string const &,
+					std::string const &, bool nice) const;
+
+	/** Returns a vector of bibliography (*.bib) file paths suitable for the
+	 *  output in the respective BibTeX/Biblatex macro
+	 */
+	std::vector<docstring> const prepareBibFilePaths(OutputParams const &,
+				    support::FileNamePairList const bibfilelist,
+				    bool const extension = true) const;
+
 	/** Returns the path where a local layout file lives.
 	 *  An empty string is returned for standard system and user layouts.
 	 *  If possible, it is always relative to the buffer path.
@@ -468,9 +472,13 @@ public:
 	/// thing from whichever Buffer it is called.
 	ListOfBuffers allRelatives() const;
 
-	/// Is buffer read-only?
+	/// Is buffer read-only? True if it has either the read-only flag or the
+	/// externally modified flag.
 	bool isReadonly() const;
 
+	/// Does the buffer have the read-only flag?
+	bool hasReadonlyFlag() const;
+
 	/// Set buffer read-only flag
 	void setReadonly(bool flag = true);
 
@@ -513,7 +521,7 @@ public:
 	/// or just for it, if it isn't a child.
 	BiblioInfo const & masterBibInfo() const;
 	/// collect bibliography info from the various insets in this buffer.
-	void collectBibKeys() const;
+	void collectBibKeys(support::FileNameList &) const;
 	/// add some BiblioInfo to our cache
 	void addBiblioInfo(BiblioInfo const & bi) const;
 	/// add a single piece of bibliography info to our cache
@@ -586,6 +594,8 @@ public:
 	void updatePreviews() const;
 	/// Remove any previewed LaTeX snippets associated with this buffer
 	void removePreviews() const;
+	///
+	graphics::Cache & graphicsCache() const;
 
 	/// Our main text (inside the top InsetText)
 	Text & text() const;
@@ -623,8 +633,8 @@ public:
 
 	/// get source code (latex/docbook) for some paragraphs, or all paragraphs
 	/// including preamble
-	/// returns NULL if Id to Row conversion is unsupported
-	std::auto_ptr<TexRow> getSourceCode(odocstream & os,
+	/// returns nullptr if Id to Row conversion is unsupported
+	unique_ptr<TexRow> getSourceCode(odocstream & os,
 			std::string const & format, pit_type par_begin,
 			pit_type par_end, OutputWhat output, bool master) const;
 
@@ -686,7 +696,7 @@ private:
 	ExportStatus doExport(std::string const & target, bool put_in_tempdir,
 		bool includeall, std::string & result_file) const;
 	///
-	ExportStatus preview(std::string const & format, bool includeall = false) const;
+	ExportStatus preview(std::string const & format, bool includeall) const;
 	///
 	void setMathFlavor(OutputParams & op) const;
 
@@ -786,7 +796,7 @@ private:
 	void updateBibfilesCache(UpdateScope scope = UpdateMaster) const;
 	/// Return the list with all bibfiles in use (including bibfiles
 	/// of loaded child documents).
-	support::FileNameList const &
+	support::FileNamePairList const &
 		getBibfilesCache(UpdateScope scope = UpdateMaster) const;
 	///
 	void collectChildren(ListOfBuffers & children, bool grand_children) const;
