@@ -216,7 +216,7 @@ enum ScreenUpdateStrategy {
 	DecorationUpdate
 };
 
-} // anon namespace
+} // namespace
 
 
 /////////////////////////////////////////////////////////////////////
@@ -351,8 +351,8 @@ BufferView::~BufferView()
 
 int BufferView::rightMargin() const
 {
-	// The value used to be hardcoded to 10, which is 2.5mm at 100dpi
-	int const default_margin = Length(2.5, Length::MM).inPixels(0);
+	// The value used to be hardcoded to 10, which is 0.1in at 100dpi
+	int const default_margin = Length(0.1, Length::IN).inPixels(0);
 	// The additional test for the case the outliner is opened.
 	if (!full_screen_ || !lyxrc.full_screen_limit
 	    || width_ < lyxrc.full_screen_width + 2 * default_margin)
@@ -368,15 +368,22 @@ int BufferView::leftMargin() const
 }
 
 
+int BufferView::inPixels(Length const & len) const
+{
+	Font const font = buffer().params().getFont();
+	return len.inPixels(workWidth(), theFontMetrics(font).em());
+}
+
+
 bool BufferView::isTopScreen() const
 {
-	return d->scrollbarParameters_.position == d->scrollbarParameters_.min;
+	return 0 == d->scrollbarParameters_.min;
 }
 
 
 bool BufferView::isBottomScreen() const
 {
-	return d->scrollbarParameters_.position == d->scrollbarParameters_.max;
+	return 0 == d->scrollbarParameters_.max;
 }
 
 
@@ -559,7 +566,6 @@ void BufferView::updateScrollbar()
 	for (size_t i = last.first + 1; i != parsize; ++i)
 		d->scrollbarParameters_.max += d->par_height_[i];
 
-	d->scrollbarParameters_.position = 0;
 	// The reference is the top position so we remove one page.
 	if (lyxrc.scroll_below_document)
 		d->scrollbarParameters_.max -= minVisiblePart();
@@ -606,17 +612,19 @@ string BufferView::contextMenu(int x, int y) const
 }
 
 
-void BufferView::scrollDocView(int value, bool update)
+
+void BufferView::scrollDocView(int const value, bool update)
 {
-	int const offset = value - d->scrollbarParameters_.position;
+	// The scrollbar values are relative to the top of the screen, therefore the
+	// offset is equal to the target value.
 
 	// No scrolling at all? No need to redraw anything
-	if (offset == 0)
+	if (value == 0)
 		return;
 
 	// If the offset is less than 2 screen height, prefer to scroll instead.
-	if (abs(offset) <= 2 * height_) {
-		d->anchor_ypos_ -= offset;
+	if (abs(value) <= 2 * height_) {
+		d->anchor_ypos_ -= value;
 		buffer_.changed(true);
 		updateHoveredInset();
 		return;
@@ -779,7 +787,7 @@ bool BufferView::moveToPosition(pit_type bottom_pit, pos_type bottom_pos,
 		if (!dit.atEnd()) {
 			dit.pos() = min(dit.paragraph().size(), top_pos);
 			// Some slices of the iterator may not be
-			// reachable (e.g. closed collapsable inset)
+			// reachable (e.g. closed collapsible inset)
 			// so the dociterator may need to be
 			// shortened. Otherwise, setCursor may crash
 			// lyx when the cursor can not be set to these
@@ -878,7 +886,7 @@ void BufferView::scrollToCursor()
 }
 
 
-bool BufferView::scrollToCursor(DocIterator const & dit, bool recenter)
+bool BufferView::scrollToCursor(DocIterator const & dit, bool const recenter)
 {
 	// We are not properly started yet, delay until resizing is
 	// done.
@@ -1027,7 +1035,10 @@ bool BufferView::getStatus(FuncRequest const & cmd, FuncStatus & flag)
 	if (buffer_.isReadonly()
 	    && !lyxaction.funcHasFlag(act, LyXAction::ReadOnly)
 	    && !lyxaction.funcHasFlag(act, LyXAction::NoBuffer)) {
-		flag.message(from_utf8(N_("Document is read-only")));
+		if (buffer_.hasReadonlyFlag())
+			flag.message(from_utf8(N_("Document is read-only")));
+		else
+			flag.message(from_utf8(N_("Document has been modified externally")));
 		flag.setEnabled(false);
 		return true;
 	}
@@ -1367,7 +1378,7 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 		break;
 
 	case LFUN_FONT_STATE:
-		dr.setMessage(cur.currentState());
+		dr.setMessage(cur.currentState(false));
 		break;
 
 	case LFUN_BOOKMARK_SAVE:
@@ -1397,7 +1408,11 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 
 	case LFUN_PARAGRAPH_GOTO: {
 		int const id = convert<int>(cmd.getArg(0));
-		int const pos = convert<int>(cmd.getArg(1));
+		pos_type const pos = convert<int>(cmd.getArg(1));
+		if (id < 0)
+			break;
+		string const str_id_end = cmd.getArg(2);
+		string const str_pos_end = cmd.getArg(3);
 		int i = 0;
 		for (Buffer * b = &buffer_; i == 0 || b != &buffer_;
 			b = theBufferList().next(b)) {
@@ -1414,10 +1429,20 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 				<< b->absFileName() << "'.");
 
 			if (b == &buffer_) {
-				// Set the cursor
-				cur.pos() = pos;
-				mouseSetCursor(cur);
-				dr.screenUpdate(Update::Force | Update::FitCursor);
+				bool success;
+				if (str_id_end.empty() || str_pos_end.empty()) {
+					// Set the cursor
+					cur.pos() = pos;
+					mouseSetCursor(cur);
+					success = true;
+				} else {
+					int const id_end = convert<int>(str_id_end);
+					pos_type const pos_end = convert<int>(str_pos_end);
+					success = setCursorFromEntries({id, pos},
+					                               {id_end, pos_end});
+				}
+				if (success)
+					dr.screenUpdate(Update::Force | Update::FitCursor);
 			} else {
 				// Switch to other buffer view and resend cmd
 				lyx::dispatch(FuncRequest(
@@ -1577,7 +1602,7 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 		break;
 
 	case LFUN_MARK_TOGGLE:
-		cur.setSelection(false);
+		cur.selection(false);
 		if (cur.mark()) {
 			cur.setMark(false);
 			dr.setMessage(from_utf8(N_("Mark removed")));
@@ -1776,7 +1801,7 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 			// Select the inset from outside.
 			cur.pop();
 			cur.resetAnchor();
-			cur.setSelection(true);
+			cur.selection(true);
 			cur.posForward();
 		} else if (cells_selected) {
 			// At least one complete cell is selected and inset is a table.
@@ -1784,7 +1809,7 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 			cur.idx() = 0;
 			cur.pos() = 0;
 			cur.resetAnchor();
-			cur.setSelection(true);
+			cur.selection(true);
 			cur.idx() = cur.lastidx();
 			cur.pos() = cur.lastpos();
 		} else {
@@ -1792,7 +1817,7 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 			cur.pit() = 0;
 			cur.pos() = 0;
 			cur.resetAnchor();
-			cur.setSelection(true);
+			cur.selection(true);
 			cur.pit() = cur.lastpit();
 			cur.pos() = cur.lastpos();
 		}
@@ -2177,7 +2202,7 @@ void BufferView::mouseEventDispatch(FuncRequest const & cmd0)
 	Cursor old = cursor();
 	Cursor cur(*this);
 	cur.push(buffer_.inset());
-	cur.setSelection(d->cursor_.selection());
+	cur.selection(d->cursor_.selection());
 
 	// Either the inset under the cursor or the
 	// surrounding Text will handle this event.
@@ -2315,58 +2340,35 @@ int BufferView::scrollUp(int offset)
 }
 
 
-void BufferView::setCursorFromRow(int row)
+bool BufferView::setCursorFromRow(int row)
 {
-	setCursorFromRow(row, buffer_.texrow());
+	TexRow::TextEntry start, end;
+	tie(start,end) = buffer_.texrow().getEntriesFromRow(row);
+	LYXERR(Debug::LATEX,
+	       "setCursorFromRow: for row " << row << ", TexRow has found "
+	       "start (id=" << start.id << ",pos=" << start.pos << "), "
+	       "end (id=" << end.id << ",pos=" << end.pos << ")");
+	return setCursorFromEntries(start, end);
 }
 
 
-void BufferView::setCursorFromRow(int row, TexRow const & texrow)
+bool BufferView::setCursorFromEntries(TexRow::TextEntry start,
+                                      TexRow::TextEntry end)
 {
-	int tmpid;
-	int tmppos;
-	pit_type newpit = 0;
-	pos_type newpos = 0;
-
-	texrow.getIdFromRow(row, tmpid, tmppos);
-
-	bool posvalid = (tmpid != -1);
-	if (posvalid) {
-		// we need to make sure that the row and position
-		// we got back are valid, because the buffer may well
-		// have changed since we last generated the LaTeX.
-		DocIterator dit = buffer_.getParFromID(tmpid);
-		if (dit == doc_iterator_end(&buffer_))
-			posvalid = false;
-		else if (dit.depth() > 1) {
-			// We are in an inset.
-			pos_type lastpos = dit.lastpos();
-			dit.pos() = tmppos > lastpos ? lastpos : tmppos;
-			setCursor(dit);
-			recenter();
-			return;
-		} else {
-			newpit = dit.pit();
-			// now have to check pos.
-			newpos = tmppos;
-			Paragraph const & par = buffer_.text().getPar(newpit);
-			if (newpos > par.size()) {
-				LYXERR0("Requested position no longer valid.");
-				newpos = par.size() - 1;
-			}
-		}
+	DocIterator dit_start, dit_end;
+	tie(dit_start,dit_end) =
+		TexRow::getDocIteratorsFromEntries(start, end, buffer_);
+	if (!dit_start)
+		return false;
+	// Setting selection start
+	d->cursor_.clearSelection();
+	setCursor(dit_start);
+	// Setting selection end
+	if (dit_end) {
+		d->cursor_.resetAnchor();
+		setCursorSelectionTo(dit_end);
 	}
-	if (!posvalid) {
-		frontend::Alert::error(_("Inverse Search Failed"),
-			_("Invalid position requested by inverse search.\n"
-		    "You need to update the viewed document."));
-		return;
-	}
-	d->cursor_.reset();
-	buffer_.text().setCursor(d->cursor_, newpit, newpos);
-	d->cursor_.setSelection(false);
-	d->cursor_.resetAnchor();
-	recenter();
+	return true;
 }
 
 
@@ -2399,8 +2401,8 @@ void BufferView::gotoLabel(docstring const & label)
 
 		// find label
 		shared_ptr<Toc> toc = buf->tocBackend().toc("label");
-		TocIterator toc_it = toc->begin();
-		TocIterator end = toc->end();
+		Toc::const_iterator toc_it = toc->begin();
+		Toc::const_iterator end = toc->end();
 		for (; toc_it != end; ++toc_it) {
 			if (label == toc_it->str()) {
 				lyx::dispatch(toc_it->action());
@@ -2450,7 +2452,7 @@ void BufferView::setCursor(DocIterator const & dit)
 		dit[i].inset().edit(d->cursor_, true);
 
 	d->cursor_.setCursor(dit);
-	d->cursor_.setSelection(false);
+	d->cursor_.selection(false);
 	d->cursor_.setCurrentFont();
 	// FIXME
 	// It seems on general grounds as if this is probably needed, but
@@ -2460,6 +2462,18 @@ void BufferView::setCursor(DocIterator const & dit)
 }
 
 
+void BufferView::setCursorSelectionTo(DocIterator const & dit)
+{
+	size_t const n = dit.depth();
+	for (size_t i = 0; i < n; ++i)
+		dit[i].inset().edit(d->cursor_, true);
+
+	d->cursor_.selection(true);
+	d->cursor_.setCursorSelectionTo(dit);
+	d->cursor_.setCurrentFont();
+}
+
+
 bool BufferView::checkDepm(Cursor & cur, Cursor & old)
 {
 	// Would be wrong to delete anything if we have a selection.
@@ -2487,7 +2501,7 @@ bool BufferView::checkDepm(Cursor & cur, Cursor & old)
 }
 
 
-bool BufferView::mouseSetCursor(Cursor & cur, bool select)
+bool BufferView::mouseSetCursor(Cursor & cur, bool const select)
 {
 	LASSERT(&cur.bv() == this, return false);
 
@@ -2505,27 +2519,23 @@ bool BufferView::mouseSetCursor(Cursor & cur, bool select)
 	if (leftinset)
 		d->cursor_.fixIfBroken();
 
-	// FIXME: shift-mouse selection doesn't work well across insets.
-	bool const do_selection =
-			select && &d->cursor_.normalAnchor().inset() == &cur.inset();
-
 	// do the dEPM magic if needed
 	// FIXME: (1) move this to InsetText::notifyCursorLeaves?
 	// FIXME: (2) if we had a working InsetText::notifyCursorLeaves,
 	// the leftinset bool would not be necessary (badcursor instead).
 	bool update = leftinset;
-	if (!do_selection && d->cursor_.inTexted())
-		update |= checkDepm(cur, d->cursor_);
 
-	if (!do_selection)
-		d->cursor_.resetAnchor();
-	d->cursor_.setCursor(cur);
-	d->cursor_.boundary(cur.boundary());
-	if (do_selection)
+	if (select) {
 		d->cursor_.setSelection();
-	else
+		d->cursor_.setCursorSelectionTo(cur);
+	} else {
+		if (d->cursor_.inTexted())
+			update |= checkDepm(cur, d->cursor_);
+		d->cursor_.resetAnchor();
+		d->cursor_.setCursor(cur);
 		d->cursor_.clearSelection();
-
+	}
+	d->cursor_.boundary(cur.boundary());
 	d->cursor_.finishUndo();
 	d->cursor_.setCurrentFont();
 	if (update)
@@ -2579,7 +2589,7 @@ bool BufferView::selectIfEmpty(DocIterator & cur)
 	d->cursor_.setCursor(cur);
 	d->cursor_.pit() = beg_pit;
 	d->cursor_.pos() = 0;
-	d->cursor_.setSelection(false);
+	d->cursor_.selection(false);
 	d->cursor_.resetAnchor();
 	d->cursor_.pit() = end_pit;
 	d->cursor_.pos() = end_pos;
@@ -2675,10 +2685,10 @@ void BufferView::updateMetrics()
 		// Complete buffer visible? Then it's easy.
 		if (scrollRange == 0)
 			d->anchor_ypos_ = anchor_pm.ascent();
-
-		// FIXME: Some clever handling needed to show
-		// the _first_ paragraph up to the top if the cursor is
-		// in the first line.
+		else {
+			// avoid empty space above the first row
+			d->anchor_ypos_ = min(d->anchor_ypos_, anchor_pm.ascent());
+		}
 	}
 	anchor_pm.setPosition(d->anchor_ypos_);
 
@@ -2791,19 +2801,7 @@ Point BufferView::coordOffset(DocIterator const & dit) const
 		}
 
 		// remember width for the case that sl.inset() is positioned in an RTL inset
-		if (i && dit[i - 1].text()) {
-			// If this Inset is inside a Text Inset, retrieve the Dimension
-			// from the containing text instead of using Inset::dimension() which
-			// might not be implemented.
-			// FIXME (Abdel 23/09/2007): this is a bit messy because of the
-			// elimination of Inset::dim_ cache. This coordOffset() method needs
-			// to be rewritten in light of the new design.
-			Dimension const & dim = coordCache().getInsets().dim(&sl.inset());
-			lastw = dim.wid;
-		} else {
-			Dimension const dim = sl.inset().dimension(*this);
-			lastw = dim.wid;
-		}
+		lastw = sl.inset().dimension(*this).wid;
 
 		//lyxerr << "Cursor::getPos, i: "
 		// << i << " x: " << xx << " y: " << y << endl;
@@ -2904,15 +2902,26 @@ int BufferView::horizScrollOffset() const
 }
 
 
-CursorSlice const & BufferView::currentRowSlice() const
+int BufferView::horizScrollOffset(Text const * text,
+                                  pit_type pit, pos_type pos) const
 {
-	return d->current_row_slice_;
+	// Is this a row that is currently scrolled?
+	if (!d->current_row_slice_.empty()
+	    && &text->inset() == d->current_row_slice_.inset().asInsetText()
+	    && pit ==  d->current_row_slice_.pit()
+	    && pos ==  d->current_row_slice_.pos())
+		return d->horiz_scroll_offset_;
+	return 0;
 }
 
 
-CursorSlice const & BufferView::lastRowSlice() const
+bool BufferView::hadHorizScrollOffset(Text const * text,
+                                      pit_type pit, pos_type pos) const
 {
-	return d->last_row_slice_;
+	return !d->last_row_slice_.empty()
+	       && &text->inset() == d->last_row_slice_.inset().asInsetText()
+	       && pit ==  d->last_row_slice_.pit()
+	       && pos ==  d->last_row_slice_.pos();
 }
 
 
@@ -2978,8 +2987,7 @@ void BufferView::checkCursorScrollOffset(PainterInfo & pi)
 		bool const drawing = pi.pain.isDrawingEnabled();
 		pi.pain.setDrawingEnabled(false);
 		// No need to care about vertical position.
-		RowPainter rp(pi, buffer().text(), d->cursor_.bottom().pit(), row,
-		              -d->horiz_scroll_offset_, 0);
+		RowPainter rp(pi, buffer().text(), row, -d->horiz_scroll_offset_, 0);
 		rp.paintText();
 		pi.pain.setDrawingEnabled(drawing);
 	}
