@@ -19,8 +19,10 @@
 #include "BufferParams.h"
 #include "Encoding.h"
 #include "LaTeXFeatures.h"
+#include "MetricsInfo.h"
 #include "OutputParams.h"
 #include "Paragraph.h"
+#include "texstream.h"
 #include "TocBackend.h"
 
 #include "support/debug.h"
@@ -30,6 +32,8 @@
 #include "support/mutex.h"
 
 #include "frontends/alert.h"
+#include "frontends/FontMetrics.h"
+#include "frontends/Painter.h"
 
 #include <ostream>
 
@@ -37,6 +41,9 @@ using namespace std;
 
 namespace lyx {
 
+using frontend::Painter;
+using frontend::FontMetrics;
+
 /*
  * Class Change has a changetime field that specifies the exact time at which
  * a specific change was made. The change time is used as a guidance for the
@@ -135,11 +142,9 @@ void Changes::set(Change const & change, pos_type const start, pos_type const en
 {
 	if (change.type != Change::UNCHANGED) {
 		LYXERR(Debug::CHANGES, "setting change (type: " << change.type
-			<< ", author: " << change.author 
+			<< ", author: " << change.author
 			<< ", time: " << long(change.changetime)
 			<< ") in range (" << start << ", " << end << ")");
-		if (!isChanged())
-			is_update_required_ = true;
 	}
 
 	Range const newRange(start, end);
@@ -208,16 +213,13 @@ void Changes::erase(pos_type const pos)
 {
 	LYXERR(Debug::CHANGES, "Erasing change at position " << pos);
 
-	ChangeTable::iterator it = table_.begin();
-	ChangeTable::iterator end = table_.end();
-
-	for (; it != end; ++it) {
+	for (ChangeRange & cr : table_) {
 		// range (pos,pos+x) becomes (pos,pos+x-1)
-		if (it->range.start > pos)
-			--(it->range.start);
+		if (cr.range.start > pos)
+			--(cr.range.start);
 		// range (pos-x,pos) stays (pos-x,pos)
-		if (it->range.end > pos)
-			--(it->range.end);
+		if (cr.range.end > pos)
+			--(cr.range.end);
 	}
 
 	merge();
@@ -231,17 +233,14 @@ void Changes::insert(Change const & change, lyx::pos_type pos)
 			<< " at position " << pos);
 	}
 
-	ChangeTable::iterator it = table_.begin();
-	ChangeTable::iterator end = table_.end();
-
-	for (; it != end; ++it) {
+	for (ChangeRange & cr : table_) {
 		// range (pos,pos+x) becomes (pos+1,pos+x+1)
-		if (it->range.start >= pos)
-			++(it->range.start);
+		if (cr.range.start >= pos)
+			++(cr.range.start);
 
 		// range (pos-x,pos) stays as it is
-		if (it->range.end > pos)
-			++(it->range.end);
+		if (cr.range.end > pos)
+			++(cr.range.end);
 	}
 
 	set(change, pos, pos + 1); // set will call merge
@@ -251,70 +250,52 @@ void Changes::insert(Change const & change, lyx::pos_type pos)
 Change const & Changes::lookup(pos_type const pos) const
 {
 	static Change const noChange = Change(Change::UNCHANGED);
-
-	ChangeTable::const_iterator it = table_.begin();
-	ChangeTable::const_iterator const end = table_.end();
-
-	for (; it != end; ++it) {
-		if (it->range.contains(pos))
-			return it->change;
-	}
-
+	for (ChangeRange const & cr : table_)
+		if (cr.range.contains(pos))
+			return cr.change;
 	return noChange;
 }
 
 
 bool Changes::isDeleted(pos_type start, pos_type end) const
 {
-	ChangeTable::const_iterator it = table_.begin();
-	ChangeTable::const_iterator const itend = table_.end();
-
-	for (; it != itend; ++it) {
-		if (it->range.contains(Range(start, end))) {
+	for (ChangeRange const & cr : table_)
+		if (cr.range.contains(Range(start, end))) {
 			LYXERR(Debug::CHANGES, "range ("
 				<< start << ", " << end << ") fully contains ("
-				<< it->range.start << ", " << it->range.end
-				<< ") of type " << it->change.type);
-			return it->change.type == Change::DELETED;
+				<< cr.range.start << ", " << cr.range.end
+				<< ") of type " << cr.change.type);
+			return cr.change.type == Change::DELETED;
 		}
-	}
 	return false;
 }
 
 
 bool Changes::isChanged(pos_type const start, pos_type const end) const
 {
-	ChangeTable::const_iterator it = table_.begin();
-	ChangeTable::const_iterator const itend = table_.end();
-
-	for (; it != itend; ++it) {
-		if (it->range.intersects(Range(start, end))) {
+	for (ChangeRange const & cr : table_)
+		if (cr.range.intersects(Range(start, end))) {
 			LYXERR(Debug::CHANGES, "found intersection of range ("
 				<< start << ", " << end << ") with ("
-				<< it->range.start << ", " << it->range.end
-				<< ") of type " << it->change.type);
+				<< cr.range.start << ", " << cr.range.end
+				<< ") of type " << cr.change.type);
 			return true;
 		}
-	}
 	return false;
 }
 
 
 bool Changes::isChanged() const
 {
-	ChangeTable::const_iterator it = table_.begin();
-	ChangeTable::const_iterator const itend = table_.end();
-	for (; it != itend; ++it) {
-		if (it->change.changed())
+	for (ChangeRange const & cr : table_)
+		if (cr.change.changed())
 			return true;
-	}
 	return false;
 }
 
 
 void Changes::merge()
 {
-	bool merged = false;
 	ChangeTable::iterator it = table_.begin();
 
 	while (it != table_.end()) {
@@ -327,7 +308,6 @@ void Changes::merge()
 				<< it->range.start);
 
 			table_.erase(it);
-			merged = true;
 			// start again
 			it = table_.begin();
 			continue;
@@ -346,7 +326,6 @@ void Changes::merge()
 			(it + 1)->change.changetime = max(it->change.changetime,
 							  (it + 1)->change.changetime);
 			table_.erase(it);
-			merged = true;
 			// start again
 			it = table_.begin();
 			continue;
@@ -354,8 +333,6 @@ void Changes::merge()
 
 		++it;
 	}
-	if (merged && !isChanged())
-		is_update_required_ = true;
 }
 
 
@@ -405,7 +382,7 @@ docstring getLaTeXMarkup(docstring const & macro, docstring const & author,
 	return ods.str();
 }
 
-} //namespace anon
+} // namespace
 
 
 int Changes::latexMarkChange(otexstream & os, BufferParams const & bparams,
@@ -417,11 +394,16 @@ int Changes::latexMarkChange(otexstream & os, BufferParams const & bparams,
 
 	int column = 0;
 
+	bool const dvipost = LaTeXFeatures::isAvailable("dvipost") &&
+			(runparams.flavor == OutputParams::LATEX
+			 || runparams.flavor == OutputParams::DVILUATEX);
+
 	if (oldChange.type != Change::UNCHANGED) {
 		// close \lyxadded or \lyxdeleted
 		os << '}';
 		column++;
-		if (oldChange.type == Change::DELETED && !runparams.wasDisplayMath)
+		if (oldChange.type == Change::DELETED
+		    && !runparams.wasDisplayMath && !dvipost)
 			--runparams.inulemcmd;
 	}
 
@@ -433,21 +415,19 @@ int Changes::latexMarkChange(otexstream & os, BufferParams const & bparams,
 	docstring macro_beg;
 	if (change.type == Change::DELETED) {
 		macro_beg = from_ascii("\\lyxdeleted{");
-		if (!runparams.inDisplayMath)
+		if (!runparams.inDisplayMath && !dvipost)
 			++runparams.inulemcmd;
 	}
 	else if (change.type == Change::INSERTED)
 		macro_beg = from_ascii("\\lyxadded{");
-	
+
 	docstring str = getLaTeXMarkup(macro_beg,
 				       bparams.authors().get(change.author).name(),
 				       chgTime, runparams);
-	
+
 	// signature needed by \lyxsout to correctly strike out display math
 	if (change.type == Change::DELETED && runparams.inDisplayMath
-	    && (!LaTeXFeatures::isAvailable("dvipost")
-		|| (runparams.flavor != OutputParams::LATEX
-		    && runparams.flavor != OutputParams::DVILUATEX))) {
+	    && !dvipost) {
 		if (os.afterParbreak())
 			str += from_ascii("\\\\\\noindent\n");
 		else
@@ -491,29 +471,25 @@ void Changes::lyxMarkChange(ostream & os, BufferParams const & bparams, int & co
 
 void Changes::checkAuthors(AuthorList const & authorList)
 {
-	ChangeTable::const_iterator it = table_.begin();
-	ChangeTable::const_iterator endit = table_.end();
-	for ( ; it != endit ; ++it) 
-		if (it->change.type != Change::UNCHANGED)
-			authorList.get(it->change.author).setUsed(true);
+	for (ChangeRange const & cr : table_)
+		if (cr.change.type != Change::UNCHANGED)
+			authorList.get(cr.change.author).setUsed(true);
 }
 
 
 void Changes::addToToc(DocIterator const & cdit, Buffer const & buffer,
-        bool output_active) const
+                       bool output_active, TocBackend & backend) const
 {
 	if (table_.empty())
 		return;
 
-	shared_ptr<Toc> change_list = buffer.tocBackend().toc("change");
+	shared_ptr<Toc> change_list = backend.toc("change");
 	AuthorList const & author_list = buffer.params().authors();
 	DocIterator dit = cdit;
 
-	ChangeTable::const_iterator it = table_.begin();
-	ChangeTable::const_iterator const itend = table_.end();
-	for (; it != itend; ++it) {
+	for (ChangeRange const & cr : table_) {
 		docstring str;
-		switch (it->change.type) {
+		switch (cr.change.type) {
 		case Change::UNCHANGED:
 			continue;
 		case Change::DELETED:
@@ -525,38 +501,79 @@ void Changes::addToToc(DocIterator const & cdit, Buffer const & buffer,
 			str.push_back(0x270d);
 			break;
 		}
-		dit.pos() = it->range.start;
+		dit.pos() = cr.range.start;
 		Paragraph const & par = dit.paragraph();
-		str += " " + par.asString(it->range.start, min(par.size(), it->range.end));
-		if (it->range.end > par.size())
+		str += " " + par.asString(cr.range.start, min(par.size(), cr.range.end));
+		if (cr.range.end > par.size())
 			// ¶ U+00B6 PILCROW SIGN
 			str.push_back(0xb6);
-		docstring const & author = author_list.get(it->change.author).name();
-		Toc::iterator it = change_list->item(0, author);
+		docstring const & author = author_list.get(cr.change.author).name();
+		Toc::iterator it = TocBackend::findItem(*change_list, 0, author);
 		if (it == change_list->end()) {
 			change_list->push_back(TocItem(dit, 0, author, true));
-			change_list->push_back(TocItem(dit, 1, str, output_active,
-				support::wrapParas(str, 4)));
+			change_list->push_back(TocItem(dit, 1, str, output_active));
 			continue;
 		}
 		for (++it; it != change_list->end(); ++it) {
 			if (it->depth() == 0 && it->str() != author)
 				break;
 		}
-		change_list->insert(it, TocItem(dit, 1, str, output_active,
-			support::wrapParas(str, 4)));
+		change_list->insert(it, TocItem(dit, 1, str, output_active));
 	}
 }
 
 
 void Changes::updateBuffer(Buffer const & buf)
 {
-	is_update_required_ = false;
-	if (!buf.areChangesPresent() && isChanged())
-		buf.setChangesPresent(true);
+	bool const changed = isChanged();
+	buf.setChangesPresent(buf.areChangesPresent() || changed);
+	previously_changed_ = changed;
 }
 
 
+void Change::paintCue(PainterInfo & pi, double const x1, double const y,
+                      double const x2, FontInfo const & font) const
+{
+	if (!changed())
+		return;
+	// Calculate 1/3 height of font
+	FontMetrics const & fm = theFontMetrics(font);
+	double const y_bar = deleted() ? y - fm.maxAscent() / 3
+		: y + 2 * pi.base.solidLineOffset() + pi.base.solidLineThickness();
+	pi.pain.line(int(x1), int(y_bar), int(x2), int(y_bar), color(),
+	             Painter::line_solid, pi.base.solidLineThickness());
+}
+
+
+void Change::paintCue(PainterInfo & pi, double const x1, double const y1,
+                      double const x2, double const y2) const
+{
+	/*
+	 * y1      /
+	 *        /
+	 *       /
+	 *      /
+	 *     /
+	 * y2 /_____
+	 *    x1  x2
+	 */
+	switch(type) {
+	case UNCHANGED:
+		return;
+	case INSERTED:
+		pi.pain.line(int(x1), int(y2) + 1, int(x2), int(y2) + 1,
+		             color(), Painter::line_solid,
+		             pi.base.solidLineThickness());
+		return;
+	case DELETED:
+		// FIXME: we cannot use antialias since we keep drawing on the same
+		// background with the current painting mechanism.
+		pi.pain.line(int(x1), int(y2), int(x2), int(y1),
+		             color(), Painter::line_solid_aliased,
+		             pi.base.solidLineThickness());
+		return;
+	}
+}
 
 
 } // namespace lyx
