@@ -19,12 +19,18 @@
 #include "Encoding.h"
 #include "ErrorList.h"
 #include "Format.h"
+#include "InsetList.h"
 #include "Language.h"
 #include "LaTeX.h"
+#include "LyXRC.h"
 #include "Mover.h"
+#include "ParagraphList.h"
+#include "Session.h"
 
 #include "frontends/alert.h"
 
+#include "insets/InsetInclude.h"
+
 #include "support/debug.h"
 #include "support/FileNameList.h"
 #include "support/filetools.h"
@@ -93,14 +99,14 @@ private:
 	string const to_;
 };
 
-} // namespace anon
+} // namespace
 
 
 Converter::Converter(string const & f, string const & t,
 		     string const & c, string const & l)
 	: from_(f), to_(t), command_(c), flags_(l),
 	  From_(0), To_(0), latex_(false), xml_(false),
-	  need_aux_(false), nice_(false)
+	  need_aux_(false), nice_(false), need_auth_(false)
 {}
 
 
@@ -128,9 +134,11 @@ void Converter::readFlags()
 			parselog_ = flag_value;
 		else if (flag_name == "nice")
 			nice_ = true;
+		else if (flag_name == "needauth")
+			need_auth_ = true;
 	}
 	if (!result_dir_.empty() && result_file_.empty())
-		result_file_ = "index." + formats.extension(to_);
+		result_file_ = "index." + theFormats().extension(to_);
 	//if (!contains(command, token_from))
 	//	latex = true;
 }
@@ -164,8 +172,8 @@ int Converters::getNumber(string const & from, string const & to) const
 void Converters::add(string const & from, string const & to,
 		     string const & command, string const & flags)
 {
-	formats.add(from);
-	formats.add(to);
+	theFormats().add(from);
+	theFormats().add(to);
 	ConverterList::iterator it = find_if(converterlist_.begin(),
 					     converterlist_.end(),
 					     ConverterEqual(from , to));
@@ -275,6 +283,107 @@ OutputParams::FLAVOR Converters::getFlavor(Graph::EdgePath const & path,
 }
 
 
+bool Converters::checkAuth(Converter const & conv, string const & doc_fname,
+			   bool use_shell_escape)
+{
+	string conv_command = conv.command();
+	bool const has_shell_escape = contains(conv_command, "-shell-escape")
+				|| contains(conv_command, "-enable-write18");
+	if (conv.latex() && has_shell_escape && !use_shell_escape) {
+		docstring const shellescape_warning =
+		      bformat(_("<p>The following LaTeX backend has been "
+		        "configured to allow execution of external programs "
+		        "for any document:</p>"
+		        "<center><p><tt>%1$s</tt></p></center>"
+		        "<p>This is a dangerous configuration. Please, "
+		        "consider using the support offered by LyX for "
+		        "allowing this privilege only to documents that "
+			"actually need it, instead.</p>"),
+		        from_utf8(conv_command));
+		frontend::Alert::error(_("Security Warning"),
+					shellescape_warning , false);
+	} else if (!conv.latex())
+		use_shell_escape = false;
+	if (!conv.need_auth() && !use_shell_escape)
+		return true;
+	size_t const token_pos = conv_command.find("$$");
+	bool const has_token = token_pos != string::npos;
+	string const command = use_shell_escape && !has_shell_escape
+		? (has_token ? conv_command.insert(token_pos, "-shell-escape ")
+			     : conv_command.append(" -shell-escape"))
+		: conv_command;
+	docstring const security_warning = (use_shell_escape
+	    ? bformat(_("<p>The following LaTeX backend has been requested "
+	        "to allow execution of external programs:</p>"
+	        "<center><p><tt>%1$s</tt></p></center>"
+	        "<p>The external programs can execute arbitrary commands on "
+	        "your system, including dangerous ones, if instructed to do "
+	        "so by a maliciously crafted LyX document.</p>"),
+	      from_utf8(command))
+	    : bformat(_("<p>The requested operation requires the use of a "
+	        "converter from %2$s to %3$s:</p>"
+	        "<blockquote><p><tt>%1$s</tt></p></blockquote>"
+	        "<p>This external program can execute arbitrary commands on "
+	        "your system, including dangerous ones, if instructed to do "
+	        "so by a maliciously crafted LyX document.</p>"),
+	      from_utf8(command), from_utf8(conv.from()),
+	      from_utf8(conv.to())));
+	if (lyxrc.use_converter_needauth_forbidden && !use_shell_escape) {
+		frontend::Alert::error(
+		    _("An external converter is disabled for security reasons"),
+		    security_warning + _(
+		    "<p><b>Your current preference settings forbid its execution.</b></p>"
+		    "<p>(To change this setting, go to <i>Preferences &#x25b9; File "
+		    "Handling &#x25b9; Converters</i> and uncheck <i>Security &#x25b9; "
+		    "Forbid needauth converters</i>.)"), false);
+		return false;
+	}
+	if (!lyxrc.use_converter_needauth && !use_shell_escape)
+		return true;
+	docstring const security_title = use_shell_escape
+		? _("A LaTeX backend requires your authorization")
+		: _("An external converter requires your authorization");
+	int choice;
+	docstring const security_warning2 = security_warning + (use_shell_escape
+		? _("<p>Should LaTeX backends be allowed to run external "
+		    "programs?</p><p><b>Allow them only if you trust the "
+		    "origin/sender of the LyX document!</b></p>")
+		: _("<p>Would you like to run this converter?</p>"
+		    "<p><b>Only run if you trust the origin/sender of the LyX "
+		    "document!</b></p>"));
+	docstring const no = use_shell_escape
+				? _("Do &not allow") : _("Do &not run");
+	docstring const yes = use_shell_escape ? _("A&llow") : _("&Run");
+	docstring const always = use_shell_escape
+					? _("&Always allow for this document")
+					: _("&Always run for this document");
+	if (!doc_fname.empty()) {
+		LYXERR(Debug::FILES, "looking up: " << doc_fname);
+		bool authorized = use_shell_escape
+			? theSession().shellescapeFiles().findAuth(doc_fname)
+			: theSession().authFiles().find(doc_fname);
+		if (!authorized) {
+			choice = frontend::Alert::prompt(security_title,
+							 security_warning2,
+							 0, 0, no, yes, always);
+			if (choice == 2) {
+				if (use_shell_escape)
+					theSession().shellescapeFiles().insert(doc_fname, true);
+				else
+					theSession().authFiles().insert(doc_fname);
+			}
+		} else {
+			choice = 1;
+		}
+	} else {
+		choice = frontend::Alert::prompt(security_title,
+						 security_warning2,
+						 0, 0, no, yes);
+	}
+	return choice != 0;
+}
+
+
 bool Converters::convert(Buffer const * buffer,
 			 FileName const & from_file, FileName const & to_file,
 			 FileName const & orig_from,
@@ -295,8 +404,8 @@ bool Converters::convert(Buffer const * buffer,
 			// default one from ImageMagic.
 			string const from_ext = from_format.empty() ?
 				getExtension(from_file.absFileName()) :
-				formats.extension(from_format);
-			string const to_ext = formats.extension(to_format);
+				theFormats().extension(from_format);
+			string const to_ext = theFormats().extension(to_format);
 			string const command =
 				os::python() + ' ' +
 				quoteName(libFileSearch("scripts", "convertDefault.py").toFilesystemEncoding()) +
@@ -339,13 +448,18 @@ bool Converters::convert(Buffer const * buffer,
 	runparams.flavor = getFlavor(edgepath, buffer);
 
 	if (buffer) {
-		runparams.use_japanese = buffer->params().bufferFormat() == "platex";
+		runparams.use_japanese =
+			(buffer->params().bufferFormat() == "latex"
+			 || suffixIs(buffer->params().bufferFormat(), "-ja"))
+			&& buffer->params().encoding().package() == Encoding::japanese;
 		runparams.use_indices = buffer->params().use_indices;
-		runparams.bibtex_command = (buffer->params().bibtex_command == "default") ?
-			string() : buffer->params().bibtex_command;
+		runparams.bibtex_command = buffer->params().bibtexCommand();
 		runparams.index_command = (buffer->params().index_command == "default") ?
 			string() : buffer->params().index_command;
 		runparams.document_language = buffer->params().language->babel();
+		runparams.only_childbibs = !buffer->params().useBiblatex()
+				&& !buffer->params().useBibtopic()
+				&& buffer->params().multibib == "child";
 	}
 
 	// Some converters (e.g. lilypond) can only output files to the
@@ -400,17 +514,62 @@ bool Converters::convert(Buffer const * buffer,
 						   "tmpfile.out"));
 		}
 
+		if (buffer && buffer->params().use_minted
+		    && lyxrc.pygmentize_command.empty() && conv.latex()) {
+			bool dowarn = false;
+			// Warn only if listings insets are actually used
+			for (Paragraph const & par : buffer->paragraphs()) {
+				InsetList const & insets = par.insetList();
+				pos_type lstpos = insets.find(LISTINGS_CODE, 0);
+				pos_type incpos = insets.find(INCLUDE_CODE, 0);
+				if (incpos >= 0) {
+					InsetInclude const * include =
+						static_cast<InsetInclude *>
+						        (insets.get(incpos));
+					if (include->params().getCmdName() !=
+					                        "inputminted") {
+						incpos = -1;
+					}
+				}
+				if (lstpos >= 0 || incpos >= 0) {
+					dowarn = true;
+					break;
+				}
+			}
+			if (dowarn) {
+				Alert::warning(_("Pygments driver command not found!"),
+				    _("The driver command necessary to use the minted package\n"
+				      "(pygmentize) has not been found. Make sure you have\n"
+				      "the python-pygments module installed or, if the driver\n"
+				      "is named differently, to add the following line to the\n"
+				      "document preamble:\n\n"
+				      "\\AtBeginDocument{\\renewcommand{\\MintedPygmentize}{driver}}\n\n"
+				      "where 'driver' is name of the driver command."));
+			}
+		}
+
+		if (!checkAuth(conv, buffer ? buffer->absFileName() : string(),
+			       buffer && buffer->params().shell_escape))
+			return false;
+
 		if (conv.latex()) {
+			// We are not importing, we have a buffer
+			LATTEST(buffer);
 			run_latex = true;
 			string command = conv.command();
 			command = subst(command, token_from, "");
-			command = subst(command, token_latex_encoding, buffer ?
-				buffer->params().encoding().latexName() : string());
+			command = subst(command, token_latex_encoding,
+			                buffer->params().encoding().latexName());
+			if (buffer->params().shell_escape
+			    && !contains(command, "-shell-escape"))
+				command += " -shell-escape ";
 			LYXERR(Debug::FILES, "Running " << command);
 			if (!runLaTeX(*buffer, command, runparams, errorList))
 				return false;
 		} else {
 			if (conv.need_aux() && !run_latex) {
+				// We are not importing, we have a buffer
+				LATTEST(buffer);
 				string command;
 				switch (runparams.flavor) {
 				case OutputParams::DVILUATEX:
@@ -627,20 +786,6 @@ bool Converters::scanLog(Buffer const & buffer, string const & /*command*/,
 }
 
 
-namespace {
-
-class ShowMessage
-	: public boost::signals::trackable {
-public:
-	ShowMessage(Buffer const & b) : buffer_(b) {}
-	void operator()(docstring const & msg) const { buffer_.message(msg); }
-private:
-	Buffer const & buffer_;
-};
-
-}
-
-
 bool Converters::runLaTeX(Buffer const & buffer, string const & command,
 			  OutputParams const & runparams, ErrorList & errorList)
 {
@@ -653,8 +798,12 @@ bool Converters::runLaTeX(Buffer const & buffer, string const & command,
 	            buffer.filePath(), buffer.layoutPos(),
 	            buffer.lastPreviewError());
 	TeXErrors terr;
-	ShowMessage show(buffer);
-	latex.message.connect(show);
+	// The connection closes itself at the end of the scope when latex is
+	// destroyed. One cannot close (and destroy) buffer while the converter is
+	// running.
+	latex.message.connect([&buffer](docstring const & msg){
+			buffer.message(msg);
+		});
 	int const result = latex.run(terr);
 
 	if (result & LaTeX::ERRORS)
@@ -699,15 +848,15 @@ bool Converters::runLaTeX(Buffer const & buffer, string const & command,
 void Converters::buildGraph()
 {
 	// clear graph's data structures
-	G_.init(formats.size());
+	G_.init(theFormats().size());
 	// each of the converters knows how to convert one format to another
 	// so, for each of them, we create an arrow on the graph, going from
 	// the one to the other
 	ConverterList::iterator it = converterlist_.begin();
 	ConverterList::iterator const end = converterlist_.end();
 	for (; it != end ; ++it) {
-		int const from = formats.getNumber(it->from());
-		int const to   = formats.getNumber(it->to());
+		int const from = theFormats().getNumber(it->from());
+		int const to   = theFormats().getNumber(it->to());
 		LASSERT(from >= 0, continue);
 		LASSERT(to >= 0, continue);
 		G_.addEdge(from, to);
@@ -723,24 +872,24 @@ FormatList const Converters::intToFormat(vector<int> const & input)
 	vector<int>::const_iterator const end = input.end();
 	FormatList::iterator rit = result.begin();
 	for ( ; it != end; ++it, ++rit) {
-		*rit = &formats.get(*it);
+		*rit = &theFormats().get(*it);
 	}
 	return result;
 }
 
 
-FormatList const Converters::getReachableTo(string const & target, 
+FormatList const Converters::getReachableTo(string const & target,
 		bool const clear_visited)
 {
 	vector<int> const & reachablesto =
-		G_.getReachableTo(formats.getNumber(target), clear_visited);
+		G_.getReachableTo(theFormats().getNumber(target), clear_visited);
 
 	return intToFormat(reachablesto);
 }
 
 
-FormatList const Converters::getReachable(string const & from, 
-		bool const only_viewable, bool const clear_visited, 
+FormatList const Converters::getReachable(string const & from,
+		bool const only_viewable, bool const clear_visited,
 		set<string> const & excludes)
 {
 	set<int> excluded_numbers;
@@ -748,10 +897,10 @@ FormatList const Converters::getReachable(string const & from,
 	set<string>::const_iterator sit = excludes.begin();
 	set<string>::const_iterator const end = excludes.end();
 	for (; sit != end; ++sit)
-		excluded_numbers.insert(formats.getNumber(*sit));
+		excluded_numbers.insert(theFormats().getNumber(*sit));
 
 	vector<int> const & reachables =
-		G_.getReachable(formats.getNumber(from),
+		G_.getReachable(theFormats().getNumber(from),
 				only_viewable,
 				clear_visited,
 				excluded_numbers);
@@ -762,15 +911,15 @@ FormatList const Converters::getReachable(string const & from,
 
 bool Converters::isReachable(string const & from, string const & to)
 {
-	return G_.isReachable(formats.getNumber(from),
-			      formats.getNumber(to));
+	return G_.isReachable(theFormats().getNumber(from),
+			      theFormats().getNumber(to));
 }
 
 
 Graph::EdgePath Converters::getPath(string const & from, string const & to)
 {
-	return G_.getPath(formats.getNumber(from),
-			  formats.getNumber(to));
+	return G_.getPath(theFormats().getNumber(from),
+			  theFormats().getNumber(to));
 }
 
 
