@@ -15,12 +15,12 @@
 #include <config.h>
 
 #include "Buffer.h"
+#include "BufferParams.h"
 #include "BufferView.h"
 #include "CoordCache.h"
 #include "Cursor.h"
 #include "CutAndPaste.h"
 #include "DispatchResult.h"
-#include "Encoding.h"
 #include "Font.h"
 #include "FuncCode.h"
 #include "FuncRequest.h"
@@ -31,6 +31,7 @@
 #include "Paragraph.h"
 #include "ParIterator.h"
 #include "Row.h"
+#include "texstream.h"
 #include "Text.h"
 #include "TextMetrics.h"
 #include "TocBackend.h"
@@ -46,12 +47,12 @@
 
 #include "mathed/InsetMath.h"
 #include "mathed/InsetMathBrace.h"
+#include "mathed/InsetMathEnsureMath.h"
 #include "mathed/InsetMathScript.h"
 #include "mathed/MacroTable.h"
 #include "mathed/MathData.h"
-#include "mathed/MathMacro.h"
-
-#include "support/bind.h"
+#include "mathed/MathFactory.h"
+#include "mathed/InsetMathMacro.h"
 
 #include <sstream>
 #include <limits>
@@ -66,32 +67,32 @@ namespace {
 
 // Find position closest to (x, y) in cell given by iter.
 // Used only in mathed
-DocIterator bruteFind2(Cursor const & c, int x, int y)
+DocIterator bruteFind(Cursor const & c, int x, int y)
 {
 	double best_dist = numeric_limits<double>::max();
 
 	DocIterator result;
 
 	DocIterator it = c;
-	it.top().pos() = 0;
+	it.pos() = 0;
 	DocIterator et = c;
-	et.top().pos() = et.top().asInsetMath()->cell(et.top().idx()).size();
+	et.pos() = et.lastpos();
 	for (size_t i = 0;; ++i) {
 		int xo;
 		int yo;
 		Inset const * inset = &it.inset();
-		CoordCache const & cache = c.bv().coordCache();
+		CoordCache::Insets const & insetCache = c.bv().coordCache().getInsets();
 
 		// FIXME: in the case where the inset is not in the cache, this
 		// means that no part of it is visible on screen. In this case
 		// we don't do elaborate search and we just return the forwarded
 		// DocIterator at its beginning.
-		if (!cache.getInsets().has(inset)) {
+		if (!insetCache.has(inset)) {
 			it.top().pos() = 0;
 			return it;
 		}
 
-		Point const o = cache.getInsets().xy(inset);
+		Point const o = insetCache.xy(inset);
 		inset->cursorPos(c.bv(), it.top(), c.boundary(), xo, yo);
 		// Convert to absolute
 		xo += o.x_;
@@ -113,133 +114,13 @@ DocIterator bruteFind2(Cursor const & c, int x, int y)
 }
 
 
-/*
-/// moves position closest to (x, y) in given box
-bool bruteFind(Cursor & cursor,
-	int x, int y, int xlow, int xhigh, int ylow, int yhigh)
-{
-	LASSERT(!cursor.empty(), return false);
-	Inset & inset = cursor[0].inset();
-	BufferView & bv = cursor.bv();
-
-	CoordCache::InnerParPosCache const & cache =
-		bv.coordCache().getParPos().find(cursor.bottom().text())->second;
-	// Get an iterator on the first paragraph in the cache
-	DocIterator it(inset);
-	it.push_back(CursorSlice(inset));
-	it.pit() = cache.begin()->first;
-	// Get an iterator after the last paragraph in the cache
-	DocIterator et(inset);
-	et.push_back(CursorSlice(inset));
-	et.pit() = prev(cache.end(), 1)->first;
-	if (et.pit() >= et.lastpit())
-		et = doc_iterator_end(inset);
-	else
-		++et.pit();
-
-	double best_dist = numeric_limits<double>::max();
-	DocIterator best_cursor = et;
-
-	for ( ; it != et; it.forwardPos(true)) {
-		// avoid invalid nesting when selecting
-		if (!cursor.selection() || positionable(it, cursor.anchor_)) {
-			Point p = bv.getPos(it, false);
-			int xo = p.x_;
-			int yo = p.y_;
-			if (xlow <= xo && xo <= xhigh && ylow <= yo && yo <= yhigh) {
-				double const dx = xo - x;
-				double const dy = yo - y;
-				double const d = dx * dx + dy * dy;
-				// '<=' in order to take the last possible position
-				// this is important for clicking behind \sum in e.g. '\sum_i a'
-				if (d <= best_dist) {
-					//	lyxerr << "*" << endl;
-					best_dist   = d;
-					best_cursor = it;
-				}
-			}
-		}
-	}
-
-	if (best_cursor != et) {
-		cursor.setCursor(best_cursor);
-		return true;
-	}
-
-	return false;
-}
-*/
-
-/*
-/// moves position closest to (x, y) in given box
-bool bruteFind3(Cursor & cur, int x, int y, bool up)
-{
-	BufferView & bv = cur.bv();
-	int ylow  = up ? 0 : y + 1;
-	int yhigh = up ? y - 1 : bv.workHeight();
-	int xlow = 0;
-	int xhigh = bv.workWidth();
-
-// FIXME: bit more work needed to get 'from' and 'to' right.
-	pit_type from = cur.bottom().pit();
-	//pit_type to = cur.bottom().pit();
-	//lyxerr << "Pit start: " << from << endl;
-
-	//lyxerr << "bruteFind3: x: " << x << " y: " << y
-	//	<< " xlow: " << xlow << " xhigh: " << xhigh
-	//	<< " ylow: " << ylow << " yhigh: " << yhigh
-	//	<< endl;
-	DocIterator it = doc_iterator_begin(cur.buffer());
-	it.pit() = from;
-	DocIterator et = doc_iterator_end(cur.buffer());
-
-	double best_dist = numeric_limits<double>::max();
-	DocIterator best_cursor = et;
-
-	for ( ; it != et; it.forwardPos()) {
-		// avoid invalid nesting when selecting
-		if (bv.cursorStatus(it) == CUR_INSIDE
-		    && (!cur.selection() || positionable(it, cur.realAnchor()))) {
-			// If this function is ever used again, check
-			// whether this is the same as "bv.getPos(it,
-			// false)" with boundary = false.
-			Point p = bv.getPos(it);
-			int xo = p.x_;
-			int yo = p.y_;
-			if (xlow <= xo && xo <= xhigh && ylow <= yo && yo <= yhigh) {
-				double const dx = xo - x;
-				double const dy = yo - y;
-				double const d = dx * dx + dy * dy;
-				//lyxerr << "itx: " << xo << " ity: " << yo << " d: " << d
-				//	<< " dx: " << dx << " dy: " << dy
-				//	<< " idx: " << it.idx() << " pos: " << it.pos()
-				//	<< " it:\n" << it
-				//	<< endl;
-				// '<=' in order to take the last possible position
-				// this is important for clicking behind \sum in e.g. '\sum_i a'
-				if (d <= best_dist) {
-					//lyxerr << "*" << endl;
-					best_dist   = d;
-					best_cursor = it;
-				}
-			}
-		}
-	}
-
-	//lyxerr << "best_dist: " << best_dist << " cur:\n" << best_cursor << endl;
-	if (best_cursor == et)
-		return false;
-	cur.setCursor(best_cursor);
-	return true;
-}
-*/
-} // namespace anon
+} // namespace
 
 
 CursorData::CursorData()
 	: DocIterator(), anchor_(),
 	  selection_(false), mark_(false), word_selection_(false),
-	  logicalpos_(false), current_font(inherit_font),
+	  current_font(inherit_font),
 	  autocorrect_(false), macromode_(false)
 {}
 
@@ -247,7 +128,7 @@ CursorData::CursorData()
 CursorData::CursorData(Buffer * buffer)
 	: DocIterator(buffer), anchor_(),
 	  selection_(false), mark_(false), word_selection_(false),
-	  logicalpos_(false), current_font(inherit_font),
+	  current_font(inherit_font),
 	  autocorrect_(false), macromode_(false)
 {}
 
@@ -255,7 +136,7 @@ CursorData::CursorData(Buffer * buffer)
 CursorData::CursorData(DocIterator const & dit)
 	: DocIterator(dit), anchor_(),
 	  selection_(false), mark_(false), word_selection_(false),
-	  logicalpos_(false), current_font(inherit_font),
+	  current_font(inherit_font),
 	  autocorrect_(false), macromode_(false)
 {}
 
@@ -313,13 +194,37 @@ void Cursor::reset()
 }
 
 
-// this (intentionally) does neither touch anchor nor selection status
 void Cursor::setCursor(DocIterator const & cur)
 {
 	DocIterator::operator=(cur);
 }
 
 
+void Cursor::setCursorSelectionTo(DocIterator dit)
+{
+	size_t i = 0;
+	// normalise dit
+	while (i < dit.depth() && i < anchor_.depth() && dit[i] == anchor_[i])
+		++i;
+	if (i != dit.depth()) {
+		// otherwise the cursor is already normal
+		if (i == anchor_.depth())
+			// dit is a proper extension of the anchor_
+			dit.cutOff(i - 1);
+		else if (i + 1 < dit.depth()) {
+			// one has dit[i] != anchor_[i] but either dit[i-1] == anchor_[i-1]
+			// or i == 0. Remove excess.
+			dit.cutOff(i);
+			if (dit[i] > anchor_[i])
+				// place dit after the inset it was in
+				++dit.pos();
+		}
+	}
+	setCursor(dit);
+	setSelection();
+}
+
+
 void Cursor::setCursorToAnchor()
 {
 	if (selection()) {
@@ -645,24 +550,6 @@ void Cursor::checkNewWordPosition()
 }
 
 
-bool Cursor::posBackward()
-{
-	if (pos() == 0)
-		return false;
-	--pos();
-	return true;
-}
-
-
-bool Cursor::posForward()
-{
-	if (pos() == lastpos())
-		return false;
-	++pos();
-	return true;
-}
-
-
 bool Cursor::posVisRight(bool skip_inset)
 {
 	Cursor new_cur = *this; // where we will move to
@@ -870,7 +757,7 @@ bool findNonVirtual(Row const & row, Row::const_iterator & cit, bool onleft)
 	return cit != row.end() && !cit->isVirtual();
 }
 
-}
+} // namespace
 
 void Cursor::getSurroundingPos(pos_type & left_pos, pos_type & right_pos) const
 {
@@ -879,12 +766,11 @@ void Cursor::getSurroundingPos(pos_type & left_pos, pos_type & right_pos) const
 	right_pos = -1;
 
 	Row const & row = textRow();
-	TextMetrics const & tm = bv_->textMetrics(text());
 	double dummy = 0;
-	Row::const_iterator cit = tm.findRowElement(row, pos(), boundary(), dummy);
+	Row::const_iterator cit = row.findElement(pos(), boundary(), dummy);
 	// Handle the case of empty row
 	if (cit == row.end()) {
-		if (paragraph().isRTL(buffer()->params()))
+		if (row.isRTL())
 			right_pos = row.pos();
 		else
 			left_pos = row.pos() - 1;
@@ -958,10 +844,8 @@ void Cursor::getSurroundingPos(pos_type & left_pos, pos_type & right_pos) const
 
 bool Cursor::posVisToNewRow(bool movingLeft)
 {
-	Paragraph const & par = paragraph();
-	Buffer const & buf = *buffer();
 	Row const & row = textRow();
-	bool par_is_LTR = !par.isRTL(buf.params());
+	bool par_is_LTR = !row.isRTL();
 
 	// Inside a table, determining whether to move to the next or
 	// previous row should be done based on the table's direction.
@@ -1061,12 +945,6 @@ CursorSlice Cursor::normalAnchor() const
 }
 
 
-DocIterator & Cursor::realAnchor()
-{
-	return anchor_;
-}
-
-
 CursorSlice Cursor::selBegin() const
 {
 	if (!selection())
@@ -1123,18 +1001,18 @@ DocIterator Cursor::selectionEnd() const
 
 void Cursor::setSelection()
 {
-	setSelection(true);
+	selection(true);
 	if (idx() == normalAnchor().idx() &&
 	    pit() == normalAnchor().pit() &&
 	    pos() == normalAnchor().pos())
-		setSelection(false);
+		selection(false);
 }
 
 
 void Cursor::setSelection(DocIterator const & where, int n)
 {
 	setCursor(where);
-	setSelection(true);
+	selection(true);
 	anchor_ = where;
 	pos() += n;
 }
@@ -1142,7 +1020,7 @@ void Cursor::setSelection(DocIterator const & where, int n)
 
 void Cursor::clearSelection()
 {
-	setSelection(false);
+	selection(false);
 	setWordSelection(false);
 	setMark(false);
 	resetAnchor();
@@ -1178,7 +1056,7 @@ void Cursor::updateTextTargetOffset()
 }
 
 
-void Cursor::info(odocstream & os) const
+void Cursor::info(odocstream & os, bool devel_mode) const
 {
 	for (int i = 1, n = depth(); i < n; ++i) {
 		operator[](i).inset().infoize(os);
@@ -1190,6 +1068,14 @@ void Cursor::info(odocstream & os) const
 		if (inset)
 			prevInset()->infoize2(os);
 	}
+	if (devel_mode) {
+		InsetMath * math = inset().asInsetMath();
+		if (math)
+			os << _(", Inset: ") << math->id();
+		os << _(", Cell: ") << idx();
+		os << _(", Position: ") << pos();
+	}
+
 }
 
 
@@ -1205,7 +1091,7 @@ bool Cursor::selHandle(bool sel)
 		cap::saveSelection(*this);
 
 	resetAnchor();
-	setSelection(sel);
+	selection(sel);
 	return true;
 }
 } // namespace lyx
@@ -1310,9 +1196,8 @@ void Cursor::plainInsert(MathAtom const & t)
 
 void Cursor::insert(docstring const & str)
 {
-	for_each(str.begin(), str.end(),
-		 bind(static_cast<void(Cursor::*)(char_type)>
-			     (&Cursor::insert), this, _1));
+	for (char_type c : str)
+		insert(c);
 }
 
 
@@ -1405,7 +1290,7 @@ void Cursor::insert(MathData const & ar)
 }
 
 
-bool Cursor::backspace()
+bool Cursor::backspace(bool const force)
 {
 	if (selection()) {
 		cap::eraseSelection(*this);
@@ -1441,11 +1326,11 @@ bool Cursor::backspace()
 		}
 	}
 
-	if (pos() != 0 && prevAtom()->nargs() > 0) {
+	if (pos() != 0 && !force && prevAtom()->confirmDeletion()) {
 		// let's require two backspaces for 'big stuff' and
 		// highlight on the first
 		resetAnchor();
-		setSelection(true);
+		selection(true);
 		--pos();
 	} else {
 		--pos();
@@ -1455,7 +1340,7 @@ bool Cursor::backspace()
 }
 
 
-bool Cursor::erase()
+bool Cursor::erase(bool const force)
 {
 	if (inMacroMode())
 		return true;
@@ -1490,9 +1375,9 @@ bool Cursor::erase()
 	}
 
 	// 'clever' UI hack: only erase large items if previously slected
-	if (pos() != lastpos() && nextAtom()->nargs() > 0) {
+	if (pos() != lastpos() && !force && nextAtom()->confirmDeletion()) {
 		resetAnchor();
-		setSelection(true);
+		selection(true);
 		++pos();
 	} else {
 		plainErase();
@@ -1528,7 +1413,7 @@ bool Cursor::down()
 }
 
 
-bool Cursor::macroModeClose(bool cancel)
+bool Cursor::macroModeClose()
 {
 	if (!inMacroMode())
 		return false;
@@ -1541,7 +1426,7 @@ bool Cursor::macroModeClose(bool cancel)
 	cell().erase(pos());
 
 	// do nothing if the macro name is empty
-	if (s == "\\" || cancel)
+	if (s == "\\")
 		return false;
 
 	// trigger updates of macros, at least, if no full
@@ -1552,12 +1437,13 @@ bool Cursor::macroModeClose(bool cancel)
 	InsetMathNest * const in = inset().asInsetMath()->asNestInset();
 	if (in && in->interpretString(*this, s))
 		return true;
-	MathAtom atom = buffer()->getMacro(name, *this, false) ?
-		MathAtom(new MathMacro(buffer(), name)) : createInsetMath(name, buffer());
+	bool const user_macro = buffer()->getMacro(name, *this, false);
+	MathAtom atom = user_macro ? MathAtom(new InsetMathMacro(buffer(), name))
+				   : createInsetMath(name, buffer());
 
 	// try to put argument into macro, if we just inserted a macro
 	bool macroArg = false;
-	MathMacro * atomAsMacro = atom.nucleus()->asMacro();
+	InsetMathMacro * atomAsMacro = atom.nucleus()->asMacro();
 	if (atomAsMacro) {
 		// macros here are still unfolded (in init mode in fact). So
 		// we have to resolve the macro here manually and check its arity
@@ -1565,17 +1451,40 @@ bool Cursor::macroModeClose(bool cancel)
 		MacroData const * data = buffer()->getMacro(atomAsMacro->name());
 		if (!selection.empty() && data && data->numargs() - data->optionals() > 0) {
 			macroArg = true;
-			atomAsMacro->setDisplayMode(MathMacro::DISPLAY_INTERACTIVE_INIT, 1);
+			atomAsMacro->setDisplayMode(InsetMathMacro::DISPLAY_INTERACTIVE_INIT, 1);
 		} else
 			// non-greedy case. Do not touch the arguments behind
-			atomAsMacro->setDisplayMode(MathMacro::DISPLAY_INTERACTIVE_INIT, 0);
+			atomAsMacro->setDisplayMode(InsetMathMacro::DISPLAY_INTERACTIVE_INIT, 0);
 	}
 
 	// insert remembered selection into first argument of a non-macro
 	else if (atom.nucleus()->nargs() > 0)
 		atom.nucleus()->cell(0).append(selection);
 
-	plainInsert(atom);
+	MathWordList const & words = mathedWordList();
+	MathWordList::const_iterator it = words.find(name);
+	bool keep_mathmode = user_macro
+		|| (it != words.end() && (it->second.inset == "font"
+		                          || it->second.inset == "oldfont"
+		                          || it->second.inset == "mbox"));
+	bool ert_macro = !user_macro && it == words.end() && atomAsMacro;
+
+	if (in && in->currentMode() == Inset::TEXT_MODE
+	    && atom.nucleus()->currentMode() == Inset::MATH_MODE
+	    && name != from_ascii("ensuremath") && !ert_macro) {
+		MathAtom at(new InsetMathEnsureMath(buffer()));
+		at.nucleus()->cell(0).push_back(atom);
+		niceInsert(at);
+		posForward();
+	} else if (in && in->currentMode() == Inset::MATH_MODE
+		   && atom.nucleus()->currentMode() == Inset::TEXT_MODE
+		   && !keep_mathmode) {
+		MathAtom at = createInsetMath("text", buffer());
+		at.nucleus()->cell(0).push_back(atom);
+		niceInsert(at);
+		posForward();
+	} else
+		plainInsert(atom);
 
 	// finally put the macro argument behind, if needed
 	if (macroArg) {
@@ -1695,8 +1604,7 @@ void Cursor::normalize()
 			<< pos() << ' ' << lastpos() <<  " in idx: " << idx()
 		       << " in atom: '";
 		odocstringstream os;
-		TexRow texrow(false);
-		otexrowstream ots(os,texrow);
+		otexrowstream ots(os);
 		WriteStream wi(ots, false, true, WriteStream::wsDefault);
 		inset().asInsetMath()->write(wi);
 		lyxerr << to_utf8(os.str()) << endl;
@@ -1800,7 +1708,7 @@ bool Cursor::upDownInMath(bool up)
 		//lyxerr << "idxUpDown triggered" << endl;
 		// try to find best position within this inset
 		if (!selection())
-			setCursor(bruteFind2(*this, xo, yo));
+			setCursor(bruteFind(*this, xo, yo));
 		return true;
 	}
 
@@ -1821,6 +1729,95 @@ bool Cursor::upDownInMath(bool up)
 }
 
 
+InsetMath & Cursor::nextMath()
+{
+	return *nextAtom().nucleus();
+}
+
+
+InsetMath & Cursor::prevMath()
+{
+	return *prevAtom().nucleus();
+}
+
+
+bool Cursor::mathForward(bool word)
+{
+	LASSERT(inMathed(), return false);
+	if (pos() < lastpos()) {
+		if (word) {
+			// word: skip a group of insets of the form X*(B*|R*|P*) (greedy
+			// match) where X is any math class, B is mathbin, R is mathrel, and
+			// P is mathpunct. Make sure that the following remains true:
+			//   mathForward(true); mathBackward(true); mathForward(true)
+			// is the same as mathForward(true) and
+			//   mathBackward(true); mathForward(true); mathBackward(true)
+			// is the same as mathBackward(true).
+			MathClass mc = nextMath().mathClass();
+			do
+				posForward();
+			while (pos() < lastpos() && mc == nextMath().mathClass());
+			if (pos() < lastpos() &&
+			    ((mc = nextMath().mathClass()) == MC_BIN ||
+			     mc == MC_REL || mc == MC_PUNCT))
+				do
+					posForward();
+				while (pos() < lastpos() && mc == nextMath().mathClass());
+		} else if (openable(nextAtom())) {
+			// single step: try to enter the next inset
+			pushBackward(nextMath());
+			inset().idxFirst(*this);
+		} else
+			posForward();
+		return true;
+	}
+	if (inset().idxForward(*this))
+		return true;
+	// try to pop forwards --- but don't pop out of math! leave that to
+	// the FINISH lfuns
+	int s = depth() - 2;
+	if (s >= 0 && operator[](s).inset().asInsetMath())
+		return popForward();
+	return false;
+}
+
+
+bool Cursor::mathBackward(bool word)
+{
+	LASSERT(inMathed(), return false);
+	if (pos() > 0) {
+		if (word) {
+			// word: skip a group of insets. See the comment in mathForward.
+			MathClass mc = prevMath().mathClass();
+			do
+				posBackward();
+			while (pos() > 0 && mc == prevMath().mathClass());
+			if (pos() > 0 && (mc == MC_BIN || mc == MC_REL || mc == MC_PUNCT)) {
+				mc = prevMath().mathClass();
+				do
+					posBackward();
+				while (pos() > 0 && mc == prevMath().mathClass());
+			}
+		} else if (openable(prevAtom())) {
+			// single step: try to enter the preceding inset
+			posBackward();
+			push(nextMath());
+			inset().idxLast(*this);
+		} else
+			posBackward();
+		return true;
+	}
+	if (inset().idxBackward(*this))
+		return true;
+	// try to pop backwards --- but don't pop out of math! leave that to
+	// the FINISH lfuns
+	int s = depth() - 2;
+	if (s >= 0 && operator[](s).inset().asInsetMath())
+		return popBackward();
+	return false;
+}
+
+
 bool Cursor::atFirstOrLastRow(bool up)
 {
 	TextMetrics const & tm = bv_->textMetrics(text());
@@ -1935,10 +1932,13 @@ bool Cursor::upDownInText(bool up, bool & updateNeeded)
 		int yo = bv().getPos(*this).y_;
 		Cursor old = *this;
 		// To next/previous row
+		// FIXME: the y position is often guessed wrongly across styles and
+		// insets, which leads to weird behaviour.
 		if (up)
 			tm.editXY(*this, xo, yo - textRow().ascent() - 1);
 		else
 			tm.editXY(*this, xo, yo + textRow().descent() + 1);
+		x_target_ = old.x_target_;
 		clearSelection();
 
 		// This happens when you move out of an inset.
@@ -1987,6 +1987,8 @@ bool Cursor::upDownInText(bool up, bool & updateNeeded)
 		bool bound = false;
 		top().pos() = tm.getPosNearX(real_next_row, xo, bound);
 		boundary(bound);
+		// When selection==false, this is done by TextMetrics::editXY
+		setCurrentFont();
 
 		updateNeeded |= bv().checkDepm(*this, old);
 	}
@@ -2062,7 +2064,7 @@ docstring parbreak(Cursor const * cur)
 	return os.str();
 }
 
-}
+} // namespace
 
 
 docstring Cursor::selectionAsString(bool with_label) const
@@ -2114,23 +2116,16 @@ docstring Cursor::selectionAsString(bool with_label) const
 }
 
 
-docstring Cursor::currentState() const
+docstring Cursor::currentState(bool devel_mode) const
 {
 	if (inMathed()) {
 		odocstringstream os;
-		info(os);
-#ifdef DEVEL_VERSION
-		InsetMath * math = inset().asInsetMath();
-		if (math)
-			os << _(", Inset: ") << math->id();
-		os << _(", Cell: ") << idx();
-		os << _(", Position: ") << pos();
-#endif
+		info(os, devel_mode);
 		return os.str();
 	}
 
 	if (inTexted())
-		return text()->currentState(*this);
+		return text()->currentState(*this, devel_mode);
 
 	return docstring();
 }
@@ -2142,18 +2137,6 @@ docstring Cursor::getPossibleLabel() const
 }
 
 
-Encoding const * Cursor::getEncoding() const
-{
-	if (empty())
-		return 0;
-	CursorSlice const & sl = innerTextSlice();
-	Text const & text = *sl.text();
-	Font font = text.getPar(sl.pit()).getFont(
-		bv().buffer().params(), sl.pos(), text.outerFont(sl.pit()));
-	return font.language()->encoding();
-}
-
-
 void Cursor::undispatched() const
 {
 	disp_.dispatched(false);
@@ -2443,4 +2426,39 @@ void Cursor::checkBufferStructure()
 }
 
 
+bool Cursor::confirmDeletion(bool const before) const
+{
+	if (!selection()) {
+		if (Inset const * inset = before ? prevInset() : nextInset())
+			return inset->confirmDeletion();
+	} else {
+		DocIterator dit = selectionBegin();
+		CursorSlice const end = selectionEnd().top();
+		for (; dit.top() < end; dit.top().forwardPos())
+			if (Inset const * inset = dit.nextInset())
+				if (inset->confirmDeletion())
+					return true;
+	}
+	return false;
+}
+
+
+void Cursor::moveToClosestEdge(int const x, bool const edit)
+{
+	if (Inset const * inset = nextInset()) {
+		// stay in front of insets for which we want to open the dialog
+		// (e.g. InsetMathSpace).
+		if (edit && (inset->hasSettings() || !inset->contextMenuName().empty()))
+			return;
+		CoordCache::Insets const & insetCache = bv().coordCache().getInsets();
+		if (!insetCache.has(inset))
+			return;
+		int const wid = insetCache.dim(inset).wid;
+		Point p = insetCache.xy(inset);
+		if (x > p.x_ + (wid + 1) / 2)
+			posForward();
+	}
+}
+
+
 } // namespace lyx
