@@ -62,14 +62,14 @@
 #include "support/lstrings.h"
 #include "support/lyxalgo.h"
 #include "support/TempFile.h"
+#include "support/unique_ptr.h"
 
 #include "frontends/alert.h"
 #include "frontends/Clipboard.h"
 #include "frontends/Selection.h"
 
-#include <boost/tuple/tuple.hpp>
-
 #include <string>
+#include <tuple>
 
 using namespace std;
 using namespace lyx::support;
@@ -102,11 +102,11 @@ bool checkPastePossible(int index)
 
 
 struct PasteReturnValue {
-	PasteReturnValue(pit_type r_par, pos_type r_pos, bool r_nu) :
-	  par(r_par), pos(r_pos), needupdate(r_nu)
+	PasteReturnValue(pit_type r_pit, pos_type r_pos, bool r_nu) :
+	  pit(r_pit), pos(r_pos), needupdate(r_nu)
 	{}
 
-	pit_type par;
+	pit_type pit;
 	pos_type pos;
 	bool needupdate;
 };
@@ -159,6 +159,41 @@ pasteSelectionHelper(DocIterator const & cur, ParagraphList const & parlist,
 		}
 	}
 
+	// Prevent to paste uncodable characters in verbatim and ERT.
+	// The encoding is inherited from the context here.
+	docstring uncodable_content;
+	if (target_inset->getLayout().isPassThru() && cur.getEncoding()) {
+		odocstringstream res;
+		Encoding const * e = cur.getEncoding();
+		for (size_t i = 0; i != insertion.size(); ++i) {
+			pos_type end = insertion[i].size();
+			for (pos_type j = 0; j != end; ++j) {
+				// skip insets
+				if (insertion[i].isInset(j))
+					continue;
+				char_type const c = insertion[i].getChar(j);
+				if (!e->encodable(c)) {
+					// do not track deletion
+					res.put(c);
+					insertion[i].eraseChar(j, false);
+					--end;
+					--j;
+				}
+			}
+		}
+		docstring const uncodable = res.str();
+		if (!uncodable.empty()) {
+			if (uncodable.size() == 1)
+				uncodable_content = bformat(_("The character \"%1$s\" is uncodable in this verbatim context "
+						      "and thus has not been pasted."),
+						    uncodable);
+			else
+				uncodable_content = bformat(_("The characters \"%1$s\" are uncodable in this verbatim context "
+						      "and thus have not been pasted."),
+						    uncodable);
+		}
+	}
+
 	// set the paragraphs to plain layout if necessary
 	DocumentClassConstPtr newDocClass = buffer.params().documentClassPtr();
 	if (cur.inset().usePlainLayout()) {
@@ -194,6 +229,9 @@ pasteSelectionHelper(DocIterator const & cur, ParagraphList const & parlist,
 	// want to invalidate them.
 	insertion.swap(in.paragraphs());
 	cap::switchBetweenClasses(oldDocClass, newDocClass, in, errorlist);
+	// Do this here since switchBetweenClasses clears the errorlist
+	if (!uncodable_content.empty())
+		errorlist.push_back(ErrorItem(_("Uncodable content"), uncodable_content));
 	insertion.swap(in.paragraphs());
 
 	ParagraphList::iterator tmpbuf = insertion.begin();
@@ -243,7 +281,7 @@ pasteSelectionHelper(DocIterator const & cur, ParagraphList const & parlist,
 
 	InsetIterator const i_end = inset_iterator_end(in);
 	for (InsetIterator it = inset_iterator_begin(in); it != i_end; ++it) {
-		// Even though this will also be done later, it has to be done here 
+		// Even though this will also be done later, it has to be done here
 		// since some inset might going to try to access
 		// the buffer() member.
 		it->setBuffer(const_cast<Buffer &>(buffer));
@@ -471,7 +509,7 @@ PitPosPair eraseSelectionHelper(BufferParams const & params,
 
 		// Separate handling of paragraph break:
 		if (merge && pit != endpit &&
-		    (pit + 1 != endpit 
+		    (pit + 1 != endpit
 		     || pars[pit].hasSameLayout(pars[endpit])
 		     || pars[endpit].size() == endpos)) {
 			if (pit + 1 == endpit)
@@ -495,16 +533,18 @@ Buffer * copyToTempBuffer(ParagraphList const & paragraphs, DocumentClassConstPt
 	// to be so, but the alternative is to construct a new one of these (with a
 	// new temporary directory, etc) every time, and then to destroy it. So maybe
 	// it's worth just keeping this one around.
-	// FIXME THREAD
 	static TempFile tempfile("clipboard.internal");
 	tempfile.setAutoRemove(false);
-	static Buffer * staticbuffer = theBufferList().newInternalBuffer(
-			tempfile.name().absFileName());
-
-	// These two things only really need doing the first time.
-	staticbuffer->setUnnamed(true);
-	staticbuffer->inset().setBuffer(*staticbuffer);
-
+	// The initialization of staticbuffer is thread-safe. Using a lambda
+	// guarantees that the properties are set only once.
+	static Buffer * staticbuffer = [&](){
+		Buffer * b =
+			theBufferList().newInternalBuffer(tempfile.name().absFileName());
+		b->setUnnamed(true);
+		b->inset().setBuffer(*b);
+		//initialize staticbuffer with b
+		return b;
+	}();
 	// Use a clone for the complicated stuff so that we do not need to clean
 	// up in order to avoid a crash.
 	Buffer * buffer = staticbuffer->cloneBufferOnly();
@@ -534,7 +574,7 @@ Buffer * copyToTempBuffer(ParagraphList const & paragraphs, DocumentClassConstPt
 }
 
 
-void putClipboard(ParagraphList const & paragraphs, 
+void putClipboard(ParagraphList const & paragraphs,
 	DocumentClassConstPtr docclass, docstring const & plaintext)
 {
 	Buffer * buffer = copyToTempBuffer(paragraphs, docclass);
@@ -634,7 +674,7 @@ void copySelectionHelper(Buffer const & buf, Text const & text,
 		// latex_language. This is invalid for others, so we
 		// need to change it to the buffer language.
 		if (it->isPassThru())
-			it->changeLanguage(buf.params(), 
+			it->changeLanguage(buf.params(),
 					   latex_language, buf.language());
 	}
 
@@ -650,16 +690,14 @@ void copySelectionHelper(Buffer const & buf, Text const & text,
 	// are not linked to something else.
 	it = copy_pars.begin();
 	for (; it != it_end; ++it) {
-		it->setBuffer(*static_cast<Buffer *>(0));
+		it->resetBuffer();
 		it->setInsetOwner(0);
 	}
 
 	cutstack.push(make_pair(copy_pars, dc));
 }
 
-} // namespace anon
-
-
+} // namespace
 
 
 namespace cap {
@@ -703,11 +741,11 @@ bool reduceSelectionToOneCell(Cursor & cur)
 	// the easy case: do nothing if only one cell is selected
 	if (i1.idx() == i2.idx())
 		return true;
-	
+
 	cur.top().pos() = 0;
 	cur.resetAnchor();
 	cur.top().pos() = cur.top().lastpos();
-	
+
 	return true;
 }
 
@@ -716,15 +754,15 @@ bool multipleCellsSelected(Cursor const & cur)
 {
 	if (!cur.selection() || !cur.inMathed())
 		return false;
-	
+
 	CursorSlice i1 = cur.selBegin();
 	CursorSlice i2 = cur.selEnd();
 	if (!i1.inset().asInsetMath())
 		return false;
-	
+
 	if (i1.idx() == i2.idx())
 		return false;
-	
+
 	return true;
 }
 
@@ -737,7 +775,7 @@ void switchBetweenClasses(DocumentClassConstPtr oldone,
 	LBUFERR(!in.paragraphs().empty());
 	if (oldone == newone)
 		return;
-	
+
 	DocumentClass const & oldtc = *oldone;
 	DocumentClass const & newtc = *newone;
 
@@ -758,8 +796,8 @@ void switchBetweenClasses(DocumentClassConstPtr oldone,
 		if (added_one || newlayouts.find(name) != newlayouts.end()) {
 			// Warn the user.
 			docstring const s = bformat(_("Layout `%1$s' was not found."), name);
-			errorlist.push_back(
-				ErrorItem(_("Layout Not Found"), s, it->id(), 0, it->size()));
+			errorlist.push_back(ErrorItem(_("Layout Not Found"), s,
+			                              {it->id(), 0}, {it->id(), -1}));
 		}
 
 		if (in.usePlainLayout())
@@ -773,14 +811,14 @@ void switchBetweenClasses(DocumentClassConstPtr oldone,
 	for (InsetIterator it = inset_iterator_begin(in); it != i_end; ++it) {
 		InsetCode const code = it->lyxCode();
 		if (code == FLEX_CODE) {
-			// FIXME: Should we verify all InsetCollapsable?
+			// FIXME: Should we verify all InsetCollapsible?
 			docstring const layoutName = it->layoutName();
 			docstring const & n = newone->insetLayout(layoutName).name();
 			bool const is_undefined = n.empty() ||
 				n == DocumentClass::plainInsetLayout().name();
 			if (!is_undefined)
 				continue;
-	
+
 			// The flex inset is undefined in newtc
 			docstring const oldname = from_utf8(oldtc.name());
 			docstring const newname = from_utf8(newtc.name());
@@ -794,8 +832,9 @@ void switchBetweenClasses(DocumentClassConstPtr oldone,
 					layoutName, oldname, newname);
 			// To warn the user that something had to be done.
 			errorlist.push_back(ErrorItem(
-					_("Undefined flex inset"),
-					s, it.paragraph().id(),	it.pos(), it.pos() + 1));
+			                              _("Undefined flex inset"), s,
+			                              {it.paragraph().id(), it.pos()},
+			                              {it.paragraph().id(), it.pos()+1}));
 		} else if (code == TABULAR_CODE) {
 			// The recursion above does not catch paragraphs in "hidden" cells,
 			// i.e., ones that are part of a multirow or multicolum. So we need
@@ -886,11 +925,9 @@ void cutSelectionHelper(Cursor & cur, CutStack & cuts, bool doclear, bool realcu
 		if (begpit != endpit)
 			cur.screenUpdateFlags(Update::Force | Update::FitCursor);
 
-		boost::tie(endpit, endpos) =
-			eraseSelectionHelper(bp,
-				text->paragraphs(),
-				begpit, endpit,
-				cur.selBegin().pos(), endpos);
+		tie(endpit, endpos) =
+			eraseSelectionHelper(bp, text->paragraphs(), begpit, endpit,
+			                     cur.selBegin().pos(), endpos);
 
 		// cutSelection can invalidate the cursor so we need to set
 		// it anew. (Lgb)
@@ -932,7 +969,7 @@ void cutSelectionHelper(Cursor & cur, CutStack & cuts, bool doclear, bool realcu
 	}
 }
 
-}
+} // namespace
 
 void cutSelection(Cursor & cur, bool doclear, bool realcut)
 {
@@ -987,7 +1024,7 @@ void copySelectionToStack(Cursor const & cur, CutStack & cutstack)
 		// and sel_end cursor
 		copySelectionHelper(*cur.buffer(), *text,
 		                    cur.selBegin().pit(), cur.selEnd().pit(),
-		                    cur.selBegin().pos(), cur.selEnd().pos(), 
+		                    cur.selBegin().pos(), cur.selEnd().pos(),
 		                    cur.buffer()->params().documentClassPtr(),
 		                    cutstack);
 		// Reset the dirty_tabular_stack_ flag only when something
@@ -1009,7 +1046,7 @@ void copySelectionToStack(Cursor const & cur, CutStack & cutstack)
 	}
 }
 
-}
+} // namespace
 
 
 void copySelectionToStack()
@@ -1047,7 +1084,7 @@ void saveSelection(Cursor const & cur)
 	// This function is called, not when a selection is formed, but when
 	// a selection is cleared. Therefore, multiple keyboard selection
 	// will not repeatively trigger this function (bug 3877).
-	if (cur.selection() 
+	if (cur.selection()
 	    && cur.selBegin() == cur.bv().cursor().selBegin()
 	    && cur.selEnd() == cur.bv().cursor().selEnd()) {
 		LYXERR(Debug::SELECTION, "saveSelection: '" << cur.selectionAsString(true) << "'");
@@ -1080,7 +1117,8 @@ docstring selection(size_t sel_index, DocumentClassConstPtr docclass)
 	if (sel_index >= theCuts.size())
 		return docstring();
 
-	boost::scoped_ptr<Buffer> buffer(copyToTempBuffer(theCuts[sel_index].first, docclass));
+	unique_ptr<Buffer> buffer(copyToTempBuffer(theCuts[sel_index].first,
+	                                           docclass));
 	if (!buffer)
 		return docstring();
 
@@ -1100,7 +1138,7 @@ void pasteParagraphList(Cursor & cur, ParagraphList const & parlist,
 			pasteSelectionHelper(cur, parlist, docclass, branchAction, errorList);
 		cur.forceBufferUpdate();
 		cur.clearSelection();
-		text->setCursor(cur, prv.par, prv.pos);
+		text->setCursor(cur, prv.pit, prv.pos);
 	}
 
 	// mathed is handled in InsetMathNest/InsetMathGrid
@@ -1365,7 +1403,7 @@ void selClearOrDel(Cursor & cur)
 	if (lyxrc.auto_region_delete)
 		selDel(cur);
 	else
-		cur.setSelection(false);
+		cur.selection(false);
 }
 
 
