@@ -25,6 +25,7 @@
 #include "LyXRC.h"
 #include "output_latex.h"
 #include "OutputParams.h"
+#include "texstream.h"
 
 #include "support/lassert.h"
 #include "support/convert.h"
@@ -93,7 +94,7 @@ char const * LaTeXSizeNames[NUM_SIZE + 4] =
 { "tiny", "scriptsize", "footnotesize", "small", "normalsize", "large",
   "Large", "LARGE", "huge", "Huge", "error8", "error9", "error10", "error11" };
 
-} // namespace anon
+} // namespace
 
 
 Font::Font(FontInfo bits, Language const * l)
@@ -165,8 +166,11 @@ docstring const stateText(FontInfo const & f)
 		os << bformat(_("Underline %1$s, "),
 			      _(GUIMiscNames[f.underbar()]));
 	if (f.strikeout() != FONT_INHERIT)
-		os << bformat(_("Strikeout %1$s, "),
+		os << bformat(_("Strike out %1$s, "),
 			      _(GUIMiscNames[f.strikeout()]));
+	if (f.xout() != FONT_INHERIT)
+		os << bformat(_("Cross out %1$s, "),
+			      _(GUIMiscNames[f.xout()]));
 	if (f.uuline() != FONT_INHERIT)
 		os << bformat(_("Double underline %1$s, "),
 			      _(GUIMiscNames[f.uuline()]));
@@ -240,6 +244,9 @@ void Font::lyxWriteChanges(Font const & orgfont,
 	if (orgfont.fontInfo().strikeout() != bits_.strikeout()) {
 		os << "\\strikeout " << LyXMiscNames[bits_.strikeout()] << "\n";
 	}
+	if (orgfont.fontInfo().xout() != bits_.xout()) {
+		os << "\\xout " << LyXMiscNames[bits_.xout()] << "\n";
+	}
 	if (orgfont.fontInfo().uuline() != bits_.uuline()) {
 		os << "\\uuline " << LyXMiscNames[bits_.uuline()] << "\n";
 	}
@@ -344,16 +351,19 @@ int Font::latexWriteStartChanges(odocstream & os, BufferParams const & bparams,
 	// the numbers are written Left-to-Right. ArabTeX package
 	// reorders the number automatically but the packages used
 	// for Hebrew and Farsi (Arabi) do not.
-	if (bits_.number() == FONT_ON && prev.fontInfo().number() != FONT_ON
-		&& (language()->lang() == "hebrew"
-			|| language()->lang() == "farsi" 
-			|| language()->lang() == "arabic_arabi")) {
+	if (!runparams.pass_thru && bits_.number() == FONT_ON
+	    && prev.fontInfo().number() != FONT_ON
+	    && (language()->lang() == "hebrew"
+		|| language()->lang() == "farsi"
+		|| language()->lang() == "arabic_arabi")) {
 		os << "{\\beginL ";
 		count += 9;
 	}
 
 	FontInfo f = bits_;
 	f.reduce(base.bits_);
+	FontInfo p = bits_;
+	p.reduce(prev.bits_);
 
 	if (f.family() != INHERIT_FAMILY) {
 		os << '\\'
@@ -377,10 +387,16 @@ int Font::latexWriteStartChanges(odocstream & os, BufferParams const & bparams,
 		env = true; //We have opened a new environment
 	}
 	if (f.color() != Color_inherit && f.color() != Color_ignore) {
-		os << "\\textcolor{"
-		   << from_ascii(lcolor.getLaTeXName(f.color()))
-		   << "}{";
-		count += lcolor.getLaTeXName(f.color()).length() + 13;
+		if (f.color() == Color_none && p.color() != Color_none) {
+			// Color none: Close previous color, if any
+			os << '}';
+			++count;
+		} else if (f.color() != Color_none) {
+			os << "\\textcolor{"
+			   << from_ascii(lcolor.getLaTeXName(f.color()))
+			   << "}{";
+			count += lcolor.getLaTeXName(f.color()).length() + 13;
+		}
 		env = true; //We have opened a new environment
 	}
 	// FIXME: uncomment this when we support background.
@@ -428,6 +444,11 @@ int Font::latexWriteStartChanges(odocstream & os, BufferParams const & bparams,
 		count += 9;
 		++runparams.inulemcmd;
 	}
+	if (f.xout() == FONT_ON) {
+		os << "\\xout{";
+		count += 9;
+		++runparams.inulemcmd;
+	}
 	if (f.uuline() == FONT_ON) {
 		os << "\\uuline{";
 		count += 11;
@@ -476,7 +497,7 @@ int Font::latexWriteEndChanges(otexstream & os, BufferParams const & bparams,
 		++count;
 		env = true; // Size change need not bother about closing env.
 	}
-	if (f.color() != Color_inherit && f.color() != Color_ignore) {
+	if (f.color() != Color_inherit && f.color() != Color_ignore && f.color() != Color_none) {
 		os << '}';
 		++count;
 		env = true; // Size change need not bother about closing env.
@@ -513,6 +534,11 @@ int Font::latexWriteEndChanges(otexstream & os, BufferParams const & bparams,
 		++count;
 		--runparams.inulemcmd;
 	}
+	if (f.xout() == FONT_ON) {
+		os << '}';
+		++count;
+		--runparams.inulemcmd;
+	}
 	if (f.uuline() == FONT_ON) {
 		os << '}';
 		++count;
@@ -528,10 +554,11 @@ int Font::latexWriteEndChanges(otexstream & os, BufferParams const & bparams,
 	// the numbers are written Left-to-Right. ArabTeX package
 	// reorders the number automatically but the packages used
 	// for Hebrew and Farsi (Arabi) do not.
-	if (bits_.number() == FONT_ON && next.fontInfo().number() != FONT_ON
-		&& (language()->lang() == "hebrew"
-			|| language()->lang() == "farsi"
-			|| language()->lang() == "arabic_arabi")) {
+	if (!runparams.pass_thru && bits_.number() == FONT_ON
+	    && next.fontInfo().number() != FONT_ON
+	    && (language()->lang() == "hebrew"
+		|| language()->lang() == "farsi"
+		|| language()->lang() == "arabic_arabi")) {
 		os << "\\endL}";
 		count += 6;
 	}
@@ -577,6 +604,7 @@ string Font::toString(bool const toggle) const
 	   << "emph " << bits_.emph() << '\n'
 	   << "underbar " << bits_.underbar() << '\n'
 	   << "strikeout " << bits_.strikeout() << '\n'
+	   << "xout " << bits_.xout() << '\n'
 	   << "uuline " << bits_.uuline() << '\n'
 	   << "uwave " << bits_.uwave() << '\n'
 	   << "noun " << bits_.noun() << '\n'
@@ -619,10 +647,10 @@ bool Font::fromString(string const & data, bool & toggle)
 			int const next = lex.getInteger();
 			bits_.setSize(FontSize(next));
 
-		} else if (token == "emph" || token == "underbar" ||
-			   token == "noun" || token == "number" ||
-			   token == "uuline" || token == "uwave" ||
-			   token == "strikeout") {
+		} else if (token == "emph" || token == "underbar"
+			|| token == "noun" || token == "number"
+			|| token == "uuline" || token == "uwave"
+			|| token == "strikeout" || token == "xout") {
 
 			int const next = lex.getInteger();
 			FontState const misc = FontState(next);
@@ -633,6 +661,8 @@ bool Font::fromString(string const & data, bool & toggle)
 				bits_.setUnderbar(misc);
 			else if (token == "strikeout")
 				bits_.setStrikeout(misc);
+			else if (token == "xout")
+				bits_.setXout(misc);
 			else if (token == "uuline")
 				bits_.setUuline(misc);
 			else if (token == "uwave")
@@ -688,7 +718,12 @@ void Font::validate(LaTeXFeatures & features) const
 	if (bits_.strikeout() == FONT_ON) {
 		LYXERR(Debug::LATEX, "font.strikeout: " << bits_.strikeout());
 		features.require("ulem");
-		LYXERR(Debug::LATEX, "Strikeout enabled. Font: " << to_utf8(stateText(0)));
+		LYXERR(Debug::LATEX, "Strike out enabled. Font: " << to_utf8(stateText(0)));
+	}
+	if (bits_.xout() == FONT_ON) {
+		LYXERR(Debug::LATEX, "font.xout: " << bits_.xout());
+		features.require("ulem");
+		LYXERR(Debug::LATEX, "Cross out enabled. Font: " << to_utf8(stateText(0)));
 	}
 	if (bits_.uuline() == FONT_ON) {
 		LYXERR(Debug::LATEX, "font.uuline: " << bits_.uuline());
@@ -760,6 +795,7 @@ ostream & operator<<(ostream & os, FontInfo const & f)
 		<< " emph " << f.emph()
 		<< " underbar " << f.underbar()
 		<< " strikeout " << f.strikeout()
+		<< " xout " << f.xout()
 		<< " uuline " << f.uuline()
 		<< " uwave " << f.uwave()
 		<< " noun " << f.noun()
