@@ -23,6 +23,7 @@
 #include "support/gettext.h"
 #include "support/lstrings.h"
 #include "support/mutex.h"
+#include "support/docstream.h"
 #include "support/os.h"
 #include "support/PathChanger.h"
 #include "support/Systemcall.h"
@@ -101,20 +102,22 @@ private:
 };
 
 
-} //namespace anon
+} // namespace
 
 bool Format::formatSorter(Format const * lhs, Format const * rhs)
 {
-	return compare_locale(_(lhs->prettyname()), _(rhs->prettyname())) < 0;
+	return compare_locale(translateIfPossible(lhs->prettyname()),
+	                      translateIfPossible(rhs->prettyname())) < 0;
 }
 
 bool operator<(Format const & a, Format const & b)
 {
-	return compare_locale(_(a.prettyname()), _(b.prettyname())) < 0;
+	return compare_locale(translateIfPossible(a.prettyname()),
+	                      translateIfPossible(b.prettyname())) < 0;
 }
 
 
-Format::Format(string const & n, string const & e, string const & p,
+Format::Format(string const & n, string const & e, docstring const & p,
 	       string const & s, string const & v, string const & ed,
 	       string const & m, int flags)
 	: name_(n), prettyname_(p), shortcut_(s), viewer_(v),
@@ -169,9 +172,9 @@ void Format::setExtensions(string const & e)
 Format const * Formats::getFormat(string const & name) const
 {
 	FormatList::const_iterator cit =
-		find_if(formatlist.begin(), formatlist.end(),
+		find_if(formatlist_.begin(), formatlist_.end(),
 			FormatNamesEqual(name));
-	if (cit != formatlist.end())
+	if (cit != formatlist_.end())
 		return &(*cit);
 	else
 		return 0;
@@ -394,7 +397,7 @@ string guessFormatFromContents(FileName const & fn)
 	return string();
 }
 
-}
+} // namespace
 
 
 string Formats::getFormatFromFile(FileName const & filename) const
@@ -428,9 +431,9 @@ string Formats::getFormatFromFile(FileName const & filename) const
 				if (!mime.empty() && mime != "application/octet-stream" &&
 				    mime != "text/plain") {
 					Formats::const_iterator cit =
-						find_if(formatlist.begin(), formatlist.end(),
+						find_if(formatlist_.begin(), formatlist_.end(),
 							FormatMimeEqual(mime));
-					if (cit != formats.end()) {
+					if (cit != formatlist_.end()) {
 						LYXERR(Debug::GRAPHICS, "\tgot format from MIME type: "
 							<< mime << " -> " << cit->name());
 						// See special eps/ps handling below
@@ -466,9 +469,9 @@ string Formats::getFormatFromFile(FileName const & filename) const
 		}
 
 		if (isZippedFileFormat(format) && !ext.empty()) {
-			string const & fmt_name = formats.getFormatFromExtension(ext);
+			string const & fmt_name = getFormatFromExtension(ext);
 			if (!fmt_name.empty()) {
-				Format const * p_format = formats.getFormat(fmt_name);
+				Format const * p_format = getFormat(fmt_name);
 				if (p_format && p_format->zippedNative())
 					return p_format->name();
 			}
@@ -497,9 +500,9 @@ string Formats::getFormatFromExtension(string const & ext) const
 		// this is ambigous if two formats have the same extension,
 		// but better than nothing
 		Formats::const_iterator cit =
-			find_if(formatlist.begin(), formatlist.end(),
+			find_if(formatlist_.begin(), formatlist_.end(),
 				FormatExtensionsEqual(ext));
-		if (cit != formats.end()) {
+		if (cit != formatlist_.end()) {
 			LYXERR(Debug::GRAPHICS, "\twill guess format from file extension: "
 				<< ext << " -> " << cit->name());
 			return cit->name();
@@ -530,6 +533,10 @@ bool Formats::isZippedFile(support::FileName const & filename) const {
 	map<string, ZippedInfo>::iterator it = zipped_.find(fname);
 	if (it != zipped_.end() && it->second.timestamp == timestamp)
 		return it->second.zipped;
+	// FIXME perf: This very expensive function is called on startup on each
+	// file whic is going to be parsed, and also on svgz icons. Maybe there is a
+	// quicker way to check whether a file is zipped?  I.e. for icons we
+	// probably just need to check the extension (svgz vs svg).
 	string const & format = getFormatFromFile(filename);
 	bool zipped = (format == "gzip" || format == "zip");
 	zipped_.insert(make_pair(fname, ZippedInfo(zipped, timestamp)));
@@ -570,8 +577,8 @@ static string fixCommand(string const & cmd, string const & ext,
 
 void Formats::setAutoOpen()
 {
-	FormatList::iterator fit = formatlist.begin();
-	FormatList::iterator const fend = formatlist.end();
+	FormatList::iterator fit = formatlist_.begin();
+	FormatList::iterator const fend = formatlist_.end();
 	for ( ; fit != fend ; ++fit) {
 		fit->setViewer(fixCommand(fit->viewer(), fit->extension(), os::VIEW));
 		fit->setEditor(fixCommand(fit->editor(), fit->extension(), os::EDIT));
@@ -582,10 +589,10 @@ void Formats::setAutoOpen()
 int Formats::getNumber(string const & name) const
 {
 	FormatList::const_iterator cit =
-		find_if(formatlist.begin(), formatlist.end(),
+		find_if(formatlist_.begin(), formatlist_.end(),
 			FormatNamesEqual(name));
-	if (cit != formatlist.end())
-		return distance(formatlist.begin(), cit);
+	if (cit != formatlist_.end())
+		return distance(formatlist_.begin(), cit);
 	else
 		return -1;
 }
@@ -594,21 +601,21 @@ int Formats::getNumber(string const & name) const
 void Formats::add(string const & name)
 {
 	if (!getFormat(name))
-		add(name, name, name, string(), string(), string(),
+		add(name, name, from_utf8(name), string(), string(), string(),
 		    string(), Format::document);
 }
 
 
 void Formats::add(string const & name, string const & extensions,
-		  string const & prettyname, string const & shortcut,
+		  docstring const & prettyname, string const & shortcut,
 		  string const & viewer, string const & editor,
 		  string const & mime, int flags)
 {
 	FormatList::iterator it =
-		find_if(formatlist.begin(), formatlist.end(),
+		find_if(formatlist_.begin(), formatlist_.end(),
 			FormatNamesEqual(name));
-	if (it == formatlist.end())
-		formatlist.push_back(Format(name, extensions, prettyname,
+	if (it == formatlist_.end())
+		formatlist_.push_back(Format(name, extensions, prettyname,
 					    shortcut, viewer, editor, mime, flags));
 	else
 		*it = Format(name, extensions, prettyname, shortcut, viewer,
@@ -619,16 +626,16 @@ void Formats::add(string const & name, string const & extensions,
 void Formats::erase(string const & name)
 {
 	FormatList::iterator it =
-		find_if(formatlist.begin(), formatlist.end(),
+		find_if(formatlist_.begin(), formatlist_.end(),
 			FormatNamesEqual(name));
-	if (it != formatlist.end())
-		formatlist.erase(it);
+	if (it != formatlist_.end())
+		formatlist_.erase(it);
 }
 
 
 void Formats::sort()
 {
-	std::sort(formatlist.begin(), formatlist.end());
+	std::sort(formatlist_.begin(), formatlist_.end());
 }
 
 
@@ -636,9 +643,9 @@ void Formats::setViewer(string const & name, string const & command)
 {
 	add(name);
 	FormatList::iterator it =
-		find_if(formatlist.begin(), formatlist.end(),
+		find_if(formatlist_.begin(), formatlist_.end(),
 			FormatNamesEqual(name));
-	if (it != formatlist.end())
+	if (it != formatlist_.end())
 		it->setViewer(command);
 }
 
@@ -647,9 +654,9 @@ void Formats::setEditor(string const & name, string const & command)
 {
 	add(name);
 	FormatList::iterator it =
-		find_if(formatlist.begin(), formatlist.end(),
+		find_if(formatlist_.begin(), formatlist_.end(),
 			FormatNamesEqual(name));
-	if (it != formatlist.end())
+	if (it != formatlist_.end())
 		it->setEditor(command);
 }
 
@@ -798,7 +805,7 @@ docstring const Formats::prettyName(string const & name) const
 {
 	Format const * format = getFormat(name);
 	if (format)
-		return from_utf8(format->prettyname());
+		return format->prettyname();
 	else
 		return from_utf8(name);
 }
@@ -850,7 +857,7 @@ FlavorTranslator const & flavorTranslator()
 	return translator;
 }
 
-}
+} // namespace
 
 
 std::string flavor2format(OutputParams::FLAVOR flavor)
@@ -865,9 +872,4 @@ OutputParams::FLAVOR format2flavor(std::string fmt)
 	return flavorTranslator().find(fmt);
 } */
 
-Formats formats;
-
-Formats system_formats;
-
-
 } // namespace lyx
