@@ -31,49 +31,44 @@ FuncRequest const FuncRequest::noaction(LFUN_NOACTION);
 
 FuncRequest::FuncRequest(Origin o)
 	: action_(LFUN_NOACTION), origin_(o), x_(0), y_(0),
-	  button_(mouse_button::none)
+	  button_(mouse_button::none), modifier_(NoModifier)
 {}
 
 
 FuncRequest::FuncRequest(FuncCode act, Origin o)
 	: action_(act), origin_(o), x_(0), y_(0),
-	button_(mouse_button::none)
+	button_(mouse_button::none), modifier_(NoModifier)
 {}
 
 
 FuncRequest::FuncRequest(FuncCode act, docstring const & arg, Origin o)
 	: action_(act), argument_(arg), origin_(o), x_(0), y_(0),
-	  button_(mouse_button::none)
+	  button_(mouse_button::none), modifier_(NoModifier)
 {}
 
 
 FuncRequest::FuncRequest(FuncCode act, string const & arg, Origin o)
 	: action_(act), argument_(from_utf8(arg)), origin_(o), x_(0), y_(0),
-	  button_(mouse_button::none)
+	  button_(mouse_button::none), modifier_(NoModifier)
 {}
 
 
 FuncRequest::FuncRequest(FuncCode act, int ax, int ay,
-			 mouse_button::state but, Origin o)
-	: action_(act), origin_(o), x_(ax), y_(ay), button_(but)
+			 mouse_button::state but, KeyModifier modifier, Origin o)
+	: action_(act), origin_(o), x_(ax), y_(ay), button_(but),
+	  modifier_(modifier)
 {}
 
 
 FuncRequest::FuncRequest(FuncRequest const & cmd, docstring const & arg, Origin o)
 	: action_(cmd.action()), argument_(arg), origin_(o),
-	  x_(cmd.x_), y_(cmd.y_), button_(cmd.button_)
-{}
-
-
-FuncRequest::FuncRequest(FuncRequest const & cmd, string const & arg, Origin o)
-	: action_(cmd.action()), argument_(from_utf8(arg)), origin_(o),
-	  x_(cmd.x_), y_(cmd.y_), button_(cmd.button_)
+	  x_(cmd.x_), y_(cmd.y_), button_(cmd.button_), modifier_(NoModifier)
 {}
 
 
 namespace {
 
-// Extracts arguments from str into args. Arguments are delimted by
+// Extracts arguments from str into args. Arguments are delimited by
 // whitespace or by double quotes.
 // We extract at most max + 1 arguments, treating args[max] as
 // continuing to eol.
@@ -106,7 +101,7 @@ void splitArg(vector<string> & args, string const & str,
 	}
 }
 
-}
+} // namespace
 
 string FuncRequest::getArg(unsigned int i) const
 {
