@@ -16,6 +16,7 @@
 
 #include "support/docstring.h"
 
+#include "frontends/KeyModifier.h"
 #include "frontends/mouse_state.h"
 
 
@@ -36,7 +37,7 @@ public:
 		MENU, // A menu entry
 		TOOLBAR, // A toolbar icon
 		KEYBOARD, // a keyboard binding
-		COMMANDBUFFER, 
+		COMMANDBUFFER,
 		LYXSERVER,
 		TOC
 	};
@@ -47,7 +48,7 @@ public:
 	explicit FuncRequest(FuncCode act, Origin o = INTERNAL);
 	/// actions without extra argument
 	FuncRequest(FuncCode act, int x, int y, mouse_button::state button,
-		    Origin o = INTERNAL);
+		    KeyModifier modifier, Origin o = INTERNAL);
 	/// actions with extra argument
 	FuncRequest(FuncCode act, docstring const & arg,
 		    Origin o = INTERNAL);
@@ -57,9 +58,6 @@ public:
 	/// for changing requests a bit
 	FuncRequest(FuncRequest const & cmd, docstring const & arg,
 		    Origin o = INTERNAL);
-	/// for changing requests a bit. FIXME: remove this
-	FuncRequest(FuncRequest const & cmd, std::string const & arg,
-		    Origin o = INTERNAL);
 
 	/// access the whole argument
 	docstring const & argument() const { return argument_; }
@@ -77,8 +75,10 @@ public:
 	int y() const { return y_; }
 	///
 	void set_y(int y) { y_ = y; }
-	/// 
+	///
 	mouse_button::state button() const { return button_; }
+	///
+	KeyModifier modifier() { return modifier_; }
 
 	/// argument parsing, extract argument i as std::string
 	std::string getArg(unsigned int i) const;
@@ -86,9 +86,9 @@ public:
 	/// eating all characters up to the end of the command line
 	std::string getLongArg(unsigned int i) const;
 
-	/// 
+	///
 	static FuncRequest const unknown;
-	/// 
+	///
 	static FuncRequest const noaction;
 private:
 	/// the action
@@ -103,6 +103,8 @@ private:
 	int y_;
 	/// some extra information (like button number)
 	mouse_button::state button_;
+	///
+	KeyModifier modifier_;
 };
 
 
