@@ -27,6 +27,7 @@ class Buffer;
 class BufferParams;
 class InsetLayout;
 class Language;
+struct TexString;
 
 /** The packages and commands that a buffer needs. This class
  *  contains a list<string>.  Each of the LaTeX packages that a buffer needs
@@ -60,11 +61,11 @@ public:
 	/// The packages needed by the document
 	std::string const getPackages() const;
 	/// The macros definitions needed by the document
-	docstring const getMacros() const;
+	TexString getMacros() const;
 	/// Extra preamble code before babel is called
-	std::string const getBabelPresettings() const;
+	docstring const getBabelPresettings() const;
 	/// Extra preamble code after babel is called
-	std::string const getBabelPostsettings() const;
+	docstring const getBabelPostsettings() const;
 	/// Do we need to pass the languages to babel directly?
 	bool needBabelLangOptions() const;
 	/// Load AMS packages when appropriate
@@ -72,7 +73,8 @@ public:
 	/// The definitions needed by the document's textclass
 	docstring const getTClassPreamble() const;
 	/// The language dependent definitions needed by the document's textclass
-	docstring const getTClassI18nPreamble(bool use_babel, bool use_polyglossia) const;
+	docstring const getTClassI18nPreamble(bool use_babel,
+				bool use_polyglossia, bool use_minted) const;
 	///
 	docstring const getTClassHTMLStyles() const;
 	///
@@ -84,18 +86,19 @@ public:
 	/// Include a file for use with the SGML entities
 	void includeFile(docstring const & key, std::string const & name);
 	/// The float definitions.
-	void getFloatDefinitions(odocstream & os) const;
+	void getFloatDefinitions(otexstream & os) const;
 	/// Print requirements to lyxerr
 	void showStruct() const;
+	/// Add preamble snippet with TexRow information
+	void addPreambleSnippet(TexString snippet, bool allowdupes = false);
+	/// Add preamble snippet without TexRow information
+	void addPreambleSnippet(docstring const & snippet, bool allowdupes = false);
 	///
-	void addPreambleSnippet(std::string const & snippet, 
-	                        bool allowdupes = false);
-	///
-	std::string getPreambleSnippets() const;
+	TexString getPreambleSnippets() const;
 	///
 	void addCSSSnippet(std::string const &);
 	///
-	std::string getCSSSnippets() const;
+	docstring getCSSSnippets() const;
 	/// Add a feature name requirements
 	void require(std::string const & name);
 	/// Add a set of feature names requirements
@@ -160,10 +163,16 @@ public:
 	bool inDeletedInset() const { return in_deleted_inset_; }
 	/// are we in a deleted inset?
 	void inDeletedInset(bool const b) { in_deleted_inset_ = b; }
+	/// set savenote environment (footnote package)
+	std::string saveNoteEnv() const { return savenote_env_; }
+	/// return savenote environment
+	void saveNoteEnv(std::string const s) { savenote_env_ = s; }
 	/// Runparams that will be used for exporting this file.
 	OutputParams const & runparams() const { return runparams_; }
 	/// Resolve alternatives like "esint|amsmath|wasysym"
 	void resolveAlternatives();
+	/// Expand multiple requirements like "textcomp,lyxmathsym,amstext"
+	void expandMultiples();
 	///
 	void setHTMLTitle(docstring const & t) { htmltitle_ = t; }
 	///
@@ -181,7 +190,7 @@ private:
 	///
 	Features features_;
 	/// Static preamble bits, from external templates, or anywhere else
-	typedef std::list<std::string> SnippetList;
+	typedef std::list<TexString> SnippetList;
 	///
 	SnippetList preamble_snippets_;
 	///
@@ -216,6 +225,8 @@ private:
 	bool in_deleted_inset_;
 	///
 	docstring htmltitle_;
+	///
+	std::string savenote_env_;
 };
 
 
