@@ -70,6 +70,7 @@ public:
 		RC_DEFAULT_DECIMAL_POINT,
 		RC_DEFAULT_LENGTH_UNIT,
 		RC_DEFAULT_OTF_VIEW_FORMAT,
+		RC_DEFAULT_PLATEX_VIEW_FORMAT,
 		RC_DEFAULT_VIEW_FORMAT,
 		RC_DEFFILE,
 		RC_DIALOGS_ICONIFY_WITH_MAIN,
@@ -98,6 +99,7 @@ public:
 		RC_INDEX_ALTERNATIVES,
 		RC_INDEX_COMMAND,
 		RC_INPUT,
+		RC_JBIBTEX_ALTERNATIVES,
 		RC_JBIBTEX_COMMAND,
 		RC_JINDEX_COMMAND,
 		RC_KBMAP,
@@ -132,6 +134,7 @@ public:
 		RC_PRINTLANDSCAPEFLAG,
 		RC_PRINTPAPERDIMENSIONFLAG,
 		RC_PRINTPAPERFLAG,
+		RC_PYGMENTIZE_COMMAND,
 		RC_SAVE_COMPRESSED,
 		RC_SAVE_ORIGIN,
 		RC_SCREEN_DPI,
@@ -167,6 +170,8 @@ public:
 		RC_USER_EMAIL,
 		RC_USER_NAME,
 		RC_USE_CONVERTER_CACHE,
+		RC_USE_CONVERTER_NEEDAUTH_FORBIDDEN,
+		RC_USE_CONVERTER_NEEDAUTH,
 		RC_USE_SYSTEM_COLORS,
 		RC_USE_TOOLTIP,
 		RC_USE_PIXMAP_CACHE,
@@ -238,6 +243,8 @@ public:
 	std::string bibtex_command;
 	/// command to run japanese bibtex incl. options
 	std::string jbibtex_command;
+	/// all available commands to run japanese bibtex incl. options
+	CommandSet jbibtex_alternatives;
 	/// all available index commands incl. options
 	CommandSet index_alternatives;
 	/// command to run makeindex incl. options or other index programs
@@ -248,6 +255,8 @@ public:
 	std::string splitindex_command;
 	/// command to run makeindex incl. options for nomencl
 	std::string nomencl_command;
+	/// command to run the python pygments syntax highlighter
+	std::string pygmentize_command;
 	///
 	std::string document_path;
 	///
@@ -287,8 +296,11 @@ public:
 	bool allow_geometry_session;
 	/// Scrolling speed of the mouse wheel
 	double mouse_wheel_speed;
-	/// Zoom factor for screen fonts
-	unsigned int zoom;
+	/// Default zoom factor for screen fonts
+	int defaultZoom;
+	/// Actual zoom factor for screen fonts
+	/// (default zoom plus buffer zoom factor)
+	int currentZoom;
 	/// Screen font sizes in points for each font size
 	std::string font_sizes[10];
 	/// Allow the use of scalable fonts? Default is yes.
@@ -381,6 +393,8 @@ public:
 	///
 	std::string default_otf_view_format;
 	///
+	std::string default_platex_view_format;
+	///
 	std::string default_view_format;
 	/// all available viewers
 	Alternatives viewer_alternatives;
@@ -443,6 +457,10 @@ public:
 	std::string texinputs_prefix;
 	/// Use the cache for file converters?
 	bool use_converter_cache;
+	/// Forbid use of external converters with 'needauth' option
+	bool use_converter_needauth_forbidden;
+	/// Ask user before calling external converters with 'needauth' option
+	bool use_converter_needauth;
 	/// The maximum age of cache files in seconds
 	unsigned int converter_cache_maxage;
 	/// Sort layouts alphabetically
