@@ -13,9 +13,10 @@
 #define OUTPUTPARAMS_H
 
 
-#include "support/shared_ptr.h"
 #include "Changes.h"
 
+#include <memory>
+
 
 namespace lyx {
 
@@ -78,6 +79,8 @@ public:
 	    This esentially seems to mean whether InsetInclude, InsetGraphics
 	    and InsetExternal should add the absolute path to any external
 	    files or not.
+	    Non-nice LaTeX also includes additional safe line breaks in order to
+	    increase the precision of forward/reverse search and error reporting.
 	*/
 	bool nice;
 
@@ -101,7 +104,7 @@ public:
 
 	/** inulemcmd > 0 means that the environment in which the
 	    inset is typeset is part of a ulem command (\uline, \uuline,
-	    \uwave, or \sout). Insets that output latex commands relying
+	    \uwave, \sout  or \xout). Insets that output latex commands relying
 	    on local assignments (such as \cite) should enclose such
 	    commands in \mbox{} in order to avoid breakage.
 	*/
@@ -182,7 +185,7 @@ public:
 	    This is a hack: Make it possible to add stuff to constant
 	    OutputParams instances.
 	*/
-	shared_ptr<ExportData> exportdata;
+	std::shared_ptr<ExportData> exportdata;
 
 	/** Whether we are entering a display math inset.
 	 *  Needed to correctly strike out deleted math in change tracking.
@@ -202,6 +205,15 @@ public:
 	 */
 	bool inComment;
 
+	/** Whether a btUnit (for multiple biblographies) is open.
+	 */
+	mutable bool openbtUnit;
+
+	/** Process only the children's aux files with BibTeX.
+	 *  This is necessary with chapterbib.
+	 */
+	bool only_childbibs;
+
 	/** Whether we are in a table cell.
 	 *  For newline, it matters whether its content is aligned or not.
          */
@@ -247,13 +259,13 @@ public:
 	mutable int lastid;
 
 	/// Last position in the last paragraph before an inset
-	mutable int lastpos;
+	mutable pos_type lastpos;
 
 	/// is this the last paragraph in the current buffer/inset?
 	bool isLastPar;
 
 
-	/** whether or not do actual file copying and image conversion
+	/** whether or not to do actual file copying and image conversion
 	 *  This mode will be used to preview the source code
 	 */
 	bool dryrun;
