@@ -17,6 +17,7 @@
 #define PARAGRAPH_H
 
 #include "FontEnums.h"
+#include "LayoutEnums.h"
 #include "SpellChecker.h"
 
 #include "support/strfwd.h"
@@ -39,17 +40,15 @@ class DocumentClass;
 class Inset;
 class InsetBibitem;
 class LaTeXFeatures;
-class Inset_code;
 class InsetList;
 class Language;
 class Layout;
 class Font;
-class Font_size;
 class MetricsInfo;
 class OutputParams;
 class PainterInfo;
 class ParagraphParameters;
-class Toc;
+class TocBackend;
 class WordLangTuple;
 class XHTMLStream;
 class otexstream;
@@ -69,7 +68,7 @@ public:
 	{
 		return first < s.first;
 	}
-	
+
 	inline bool operator==(FontSpan const & s) const
 	{
 		return first == s.first && last == s.last;
@@ -84,7 +83,7 @@ public:
 	{
 		return empty() ? 0 : last - first;
 	}
-	
+
 
 	inline FontSpan intersect(FontSpan const & f) const
 	{
@@ -101,7 +100,7 @@ public:
 			result.last = last;
 		return result;
 	}
-	
+
 	inline bool empty() const
 	{
 		return first > last;
@@ -153,7 +152,7 @@ public:
 
 	///
 	void addChangesToToc(DocIterator const & cdit, Buffer const & buf,
-	        bool output_active) const;
+	                     bool output_active, TocBackend & backend) const;
 	/// set the buffer flag if there are changes in the paragraph
 	void addChangesToBuffer(Buffer const & buf) const;
 	///
@@ -184,8 +183,8 @@ public:
 			   int options = AS_STR_NONE,
 			   const OutputParams *runparams = 0) const;
 	///
-	void forOutliner(docstring &, size_t const maxlen,
-					 bool const shorten = true) const;
+	void forOutliner(docstring &, size_t maxlen, bool shorten = true,
+	                 bool label = true) const;
 
 	///
 	void write(std::ostream &, BufferParams const &,
@@ -311,7 +310,7 @@ public:
 	/// Set label width string.
 	void setLabelWidthString(docstring const & s);
 	/// Actual paragraph alignment used
-	char getAlign() const;
+	LyXAlignment getAlign() const;
 	/// The nesting depth of a paragraph
 	depth_type getDepth() const;
 	/// The maximal possible depth of a paragraph after this one
@@ -362,9 +361,6 @@ public:
 	char_type getUChar(BufferParams const &, pos_type pos) const;
 	/// pos <= size() (there is a dummy font change at the end of each par)
 	void setFont(pos_type pos, Font const & font);
-	/// Returns the height of the highest font in range
-	FontSize highestFontInRange(pos_type startpos,
-					pos_type endpos, FontSize def_size) const;
 	///
 	void insert(pos_type pos, docstring const & str,
 		    Font const & font, Change const & change);
@@ -402,6 +398,8 @@ public:
 	InsetList const & insetList() const;
 	///
 	void setBuffer(Buffer &);
+	///
+	void resetBuffer();
 
 	///
 	bool isHfill(pos_type pos) const;
@@ -470,7 +468,7 @@ public:
 		bool mw, ///<
 		pos_type pos, ///< start from here.
 		bool del = true) const;
-	
+
 	void locateWord(pos_type & from, pos_type & to,
 		word_location const loc) const;
 	///
