@@ -26,7 +26,6 @@
 #include "Row.h"
 #include "MetricsInfo.h"
 #include "Paragraph.h"
-#include "ParagraphMetrics.h"
 #include "ParagraphParameters.h"
 #include "TextMetrics.h"
 #include "VSpace.h"
@@ -45,6 +44,8 @@
 #include "support/lassert.h"
 #include <boost/crc.hpp>
 
+#include <stdlib.h>
+
 using namespace std;
 
 namespace lyx {
@@ -54,38 +55,21 @@ using frontend::FontMetrics;
 
 
 RowPainter::RowPainter(PainterInfo & pi,
-	Text const & text, pit_type pit, Row const & row, int x, int y)
+	Text const & text, Row const & row, int x, int y)
 	: pi_(pi), text_(text),
-	  text_metrics_(pi_.base.bv->textMetrics(&text)),
+	  tm_(pi_.base.bv->textMetrics(&text)),
 	  pars_(text.paragraphs()),
-	  row_(row), pit_(pit), par_(text.paragraphs()[pit]),
-	  pm_(text_metrics_.parMetrics(pit)), change_(pi_.change_),
-	  xo_(x), yo_(y), width_(text_metrics_.width()),
-	  solid_line_thickness_(1), solid_line_offset_(1),
-	  dotted_line_thickness_(1)
+	  row_(row), par_(text.paragraphs()[row.pit()]),
+	  change_(pi_.change_),
+	  xo_(x), yo_(y)
 {
-	if (lyxrc.zoom >= 200) {
-		// derive the line thickness from zoom factor
-		// the zoom is given in percent
-		// (increase thickness at 250%, 450% etc.)
-		solid_line_thickness_ = (lyxrc.zoom + 50) / 200;
-		// adjust line_offset_ too
-		solid_line_offset_ = 1 + solid_line_thickness_ / 2;
-	}
-	if (lyxrc.zoom >= 100) {
-		// derive the line thickness from zoom factor
-		// the zoom is given in percent
-		// (increase thickness at 150%, 250% etc.)
-		dotted_line_thickness_ = (lyxrc.zoom + 50) / 100;
-	}
-
 	x_ = row_.left_margin + xo_;
 
 	//lyxerr << "RowPainter: x: " << x_ << " xo: " << xo_ << " yo: " << yo_ << endl;
 	//row_.dump();
 
-	LBUFERR(pit >= 0);
-	LBUFERR(pit < int(text.paragraphs().size()));
+	LBUFERR(row.pit() >= 0);
+	LBUFERR(row.pit() < int(text.paragraphs().size()));
 }
 
 
@@ -104,44 +88,36 @@ FontInfo RowPainter::labelFont() const
 // This draws green lines around each inset.
 
 
-void RowPainter::paintInset(Inset const * inset, Font const & font,
-                            Change const & change,
-                            pos_type const pos)
+void RowPainter::paintInset(Row::Element const & e) const
 {
 	// Handle selection
 	bool const pi_selected = pi_.selected;
 	Cursor const & cur = pi_.base.bv->cursor();
 	if (cur.selection() && cur.text() == &text_
 		&& cur.normalAnchor().text() == &text_)
-		pi_.selected = row_.sel_beg <= pos && row_.sel_end > pos;
+		pi_.selected = row_.sel_beg <= e.pos && row_.sel_end > e.pos;
 
-	LASSERT(inset, return);
+	LASSERT(e.inset, return);
 	// Backup full_repaint status because some insets (InsetTabular)
 	// requires a full repaint
 	bool const pi_full_repaint = pi_.full_repaint;
 	bool const pi_do_spellcheck = pi_.do_spellcheck;
 	Change const pi_change = pi_.change_;
 
-	pi_.base.font = inset->inheritFont() ? font.fontInfo() :
+	pi_.base.font = e.inset->inheritFont() ? e.font.fontInfo() :
 		pi_.base.bv->buffer().params().getFont().fontInfo();
-	pi_.ltr_pos = !font.isVisibleRightToLeft();
-	pi_.change_ = change_.changed() ? change_ : change;
-	pi_.do_spellcheck &= inset->allowSpellCheck();
+	pi_.ltr_pos = !e.font.isVisibleRightToLeft();
+	pi_.change_ = change_.changed() ? change_ : e.change;
+	pi_.do_spellcheck &= e.inset->allowSpellCheck();
 
 	int const x1 = int(x_);
-	pi_.base.bv->coordCache().insets().add(inset, x1, yo_);
+	pi_.base.bv->coordCache().insets().add(e.inset, x1, yo_);
 	// insets are painted completely. Recursive
 	// FIXME: it is wrong to completely paint the background
 	// if we want to do single row painting.
-	inset->drawBackground(pi_, x1, yo_);
-	inset->drawSelection(pi_, x1, yo_);
-	inset->draw(pi_, x1, yo_);
-
-	Dimension const & dim = pi_.base.bv->coordCache().insets().dim(inset);
-
-	paintForeignMark(x_, font.language(), dim.descent());
-
-	x_ += dim.width();
+	e.inset->drawBackground(pi_, x1, yo_);
+	e.inset->drawSelection(pi_, x1, yo_);
+	e.inset->draw(pi_, x1, yo_);
 
 	// Restore full_repaint status.
 	pi_.full_repaint = pi_full_repaint;
@@ -150,6 +126,7 @@ void RowPainter::paintInset(Inset const * inset, Font const & font,
 	pi_.selected = pi_selected;
 
 #ifdef DEBUG_METRICS
+	Dimension const & dim = pi_.base.bv->coordCache().insets().dim(e.inset);
 	int const x2 = x1 + dim.wid;
 	int const y1 = yo_ + dim.des;
 	int const y2 = yo_ - dim.asc;
@@ -161,8 +138,9 @@ void RowPainter::paintInset(Inset const * inset, Font const & font,
 }
 
 
-void RowPainter::paintForeignMark(double orig_x, Language const * lang, int desc) const
+void RowPainter::paintForeignMark(Row::Element const & e) const
 {
+	Language const * lang = e.font.language();
 	if (!lyxrc.mark_foreign_language)
 		return;
 	if (lang == latex_language)
@@ -170,21 +148,22 @@ void RowPainter::paintForeignMark(double orig_x, Language const * lang, int desc
 	if (lang == pi_.base.bv->buffer().params().language)
 		return;
 
-	int const y = yo_ + solid_line_offset_ + desc + solid_line_thickness_ / 2;
-	pi_.pain.line(int(orig_x), y, int(x_), y, Color_language,
-		Painter::line_solid, solid_line_thickness_);
+	int const desc = e.inset ? e.dim.descent() : 0;
+	int const y = yo_ + pi_.base.solidLineOffset()
+		+ desc + pi_.base.solidLineThickness() / 2;
+	pi_.pain.line(int(x_), y, int(x_ + e.full_width()), y, Color_language,
+	              Painter::line_solid, pi_.base.solidLineThickness());
 }
 
 
-void RowPainter::paintMisspelledMark(double const orig_x,
-                                     Row::Element const & e) const
+void RowPainter::paintMisspelledMark(Row::Element const & e) const
 {
 	// if changed the misspelled marker gets placed slightly lower than normal
 	// to avoid drawing at the same vertical offset
 	FontMetrics const & fm = theFontMetrics(e.font);
 	int const thickness = max(fm.lineWidth(), 2);
-	int const y = yo_ + solid_line_offset_ + solid_line_thickness_
-		+ (e.change.changed() ? solid_line_thickness_ + 1 : 0)
+	int const y = yo_ + pi_.base.solidLineOffset() + pi_.base.solidLineThickness()
+		+ (e.change.changed() ? pi_.base.solidLineThickness() + 1 : 0)
 		+ 1 + thickness / 2;
 
 	//FIXME: this could be computed only once, it is probably not costly.
@@ -226,7 +205,7 @@ void RowPainter::paintMisspelledMark(double const orig_x,
 		if (x1 > x2)
 			swap(x1, x2);
 
-		pi_.pain.line(int(orig_x) + x1, y, int(orig_x) + x2, y,
+		pi_.pain.line(int(x_ + x1), y, int(x_ + x2), y,
 		              Color_error,
 		              Painter::line_onoffdash, thickness);
 		pos = range.last + 1;
@@ -234,7 +213,7 @@ void RowPainter::paintMisspelledMark(double const orig_x,
 }
 
 
-void RowPainter::paintStringAndSel(Row::Element const & e)
+void RowPainter::paintStringAndSel(Row::Element const & e) const
 {
 	// at least part of text selected?
 	bool const some_sel = (e.endpos >= row_.sel_beg && e.pos < row_.sel_end)
@@ -248,29 +227,21 @@ void RowPainter::paintStringAndSel(Row::Element const & e)
 		Color const col = e.change.changed() ? e.change.color()
 		                                     : Color_selectiontext;
 		copy.fontInfo().setPaintColor(col);
-		pi_.pain.text(int(x_), yo_, e.str, copy, e.extra);
+		pi_.pain.text(int(x_), yo_, e.str, copy, e.extra, e.full_width());
 	} else if (!some_sel) {
-		pi_.pain.text(int(x_), yo_, e.str, e.font, e.extra);
+		pi_.pain.text(int(x_), yo_, e.str, e.font, e.extra, e.full_width());
 	} else {
 		pi_.pain.text(int(x_), yo_, e.str, e.font, Color_selectiontext,
-				    max(row_.sel_beg, e.pos) - e.pos,
-                        min(row_.sel_end, e.endpos) - e.pos, e.extra);
+		              max(row_.sel_beg, e.pos) - e.pos,
+		              min(row_.sel_end, e.endpos) - e.pos,
+		              e.extra, e.full_width());
 	}
-	x_ += e.full_width();
 }
 
 
-void RowPainter::paintChange(double orig_x, Font const & font,
-                             Change const & change) const
+void RowPainter::paintChange(Row::Element const & e) const
 {
-	if (!change.changed())
-		return;
-	// Calculate 1/3 height of font
-	FontMetrics const & fm = theFontMetrics(font);
-	int const y_bar = change.deleted() ? yo_ - fm.maxAscent() / 3
-		: yo_ + 2 * solid_line_offset_ + solid_line_thickness_;
-	pi_.pain.line(int(orig_x), y_bar, int(x_), y_bar,
-	              change.color(), Painter::line_solid, solid_line_thickness_);
+	e.change.paintCue(pi_, x_, yo_, x_ + e.full_width(), e.font.fontInfo());
 }
 
 
@@ -288,7 +259,7 @@ void RowPainter::paintChangeBar() const
 	if (start == end || !par_.isChanged(start, end))
 		return;
 
-	int const height = text_metrics_.isLastRow(pit_, row_)
+	int const height = tm_.isLastRow(row_)
 		? row_.ascent()
 		: row_.height();
 
@@ -308,7 +279,7 @@ void RowPainter::paintAppendix() const
 		y += 2 * defaultRowHeight();
 
 	pi_.pain.line(1, y, 1, yo_ + row_.height(), Color_appendix);
-	pi_.pain.line(width_ - 2, y, width_ - 2, yo_ + row_.height(), Color_appendix);
+	pi_.pain.line(tm_.width() - 2, y, tm_.width() - 2, yo_ + row_.height(), Color_appendix);
 }
 
 
@@ -320,16 +291,16 @@ void RowPainter::paintDepthBar() const
 		return;
 
 	depth_type prev_depth = 0;
-	if (!text_metrics_.isFirstRow(pit_, row_)) {
-		pit_type pit2 = pit_;
+	if (!tm_.isFirstRow(row_)) {
+		pit_type pit2 = row_.pit();
 		if (row_.pos() == 0)
 			--pit2;
 		prev_depth = pars_[pit2].getDepth();
 	}
 
 	depth_type next_depth = 0;
-	if (!text_metrics_.isLastRow(pit_, row_)) {
-		pit_type pit2 = pit_;
+	if (!tm_.isLastRow(row_)) {
+		pit_type pit2 = row_.pit();
 		if (row_.endpos() >= pars_[pit2].size())
 			++pit2;
 		next_depth = pars_[pit2].getDepth();
@@ -368,29 +339,30 @@ void RowPainter::paintAppendixStart(int y) const
 	docstring const label = _("Appendix");
 	theFontMetrics(pb_font).rectText(label, w, a, d);
 
-	int const text_start = int(xo_ + (width_ - w) / 2);
+	int const text_start = int(xo_ + (tm_.width() - w) / 2);
 	int const text_end = text_start + w;
 
 	pi_.pain.rectText(text_start, y + d, label, pb_font, Color_none, Color_none);
 
 	pi_.pain.line(int(xo_ + 1), y, text_start, y, Color_appendix);
-	pi_.pain.line(text_end, y, int(xo_ + width_ - 2), y, Color_appendix);
+	pi_.pain.line(text_end, y, int(xo_ + tm_.width() - 2), y, Color_appendix);
 }
 
 
 void RowPainter::paintTooLargeMarks(bool const left, bool const right) const
 {
 	if (left)
-		pi_.pain.line(dotted_line_thickness_, yo_ - row_.ascent(),
-					  dotted_line_thickness_, yo_ + row_.descent(),
-					  Color_scroll,
-					  Painter::line_onoffdash, dotted_line_thickness_);
+		pi_.pain.line(pi_.base.dottedLineThickness(), yo_ - row_.ascent(),
+					  pi_.base.dottedLineThickness(), yo_ + row_.descent(),
+					  Color_scroll, Painter::line_onoffdash,
+		              pi_.base.dottedLineThickness());
 	if (right) {
-		int const wwidth = pi_.base.bv->workWidth() - dotted_line_thickness_;
+		int const wwidth =
+			pi_.base.bv->workWidth() - pi_.base.dottedLineThickness();
 		pi_.pain.line(wwidth, yo_ - row_.ascent(),
 					  wwidth, yo_ + row_.descent(),
-					  Color_scroll,
-					  Painter::line_onoffdash, dotted_line_thickness_);
+					  Color_scroll, Painter::line_onoffdash,
+		              pi_.base.dottedLineThickness());
 	}
 }
 
@@ -404,7 +376,7 @@ void RowPainter::paintFirst() const
 	    paintAppendixStart(yo_ - row_.ascent() + 2 * defaultRowHeight());
 
 	bool const is_first =
-		text_.isFirstInSequence(pit_) || !layout.isParagraphGroup();
+		text_.isFirstInSequence(row_.pit()) || !layout.isParagraphGroup();
 	//lyxerr << "paintFirst: " << par_.id() << " is_seq: " << is_seq << endl;
 
 	if (layout.labelIsInline()
@@ -417,18 +389,17 @@ void RowPainter::paintFirst() const
 
 void RowPainter::paintLabel() const
 {
-	docstring const str = par_.labelString();
+	docstring const & str = par_.labelString();
 	if (str.empty())
 		return;
 
-	bool const is_rtl = text_.isRTL(par_);
 	Layout const & layout = par_.layout();
 	FontInfo const font = labelFont();
 	FontMetrics const & fm = theFontMetrics(font);
 	double x = x_;
 
-	if (is_rtl)
-		x = width_ - row_.right_margin + fm.width(layout.labelsep);
+	if (row_.isRTL())
+		x = tm_.width() - row_.right_margin + fm.width(layout.labelsep);
 	else
 		x = x_ - fm.width(layout.labelsep) - fm.width(str);
 
@@ -439,7 +410,6 @@ void RowPainter::paintLabel() const
 void RowPainter::paintTopLevelLabel() const
 {
 	BufferParams const & bparams = pi_.base.bv->buffer().params();
-	bool const is_rtl = text_.isRTL(par_);
 	ParagraphParameters const & pparams = par_.params();
 	Layout const & layout = par_.layout();
 	FontInfo const font = labelFont();
@@ -464,10 +434,10 @@ void RowPainter::paintTopLevelLabel() const
 
 	double x = x_;
 	if (layout.labeltype == LABEL_CENTERED) {
-		x = row_.left_margin + (width_ - row_.left_margin - row_.right_margin) / 2;
+		x = row_.left_margin + (tm_.width() - row_.left_margin - row_.right_margin) / 2;
 		x -= fm.width(str) / 2;
-	} else if (is_rtl) {
-		x = width_ - row_.right_margin - fm.width(str);
+	} else if (row_.isRTL()) {
+		x = tm_.width() - row_.right_margin - fm.width(str);
 	}
 	pi_.pain.text(int(x), yo_ - maxdesc - labeladdon, str, font);
 }
@@ -505,10 +475,9 @@ static int getEndLabel(pit_type p, Text const & text)
 }
 
 
-void RowPainter::paintLast()
+void RowPainter::paintLast() const
 {
-	bool const is_rtl = text_.isRTL(par_);
-	int const endlabel = getEndLabel(pit_, text_);
+	int const endlabel = getEndLabel(row_.pit(), text_);
 
 	// paint imaginary end-of-paragraph character
 
@@ -543,11 +512,11 @@ void RowPainter::paintLast()
 
 		// If needed, move the box a bit to avoid overlapping with text.
 		int x = 0;
-		if (is_rtl) {
+		if (row_.isRTL()) {
 			int const normal_x = nestMargin() + changebarMargin();
 			x = min(normal_x, row_.left_margin - size - Inset::TEXT_TO_INSET_OFFSET);
 		} else {
-			int const normal_x = width_ - row_.right_margin
+			int const normal_x = tm_.width() - row_.right_margin
 				- size - Inset::TEXT_TO_INSET_OFFSET;
 			x = max(normal_x, row_.width());
 		}
@@ -563,7 +532,7 @@ void RowPainter::paintLast()
 		FontInfo const font = labelFont();
 		FontMetrics const & fm = theFontMetrics(font);
 		docstring const & str = par_.layout().endlabelstring();
-		double const x = is_rtl ? x_ - fm.width(str) : x_;
+		double const x = row_.isRTL() ? x_ - fm.width(str) : x_;
 		pi_.pain.text(int(x), yo_, str, font);
 		break;
 	}
@@ -581,24 +550,15 @@ void RowPainter::paintOnlyInsets()
 	for ( ; cit != end ; ++cit) {
 		Row::Element const & e = *cit;
 		if (e.type == Row::INSET) {
-			// If outer row has changed, nested insets are repainted completely.
-			pi_.base.bv->coordCache().insets().add(e.inset, int(x_), yo_);
-			bool const nested_inset =
-				(e.inset->asInsetMath() && !e.inset->asInsetMath()->asMacroTemplate())
-				|| e.inset->asInsetText() || e.inset->asInsetTabular();
-			if (!nested_inset) {
-				x_ += e.full_width();
-				continue;
-			}
-			double orig_x = x_;
-			paintInset(e.inset, e.font, e.change, e.pos);
+			paintInset(e);
 			// The line that indicates word in a different language
-			paintForeignMark(orig_x, e.font.language(), e.dim.descent());
+			paintForeignMark(e);
 			// change tracking (not for insets that handle it themselves)
-			if (! e.inset->canTrackChanges())
-				paintChange(orig_x, e.font, e.change);
-		} else
-			x_ += e.full_width();
+			if (!e.inset->canPaintChange(*pi_.base.bv))
+				paintChange(e);
+		}
+
+		x_ += e.full_width();
 	}
 }
 
@@ -608,9 +568,7 @@ void RowPainter::paintText()
 	Row::const_iterator cit = row_.begin();
 	Row::const_iterator const & end = row_.end();
 	for ( ; cit != end ; ++cit) {
-		double const orig_x = x_;
 		Row::Element const & e = *cit;
-		int foreign_descent = 0;
 
 		switch (e.type) {
 		case Row::STRING:
@@ -619,26 +577,25 @@ void RowPainter::paintText()
 
 			// Paint the spelling marks if enabled.
 			if (lyxrc.spellcheck_continuously && pi_.do_spellcheck && pi_.pain.isDrawingEnabled())
-				paintMisspelledMark(orig_x, e);
+				paintMisspelledMark(e);
 			break;
-		case Row::INSET: {
-			// If outer row has changed, nested insets are repaint completely.
-			pi_.base.bv->coordCache().insets().add(e.inset, int(x_), yo_);
-			paintInset(e.inset, e.font, e.change, e.pos);
-			foreign_descent = e.dim.descent();
-		}
+
+		case Row::INSET:
+			paintInset(e);
 			break;
+
 		case Row::SPACE:
 			pi_.pain.textDecoration(e.font.fontInfo(), int(x_), yo_, int(e.full_width()));
-			x_ += e.full_width();
 		}
 
 		// The line that indicates word in a different language
-		paintForeignMark(orig_x, e.font.language(), foreign_descent);
+		paintForeignMark(e);
 
-		// change tracking (not for insets that track their own changes)
-		if (e.type != Row::INSET || ! e.inset->canTrackChanges())
-			paintChange(orig_x, e.font, e.change);
+		// change tracking (not for insets that handle it themselves)
+		if (e.type != Row::INSET || ! e.inset->canPaintChange(*pi_.base.bv))
+			paintChange(e);
+
+		x_ += e.full_width();
 	}
 }
 
@@ -650,19 +607,15 @@ void RowPainter::paintSelection() const
 
 	int const y1 = yo_ - row_.ascent();
 	int const y2 = y1 + row_.height();
-	bool const rtl = text_.isRTL(par_);
 
 	// draw the margins
-	if (rtl ? row_.end_margin_sel : row_.begin_margin_sel)
+	if (row_.isRTL() ? row_.end_margin_sel : row_.begin_margin_sel)
 		pi_.pain.fillRectangle(int(xo_), y1, row_.left_margin, y2 - y1,
 		                       Color_selection);
 
 	// go through row and draw from RTL boundary to RTL boundary
 	double x = xo_ + row_.left_margin;
-	Row::const_iterator cit = row_.begin();
-	Row::const_iterator const & end = row_.end();
-	for ( ; cit != end ; ++cit) {
-		Row::Element const & e = *cit;
+	for (auto const & e : row_) {
 		// These are the same tests as in paintStringAndSel, except
 		// that all_sel has an additional clause that triggers for end
 		// of paragraph markers. The clause was not used in
@@ -694,9 +647,9 @@ void RowPainter::paintSelection() const
 		x += e.full_width();
 	}
 
-	if (rtl ? row_.begin_margin_sel : row_.end_margin_sel)
+	if (row_.isRTL() ? row_.begin_margin_sel : row_.end_margin_sel)
 		pi_.pain.fillRectangle(int(x), y1,
-		                       int(xo_ + text_metrics_.width()) - int(x), y2 - y1,
+		                       int(xo_ + tm_.width()) - int(x), y2 - y1,
 		                       Color_selection);
 
 }
