@@ -17,6 +17,7 @@
 #include "TextClass.h"
 
 #include "LayoutFile.h"
+#include "CiteEnginesList.h"
 #include "Color.h"
 #include "Counters.h"
 #include "Floating.h"
@@ -61,7 +62,13 @@ namespace lyx {
 // You should also run the development/tools/updatelayouts.py script,
 // to update the format of all of our layout files.
 //
-int const LAYOUT_FORMAT = 60; //lasgouttes LongTableNoNumber => Unnumbered
+int const LAYOUT_FORMAT = 66; //spitz: New layout tag AutoNests
+
+
+// Layout format for the current lyx file format. Controls which format is
+// targeted by Local Layout > Convert. In master, equal to LAYOUT_FORMAT.
+int const LYXFILE_LAYOUT_FORMAT = LAYOUT_FORMAT;
+
 
 namespace {
 
@@ -79,26 +86,29 @@ private:
 };
 
 
-bool layout2layout(FileName const & filename, FileName const & tempfile)
+bool layout2layout(FileName const & filename, FileName const & tempfile,
+                   int const format = LAYOUT_FORMAT)
 {
 	FileName const script = libFileSearch("scripts", "layout2layout.py");
 	if (script.empty()) {
 		LYXERR0("Could not find layout conversion "
-			  "script layout2layout.py.");
+		        "script layout2layout.py.");
 		return false;
 	}
 
 	ostringstream command;
 	command << os::python() << ' ' << quoteName(script.toFilesystemEncoding())
-		<< ' ' << quoteName(filename.toFilesystemEncoding())
-		<< ' ' << quoteName(tempfile.toFilesystemEncoding());
+	        << " -t " << format
+	        << ' ' << quoteName(filename.toFilesystemEncoding())
+	        << ' ' << quoteName(tempfile.toFilesystemEncoding());
 	string const command_str = command.str();
 
 	LYXERR(Debug::TCLASS, "Running `" << command_str << '\'');
 
 	cmd_ret const ret = runCommand(command_str);
 	if (ret.first != 0) {
-		LYXERR0("Could not run layout conversion script layout2layout.py.");
+		if (format == LAYOUT_FORMAT)
+			LYXERR0("Conversion of layout with layout2layout.py has failed.");
 		return false;
 	}
 	return true;
@@ -114,6 +124,8 @@ string translateReadType(TextClass::ReadType rt)
 		return "input file";
 	case TextClass::MODULE:
 		return "module file";
+	case TextClass::CITE_ENGINE:
+		return "cite engine";
 	case TextClass::VALIDATION:
 		return "validation";
 	}
@@ -121,7 +133,7 @@ string translateReadType(TextClass::ReadType rt)
 	return string();
 }
 
-} // namespace anon
+} // namespace
 
 
 // This string should not be translated here,
@@ -140,10 +152,10 @@ TextClass::TextClass()
 	  opt_enginetype_("authoryear|numerical"), opt_fontsize_("10|11|12"),
 	  opt_pagestyle_("empty|plain|headings|fancy"), pagestyle_("default"),
 	  columns_(1), sides_(OneSide), secnumdepth_(3), tocdepth_(3),
-	  outputType_(LATEX), outputFormat_("latex"),
+	  outputType_(LATEX), outputFormat_("latex"), has_output_format_(false),
 	  defaultfont_(sane_font),
 	  titletype_(TITLE_COMMAND_AFTER), titlename_("maketitle"),
-	  min_toclevel_(0), max_toclevel_(0),
+	  min_toclevel_(0), max_toclevel_(0), maxcitenames_(2),
 	  cite_full_author_list_(true)
 {
 }
@@ -209,6 +221,8 @@ enum TextClassTags {
 	TC_CITEENGINE,
 	TC_CITEENGINETYPE,
 	TC_CITEFORMAT,
+	TC_CITEFRAMEWORK,
+	TC_MAXCITENAMES,
 	TC_DEFAULTBIBLIO,
 	TC_FULLAUTHORLIST,
 	TC_OUTLINERNAME
@@ -224,6 +238,7 @@ LexerKeyword textClassTags[] = {
 	{ "citeengine",        TC_CITEENGINE },
 	{ "citeenginetype",    TC_CITEENGINETYPE },
 	{ "citeformat",        TC_CITEFORMAT },
+	{ "citeframework",     TC_CITEFRAMEWORK },
 	{ "classoptions",      TC_CLASSOPTIONS },
 	{ "columns",           TC_COLUMNS },
 	{ "counter",           TC_COUNTER },
@@ -242,6 +257,7 @@ LexerKeyword textClassTags[] = {
 	{ "input",             TC_INPUT },
 	{ "insetlayout",       TC_INSETLAYOUT },
 	{ "leftmargin",        TC_LEFTMARGIN },
+	{ "maxcitenames",      TC_MAXCITENAMES },
 	{ "modifystyle",       TC_MODIFYSTYLE },
 	{ "nocounter",         TC_NOCOUNTER },
 	{ "nofloat",           TC_NOFLOAT },
@@ -266,7 +282,7 @@ LexerKeyword textClassTags[] = {
 	{ "tocdepth",          TC_TOCDEPTH }
 };
 
-} //namespace anon
+} // namespace
 
 
 bool TextClass::convertLayoutFormat(support::FileName const & filename, ReadType rt)
@@ -290,7 +306,7 @@ std::string TextClass::convert(std::string const & str)
 	os.close();
 	TempFile tmp2("convert_localXXXXXX.layout");
 	FileName const tempfile = tmp2.name();
-	bool success = layout2layout(fn, tempfile);
+	bool success = layout2layout(fn, tempfile, LYXFILE_LAYOUT_FORMAT);
 	if (!success)
 		return "";
 	ifstream is(tempfile.toFilesystemEncoding().c_str());
@@ -375,13 +391,13 @@ TextClass::ReturnValues TextClass::read(std::string const & str, ReadType rt)
 	os << str;
 	os.close();
 
-	// now try to convert it
-	bool const worx = convertLayoutFormat(tempfile, rt);
-	if (!worx) {
+	// now try to convert it to LAYOUT_FORMAT
+	if (!convertLayoutFormat(tempfile, rt)) {
 		LYXERR0("Unable to convert internal layout information to format "
 			<< LAYOUT_FORMAT);
 		return ERROR;
 	}
+
 	return OK_OLDFORMAT;
 }
 
@@ -392,11 +408,14 @@ TextClass::ReturnValues TextClass::read(Lexer & lexrc, ReadType rt)
 	if (!lexrc.isOK())
 		return ERROR;
 
-	// Format of files before the 'Format' tag was introduced
-	int format = 1;
-	bool error = false;
+	// The first usable line should be
+	// Format LAYOUT_FORMAT
+	if (lexrc.lex() != TC_FORMAT || !lexrc.next()
+	    || lexrc.getInteger() != LAYOUT_FORMAT)
+		return FORMAT_MISMATCH;
 
 	// parsing
+	bool error = false;
 	while (lexrc.isOK() && !error) {
 		int le = lexrc.lex();
 
@@ -421,13 +440,15 @@ TextClass::ReturnValues TextClass::read(Lexer & lexrc, ReadType rt)
 		switch (static_cast<TextClassTags>(le)) {
 
 		case TC_FORMAT:
-			if (lexrc.next())
-				format = lexrc.getInteger();
+			lexrc.next();
+			lexrc.printError("Duplicate Format directive");
 			break;
 
 		case TC_OUTPUTFORMAT:
-			if (lexrc.next())
+			if (lexrc.next()) {
 				outputFormat_ = lexrc.getString();
+				has_output_format_ = true;
+			}
 			break;
 
 		case TC_OUTPUTTYPE:
@@ -501,9 +522,9 @@ TextClass::ReturnValues TextClass::read(Lexer & lexrc, ReadType rt)
 				error = !readStyle(lexrc, lay);
 				break;
 			}
-			
+
 			bool const have_layout = hasLayout(name);
-			
+
 			// If the layout already exists, then we want to add it to
 			// the existing layout, as long as we are not in an ProvideStyle
 			// block.
@@ -533,7 +554,8 @@ TextClass::ReturnValues TextClass::read(Lexer & lexrc, ReadType rt)
 			// Either way, we just scan the rest and discard it
 			else {
 				Layout lay;
-				readStyle(lexrc, lay);
+				// signal to coverity that we do not care about the result
+				(void)readStyle(lexrc, lay);
 			}
 			break;
 		}
@@ -608,15 +630,15 @@ TextClass::ReturnValues TextClass::read(Lexer & lexrc, ReadType rt)
 			break;
 
 		case TC_PREAMBLE:
-			preamble_ = from_utf8(lexrc.getLongString("EndPreamble"));
+			preamble_ = lexrc.getLongString(from_ascii("EndPreamble"));
 			break;
 
 		case TC_HTMLPREAMBLE:
-			htmlpreamble_ = from_utf8(lexrc.getLongString("EndPreamble"));
+			htmlpreamble_ = lexrc.getLongString(from_ascii("EndPreamble"));
 			break;
 
 		case TC_HTMLSTYLES:
-			htmlstyles_ = from_utf8(lexrc.getLongString("EndStyles"));
+			htmlstyles_ = lexrc.getLongString(from_ascii("EndStyles"));
 			break;
 
 		case TC_HTMLTOCSECTION:
@@ -624,15 +646,15 @@ TextClass::ReturnValues TextClass::read(Lexer & lexrc, ReadType rt)
 			break;
 
 		case TC_ADDTOPREAMBLE:
-			preamble_ += from_utf8(lexrc.getLongString("EndPreamble"));
+			preamble_ += lexrc.getLongString(from_ascii("EndPreamble"));
 			break;
 
 		case TC_ADDTOHTMLPREAMBLE:
-			htmlpreamble_ += from_utf8(lexrc.getLongString("EndPreamble"));
+			htmlpreamble_ += lexrc.getLongString(from_ascii("EndPreamble"));
 			break;
 
 		case TC_ADDTOHTMLSTYLES:
-			htmlstyles_ += from_utf8(lexrc.getLongString("EndStyles"));
+			htmlstyles_ += lexrc.getLongString(from_ascii("EndStyles"));
 			break;
 
 		case TC_PROVIDES: {
@@ -749,9 +771,35 @@ TextClass::ReturnValues TextClass::read(Lexer & lexrc, ReadType rt)
 			error = !readCiteFormat(lexrc);
 			break;
 
+		case TC_CITEFRAMEWORK:
+			lexrc.next();
+			citeframework_ = rtrim(lexrc.getString());
+			break;
+
+		case TC_MAXCITENAMES:
+			lexrc.next();
+			maxcitenames_ = size_t(lexrc.getInteger());
+			break;
+
 		case TC_DEFAULTBIBLIO:
-			if (lexrc.next())
-				cite_default_biblio_style_ = rtrim(lexrc.getString());
+			if (lexrc.next()) {
+				vector<string> const dbs =
+					getVectorFromString(rtrim(lexrc.getString()), "|");
+				vector<string>::const_iterator it  = dbs.begin();
+				vector<string>::const_iterator end = dbs.end();
+				for (; it != end; ++it) {
+					if (!contains(*it, ':')) {
+						vector<string> const enginetypes =
+							getVectorFromString(opt_enginetype_, "|");
+						for (string const &s: enginetypes)
+							cite_default_biblio_style_[s] = *it;
+					} else {
+						string eng;
+						string const db = split(*it, eng, ':');
+						cite_default_biblio_style_[eng] = db;
+					}
+				}
+			}
 			break;
 
 		case TC_FULLAUTHORLIST:
@@ -810,11 +858,6 @@ TextClass::ReturnValues TextClass::read(Lexer & lexrc, ReadType rt)
 			error = !readOutlinerName(lexrc);
 			break;
 		} // end of switch
-
-		// Note that this is triggered the first time through the loop unless
-		// we hit a format tag.
-		if (format != LAYOUT_FORMAT)
-			return FORMAT_MISMATCH;
 	}
 
 	// at present, we abort if we encounter an error,
@@ -1007,30 +1050,85 @@ bool TextClass::readCiteEngine(Lexer & lexrc)
 			getout = true;
 			continue;
 		}
-		string cmd;
 		CitationStyle cs;
 		char ichar = def[0];
 		if (ichar == '#')
 			continue;
-		if (ichar == 'C') {
+		if (isUpperCase(ichar)) {
 			cs.forceUpperCase = true;
-			def[0] = 'c';
+			def[0] = lowercase(ichar);
 		}
 
+		/** For portability reasons (between different
+		 *  cite engines such as natbib and biblatex),
+		 *  we distinguish between:
+		 *  1. The LyX name as output in the LyX file
+		 *  2. Possible aliases that might fall back to
+		 *     the given LyX name in the current engine
+		 *  3. The actual LaTeX command that is output
+		 *  (2) and (3) are optional.
+		 *  Also, the GUI string for the starred version can
+		 *  be changed
+		 *  The syntax is:
+		 *  LyXName|alias,nextalias*<!stardesc!stardesctooltip>[][]=latexcmd
+		 */
+		enum ScanMode {
+			LyXName,
+			Alias,
+			LaTeXCmd,
+			StarDesc
+		};
+
+		ScanMode mode = LyXName;
+		ScanMode oldmode = LyXName;
+		string lyx_cmd;
+		string alias;
+		string latex_cmd;
+		string stardesc;
 		size_t const n = def.size();
 		for (size_t i = 0; i != n; ++i) {
 			ichar = def[i];
-			if (ichar == '*')
-				cs.fullAuthorList = true;
+			if (ichar == '|')
+				mode = Alias;
+			else if (ichar == '=')
+				mode = LaTeXCmd;
+			else if (ichar == '<') {
+				oldmode = mode;
+				mode = StarDesc;
+			} else if (ichar == '>')
+				mode = oldmode;
+			else if (mode == LaTeXCmd)
+				latex_cmd += ichar;
+			else if (mode == StarDesc)
+				stardesc += ichar;
+			else if (ichar == '$')
+				cs.hasQualifiedList = true;
+			else if (ichar == '*')
+				cs.hasStarredVersion = true;
 			else if (ichar == '[' && cs.textAfter)
 				cs.textBefore = true;
 			else if (ichar == '[')
 				cs.textAfter = true;
-			else if (ichar != ']')
-				cmd += ichar;
+			else if (ichar != ']') {
+				if (mode == Alias)
+					alias += ichar;
+				else
+					lyx_cmd += ichar;
+			}
 		}
-
-		cs.cmd = cmd;
+		cs.name = lyx_cmd;
+		cs.cmd = latex_cmd.empty() ? lyx_cmd : latex_cmd;
+		if (!alias.empty()) {
+			vector<string> const aliases = getVectorFromString(alias);
+			for (string const &s: aliases)
+				cite_command_aliases_[s] = lyx_cmd;
+		}
+		vector<string> const stardescs = getVectorFromString(stardesc, "!");
+		int size = stardesc.size();
+		if (size > 0)
+			cs.stardesc = stardescs[0];
+		if (size > 1)
+			cs.startooltip = stardescs[1];
 		if (type & ENGINE_TYPE_AUTHORYEAR)
 			cite_styles_[ENGINE_TYPE_AUTHORYEAR].push_back(cs);
 		if (type & ENGINE_TYPE_NUMERICAL)
@@ -1081,7 +1179,7 @@ bool TextClass::readCiteFormat(Lexer & lexrc)
 		char initchar = etype[0];
 		if (initchar == '#')
 			continue;
-		if (initchar == '!' || initchar == '_') {
+		if (initchar == '!' || initchar == '_' || prefixIs(etype, "B_")) {
 			if (type & ENGINE_TYPE_AUTHORYEAR)
 				cite_macros_[ENGINE_TYPE_AUTHORYEAR][etype] = definition;
 			if (type & ENGINE_TYPE_NUMERICAL)
@@ -1149,7 +1247,7 @@ bool TextClass::readFloat(Lexer & lexrc)
 
 	string ext;
 	string htmlattr;
-	string htmlstyle;
+	docstring htmlstyle;
 	string htmltag;
 	string listname;
 	string listcommand;
@@ -1253,7 +1351,7 @@ bool TextClass::readFloat(Lexer & lexrc)
 			break;
 		case FT_HTMLSTYLE:
 			lexrc.next();
-			htmlstyle = lexrc.getLongString("EndHTMLStyle");
+			htmlstyle = lexrc.getLongString(from_ascii("EndHTMLStyle"));
 			break;
 		case FT_HTMLTAG:
 			lexrc.next();
@@ -1324,18 +1422,6 @@ bool TextClass::readOutlinerName(Lexer & lexrc)
 }
 
 
-docstring TextClass::outlinerName(std::string const & type) const
-{
-	std::map<std::string,docstring>::const_iterator const it
-		= outliner_names_.find(type);
-	if (it == outliner_names_.end()) {
-		LYXERR0("Missing OutlinerName for " << type << "!");
-		return from_utf8(type);
-	} else
-		return it->second;
-}
-
-
 string const & TextClass::prerequisites(string const & sep) const
 {
 	if (contains(prerequisites_, ',')) {
@@ -1579,6 +1665,7 @@ Layout TextClass::createBasicLayout(docstring const & name, bool unknown) const
 
 DocumentClassPtr getDocumentClass(
 		LayoutFile const & baseClass, LayoutModuleList const & modlist,
+		LayoutModuleList const & celist,
 		bool const clone)
 {
 	DocumentClassPtr doc_class =
@@ -1617,6 +1704,42 @@ DocumentClassPtr getDocumentClass(
 			frontend::Alert::warning(_("Read Error"), msg);
 		}
 	}
+
+	LayoutModuleList::const_iterator cit = celist.begin();
+	LayoutModuleList::const_iterator cen = celist.end();
+	for (; cit != cen; ++cit) {
+		string const ceName = *cit;
+		LyXCiteEngine * ce = theCiteEnginesList[ceName];
+		if (!ce) {
+			docstring const msg =
+						bformat(_("The cite engine %1$s has been requested by\n"
+						"this document but has not been found in the list of\n"
+						"available engines. If you recently installed it, you\n"
+						"probably need to reconfigure LyX.\n"), from_utf8(ceName));
+			if (!clone)
+				frontend::Alert::warning(_("Cite Engine not available"), msg);
+			continue;
+		}
+		if (!ce->isAvailable() && !clone) {
+			docstring const prereqs = from_utf8(getStringFromVector(ce->prerequisites(), "\n\t"));
+			docstring const msg =
+				bformat(_("The cite engine %1$s requires a package that is not\n"
+					"available in your LaTeX installation, or a converter that\n"
+					"you have not installed. LaTeX output may not be possible.\n"
+					"Missing prerequisites:\n"
+						"\t%2$s\n"
+					"See section 3.1.2.3 (Modules) of the User's Guide for more information."),
+				from_utf8(ceName), prereqs);
+			frontend::Alert::warning(_("Package not available"), msg, true);
+		}
+		FileName layout_file = libFileSearch("citeengines", ce->getFilename());
+		if (!doc_class->read(layout_file, TextClass::CITE_ENGINE)) {
+			docstring const msg =
+						bformat(_("Error reading cite engine %1$s\n"), from_utf8(ceName));
+			frontend::Alert::warning(_("Read Error"), msg);
+		}
+	}
+
 	return doc_class;
 }
 
@@ -1665,7 +1788,7 @@ Layout const & DocumentClass::getTOCLayout() const
 	for (; lit != len; ++lit) {
 		int const level = lit->toclevel;
 		// we don't want Part or unnumbered sections
-		if (level == Layout::NOT_IN_TOC || level < 0 
+		if (level == Layout::NOT_IN_TOC || level < 0
 		    || level >= minlevel || lit->counter.empty())
 			continue;
 		lay = &*lit;
@@ -1686,10 +1809,12 @@ Layout const & DocumentClass::htmlTOCLayout() const
 }
 
 
-string const & DocumentClass::getCiteFormat(CiteEngineType const & type,
-	string const & entry, string const & fallback) const
+string const DocumentClass::getCiteFormat(CiteEngineType const & type,
+	string const & entry, bool const punct, string const & fallback) const
 {
-	static string default_format = "{%author%[[%author%, ]][[{%editor%[[%editor%, ed., ]]}]]}\"%title%\"{%journal%[[, {!<i>!}%journal%{!</i>!}]][[{%publisher%[[, %publisher%]][[{%institution%[[, %institution%]]}]]}]]}{%year%[[ (%year%)]]}{%pages%[[, %pages%]]}.";
+	string default_format = "{%fullnames:author%[[%fullnames:author%, ]][[{%fullnames:editor%[[%fullnames:editor%, ed., ]]}]]}\"%title%\"{%journal%[[, {!<i>!}%journal%{!</i>!}]][[{%publisher%[[, %publisher%]][[{%institution%[[, %institution%]]}]]}]]}{%year%[[ (%year%)]]}{%pages%[[, %pages%]]}";
+	if (punct)
+		default_format += ".";
 
 	map<CiteEngineType, map<string, string> >::const_iterator itype = cite_formats_.find(type);
 	if (itype == cite_formats_.end())
@@ -1699,6 +1824,8 @@ string const & DocumentClass::getCiteFormat(CiteEngineType const & type,
 		it = itype->second.find(fallback);
 	if (it == itype->second.end())
 		return default_format;
+	if (punct)
+		return it->second + ".";
 	return it->second;
 }
 
@@ -1726,7 +1853,7 @@ vector<string> const DocumentClass::citeCommands(
 	vector<string> cmds;
 	for (; it != end; ++it) {
 		CitationStyle const cite = *it;
-		cmds.push_back(cite.cmd);
+		cmds.push_back(cite.name);
 	}
 	return cmds;
 }
