@@ -38,12 +38,7 @@ namespace frontend {
  *
  * The intention for a toolkit is that it uses these methods to paint
  * onto a backing pixmap. Only when expose events arrive via the event
- * queue (perhaps generated via Screen::expose), does the copy onto
- * the actual WorkArea widget take place. Paints are wrapped in (possibly
- * recursive) calls to start() and end() to facilitate the backing pixmap
- * management.
- *
- * Note that the methods return *this for convenience.
+ * queue, does the copy onto the actual WorkArea widget take place.
  *
  * Caution: All char_type and docstring arguments of the text drawing
  * methods of this class are no UCS4 chars or strings if the font is a
@@ -61,6 +56,8 @@ public:
 	/// possible line styles
 	enum line_style {
 		line_solid, //< solid line
+		line_solid_aliased, //< solid line, no anti-aliasing (used as a
+		                    // workaround to painting issues)
 		line_onoffdash //< dashes with spaces
 	};
 
@@ -126,34 +123,29 @@ public:
 	/// draw a pixel
 	virtual void point(int x, int y, Color) = 0;
 
-	/// draw a filled rectangle with the shape of a 3D button
-	virtual void button(int x, int y, int w, int h, bool mouseHover) = 0;
-
 	/// draw an image from the image cache
 	virtual void image(int x, int y, int w, int h,
 		graphics::Image const & image) = 0;
 
-	/** draw a string at position x, y (y is the baseline). The text
-	 * direction depends on the string itself.
-	 * \return the width of the drawn text.
-	 */
-	virtual int text(int x, int y, docstring const & str, FontInfo const & f) = 0;
+	/// draw a string at position x, y (y is the baseline).
+	virtual void text(int x, int y, docstring const & str, FontInfo const & f) = 0;
 
-	/** draw a string at position x, y (y is the baseline). The text
-	 * direction is enforced by the \c Font.
-	 * \return the width of the drawn text.
+	/// draw a char at position x, y (y is the baseline)
+	virtual void text(int x, int y, char_type c, FontInfo const & f) = 0;
+
+	/** draw a string at position x, y (y is the baseline). The
+	 * text direction is enforced by the \c Font.
 	 */
-	virtual int text(int x, int y, docstring const & str, Font const & f,
-                     double wordspacing = 0.0) = 0;
+	virtual void text(int x, int y, docstring const & str, Font const & f,
+                      double wordspacing, double textwidth) = 0;
 
 	/** draw a string at position x, y (y is the baseline), but
 	 * make sure that the part between \c from and \c to is in
 	 * \c other color. The text direction is enforced by the \c Font.
-	 * \return the width of the drawn text.
 	 */
-	virtual int text(int x, int y, docstring const & str, Font const & f,
-                     Color other, size_type from, size_type to,
-                     double const wordspacing) = 0;
+	virtual void text(int x, int y, docstring const & str, Font const & f,
+	                  Color other, size_type from, size_type to,
+                      double wordspacing, double textwidth) = 0;
 
 	void setDrawingEnabled(bool drawing_enabled)
 	{ drawing_enabled_ = drawing_enabled; }
@@ -163,13 +155,7 @@ public:
 
 	double pixelRatio() const { return pixel_ratio_; }
 
-	/// draw a char at position x, y (y is the baseline)
-	/**
-	* \return the width of the drawn text.
-	*/
-	virtual int text(int x, int y, char_type c, FontInfo const & f) = 0;
-
-	/// draw the underbar, strikeout, uuline and uwave font attributes
+	/// draw the underbar, strikeout, xout, uuline and uwave font attributes
 	virtual void textDecoration(FontInfo const & f, int x, int y, int width) = 0;
 
 	/**
@@ -183,14 +169,14 @@ public:
 
 	/// draw a string and enclose it inside a button frame
 	virtual void buttonText(int x, int baseline, docstring const & s,
-		FontInfo const & font, bool mouseHover) = 0;
+		FontInfo const & font, Color back, Color frame, int offset) = 0;
 
 	/// draw a character of a preedit string for cjk support.
 	virtual int preeditText(int x, int y,
 		char_type c, FontInfo const & f, preedit_style style) = 0;
 
 	/// start monochrome painting mode, i.e. map every color into [min,max]
-	virtual void enterMonochromeMode(Color const & min, 
+	virtual void enterMonochromeMode(Color const & min,
 		Color const & max) = 0;
 	/// leave monochrome painting mode
 	virtual void leaveMonochromeMode() = 0;
