@@ -18,13 +18,13 @@
 
 #include "GuiApplication.h"
 #include "GuiSelectionManager.h"
-#include "LyXToolBox.h"
 #include "qt_helpers.h"
 
 #include "Buffer.h"
 #include "BufferView.h"
 #include "BiblioInfo.h"
 #include "BufferParams.h"
+#include "TextClass.h"
 #include "FuncRequest.h"
 
 #include "insets/InsetCommand.h"
@@ -38,6 +38,7 @@
 #include <QMenu>
 #include <QSettings>
 #include <QShowEvent>
+#include <QStandardItemModel>
 #include <QVariant>
 
 #include <vector>
@@ -91,21 +92,17 @@ static vector<lyx::docstring> to_docstring_vector(QStringList const & qlist)
 
 GuiCitation::GuiCitation(GuiView & lv)
 	: DialogView(lv, "citation", qt_("Citation")),
-	  params_(insetCode("citation"))
+	  style_(QString()), literal_(false), params_(insetCode("citation"))
 {
 	setupUi(this);
 
 	// The filter bar
 	filter_ = new FancyLineEdit(this);
-#if QT_VERSION >= 0x040600
 	filter_->setButtonPixmap(FancyLineEdit::Right, getPixmap("images/", "editclear", "svgz,png"));
 	filter_->setButtonVisible(FancyLineEdit::Right, true);
 	filter_->setButtonToolTip(FancyLineEdit::Right, qt_("Clear text"));
 	filter_->setAutoHideButton(FancyLineEdit::Right, true);
-#endif
-#if QT_VERSION >= 0x040700
 	filter_->setPlaceholderText(qt_("All avail. citations"));
-#endif
 
 	filterBarL->addWidget(filter_, 0);
 	findKeysLA->setBuddy(filter_);
@@ -127,10 +124,12 @@ GuiCitation::GuiCitation(GuiView & lv)
 
 	connect(citationStyleCO, SIGNAL(activated(int)),
 		this, SLOT(on_citationStyleCO_currentIndexChanged(int)));
-	connect(fulllistCB, SIGNAL(clicked()),
+	connect(starredCB, SIGNAL(clicked()),
+		this, SLOT(updateStyles()));
+	connect(literalCB, SIGNAL(clicked()),
 		this, SLOT(changed()));
 	connect(forceuppercaseCB, SIGNAL(clicked()),
-		this, SLOT(changed()));
+		this, SLOT(updateStyles()));
 	connect(textBeforeED, SIGNAL(textChanged(QString)),
 		this, SLOT(updateStyles()));
 	connect(textAfterED, SIGNAL(textChanged(QString)),
@@ -140,8 +139,8 @@ GuiCitation::GuiCitation(GuiView & lv)
 	connect(textAfterED, SIGNAL(returnPressed()),
 		this, SLOT(on_okPB_clicked()));
 
-	selectionManager = new GuiSelectionManager(availableLV, selectedLV,
-			addPB, deletePB, upPB, downPB, &available_model_, &selected_model_);
+	selectionManager = new GuiSelectionManager(this, availableLV, selectedLV,
+			addPB, deletePB, upPB, downPB, &available_model_, &selected_model_, 1);
 	connect(selectionManager, SIGNAL(selectionChanged()),
 		this, SLOT(setCitedKeys()));
 	connect(selectionManager, SIGNAL(updateHook()),
@@ -155,6 +154,13 @@ GuiCitation::GuiCitation(GuiView & lv)
 		this, SLOT(filterChanged(QString)));
 	connect(filter_, SIGNAL(returnPressed()),
 		this, SLOT(filterPressed()));
+#if (QT_VERSION < 0x050000)
+	connect(filter_, SIGNAL(downPressed()),
+	        availableLV, SLOT(setFocus()));
+#else
+	connect(filter_, &FancyLineEdit::downPressed,
+	        availableLV, [=](){ focusAndHighlight(availableLV); });
+#endif
 	connect(regexp_, SIGNAL(triggered()),
 		this, SLOT(regexChanged()));
 	connect(casesense_, SIGNAL(triggered()),
@@ -162,16 +168,16 @@ GuiCitation::GuiCitation(GuiView & lv)
 	connect(instant_, SIGNAL(triggered(bool)),
 		this, SLOT(instantChanged(bool)));
 
+#if (QT_VERSION < 0x050000)
+	selectedLV->horizontalHeader()->setResizeMode(QHeaderView::Stretch);
+#else
+	selectedLV->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
+#endif
+
 	setFocusProxy(filter_);
 }
 
 
-GuiCitation::~GuiCitation()
-{
-	delete selectionManager;
-}
-
-
 void GuiCitation::closeEvent(QCloseEvent * e)
 {
 	clearSelection();
@@ -182,8 +188,8 @@ void GuiCitation::closeEvent(QCloseEvent * e)
 void GuiCitation::applyView()
 {
 	int const choice = max(0, citationStyleCO->currentIndex());
-	style_ = choice;
-	bool const full  = fulllistCB->isChecked();
+	style_ = citationStyleCO->itemData(citationStyleCO->currentIndex()).toString();
+	bool const full  = starredCB->isChecked();
 	bool const force = forceuppercaseCB->isChecked();
 
 	QString const before = textBeforeED->text();
@@ -229,6 +235,13 @@ void GuiCitation::on_restorePB_clicked()
 }
 
 
+void GuiCitation::on_literalCB_clicked()
+{
+	literal_ = literalCB->isChecked();
+	changed();
+}
+
+
 void GuiCitation::updateControls()
 {
 	BiblioInfo const & bi = bibInfo();
@@ -242,35 +255,102 @@ void GuiCitation::updateControls()
 // will not have changed.
 void GuiCitation::updateControls(BiblioInfo const & bi)
 {
-	QModelIndex idx = selectionManager->getSelectedIndex();
+	QModelIndex idx = selectionManager->getSelectedIndex(1);
 	updateInfo(bi, idx);
 	int i = citationStyleCO->currentIndex();
-	if (i == -1)
+	if (i == -1 || i > int(citeStyles_.size()))
 		i = 0;
 	updateFormatting(citeStyles_[i]);
 	selectionManager->update();
 }
 
 
-void GuiCitation::updateFormatting(CitationStyle currentStyle)
+void GuiCitation::updateFormatting(CitationStyle const & currentStyle)
 {
+	BufferParams const bp = documentBuffer().params();
 	bool const force = currentStyle.forceUpperCase;
-	bool const full = currentStyle.fullAuthorList &&
-		documentBuffer().params().fullAuthorList();
+	bool const starred = currentStyle.hasStarredVersion;
+	bool const full = starred && bp.fullAuthorList();
 	bool const textbefore = currentStyle.textBefore;
 	bool const textafter = currentStyle.textAfter;
 
-	bool const haveSelection =
-		selectedLV->model()->rowCount() > 0;
+	int const rows = selectedLV->model()->rowCount();
+
+	bool const qualified = currentStyle.hasQualifiedList
+		&& (rows > 1
+		    || !params_["pretextlist"].empty()
+		    || !params_["posttextlist"].empty());
+	selectedLV->horizontalHeader()->setVisible(qualified);
+	selectedLV->setColumnHidden(0, !qualified);
+	selectedLV->setColumnHidden(2, !qualified);
+	bool const haveSelection = rows > 0;
+	if (qualified) {
+		textBeforeLA->setText(qt_("General text befo&re:"));
+		textAfterLA->setText(qt_("General &text after:"));
+		textBeforeED->setToolTip(qt_("Text that precedes the whole reference list. "
+					     "For text that precedes individual items, "
+					     "double-click on the respective entry above."));
+		textAfterLA->setToolTip(qt_("General &text after:"));
+		textAfterED->setToolTip(qt_("Text that follows the whole reference list. "
+					     "For text that follows individual items, "
+					     "double-click on the respective entry above."));
+	} else {
+		textBeforeLA->setText(qt_("Text befo&re:"));
+		if (textbefore && haveSelection)
+			textBeforeED->setToolTip(qt_("Text that precedes the reference (e.g., \"cf.\")"));
+		else
+			textBeforeED->setToolTip(qt_("Text that precedes the reference (e.g., \"cf.\"), "
+						     "if the current citation style supports this."));
+		textAfterLA->setText(qt_("&Text after:"));
+		if (textafter && haveSelection)
+			textAfterED->setToolTip(qt_("Text that follows the reference (e.g., pages)"));
+		else
+			textAfterED->setToolTip(qt_("Text that follows the reference (e.g., pages), "
+						    "if the current citation style supports this."));
+	}
 
 	forceuppercaseCB->setEnabled(force && haveSelection);
-	fulllistCB->setEnabled(full && haveSelection);
+	if (force && haveSelection)
+		forceuppercaseCB->setToolTip(qt_("Force upper case in names (\"Del Piero\", not \"del Piero\")."));
+	else
+		forceuppercaseCB->setToolTip(qt_("Force upper case in names (\"Del Piero\", not \"del Piero\"), "
+					     "if the current citation style supports this."));
+	starredCB->setEnabled(full && haveSelection);
 	textBeforeED->setEnabled(textbefore && haveSelection);
 	textBeforeLA->setEnabled(textbefore && haveSelection);
 	textAfterED->setEnabled(textafter && haveSelection);
 	textAfterLA->setEnabled(textafter && haveSelection);
+	literalCB->setEnabled(textbefore || textafter);
 	citationStyleCO->setEnabled(haveSelection);
 	citationStyleLA->setEnabled(haveSelection);
+
+	// Check if we have a custom string/tooltip for the starred version
+	if (starred && !currentStyle.stardesc.empty()) {
+		string val =
+			bp.documentClass().getCiteMacro(bp.citeEngineType(), currentStyle.stardesc);
+		docstring guistring;
+		if (!val.empty()) {
+			guistring = translateIfPossible(from_utf8(val));
+			starredCB->setText(toqstr(guistring));
+			starredCB->setEnabled(haveSelection);
+		}
+		if (!currentStyle.startooltip.empty()) {
+			val = bp.documentClass().getCiteMacro(bp.citeEngineType(),
+							      currentStyle.startooltip);
+			if (!val.empty())
+				guistring = translateIfPossible(from_utf8(val));
+		}
+		// Tooltip might also be empty
+		starredCB->setToolTip(toqstr(guistring));
+	} else {
+		// This is the default meaning of the starred commands
+		starredCB->setText(qt_("All aut&hors"));
+		if (full && haveSelection)
+			starredCB->setToolTip(qt_("Always list all authors (rather than using \"et al.\")"));
+		else
+			starredCB->setToolTip(qt_("Always list all authors (rather than using \"et al.\"), "
+						  "if the current citation style supports this."));
+	}
 }
 
 
@@ -288,7 +368,7 @@ void GuiCitation::updateStyles()
 // Update the styles for the style combo, citationStyleCO.
 void GuiCitation::updateStyles(BiblioInfo const & bi)
 {
-	QStringList selected_keys = selected_model_.stringList();
+	QStringList selected_keys = selectedKeys();
 	int curr = selectedLV->model()->rowCount() - 1;
 
 	if (curr < 0 || selected_keys.empty()) {
@@ -299,9 +379,9 @@ void GuiCitation::updateStyles(BiblioInfo const & bi)
 	}
 
 	static const size_t max_length = 80;
-	QStringList sty = citationStyles(bi, max_length);
+	BiblioInfo::CiteStringMap sty = citationStyles(bi, max_length);
 
-	if (sty.isEmpty()) {
+	if (sty.empty()) {
 		// some error
 		citationStyleCO->setEnabled(false);
 		citationStyleLA->setEnabled(false);
@@ -311,15 +391,21 @@ void GuiCitation::updateStyles(BiblioInfo const & bi)
 
 	citationStyleCO->blockSignals(true);
 
-	// save old index
-	int const oldIndex = citationStyleCO->currentIndex();
+	// save old style selection
+	QString const curdata =
+		citationStyleCO->itemData(citationStyleCO->currentIndex()).toString();
+	QString const olddata = (curdata.isEmpty()) ? style_ : curdata;
 	citationStyleCO->clear();
-	citationStyleCO->insertItems(0, sty);
+	BiblioInfo::CiteStringMap::const_iterator cit = sty.begin();
+	BiblioInfo::CiteStringMap::const_iterator end = sty.end();
+	for (int ii = 1; cit != end; ++cit, ++ii)
+		citationStyleCO->addItem(toqstr(cit->second), toqstr(cit->first));
 	citationStyleCO->setEnabled(true);
 	citationStyleLA->setEnabled(true);
-	// restore old index
-	if (oldIndex != -1 && oldIndex < citationStyleCO->count())
-		citationStyleCO->setCurrentIndex(oldIndex);
+	// restore old style selection
+	int const i = citationStyleCO->findData(olddata);
+	if (i != -1)
+		citationStyleCO->setCurrentIndex(i);
 
 	citationStyleCO->blockSignals(false);
 }
@@ -357,7 +443,7 @@ void GuiCitation::fillEntries(BiblioInfo const & bi)
 bool GuiCitation::isSelected(QModelIndex const & idx)
 {
 	QString const str = idx.data().toString();
-	return selected_model_.stringList().contains(str);
+	return selectedKeys().contains(str);
 }
 
 
@@ -378,8 +464,10 @@ void GuiCitation::updateInfo(BiblioInfo const & bi, QModelIndex const & idx)
 	}
 
 	infoML->setToolTip(qt_("Sketchy preview of the selected citation"));
+	CiteItem ci;
+	ci.richtext = true;
 	QString const keytxt = toqstr(
-		bi.getInfo(qstring_to_ucs4(idx.data().toString()), documentBuffer(), true));
+		bi.getInfo(qstring_to_ucs4(idx.data().toString()), documentBuffer(), ci));
 	infoML->document()->setHtml(keytxt);
 }
 
@@ -523,13 +611,18 @@ void GuiCitation::applyParams(int const choice, bool full, bool force,
 		after.clear();
 
 	cs.forceUpperCase &= force;
-	cs.fullAuthorList &= full;
+	cs.hasStarredVersion &= full;
 	string const command = citationStyleToString(cs);
 
 	params_.setCmdName(command);
 	params_["key"] = qstring_to_ucs4(cited_keys_.join(","));
 	params_["before"] = qstring_to_ucs4(before);
 	params_["after"] = qstring_to_ucs4(after);
+	if (cs.hasQualifiedList) {
+		params_["pretextlist"] = getStringFromVector(getPreTexts(), from_ascii("\t"));
+		params_["posttextlist"] = getStringFromVector(getPostTexts(), from_ascii("\t"));
+	}
+	params_["literal"] = literalCB->isChecked() ? from_ascii("true") : from_ascii("false");
 	dispatchParams();
 }
 
@@ -537,7 +630,107 @@ void GuiCitation::applyParams(int const choice, bool full, bool force,
 void GuiCitation::clearSelection()
 {
 	cited_keys_.clear();
-	selected_model_.setStringList(cited_keys_);
+	setSelectedKeys(cited_keys_);
+}
+
+
+void GuiCitation::setSelectedKeys(QStringList const sl)
+{
+	selected_model_.clear();
+	selected_model_.setColumnCount(3);
+	QStringList headers;
+	headers << qt_("Text before")
+		<< qt_("Cite key")
+		<< qt_("Text after");
+	selected_model_.setHorizontalHeaderLabels(headers);
+	selectedLV->setColumnHidden(0, true);
+	selectedLV->setColumnHidden(2, true);
+	selectedLV->verticalHeader()->setVisible(false);
+	selectedLV->horizontalHeader()->setVisible(false);
+	QStringList::const_iterator it  = sl.begin();
+	QStringList::const_iterator end = sl.end();
+	for (int i = 0; it != end; ++it, ++i) {
+		QStandardItem * si = new QStandardItem();
+		si->setData(*it);
+		si->setText(*it);
+		si->setToolTip(*it);
+		si->setEditable(false);
+		selected_model_.setItem(i, 1, si);
+	}
+}
+
+
+QStringList GuiCitation::selectedKeys()
+{
+	QStringList res;
+	for (int i = 0; i != selected_model_.rowCount(); ++i) {
+		QStandardItem const * item = selected_model_.item(i, 1);
+		if (item)
+			res.append(item->text());
+	}
+	return res;
+}
+
+
+void GuiCitation::setPreTexts(vector<docstring> const m)
+{
+	for (docstring const & s: m) {
+		QStandardItem * si = new QStandardItem();
+		docstring key;
+		docstring pre = split(s, key, ' ');
+		si->setData(toqstr(pre));
+		si->setText(toqstr(pre));
+		QModelIndexList qmil =
+				selected_model_.match(selected_model_.index(0, 1),
+						     Qt::DisplayRole, toqstr(key), 1,
+						     Qt::MatchFlags(Qt::MatchExactly | Qt::MatchWrap));
+		if (!qmil.empty())
+			selected_model_.setItem(qmil.front().row(), 0, si);
+	}
+}
+
+
+vector<docstring> GuiCitation::getPreTexts()
+{
+	vector<docstring> res;
+	for (int i = 0; i != selected_model_.rowCount(); ++i) {
+		QStandardItem const * key = selected_model_.item(i, 1);
+		QStandardItem const * pre = selected_model_.item(i, 0);
+		if (key && pre && !key->text().isEmpty() && !pre->text().isEmpty())
+			res.push_back(qstring_to_ucs4(key->text()) + " " + qstring_to_ucs4(pre->text()));
+	}
+	return res;
+}
+
+
+void GuiCitation::setPostTexts(vector<docstring> const m)
+{
+	for (docstring const & s: m) {
+		QStandardItem * si = new QStandardItem();
+		docstring key;
+		docstring post = split(s, key, ' ');
+		si->setData(toqstr(post));
+		si->setText(toqstr(post));
+		QModelIndexList qmil =
+				selected_model_.match(selected_model_.index(0, 1),
+						     Qt::DisplayRole, toqstr(key), 1,
+						     Qt::MatchFlags(Qt::MatchExactly | Qt::MatchWrap));
+		if (!qmil.empty())
+			selected_model_.setItem(qmil.front().row(), 2, si);
+	}
+}
+
+
+vector<docstring> GuiCitation::getPostTexts()
+{
+	vector<docstring> res;
+	for (int i = 0; i != selected_model_.rowCount(); ++i) {
+		QStandardItem const * key = selected_model_.item(i, 1);
+		QStandardItem const * post = selected_model_.item(i, 2);
+		if (key && post)
+			res.push_back(qstring_to_ucs4(key->text()) + " " + qstring_to_ucs4(post->text()));
+	}
+	return res;
 }
 
 
@@ -546,6 +739,7 @@ void GuiCitation::init()
 	// Make the list of all available bibliography keys
 	BiblioInfo const & bi = bibInfo();
 	all_keys_ = to_qstring_list(bi.getKeys());
+
 	available_model_.setStringList(all_keys_);
 
 	// Ditto for the keys cited in this inset
@@ -554,7 +748,7 @@ void GuiCitation::init()
 		cited_keys_.clear();
 	else
 		cited_keys_ = str.split(",");
-	selected_model_.setStringList(cited_keys_);
+	setSelectedKeys(cited_keys_);
 
 	// Initialize the drop downs
 	fillEntries(bi);
@@ -562,28 +756,38 @@ void GuiCitation::init()
 
 	// Initialize the citation formatting
 	string const & cmd = params_.getCmdName();
-	CitationStyle const cs = citationStyleFromString(cmd);
+	CitationStyle const cs =
+		citationStyleFromString(cmd, documentBuffer().params());
+
 	forceuppercaseCB->setChecked(cs.forceUpperCase);
-	fulllistCB->setChecked(cs.fullAuthorList &&
+	starredCB->setChecked(cs.hasStarredVersion &&
 		documentBuffer().params().fullAuthorList());
 	textBeforeED->setText(toqstr(params_["before"]));
 	textAfterED->setText(toqstr(params_["after"]));
 
+	// if this is a new citation, we set the literal checkbox
+	// to its last set value.
+	if (cited_keys_.isEmpty())
+		literalCB->setChecked(literal_);
+	else
+		literalCB->setChecked(params_["literal"] == "true");
+
+	setPreTexts(getVectorFromString(params_["pretextlist"], from_ascii("\t")));
+	setPostTexts(getVectorFromString(params_["posttextlist"], from_ascii("\t")));
+
 	// Update the interface
 	updateControls(bi);
 	updateStyles(bi);
 	if (selected_model_.rowCount()) {
 		selectedLV->blockSignals(true);
 		selectedLV->setFocus();
-		QModelIndex idx = selected_model_.index(0, 0);
-		selectedLV->selectionModel()->select(idx,
-				QItemSelectionModel::ClearAndSelect);
+		selectedLV->selectRow(0);
 		selectedLV->blockSignals(false);
 
 		// Find the citation style
 		vector<string> const & cmds = citeCmds_;
 		vector<string>::const_iterator cit =
-			std::find(cmds.begin(), cmds.end(), cs.cmd);
+			std::find(cmds.begin(), cmds.end(), cs.name);
 		int i = 0;
 		if (cit != cmds.end())
 			i = int(cit - cmds.begin());
@@ -607,6 +811,7 @@ void GuiCitation::findKey(BiblioInfo const & bi,
 	bool case_sensitive, bool reg_exp, bool reset)
 {
 	// FIXME THREAD
+	// This should be moved to a class member.
 	// Used for optimisation: store last searched string.
 	static QString last_searched_string;
 	// Used to disable the above optimisation.
@@ -655,22 +860,50 @@ void GuiCitation::findKey(BiblioInfo const & bi,
 }
 
 
-QStringList GuiCitation::citationStyles(BiblioInfo const & bi, size_t max_size)
+BiblioInfo::CiteStringMap GuiCitation::citationStyles(BiblioInfo const & bi, size_t max_size)
 {
-	docstring const before = qstring_to_ucs4(textBeforeED->text());
-	docstring const after = qstring_to_ucs4(textAfterED->text());
 	vector<docstring> const keys = to_docstring_vector(cited_keys_);
 	vector<CitationStyle> styles = citeStyles_;
-	// FIXME: pass a dictionary instead of individual before, after, dialog, etc.
-	vector<docstring> ret = bi.getCiteStrings(keys, styles, documentBuffer(),
-		before, after, from_utf8("dialog"), max_size);
-	return to_qstring_list(ret);
+	int ind = citationStyleCO->currentIndex();
+	if (ind == -1)
+		ind = 0;
+	CitationStyle cs = styles[ind];
+	vector<docstring> pretexts = getPreTexts();
+	vector<docstring> posttexts = getPostTexts();
+	bool const qualified = cs.hasQualifiedList
+		&& (selectedLV->model()->rowCount() > 1
+		    || !pretexts.empty()
+		    || !posttexts.empty());
+	std::map<docstring, docstring> pres;
+	for (docstring const & s: pretexts) {
+		docstring key;
+		docstring val = split(s, key, ' ');
+		pres[key] = val;
+	}
+	std::map<docstring, docstring> posts;
+	for (docstring const & s: posttexts) {
+		docstring key;
+		docstring val = split(s, key, ' ');
+		posts[key] = val;
+	}
+	CiteItem ci;
+	ci.textBefore = qstring_to_ucs4(textBeforeED->text());
+	ci.textAfter = qstring_to_ucs4(textAfterED->text());
+	ci.forceUpperCase = forceuppercaseCB->isChecked();
+	ci.Starred = starredCB->isChecked();
+	ci.context = CiteItem::Dialog;
+	ci.max_size = max_size;
+	ci.isQualified = qualified;
+	ci.pretexts = pres;
+	ci.posttexts = posts;
+	BiblioInfo::CiteStringMap ret = bi.getCiteStrings(keys, styles, documentBuffer(), ci);
+	return ret;
 }
 
 
 void GuiCitation::setCitedKeys()
 {
-	cited_keys_ = selected_model_.stringList();
+	cited_keys_ = selectedKeys();
 	updateStyles();
 }
 
@@ -726,7 +959,7 @@ static docstring escape_special_chars(docstring const & expr)
 
 	// $& is an ECMAScript format expression that expands to all
 	// of the current match
-#if defined(LYX_USE_CXX11) && defined(LYX_USE_STD_REGEX)
+#ifdef LYX_USE_STD_REGEX
 	// To prefix a matched expression with a single literal backslash, we
 	// need to escape it for the C++ compiler and use:
 	// FIXME: UNICODE
@@ -825,6 +1058,10 @@ void GuiCitation::saveSession(QSettings & settings) const
 		sessionKey() + "/casesensitive", casesense_->isChecked());
 	settings.setValue(
 		sessionKey() + "/autofind", instant_->isChecked());
+	settings.setValue(
+		sessionKey() + "/citestyle", style_);
+	settings.setValue(
+		sessionKey() + "/literal", literal_);
 }
 
 
@@ -834,7 +1071,9 @@ void GuiCitation::restoreSession()
 	QSettings settings;
 	regexp_->setChecked(settings.value(sessionKey() + "/regex").toBool());
 	casesense_->setChecked(settings.value(sessionKey() + "/casesensitive").toBool());
-	instant_->setChecked(settings.value(sessionKey() + "/autofind").toBool());
+	instant_->setChecked(settings.value(sessionKey() + "/autofind", true).toBool());
+	style_ = settings.value(sessionKey() + "/citestyle").toString();
+	literal_ = settings.value(sessionKey() + "/literal", false).toBool();
 	updateFilterHint();
 }
 
