@@ -21,13 +21,13 @@
 #include "qt_helpers.h"
 
 #include "Font.h"
-#include "Language.h"
 #include "LyXRC.h"
 
-#include "insets/Inset.h"
-
-#include "support/lassert.h"
 #include "support/debug.h"
+#include "support/lassert.h"
+#include "support/lyxlib.h"
+
+#include <algorithm>
 
 #include <QPixmapCache>
 #include <QTextLayout>
@@ -50,7 +50,7 @@ const int Painter::thin_line = 1;
 
 GuiPainter::GuiPainter(QPaintDevice * device, double pixel_ratio)
 	: QPainter(device), Painter(pixel_ratio),
-	  use_pixmap_cache_(lyxrc.use_pixmap_cache && USE_PIXMAP_CACHE)
+	  use_pixmap_cache_(false)
 {
 	// new QPainter has default QPen:
 	current_color_ = guiApp->colorCache().get(Color_black);
@@ -80,8 +80,11 @@ void GuiPainter::setQPainterPen(QColor const & col,
 	pen.setColor(col);
 
 	switch (ls) {
-		case line_solid: pen.setStyle(Qt::SolidLine); break;
-		case line_onoffdash: pen.setStyle(Qt::DotLine); break;
+	case line_solid:
+	case line_solid_aliased:
+		pen.setStyle(Qt::SolidLine); break;
+	case line_onoffdash:
+		pen.setStyle(Qt::DotLine); break;
 	}
 
 	pen.setWidth(lw);
@@ -131,15 +134,15 @@ QColor GuiPainter::filterColor(QColor const & col)
 	// map into [min,max] interval
 	QColor const & min = monochrome_min_.top();
 	QColor const & max = monochrome_max_.top();
-			
+
 	qreal v = col.valueF();
 	v *= v; // make it a bit steeper (i.e. darker)
-		
+
 	qreal minr, ming, minb;
 	qreal maxr, maxg, maxb;
 	min.getRgbF(&minr, &ming, &minb);
 	max.getRgbF(&maxr, &maxg, &maxb);
-			
+
 	QColor c;
 	c.setRgbF(
 		v * (minr - maxr) + maxr,
@@ -186,7 +189,7 @@ void GuiPainter::line(int x1, int y1, int x2, int y2,
 
 	setQPainterPen(computeColor(col), ls, lw);
 	bool const do_antialiasing = renderHints() & TextAntialiasing
-		&& x1 != x2 && y1 != y2;
+		&& x1 != x2 && y1 != y2 && ls != line_solid_aliased;
 	setRenderHint(Antialiasing, do_antialiasing);
 	drawLine(x1, y1, x2, y2);
 	setRenderHint(Antialiasing, false);
@@ -208,6 +211,8 @@ void GuiPainter::lines(int const * xp, int const * yp, int np,
 	if (np > points.size())
 		points.resize(2 * np);
 
+	// Note: the proper way to not get blurry vertical and horizontal lines is
+	// to add 0.5 to all coordinates.
 	bool antialias = false;
 	for (int i = 0; i < np; ++i) {
 		points[i].setX(xp[i]);
@@ -218,7 +223,8 @@ void GuiPainter::lines(int const * xp, int const * yp, int np,
 	QColor const color = computeColor(col);
 	setQPainterPen(color, ls, lw);
 	bool const text_is_antialiased = renderHints() & TextAntialiasing;
-	setRenderHint(Antialiasing, antialias && text_is_antialiased);
+	setRenderHint(Antialiasing,
+	              antialias && text_is_antialiased && ls != line_solid_aliased);
 	if (fs == fill_none) {
 		drawPolyline(points.data(), np);
 	} else {
@@ -259,7 +265,7 @@ void GuiPainter::path(int const * xp, int const * yp,
 	QColor const color = computeColor(col);
 	setQPainterPen(color, ls, lw);
 	bool const text_is_antialiased = renderHints() & TextAntialiasing;
-	setRenderHint(Antialiasing, text_is_antialiased);
+	setRenderHint(Antialiasing, text_is_antialiased && ls != line_solid_aliased);
 	drawPath(bpath);
 	if (fs != fill_none)
 		fillPath(bpath, QBrush(color));
@@ -327,19 +333,20 @@ void GuiPainter::image(int x, int y, int w, int h, graphics::Image const & i)
 }
 
 
-int GuiPainter::text(int x, int y, char_type c, FontInfo const & f)
+void GuiPainter::text(int x, int y, char_type c, FontInfo const & f)
 {
-	return text(x, y, docstring(1, c), f);
+	text(x, y, docstring(1, c), f);
 }
 
 
-int GuiPainter::text(int x, int y, docstring const & s, FontInfo const & f)
+void GuiPainter::text(int x, int y, docstring const & s, FontInfo const & f)
 {
-	return text(x, y, s, f, Auto, 0.0);
+	text(x, y, s, f, Auto, 0.0, 0.0);
 }
 
 
-void GuiPainter::do_drawText(int x, int y, QString str, Direction const dir,
+void GuiPainter::do_drawText(int x, int y, QString str,
+                             GuiPainter::Direction const dir,
                              FontInfo const & f, QFont ff)
 {
 	setQPainterPen(computeColor(f.realColor()));
@@ -379,13 +386,13 @@ void GuiPainter::do_drawText(int x, int y, QString str, Direction const dir,
 }
 
 
-int GuiPainter::text(int x, int y, docstring const & s,
-                     FontInfo const & f, Direction const dir,
-                     double const wordspacing)
+void GuiPainter::text(int x, int y, docstring const & s,
+                      FontInfo const & f, Direction const dir,
+                      double const wordspacing, double const tw)
 {
 	//LYXERR0("text: x=" << x << ", s=" << s);
-	if (s.empty())
-		return 0;
+	if (s.empty() || !isDrawingEnabled())
+		return;
 
 	/* Caution: The following ucs4 to QString conversions work for symbol fonts
 	only because they are no real conversions but simple casts in reality.
@@ -393,8 +400,8 @@ int GuiPainter::text(int x, int y, docstring const & s,
 	of the symbol in the font (as given in lib/symbols) as a char_type to the
 	frontend. This is just wrong, because the symbol is no UCS4 character at
 	all. You can think of this number as the code point of the symbol in a
-	custom symbol encoding. It works because this char_type is lateron again
-	interpreted as a position in the font again.
+	custom symbol encoding. It works because this char_type is later on again
+	interpreted as a position in the font.
 	The correct solution would be to have extra functions for symbols, but that
 	would require to duplicate a lot of frontend and mathed support code.
 	*/
@@ -411,14 +418,13 @@ int GuiPainter::text(int x, int y, docstring const & s,
 	ff.setWordSpacing(wordspacing);
 	GuiFontMetrics const & fm = getFontMetrics(f);
 
-	// Here we use the font width cache instead of
-	//   textwidth = fontMetrics().width(str);
-	// because the above is awfully expensive on MacOSX
-	// Note that we have to take in account space stretching (word spacing)
-	int const textwidth = fm.width(s) + count(s.begin(), s.end(), ' ') * wordspacing;
-
-	if (!isDrawingEnabled())
-		return textwidth;
+	int textwidth = 0;
+	if (tw == 0.0)
+		// Note that we have to take in account space stretching (word spacing)
+		textwidth = fm.width(s) +
+			static_cast<int>(fm.countExpanders(s) * wordspacing);
+	else
+		textwidth = static_cast<int>(tw);
 
 	textDecoration(f, x, y, textwidth);
 
@@ -432,14 +438,14 @@ int GuiPainter::text(int x, int y, docstring const & s,
 		// Only the left bearing of the first character is important
 		// as we always write from left to right, even for
 		// right-to-left languages.
-		// FIXME: this is probably broken for RtL now that we draw full strings.
+		// FIXME: this is probably broken for RTL now that we draw full strings.
 		// Morover the first/last element is possibly not the right one since the glyph may have changed.
 		int const lb = min(fm.lbearing(s[0]), 0);
 		int const mA = fm.maxAscent();
 		if (QPixmapCache::find(key, pm)) {
 			// Draw the cached pixmap.
 			drawPixmap(x + lb, y - mA, pm);
-			return textwidth;
+			return;
 		}
 
 		// Only the right bearing of the last character is
@@ -467,7 +473,7 @@ int GuiPainter::text(int x, int y, docstring const & s,
 			drawPixmap(x + lb, y - mA, pm);
 			//rectangle(x-lb, y-mA, w, h, Color_green);
 		}
-		return textwidth;
+		return;
 	}
 
 	// don't use the pixmap cache
@@ -484,20 +490,20 @@ int GuiPainter::text(int x, int y, docstring const & s,
 	}
 	//LYXERR(Debug::PAINTING, "draw " << string(str.toUtf8())
 	//	<< " at " << x << "," << y);
-	return textwidth;
 }
 
 
-int GuiPainter::text(int x, int y, docstring const & str, Font const & f,
-                     double const wordspacing)
+void GuiPainter::text(int x, int y, docstring const & str, Font const & f,
+                      double const wordspacing, double const tw)
 {
-	return text(x, y, str, f.fontInfo(), f.isVisibleRightToLeft() ? RtL : LtR, wordspacing);
+	text(x, y, str, f.fontInfo(), f.isVisibleRightToLeft() ? RtL : LtR,
+	     wordspacing, tw);
 }
 
 
-int GuiPainter::text(int x, int y, docstring const & str, Font const & f,
-                     Color other, size_type const from, size_type const to,
-                     double const wordspacing)
+void GuiPainter::text(int x, int y, docstring const & str, Font const & f,
+                      Color other, size_type const from, size_type const to,
+                      double const wordspacing, double const tw)
 {
 	GuiFontMetrics const & fm = getFontMetrics(f.fontInfo());
 	FontInfo fi = f.fontInfo();
@@ -516,7 +522,7 @@ int GuiPainter::text(int x, int y, docstring const & str, Font const & f,
 	fi.setPaintColor(other);
 	QRegion const clip(x + xmin, y - ascent, xmax - xmin, height);
 	setClipRegion(clip);
-	int const textwidth = text(x, y, str, fi, dir, wordspacing);
+	text(x, y, str, fi, dir, wordspacing, tw);
 
 	// Then the part in normal color
 	// Note that in Qt5, it is not possible to use Qt::UniteClip,
@@ -524,10 +530,8 @@ int GuiPainter::text(int x, int y, docstring const & str, Font const & f,
 	fi.setPaintColor(orig);
 	QRegion region(viewport());
 	setClipRegion(region - clip);
-	text(x, y, str, fi, dir, wordspacing);
+	text(x, y, str, fi, dir, wordspacing, tw);
 	setClipping(false);
-
-	return textwidth;
 }
 
 
@@ -537,6 +541,8 @@ void GuiPainter::textDecoration(FontInfo const & f, int x, int y, int width)
 		underline(f, x, y, width);
 	if (f.strikeout() == FONT_ON)
 		strikeoutLine(f, x, y, width);
+	if (f.xout() == FONT_ON)
+		crossoutLines(f, x, y, width);
 	if (f.uuline() == FONT_ON)
 		doubleUnderline(f, x, y, width);
 	if (f.uwave() == FONT_ON)
@@ -548,31 +554,10 @@ void GuiPainter::textDecoration(FontInfo const & f, int x, int y, int width)
 static int max(int a, int b) { return a > b ? a : b; }
 
 
-void GuiPainter::button(int x, int y, int w, int h, bool mouseHover)
-{
-	if (mouseHover)
-		fillRectangle(x, y, w, h, Color_buttonhoverbg);
-	else
-		fillRectangle(x, y, w, h, Color_buttonbg);
-	buttonFrame(x, y, w, h);
-}
-
-
-void GuiPainter::buttonFrame(int x, int y, int w, int h)
-{
-	line(x, y, x, y + h - 1, Color_buttonframe);
-	line(x - 1 + w, y, x - 1 + w, y + h - 1, Color_buttonframe);
-	line(x, y - 1, x - 1 + w, y - 1, Color_buttonframe);
-	line(x, y + h - 1, x - 1 + w, y + h - 1, Color_buttonframe);
-}
-
-
 void GuiPainter::rectText(int x, int y, docstring const & str,
 	FontInfo const & font, Color back, Color frame)
 {
-	int width;
-	int ascent;
-	int descent;
+	int width, ascent, descent;
 
 	FontMetrics const & fm = theFontMetrics(font);
 	fm.rectText(str, width, ascent, descent);
@@ -584,24 +569,25 @@ void GuiPainter::rectText(int x, int y, docstring const & str,
 	if (frame != Color_none)
 		rectangle(x, y - ascent, width, ascent + descent, frame);
 
+	// FIXME: let offset depend on font
 	text(x + 3, y, str, font);
 }
 
 
-void GuiPainter::buttonText(int x, int y, docstring const & str,
-	FontInfo const & font, bool mouseHover)
+void GuiPainter::buttonText(int x, int baseline, docstring const & s,
+	FontInfo const & font, Color back, Color frame, int offset)
 {
-	int width;
-	int ascent;
-	int descent;
+	int width, ascent, descent;
 
 	FontMetrics const & fm = theFontMetrics(font);
-	fm.buttonText(str, width, ascent, descent);
+	fm.buttonText(s, offset, width, ascent, descent);
 
-	static int const d = Inset::TEXT_TO_INSET_OFFSET / 2;
+	static int const d = offset / 2;
 
-	button(x + d, y - ascent, width - Inset::TEXT_TO_INSET_OFFSET, descent + ascent, mouseHover);
-	text(x + Inset::TEXT_TO_INSET_OFFSET, y, str, font);
+	fillRectangle(x + d + 1, baseline - ascent + 1, width - offset - 1,
+			      ascent + descent - 1, back);
+	rectangle(x + d, baseline - ascent, width - offset, ascent + descent, frame);
+	text(x + offset, baseline, s, font);
 }
 
 
@@ -659,6 +645,20 @@ void GuiPainter::strikeoutLine(FontInfo const & f, int x, int y, int width)
 }
 
 
+void GuiPainter::crossoutLines(FontInfo const & f, int x, int y, int width)
+{
+	FontInfo tmpf = f;
+	tmpf.setXout(FONT_OFF);
+
+	// the definition of \xout in ulem.sty is
+    //  \def\xout{\bgroup \markoverwith{\hbox to.35em{\hss/\hss}}\ULon}
+	// Let's mimick it somewhat.
+	double offset = max(0.35 * theFontMetrics(tmpf).em(), 1);
+	for (int i = 0 ; i < iround(width / offset) ; ++i)
+		text(x + iround(i * offset), y, '/', tmpf);
+}
+
+
 void GuiPainter::doubleUnderline(FontInfo const & f, int x, int y, int width)
 {
 	FontMetrics const & fm = theFontMetrics(f);
