@@ -105,44 +105,39 @@ public:
 	virtual void image(int x, int y, int w, int h,
 		lyx::graphics::Image const & image);
 
-	/** draw a string at position x, y (y is the baseline). The text
-	 * direction depends on the string itself.
-	 * \return the width of the drawn text.
-	 */
-	virtual int text(int x, int y, docstring const & str, FontInfo const & f);
+	/// draw a string at position x, y (y is the baseline).
+	virtual void text(int x, int y, docstring const & str, FontInfo const & f);
 
-	/** draw a string at position x, y (y is the baseline). The text
-	 * direction is enforced by the \c Font.
-	 * \return the width of the drawn text.
+	/// draw a char at position x, y (y is the baseline)
+	virtual void text(int x, int y, char_type c, FontInfo const & f);
+
+	/** draw a string at position x, y (y is the baseline). The
+	 * text direction is enforced by the \c Font.
 	 */
-	virtual int text(int x, int y, docstring const & str, Font const & f,
-                     double wordspacing = 0.0);
+	virtual void text(int x, int y, docstring const & str, Font const & f,
+                      double wordspacing, double textwidth);
 
 	/** draw a string at position x, y (y is the baseline), but
 	 * make sure that the part between \c from and \c to is in
 	 * \c other color. The text direction is enforced by the \c Font.
-	 * \return the width of the drawn text.
 	 */
-	virtual int text(int x, int y, docstring const & str, Font const & f,
-                     Color other, size_type from, size_type to,
-                     double const wordspacing);
-
-	/// draw a char at position x, y (y is the baseline)
-	virtual int text(int x, int y, char_type c, FontInfo const & f);
+	virtual void text(int x, int y, docstring const & str, Font const & f,
+	                  Color other, size_type from, size_type to,
+                      double wordspacing, double textwidth);
 
 	///
 	virtual void textDecoration(FontInfo const & f, int x, int y, int width);
 
 	/// draw a string and enclose it inside a button frame
 	virtual void buttonText(int x, int baseline, docstring const & s,
-		FontInfo const & font, bool mouseHover);
+		FontInfo const & font, Color back, Color frame, int offset);
 
 	/// start monochrome painting mode, i.e. map every color into [min,max]
-	virtual void enterMonochromeMode(Color const & min, 
+	virtual void enterMonochromeMode(Color const & min,
 		Color const & max);
 	/// leave monochrome painting mode
 	virtual void leaveMonochromeMode();
-	
+
 	/**
 	 * Draw a string and enclose it inside a rectangle. If
 	 * back color is specified, the background is cleared with
@@ -152,9 +147,6 @@ public:
 	virtual void rectText(int x, int baseline, docstring const & str,
 		FontInfo const & font, Color back, Color frame);
 
-	/// draw a filled rectangle with the shape of a 3D button
-	virtual void button(int x, int y, int w, int h, bool mouseHover);
-
 	/// draw a character of a preedit string for cjk support.
 	virtual int preeditText(int x, int y,
 		char_type c, FontInfo const & f, preedit_style style);
@@ -174,25 +166,28 @@ private:
 	void strikeoutLine(FontInfo const & f,
 		int x, int y, int width);
 
+	/// check the font, and if set, draw cross-through lines
+	void crossoutLines(FontInfo const & f,
+		int x, int y, int width);
+
 	/// check the font, and if set, draw double underline
 	void doubleUnderline(FontInfo const & f,
 		int x, int y, int width);
 
-	/// draw a bevelled button border
-	void buttonFrame(int x, int y, int w, int h);
-
 	/// set pen parameters
 	void setQPainterPen(QColor const & col,
 		line_style ls = line_solid, int lw = thin_line);
 
-	/// Direction for painting text
+	// Direction for painting text
 	enum Direction { LtR, RtL, Auto };
 
-	/// Helpers for text() method
+	// Helper for text() method
 	void do_drawText(int x, int y, QString str, Direction dir, FontInfo const & f, QFont ff);
-	///
-	virtual int text(int x, int y, docstring const & str, FontInfo const & f,
-                     Direction dir, double wordspacing);
+
+	// Real text() method
+	void text(int x, int y, docstring const & s,
+              FontInfo const & f, Direction const dir,
+              double const wordspacing, double tw);
 
 	QColor current_color_;
 	Painter::line_style current_ls_;
