@@ -45,7 +45,9 @@ namespace lyx {
 namespace frontend {
 
 GuiTabular::GuiTabular(QWidget * parent)
-	: InsetParamsWidget(parent)
+	: InsetParamsWidget(parent), firstheader_suppressable_(false),
+	  lastfooter_suppressable_(false)
+
 {
 	setupUi(this);
 
@@ -217,7 +219,7 @@ void GuiTabular::enableWidgets() const
 	else if (!multicolumnCB->isChecked() && !multirowCB->isChecked()
 		&& hAlignCO->findData(toqstr("decimal")) == -1)
 		hAlignCO->addItem(qt_("At Decimal Separator"), toqstr("decimal"));
-	bool const dalign = 
+	bool const dalign =
 		hAlignCO->itemData(hAlignCO->currentIndex()).toString() == QString("decimal");
 	decimalPointED->setEnabled(dalign);
 	decimalLA->setEnabled(dalign);
@@ -256,6 +258,12 @@ void GuiTabular::enableWidgets() const
 	// longtables and tabular* cannot have a vertical alignment
 	TableAlignLA->setDisabled(is_tabular_star || longtabular);
 	TableAlignCO->setDisabled(is_tabular_star || longtabular);
+	// longtable cannot be rotated (with rotating package)
+	// FIXME: Add support for [pdf]lscape
+	rotateTabularCB->setDisabled(longtabular);
+	rotateTabularLA->setDisabled(longtabular);
+	// this one would also be disabled with [pdf]lscape
+	rotateTabularAngleSB->setDisabled(longtabular);
 
 	// FIXME: This Dialog is really horrible, disabling/enabling a checkbox
 	// depending on the cursor position is very very unintuitive...
