@@ -20,12 +20,12 @@
 #include "support/strfwd.h"
 
 #include <QMainWindow>
+#include <QMenu>
 
 class QCloseEvent;
 class QDragEnterEvent;
 class QDropEvent;
 class QLabel;
-class QMenu;
 class QShowEvent;
 
 
@@ -110,7 +110,7 @@ public:
 	/// display a message in the view
 	/// could be called from any thread
 	void message(docstring const &);
-	
+
 	bool getStatus(FuncRequest const & cmd, FuncStatus & flag);
 	/// dispatch command.
 	/// \return true if the \c FuncRequest has been dispatched.
@@ -130,8 +130,10 @@ public:
 
 	///
 	void focusInEvent(QFocusEvent * e);
-	/// set a buffer to the current workarea.
-	void setBuffer(Buffer * b); ///< \c Buffer to set.
+	/// Add a Buffer to the View
+	/// \param b Buffer to set.
+	/// \param switch_to Whether to set it to the current workarea.
+	void setBuffer(Buffer * b, bool switch_to = true);
 
 	/// load a document into the current workarea.
 	Buffer * loadDocument(
@@ -171,7 +173,7 @@ public:
 
 	///
 	TocModels & tocModels();
-	
+
 	/// called on timeout
 	void autoSave();
 
@@ -189,7 +191,7 @@ public:
 	/// \return the \c Workarea at index \c index
 	GuiWorkArea * workArea(int index);
 
-	/// Add a \c WorkArea 
+	/// Add a \c WorkArea
 	/// \return the \c Workarea associated to \p  Buffer
 	/// \retval 0 if no \c WorkArea is found.
 	GuiWorkArea * addWorkArea(Buffer & buffer);
@@ -206,13 +208,15 @@ public:
 	GuiWorkArea const * currentMainWorkArea() const;
 	/// return the current document WorkArea (it may not have the focus).
 	GuiWorkArea * currentMainWorkArea();
-	
+
 	/// Current ratio between physical pixels and device-independent pixels
 	double pixelRatio() const;
 
 Q_SIGNALS:
 	void closing(int);
 	void triggerShowDialog(QString const & qname, QString const & qdata, Inset * inset);
+	// emitted when the work area or its buffer view changed
+	void bufferViewChanged();
 
 public Q_SLOTS:
 	///
@@ -222,6 +226,8 @@ public Q_SLOTS:
 	void clearMessage();
 	///
 	void updateWindowTitle(GuiWorkArea * wa);
+	///
+	void disableShellEscape();
 
 private Q_SLOTS:
 	///
@@ -230,6 +236,8 @@ private Q_SLOTS:
 	///
 	void on_currentWorkAreaChanged(GuiWorkArea *);
 	///
+	void onBufferViewChanged();
+	///
 	void on_lastWorkAreaRemoved();
 
 	/// For completion of autosave or export threads.
@@ -298,7 +306,7 @@ public:
 	void hideAll() const;
 
 	/// Update all visible dialogs.
-	/** 
+	/**
 	 *  Check the status of all visible dialogs and disable or reenable
 	 *  them as appropriate.
 	 *
@@ -337,6 +345,8 @@ public:
 	void hideDialog(std::string const & name, Inset * inset);
 	///
 	void disconnectDialog(std::string const & name);
+	///
+	bool develMode() const { return devel_mode_; }
 
 private:
 	/// Saves the layout and geometry of the window
@@ -353,6 +363,8 @@ private:
 	void initToolbars();
 	///
 	void initToolbar(std::string const & name);
+	/// Update lock (all) toolbars position
+	void updateLockToolbars();
 	///
 	bool lfunUiToggle(std::string const & ui_component);
 	///
@@ -360,15 +372,13 @@ private:
 	///
 	void insertLyXFile(docstring const & fname);
 	///
-	void insertPlaintextFile(docstring const & fname,
-		bool asParagraph);
 	/// Open Export As ... dialog. \p iformat is the format the
 	/// filter is initially set to.
 	bool exportBufferAs(Buffer & b, docstring const & iformat);
 
 	///
 	enum RenameKind { LV_WRITE_AS, LV_VC_RENAME, LV_VC_COPY };
-	/// Save a buffer as a new file. 
+	/// Save a buffer as a new file.
 	/**
 	Write a buffer to a new file name and rename the buffer
     according to the new file name.
@@ -401,7 +411,7 @@ private:
 	/// closes the tabworkarea and all tabs. If we are in a close event,
 	/// all buffers will be closed, otherwise they will be hidden.
 	bool closeTabWorkArea(TabWorkArea * twa);
-	/// gives the user the possibility to save his work 
+	/// gives the user the possibility to save his work
 	/// or to discard the changes. If hiding is true, the
 	/// document will be reloaded.
 	bool saveBufferIfNeeded(Buffer & buf, bool hiding);
@@ -453,11 +463,35 @@ private:
 	/// Request to give focus to minibuffer
 	bool minibuffer_focus_;
 
+	/// Statusbar widget that shows shell-escape status
+	QLabel * shell_escape_;
 	/// Statusbar widget that shows read-only status
 	QLabel * read_only_;
 	/// Statusbar widget that shows version control status
 	QLabel * version_control_;
 
+	/// The rate from which the actual zoom value is calculated
+	/// from the default zoom pref
+	double zoom_ratio_ = 1.0;
+	/// Minimum zoom percentage
+	static int const zoom_min_ = 10;
+
+	// movability flag of all toolbars
+	bool toolbarsMovable_;
+
+	// developer mode
+	bool devel_mode_;
+};
+
+
+class SEMenu : public QMenu
+{
+	Q_OBJECT
+public:
+	explicit SEMenu(QWidget * parent);
+
+public Q_SLOTS:
+	void showMenu(QPoint const &) { exec(QCursor::pos()); }
 };
 
 } // namespace frontend
