@@ -80,6 +80,7 @@
 #include "support/bind.h"
 
 #include <cmath>
+#include <iostream>
 
 int const TabIndicatorWidth = 3;
 
@@ -211,8 +212,8 @@ public:
 	int cursorWidth() const { return cursor_width_; }
 	void recomputeWidth() {
 		cursor_width_ = lyxrc.cursor_width
-			? lyxrc.cursor_width 
-			: 1 + int((lyxrc.zoom + 50) / 200.0);
+			? lyxrc.cursor_width
+			: 1 + int((lyxrc.currentZoom + 50) / 200.0);
 	}
 
 	QRect const & rect() { return rect_; }
@@ -232,7 +233,7 @@ private:
 	QRect rect_;
 	/// x position (were the vertical line is drawn)
 	int x_;
-	
+
 	int cursor_width_;
 };
 
@@ -249,21 +250,23 @@ GuiWorkArea::Private::Private(GuiWorkArea * parent)
   cursor_visible_(false), cursor_(0),
   need_resize_(false), schedule_redraw_(false), preedit_lines_(1),
   pixel_ratio_(1.0),
-  completer_(new GuiCompleter(p, p)), dialog_mode_(false),
-  read_only_(false), clean_(true)
+  completer_(new GuiCompleter(p, p)), dialog_mode_(false), shell_escape_(false),
+  read_only_(false), clean_(true), externally_modified_(false)
 {
 }
 
 
 GuiWorkArea::GuiWorkArea(QWidget * /* w */)
-: d(new Private(this)) 
+: d(new Private(this))
 {
+	new CompressorProxy(this); // not a leak
 }
 
 
 GuiWorkArea::GuiWorkArea(Buffer & buffer, GuiView & gv)
 : d(new Private(this))
 {
+	new CompressorProxy(this); // not a leak
 	setGuiView(gv);
 	buffer.params().display_pixel_ratio = theGuiApp()->pixelRatio();
 	setBuffer(buffer);
@@ -274,7 +277,7 @@ GuiWorkArea::GuiWorkArea(Buffer & buffer, GuiView & gv)
 double GuiWorkArea::pixelRatio() const
 {
 #if QT_VERSION >= 0x050000
-	return devicePixelRatio();
+	return qt_scale_factor * devicePixelRatio();
 #else
 	return 1.0;
 #endif
@@ -319,13 +322,10 @@ void GuiWorkArea::init()
 
 	d->setCursorShape(Qt::IBeamCursor);
 
-	d->synthetic_mouse_event_.timeout.timeout.connect(
-		bind(&GuiWorkArea::generateSyntheticMouseEvent,
-					this));
-
-	// Initialize the vertical Scroll Bar
-	QObject::connect(verticalScrollBar(), SIGNAL(valueChanged(int)),
-		this, SLOT(scrollTo(int)));
+	// This connection is closed at the same time as this is destroyed.
+	d->synthetic_mouse_event_.timeout.timeout.connect([this](){
+			generateSyntheticMouseEvent();
+		});
 
 	LYXERR(Debug::GUI, "viewport width: " << viewport()->width()
 		<< "  viewport height: " << viewport()->height());
@@ -340,7 +340,10 @@ void GuiWorkArea::init()
 
 GuiWorkArea::~GuiWorkArea()
 {
-	d->buffer_view_->buffer().workAreaManager().remove(this);
+	// If something is wrong with the buffer, we can ignore it safely
+	try {
+		d->buffer_view_->buffer().workAreaManager().remove(this);
+	} catch(...) {}
 	delete d->screen_;
 	delete d->buffer_view_;
 	delete d->cursor_;
@@ -365,7 +368,7 @@ void GuiWorkArea::Private::setCursorShape(Qt::CursorShape shape)
 
 void GuiWorkArea::Private::updateCursorShape()
 {
-	setCursorShape(buffer_view_->clickableInset() 
+	setCursorShape(buffer_view_->clickableInset()
 		? Qt::PointingHandCursor : Qt::IBeamCursor);
 }
 
@@ -391,6 +394,7 @@ void GuiWorkArea::setBuffer(Buffer & buffer)
 	if (buffer.text().paragraphs().size() > 4)
 		setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
 	QTimer::singleShot(50, this, SLOT(fixVerticalScrollBar()));
+	Q_EMIT bufferViewChanged();
 }
 
 
@@ -529,28 +533,15 @@ void GuiWorkArea::processKeySym(KeySymbol const & key, KeyModifier mod)
 }
 
 
-void GuiWorkArea::Private::dispatch(FuncRequest const & cmd0, KeyModifier mod)
+void GuiWorkArea::Private::dispatch(FuncRequest const & cmd)
 {
 	// Handle drag&drop
-	if (cmd0.action() == LFUN_FILE_OPEN) {
+	if (cmd.action() == LFUN_FILE_OPEN) {
 		DispatchResult dr;
-		lyx_view_->dispatch(cmd0, dr);
+		lyx_view_->dispatch(cmd, dr);
 		return;
 	}
 
-	FuncRequest cmd;
-
-	if (cmd0.action() == LFUN_MOUSE_PRESS) {
-		if (mod == ShiftModifier)
-			cmd = FuncRequest(cmd0, "region-select");
-		else if (mod == ControlModifier)
-			cmd = FuncRequest(cmd0, "paragraph-select");
-		else
-			cmd = cmd0;
-	}
-	else
-		cmd = cmd0;
-
 	bool const notJustMovingTheMouse =
 		cmd.action() != LFUN_MOUSE_MOTION || cmd.button() != mouse_button::none;
 
@@ -682,14 +673,18 @@ void GuiWorkArea::toggleCursor()
 
 void GuiWorkArea::Private::updateScrollbar()
 {
+	// Prevent setRange() and setSliderPosition from causing recursive calls via
+	// the signal valueChanged. (#10311)
+	QObject::disconnect(p->verticalScrollBar(), SIGNAL(valueChanged(int)),
+	                    p, SLOT(scrollTo(int)));
 	ScrollbarParameters const & scroll_ = buffer_view_->scrollbarParameters();
-	// Block signals to prevent setRange() and setSliderPosition from causing
-	// recursive calls via the signal valueChanged. (#10311)
-	QSignalBlocker blocker(p->verticalScrollBar());
 	p->verticalScrollBar()->setRange(scroll_.min, scroll_.max);
 	p->verticalScrollBar()->setPageStep(scroll_.page_step);
 	p->verticalScrollBar()->setSingleStep(scroll_.single_step);
-	p->verticalScrollBar()->setSliderPosition(scroll_.position);
+	p->verticalScrollBar()->setSliderPosition(0);
+	// Connect to the vertical scroll bar
+	QObject::connect(p->verticalScrollBar(), SIGNAL(valueChanged(int)),
+	                 p, SLOT(scrollTo(int)));
 }
 
 
@@ -721,7 +716,7 @@ bool GuiWorkArea::event(QEvent * e)
 			QPoint pos = helpEvent->pos();
 			if (pos.x() < viewport()->width()) {
 				QString s = toqstr(d->buffer_view_->toolTip(pos.x(), pos.y()));
-				QToolTip::showText(helpEvent->globalPos(), s);
+				QToolTip::showText(helpEvent->globalPos(), formatToolTip(s,35));
 			}
 			else
 				QToolTip::hideText();
@@ -782,7 +777,7 @@ void GuiWorkArea::contextMenuEvent(QContextMenuEvent * e)
 		}
 		name = d->buffer_view_->contextMenu(pos.x(), pos.y());
 	}
-	
+
 	if (name.empty()) {
 		e->accept();
 		return;
@@ -828,7 +823,7 @@ void GuiWorkArea::mousePressEvent(QMouseEvent * e)
 	if (d->dc_event_.active && d->dc_event_ == *e) {
 		d->dc_event_.active = false;
 		FuncRequest cmd(LFUN_MOUSE_TRIPLE, e->x(), e->y(),
-			q_button_state(e->button()));
+			q_button_state(e->button()), q_key_state(e->modifiers()));
 		d->dispatch(cmd);
 		e->accept();
 		return;
@@ -839,8 +834,8 @@ void GuiWorkArea::mousePressEvent(QMouseEvent * e)
 #endif
 
 	FuncRequest const cmd(LFUN_MOUSE_PRESS, e->x(), e->y(),
-		q_button_state(e->button()));
-	d->dispatch(cmd, q_key_state(e->modifiers()));
+			q_button_state(e->button()), q_key_state(e->modifiers()));
+	d->dispatch(cmd);
 
 	// Save the context menu on mouse press, because also the mouse
 	// cursor is set on mouse press. Afterwards, we can either release
@@ -860,7 +855,7 @@ void GuiWorkArea::mouseReleaseEvent(QMouseEvent * e)
 		d->synthetic_mouse_event_.timeout.stop();
 
 	FuncRequest const cmd(LFUN_MOUSE_RELEASE, e->x(), e->y(),
-			      q_button_state(e->button()));
+			q_button_state(e->button()), q_key_state(e->modifiers()));
 	d->dispatch(cmd);
 	e->accept();
 }
@@ -871,7 +866,7 @@ void GuiWorkArea::mouseMoveEvent(QMouseEvent * e)
 	// we kill the triple click if we move
 	doubleClickTimeout();
 	FuncRequest cmd(LFUN_MOUSE_MOTION, e->x(), e->y(),
-		q_motion_state(e->buttons()));
+			q_motion_state(e->buttons()), q_key_state(e->modifiers()));
 
 	e->accept();
 
@@ -898,7 +893,7 @@ void GuiWorkArea::mouseMoveEvent(QMouseEvent * e)
 			// in the first place.
 			return;
 		}
-		
+
 		d->synthetic_mouse_event_.restart_timeout = true;
 		d->synthetic_mouse_event_.timeout.start();
 		// Fall through to handle this event...
@@ -950,8 +945,10 @@ void GuiWorkArea::wheelEvent(QWheelEvent * ev)
 	int const lines = qApp->wheelScrollLines();
 	int const page_step = verticalScrollBar()->pageStep();
 	// Test if the wheel mouse is set to one screen at a time.
-	int scroll_value = lines > page_step
-		? page_step : lines * verticalScrollBar()->singleStep();
+	// This is according to
+	// https://doc.qt.io/qt-5/qapplication.html#wheelScrollLines-prop
+	int scroll_value =
+		min(lines * verticalScrollBar()->singleStep(), page_step);
 
 	// Take into account the rotation and the user preferences.
 	scroll_value = int(scroll_value * delta * lyxrc.mouse_wheel_speed);
@@ -981,7 +978,7 @@ void GuiWorkArea::generateSyntheticMouseEvent()
 		if (up || down) {
 			int dist = up ? -e_y : e_y - wh;
 			time = max(min(200, 250000 / (dist * dist)), 1) ;
-			
+
 			if (time < 40) {
 				step = 80000 / (time * time);
 				time = 40;
@@ -1055,6 +1052,39 @@ void GuiWorkArea::generateSyntheticMouseEvent()
 }
 
 
+// CompressorProxy adapted from Kuba Ober https://stackoverflow.com/a/21006207
+CompressorProxy::CompressorProxy(GuiWorkArea * wa) : QObject(wa), flag_(false)
+{
+	qRegisterMetaType<KeySymbol>("KeySymbol");
+	qRegisterMetaType<KeyModifier>("KeyModifier");
+	connect(wa, SIGNAL(compressKeySym(KeySymbol, KeyModifier, bool)),
+		this, SLOT(slot(KeySymbol, KeyModifier, bool)),
+	        Qt::QueuedConnection);
+	connect(this, SIGNAL(signal(KeySymbol, KeyModifier)),
+		wa, SLOT(processKeySym(KeySymbol, KeyModifier)));
+}
+
+
+bool CompressorProxy::emitCheck(bool isAutoRepeat)
+{
+	flag_ = true;
+	if (isAutoRepeat)
+		QCoreApplication::sendPostedEvents(this, QEvent::MetaCall); // recurse
+	bool result = flag_;
+	flag_ = false;
+	return result;
+}
+
+
+void CompressorProxy::slot(KeySymbol sym, KeyModifier mod, bool isAutoRepeat)
+{
+	if (emitCheck(isAutoRepeat))
+		Q_EMIT signal(sym, mod);
+	else
+		LYXERR(Debug::KEY, "system is busy: autoRepeat key event ignored");
+}
+
+
 void GuiWorkArea::keyPressEvent(QKeyEvent * ev)
 {
 	// this is also called for ShortcutOverride events. In this case, one must
@@ -1062,13 +1092,16 @@ void GuiWorkArea::keyPressEvent(QKeyEvent * ev)
 	bool const act = (ev->type() != QEvent::ShortcutOverride);
 
 	// Do not process here some keys if dialog_mode_ is set
-	if (d->dialog_mode_
+	bool const for_dialog_mode = d->dialog_mode_
 		&& (ev->modifiers() == Qt::NoModifier
 		    || ev->modifiers() == Qt::ShiftModifier)
 		&& (ev->key() == Qt::Key_Escape
 		    || ev->key() == Qt::Key_Enter
-		    || ev->key() == Qt::Key_Return)
-	    ) {
+		    || ev->key() == Qt::Key_Return);
+	// also do not use autoRepeat to input shortcuts
+	bool const autoRepeat = ev->isAutoRepeat();
+
+	if (for_dialog_mode || (!act && autoRepeat)) {
 		ev->ignore();
 		return;
 	}
@@ -1085,51 +1118,31 @@ void GuiWorkArea::keyPressEvent(QKeyEvent * ev)
 		}
 	}
 
-	// do nothing if there are other events
-	// (the auto repeated events come too fast)
-	// it looks like this is only needed on X11
-#if defined(Q_WS_X11) || defined(QPA_XCB)
-	// FIXME: this is a weird way to implement event compression. Also, this is
-	// broken with IBus.
-	if (act && qApp->hasPendingEvents() && ev->isAutoRepeat()) {
-		switch (ev->key()) {
-		case Qt::Key_PageDown:
-		case Qt::Key_PageUp:
-		case Qt::Key_Left:
-		case Qt::Key_Right:
-		case Qt::Key_Up:
-		case Qt::Key_Down:
-			LYXERR(Debug::KEY, "system is busy: scroll key event ignored");
-			ev->ignore();
-			return;
-		}
-	}
-#endif
-
 	KeyModifier const m = q_key_state(ev->modifiers());
 
-	std::string str;
-	if (m & ShiftModifier)
-		str += "Shift-";
-	if (m & ControlModifier)
-		str += "Control-";
-	if (m & AltModifier)
-		str += "Alt-";
-	if (m & MetaModifier)
-		str += "Meta-";
-
-	if (act)
+	if (act && lyxerr.debugging(Debug::KEY)) {
+		std::string str;
+		if (m & ShiftModifier)
+			str += "Shift-";
+		if (m & ControlModifier)
+			str += "Control-";
+		if (m & AltModifier)
+			str += "Alt-";
+		if (m & MetaModifier)
+			str += "Meta-";
 		LYXERR(Debug::KEY, " count: " << ev->count() << " text: " << ev->text()
 		       << " isAutoRepeat: " << ev->isAutoRepeat() << " key: " << ev->key()
 		       << " keyState: " << str);
+	}
 
 	KeySymbol sym;
 	setKeySymbol(&sym, ev);
 	if (sym.isOK()) {
 		if (act) {
-			processKeySym(sym, m);
+			Q_EMIT compressKeySym(sym, m, autoRepeat);
 			ev->accept();
 		} else
+			// here, !autoRepeat, as determined at the beginning
 			ev->setAccepted(queryKeySym(sym, m));
 	} else {
 		ev->ignore();
@@ -1147,10 +1160,9 @@ void GuiWorkArea::mouseDoubleClickEvent(QMouseEvent * ev)
 {
 	d->dc_event_ = DoubleClick(ev);
 	QTimer::singleShot(QApplication::doubleClickInterval(), this,
-			   SLOT(doubleClickTimeout()));
-	FuncRequest cmd(LFUN_MOUSE_DOUBLE,
-			ev->x(), ev->y(),
-			q_button_state(ev->button()));
+			SLOT(doubleClickTimeout()));
+	FuncRequest cmd(LFUN_MOUSE_DOUBLE, ev->x(), ev->y(),
+			q_button_state(ev->button()), q_key_state(ev->modifiers()));
 	d->dispatch(cmd);
 	ev->accept();
 }
@@ -1268,7 +1280,7 @@ void GuiWorkArea::inputMethodEvent(QInputMethodEvent * e)
 		stopBlinkingCursor();
 
 	// last_width : for checking if last preedit string was/wasn't empty.
-	// FIXME THREAD
+	// FIXME THREAD && FIXME
 	// We could have more than one work area, right?
 	static bool last_width = false;
 	if (!last_width && preedit_string.empty()) {
@@ -1406,13 +1418,19 @@ QVariant GuiWorkArea::inputMethodQuery(Qt::InputMethodQuery query) const
 void GuiWorkArea::updateWindowTitle()
 {
 	Buffer const & buf = bufferView().buffer();
-	if (buf.fileName() != d->file_name_ || buf.isReadonly() != d->read_only_
-	    || buf.lyxvc().vcstatus() != d->vc_status_ || buf.isClean() != d->clean_) {
+	if (buf.fileName() != d->file_name_
+	    || buf.params().shell_escape != d->shell_escape_
+	    || buf.hasReadonlyFlag() != d->read_only_
+	    || buf.lyxvc().vcstatus() != d->vc_status_
+	    || buf.isClean() != d->clean_
+	    || buf.notifiesExternalModification() != d->externally_modified_) {
 		d->file_name_ = buf.fileName();
-		d->read_only_ = buf.isReadonly();
+		d->shell_escape_ = buf.params().shell_escape;
+		d->read_only_ = buf.hasReadonlyFlag();
 		d->vc_status_ = buf.lyxvc().vcstatus();
 		d->clean_ = buf.isClean();
-		titleChanged(this);
+		d->externally_modified_ = buf.notifiesExternalModification();
+		Q_EMIT titleChanged(this);
 	}
 }
 
@@ -1555,7 +1573,7 @@ NoTabFrameMacStyle noTabFrameMacStyle;
 
 
 TabWorkArea::TabWorkArea(QWidget * parent)
-	: QTabWidget(parent), clicked_tab_(-1)
+	: QTabWidget(parent), clicked_tab_(-1), midpressed_tab_(-1)
 {
 #ifdef Q_OS_MAC
 	setStyle(&noTabFrameMacStyle);
@@ -1603,12 +1621,32 @@ TabWorkArea::TabWorkArea(QWidget * parent)
 }
 
 
+void TabWorkArea::mousePressEvent(QMouseEvent *me)
+{
+	if (me->button() == Qt::MidButton)
+		midpressed_tab_ = tabBar()->tabAt(me->pos());
+	else
+		QTabWidget::mousePressEvent(me);
+}
+
+
+void TabWorkArea::mouseReleaseEvent(QMouseEvent *me)
+{
+	if (me->button() == Qt::MidButton) {
+		int const midreleased_tab = tabBar()->tabAt(me->pos());
+		if (midpressed_tab_ == midreleased_tab && posIsTab(me->pos()))
+			closeTab(midreleased_tab);
+	} else
+		QTabWidget::mouseReleaseEvent(me);
+}
+
+
 void TabWorkArea::paintEvent(QPaintEvent * event)
 {
 	if (tabBar()->isVisible()) {
 		QTabWidget::paintEvent(event);
 	} else {
-		// Prevent the selected tab to influence the 
+		// Prevent the selected tab to influence the
 		// painting of the frame of the tab widget.
 		// This is needed for gtk style in Qt.
 		QStylePainter p(this);
@@ -1626,15 +1664,27 @@ void TabWorkArea::paintEvent(QPaintEvent * event)
 }
 
 
+bool TabWorkArea::posIsTab(QPoint position)
+{
+	// tabAt returns -1 if tab does not covers position
+	return tabBar()->tabAt(position) > -1;
+}
+
+
 void TabWorkArea::mouseDoubleClickEvent(QMouseEvent * event)
 {
 	if (event->button() != Qt::LeftButton)
 		return;
 
+	// this code chunk is unnecessary because it seems the event only makes
+	// it this far if it is not on a tab. I'm not sure why this is (maybe
+	// it is handled and ended in DragTabBar?), and thus I'm not sure if
+	// this is true in all cases and if it will be true in the future so I
+	// leave this code for now. (skostysh, 2016-07-21)
+	//
 	// return early if double click on existing tabs
-	for (int i = 0; i < count(); ++i)
-		if (tabBar()->tabRect(i).contains(event->pos()))
-			return;
+	if (posIsTab(event->pos()))
+		return;
 
 	dispatch(FuncRequest(LFUN_BUFFER_NEW));
 }
@@ -1663,26 +1713,41 @@ void TabWorkArea::showBar(bool show)
 }
 
 
-GuiWorkArea * TabWorkArea::currentWorkArea()
+GuiWorkAreaContainer * TabWorkArea::widget(int index) const
 {
-	if (count() == 0)
-		return 0;
-
-	GuiWorkArea * wa = dynamic_cast<GuiWorkArea *>(currentWidget());
-	LATTEST(wa);
-	return wa;
+	QWidget * w = QTabWidget::widget(index);
+	if (!w)
+		return nullptr;
+	GuiWorkAreaContainer * wac = dynamic_cast<GuiWorkAreaContainer *>(w);
+	LATTEST(wac);
+	return wac;
 }
 
 
-GuiWorkArea * TabWorkArea::workArea(int index)
+GuiWorkAreaContainer * TabWorkArea::currentWidget() const
 {
-	return dynamic_cast<GuiWorkArea *>(widget(index));
+	return widget(currentIndex());
 }
 
 
-GuiWorkArea * TabWorkArea::workArea(Buffer & buffer)
+GuiWorkArea * TabWorkArea::workArea(int index) const
 {
-	// FIXME: this method doesn't work if we have more than work area
+	GuiWorkAreaContainer * w = widget(index);
+	if (!w)
+		return nullptr;
+	return w->workArea();
+}
+
+
+GuiWorkArea * TabWorkArea::currentWorkArea() const
+{
+	return workArea(currentIndex());
+}
+
+
+GuiWorkArea * TabWorkArea::workArea(Buffer & buffer) const
+{
+	// FIXME: this method doesn't work if we have more than one work area
 	// showing the same buffer.
 	for (int i = 0; i != count(); ++i) {
 		GuiWorkArea * wa = workArea(i);
@@ -1697,18 +1762,27 @@ GuiWorkArea * TabWorkArea::workArea(Buffer & buffer)
 void TabWorkArea::closeAll()
 {
 	while (count()) {
-		GuiWorkArea * wa = workArea(0);
-		LASSERT(wa, return);
+		QWidget * wac = widget(0);
+		LASSERT(wac, return);
 		removeTab(0);
-		delete wa;
+		delete wac;
 	}
 }
 
 
+int TabWorkArea::indexOfWorkArea(GuiWorkArea * w) const
+{
+	for (int index = 0; index < count(); ++index)
+		if (workArea(index) == w)
+			return index;
+	return -1;
+}
+
+
 bool TabWorkArea::setCurrentWorkArea(GuiWorkArea * work_area)
 {
 	LASSERT(work_area, return false);
-	int index = indexOf(work_area);
+	int index = indexOfWorkArea(work_area);
 	if (index == -1)
 		return false;
 
@@ -1727,12 +1801,13 @@ bool TabWorkArea::setCurrentWorkArea(GuiWorkArea * work_area)
 GuiWorkArea * TabWorkArea::addWorkArea(Buffer & buffer, GuiView & view)
 {
 	GuiWorkArea * wa = new GuiWorkArea(buffer, view);
+	GuiWorkAreaContainer * wac = new GuiWorkAreaContainer(wa);
 	wa->setUpdatesEnabled(false);
 	// Hide tabbar if there's no tab (avoid a resize and a flashing tabbar
 	// when hiding it again below).
 	if (!(currentWorkArea() && currentWorkArea()->isFullScreen()))
 		showBar(count() > 0);
-	addTab(wa, wa->windowTitle());
+	addTab(wac, wa->windowTitle());
 	QObject::connect(wa, SIGNAL(titleChanged(GuiWorkArea *)),
 		this, SLOT(updateTabTexts()));
 	if (currentWorkArea() && currentWorkArea()->isFullScreen())
@@ -1750,13 +1825,14 @@ GuiWorkArea * TabWorkArea::addWorkArea(Buffer & buffer, GuiView & view)
 bool TabWorkArea::removeWorkArea(GuiWorkArea * work_area)
 {
 	LASSERT(work_area, return false);
-	int index = indexOf(work_area);
+	int index = indexOfWorkArea(work_area);
 	if (index == -1)
 		return false;
 
 	work_area->setUpdatesEnabled(false);
+	QWidget * wac = widget(index);
 	removeTab(index);
-	delete work_area;
+	delete wac;
 
 	if (count()) {
 		// make sure the next work area is enabled.
@@ -1837,7 +1913,7 @@ void TabWorkArea::closeTab(int index)
 class DisplayPath {
 public:
 	/// make vector happy
-	DisplayPath() {}
+	DisplayPath() : tab_(-1), dottedPrefix_(false) {}
 	///
 	DisplayPath(int tab, FileName const & filename)
 		: tab_(tab)
@@ -1988,15 +2064,18 @@ void TabWorkArea::updateTabTexts()
 				continue;
 			}
 
-			// we found a non-atomic segment segStart <= sit <= it < next.
+			// We found a non-atomic segment
+			// We know that segStart <= it < next <= paths.end().
+			// The assertion below tells coverity about it.
+			LATTEST(segStart != paths.end());
+			QString dspString = segStart->forecastPathString();
+			LYXERR(Debug::GUI, "first forecast found for "
+			       << segStart->abs() << " => " << dspString);
+			It sit = segStart;
+			++sit;
 			// Shift path segments and hope for the best
 			// that it makes the path more unique.
 			somethingChanged = true;
-			It sit = segStart;
-			QString dspString = sit->forecastPathString();
-			LYXERR(Debug::GUI, "first forecast found for "
-			       << sit->abs() << " => " << dspString);
-			++sit;
 			bool moreUnique = false;
 			for (; sit != next; ++sit) {
 				if (sit->forecastPathString() != dspString) {
@@ -2032,11 +2111,16 @@ void TabWorkArea::updateTabTexts()
 		if (!buf.fileName().empty() && !buf.isClean())
 			tab_text += "*";
 		QString tab_tooltip = it->abs();
-		if (buf.isReadonly()) {
+		if (buf.hasReadonlyFlag()) {
 			setTabIcon(tab_index, QIcon(getPixmap("images/", "emblem-readonly", "svgz,png")));
-			tab_tooltip = qt_("%1 (read only)").arg(it->abs());
+			tab_tooltip = qt_("%1 (read only)").arg(tab_tooltip);
 		} else
 			setTabIcon(tab_index, QIcon());
+		if (buf.notifiesExternalModification()) {
+			QString const warn = qt_("%1 (modified externally)");
+			tab_tooltip = warn.arg(tab_tooltip);
+			tab_text += QChar(0x26a0);
+		}
 		setTabText(tab_index, tab_text);
 		setTabToolTip(tab_index, tab_tooltip);
 	}
@@ -2153,6 +2237,58 @@ void DragTabBar::dropEvent(QDropEvent * event)
 }
 
 
+GuiWorkAreaContainer::GuiWorkAreaContainer(GuiWorkArea * wa, QWidget * parent)
+	: QWidget(parent), wa_(wa)
+{
+	LASSERT(wa, return);
+	Ui::WorkAreaUi::setupUi(this);
+	layout()->addWidget(wa);
+	connect(wa, SIGNAL(titleChanged(GuiWorkArea *)),
+	        this, SLOT(updateDisplay()));
+	connect(reloadPB, SIGNAL(clicked()), this, SLOT(reload()));
+	connect(ignorePB, SIGNAL(clicked()), this, SLOT(ignore()));
+	setMessageColour({notificationFrame}, {reloadPB, ignorePB});
+	updateDisplay();
+}
+
+
+void GuiWorkAreaContainer::updateDisplay()
+{
+	Buffer const & buf = wa_->bufferView().buffer();
+	notificationFrame->setHidden(!buf.notifiesExternalModification());
+	QString const label = qt_("<b>The file %1 changed on disk.</b>")
+		.arg(toqstr(buf.fileName().displayName()));
+	externalModificationLabel->setText(label);
+}
+
+
+void GuiWorkAreaContainer::dispatch(FuncRequest f) const
+{
+	lyx::dispatch(FuncRequest(LFUN_BUFFER_SWITCH,
+	                          wa_->bufferView().buffer().absFileName()));
+	lyx::dispatch(f);
+}
+
+
+void GuiWorkAreaContainer::reload() const
+{
+	dispatch(FuncRequest(LFUN_BUFFER_RELOAD));
+}
+
+
+void GuiWorkAreaContainer::ignore() const
+{
+	dispatch(FuncRequest(LFUN_BUFFER_EXTERNAL_MODIFICATION_CLEAR));
+}
+
+
+void GuiWorkAreaContainer::mouseDoubleClickEvent(QMouseEvent * event)
+{
+	// prevent TabWorkArea from opening a new buffer on double click
+	event->accept();
+}
+
+
 } // namespace frontend
 } // namespace lyx
 
