@@ -13,7 +13,10 @@
 #ifndef WORKAREA_H
 #define WORKAREA_H
 
+#include "ui_WorkAreaUi.h"
+
 #include "frontends/WorkArea.h"
+#include "frontends/KeySymbol.h"
 
 #include <QAbstractScrollArea>
 #include <QTabBar>
@@ -31,6 +34,7 @@ class QWidget;
 namespace lyx {
 
 class Buffer;
+class FuncRequest;
 
 namespace frontend {
 
@@ -70,9 +74,6 @@ public:
 
 	/// return true if the key is part of a shortcut
 	bool queryKeySym(KeySymbol const & key, KeyModifier mod) const;
-	/// Process Key pressed event.
-	/// This needs to be public because it is accessed externally by GuiView.
-	void processKeySym(KeySymbol const & key, KeyModifier mod);
 
 	bool inDialogMode() const;
 	void setDialogMode(bool mode);
@@ -90,6 +91,9 @@ public:
 	double pixelRatio() const;
 
 public Q_SLOTS:
+	/// Process Key pressed event.
+	/// This needs to be public because it is accessed externally by GuiView.
+	void processKeySym(KeySymbol const & key, KeyModifier mod);
 	///
 	void stopBlinkingCursor();
 	///
@@ -100,6 +104,10 @@ Q_SIGNALS:
 	void titleChanged(GuiWorkArea *);
 	///
 	void busy(bool);
+	///
+	void bufferViewChanged();
+	/// send key event to CompressorProxy
+	void compressKeySym(KeySymbol sym, KeyModifier mod, bool isAutoRepeat);
 
 private Q_SLOTS:
 	/// Scroll the BufferView.
@@ -160,6 +168,23 @@ private:
 }; // GuiWorkArea
 
 
+/// CompressorProxy adapted from Kuba Ober https://stackoverflow.com/a/21006207
+class CompressorProxy : public QObject
+{
+    Q_OBJECT
+	bool emitCheck(bool isAutoRepeat);
+	bool flag_;
+	// input: event to compress
+	Q_SLOT void slot(KeySymbol sym, KeyModifier mod, bool isAutoRepeat);
+	// output: compressed event
+    Q_SIGNAL void signal(KeySymbol sym, KeyModifier mod);
+public:
+    // No default constructor, since the proxy must be a child of the
+    // target object.
+	explicit CompressorProxy(GuiWorkArea * wa);
+};
+
+
 class EmbeddedWorkArea : public GuiWorkArea
 {
 	Q_OBJECT
@@ -188,6 +213,8 @@ private:
 }; // EmbeddedWorkArea
 
 
+class GuiWorkAreaContainer;
+
 /// A tabbed set of GuiWorkAreas.
 class TabWorkArea : public QTabWidget
 {
@@ -195,6 +222,10 @@ class TabWorkArea : public QTabWidget
 public:
 	TabWorkArea(QWidget * parent = 0);
 
+	/// hide QTabWidget methods
+	GuiWorkAreaContainer * currentWidget() const;
+	GuiWorkAreaContainer * widget(int index) const;
+
 	///
 	void setFullScreen(bool full_screen);
 	void showBar(bool show);
@@ -202,9 +233,9 @@ public:
 	bool setCurrentWorkArea(GuiWorkArea *);
 	GuiWorkArea * addWorkArea(Buffer & buffer, GuiView & view);
 	bool removeWorkArea(GuiWorkArea *);
-	GuiWorkArea * currentWorkArea();
-	GuiWorkArea * workArea(Buffer & buffer);
-	GuiWorkArea * workArea(int index);
+	GuiWorkArea * currentWorkArea() const;
+	GuiWorkArea * workArea(Buffer & buffer) const;
+	GuiWorkArea * workArea(int index) const;
 	void paintEvent(QPaintEvent *);
 
 Q_SIGNALS:
@@ -224,19 +255,31 @@ public Q_SLOTS:
 	void moveTab(int fromIndex, int toIndex);
 	///
 	void updateTabTexts();
-	
+
 private Q_SLOTS:
 	///
 	void on_currentTabChanged(int index);
 	///
 	void showContextMenu(const QPoint & pos);
+	/// enable closing tab on middle-click
+	void mousePressEvent(QMouseEvent * me);
+	void mouseReleaseEvent(QMouseEvent * me);
 	///
 	void mouseDoubleClickEvent(QMouseEvent * event);
+	///
+	int indexOfWorkArea(GuiWorkArea * w) const;
 
 private:
-	///
+	using QTabWidget::addTab;
+	using QTabWidget::insertTab;
+
+	/// true if position is a tab (rather than the blank space in tab bar)
+	bool posIsTab(QPoint position);
+
 	int clicked_tab_;
 	///
+	int midpressed_tab_;
+	///
 	QToolButton * closeBufferButton;
 }; // TabWorkArea
 
@@ -267,6 +310,31 @@ Q_SIGNALS:
 	void tabMoveRequested(int fromIndex, int toIndex);
 };
 
+
+class GuiWorkAreaContainer : public QWidget, public Ui::WorkAreaUi
+{
+	Q_OBJECT
+	// non-null
+	GuiWorkArea * const wa_;
+	void dispatch(FuncRequest f) const;
+
+private Q_SLOTS:
+	void updateDisplay();
+	void reload() const;
+	void ignore() const;
+
+protected:
+	void mouseDoubleClickEvent(QMouseEvent * event); //override
+
+public:
+	/// wa != 0
+	GuiWorkAreaContainer(GuiWorkArea * wa, QWidget * parent = 0);
+	/// non-null
+	GuiWorkArea * workArea() const { return wa_; }
+};
+
+
+
 } // namespace frontend
 } // namespace lyx
 
