@@ -60,6 +60,7 @@
 #include "insets/Inset.h"
 #include "insets/InsetCitation.h"
 #include "insets/InsetGraphics.h"
+#include "insets/InsetQuotes.h"
 
 #include "support/lassert.h"
 #include "support/convert.h"
@@ -78,9 +79,8 @@
 #include <QProxyStyle>
 #endif
 
-#include "support/shared_ptr.h"
-
 #include <algorithm>
+#include <memory>
 #include <vector>
 
 using namespace std;
@@ -135,6 +135,9 @@ public:
 		/** This is a list of exportable formats
 		    typically for the File->Export menu. */
 		ExportFormats,
+		/** This exports the document default format
+		    typically for the File->Export menu. */
+		ExportFormat,
 		/** This is a list of importable formats
 		    typically for the File->Import menu. */
 		ImportFormats,
@@ -189,7 +192,11 @@ public:
 		in the InsetCaption context menu. */
 		SwitchCaptions,
 		/** Commands to separate environments. */
-		EnvironmentSeparators
+		EnvironmentSeparators,
+		/** Commands to separate environments (context menu version). */
+		EnvironmentSeparatorsContext,
+		/** This is the list of quotation marks available */
+		SwitchQuotes
 	};
 
 	explicit MenuItem(Kind kind) : kind_(kind), optional_(false) {}
@@ -199,8 +206,8 @@ public:
 		 QString const & submenu = QString(),
 		 QString const & tooltip = QString(),
 		 bool optional = false)
-		: kind_(kind), label_(label), submenuname_(submenu),
-		  tooltip_(tooltip), optional_(optional)
+		: kind_(kind), label_(label), func_(make_shared<FuncRequest>()),
+		  submenuname_(submenu), tooltip_(tooltip), optional_(optional)
 	{
 		LATTEST(kind == Submenu || kind == Help || kind == Info);
 	}
@@ -211,15 +218,12 @@ public:
 		 QString const & tooltip = QString(),
 		 bool optional = false,
 		 FuncRequest::Origin origin = FuncRequest::MENU)
-		: kind_(kind), label_(label), func_(func),
+		: kind_(kind), label_(label), func_(make_shared<FuncRequest>(func)),
 		  tooltip_(tooltip), optional_(optional)
 	{
-		func_.setOrigin(origin);
+		func_->setOrigin(origin);
 	}
 
-	// shared_ptr<MenuDefinition> needs this apprently...
-	~MenuItem() {}
-
 	/// The label of a given menuitem
 	QString label() const
 	{
@@ -238,7 +242,7 @@ public:
 	/// The kind of entry
 	Kind kind() const { return kind_; }
 	/// the action (if relevant)
-	FuncRequest const & func() const { return func_; }
+	shared_ptr<FuncRequest const> func() const { return func_; }
 	/// the tooltip
 	QString const & tooltip() const { return tooltip_; }
 	/// returns true if the entry should be omitted when disabled
@@ -257,13 +261,13 @@ public:
 			return QString();
 		// Get the keys bound to this action, but keep only the
 		// first one later
-		KeyMap::Bindings bindings = theTopLevelKeymap().findBindings(func_);
+		KeyMap::Bindings bindings = theTopLevelKeymap().findBindings(*func_);
 		if (!bindings.empty())
 			return toqstr(bindings.begin()->print(KeySequence::ForGui));
 
 		LYXERR(Debug::KBMAP, "No binding for "
-			<< lyxaction.getActionName(func_.action())
-			<< '(' << func_.argument() << ')');
+			<< lyxaction.getActionName(func_->action())
+			<< '(' << func_->argument() << ')');
 		return QString();
 	}
 
@@ -289,7 +293,7 @@ private:
 	///
 	QString label_;
 	///
-	FuncRequest func_;
+	shared_ptr<FuncRequest> func_;// non-null
 	///
 	QString submenuname_;
 	///
@@ -352,6 +356,7 @@ public:
 	void expandFloatListInsert(Buffer const * buf);
 	void expandFloatInsert(Buffer const * buf);
 	void expandFlexInsert(Buffer const * buf, InsetLayout::InsetLyXType type);
+	void expandTocSubmenu(std::string const & type, Toc const & toc_list);
 	void expandToc2(Toc const & toc_list, size_t from, size_t to, int depth, string toc_type);
 	void expandToc(Buffer const * buf);
 	void expandPasteRecent(Buffer const * buf);
@@ -365,7 +370,8 @@ public:
 	void expandLanguageSelector(Buffer const * buf);
 	void expandArguments(BufferView const *, bool switcharg = false);
 	void expandCaptions(Buffer const * buf, bool switchcap = false);
-	void expandEnvironmentSeparators(BufferView const *);
+	void expandEnvironmentSeparators(BufferView const *, bool contextmenu = false);
+	void expandQuotes(BufferView const *);
 	///
 	ItemList items_;
 	///
@@ -400,7 +406,7 @@ void MenuDefinition::addWithStatusCheck(MenuItem const & i)
 	switch (i.kind()) {
 
 	case MenuItem::Command: {
-		FuncStatus status = lyx::getStatus(i.func());
+		FuncStatus status = lyx::getStatus(*i.func());
 		if (status.unknown() || (!status.enabled() && i.optional()))
 			break;
 		items_.push_back(i);
@@ -453,6 +459,7 @@ void MenuDefinition::read(Lexer & lex)
 		md_custom,
 		md_elements,
 		md_endmenu,
+		md_exportformat,
 		md_exportformats,
 		md_importformats,
 		md_indices,
@@ -478,7 +485,9 @@ void MenuDefinition::read(Lexer & lex)
 		md_switcharguments,
 		md_captions,
 		md_switchcaptions,
-		md_env_separators
+		md_env_separators,
+		md_env_separatorscontext,
+		md_switchquotes
 	};
 
 	LexerKeyword menutags[] = {
@@ -493,6 +502,8 @@ void MenuDefinition::read(Lexer & lex)
 		{ "elements", md_elements },
 		{ "end", md_endmenu },
 		{ "environmentseparators", md_env_separators },
+		{ "environmentseparatorscontext", md_env_separatorscontext },
+		{ "exportformat", md_exportformat },
 		{ "exportformats", md_exportformats },
 		{ "floatinsert", md_floatinsert },
 		{ "floatlistinsert", md_floatlistinsert },
@@ -513,6 +524,7 @@ void MenuDefinition::read(Lexer & lex)
 		{ "submenu", md_submenu },
 		{ "switcharguments", md_switcharguments },
 		{ "switchcaptions", md_switchcaptions },
+		{ "switchquotes", md_switchquotes },
 		{ "toc", md_toc },
 		{ "toolbars", md_toolbars },
 		{ "updateformats", md_updateformats },
@@ -584,6 +596,10 @@ void MenuDefinition::read(Lexer & lex)
 			add(MenuItem(MenuItem::ExportFormats));
 			break;
 
+		case md_exportformat:
+			add(MenuItem(MenuItem::ExportFormat));
+			break;
+
 		case md_importformats:
 			add(MenuItem(MenuItem::ImportFormats));
 			break;
@@ -660,6 +676,14 @@ void MenuDefinition::read(Lexer & lex)
 			add(MenuItem(MenuItem::EnvironmentSeparators));
 			break;
 
+		case md_env_separatorscontext:
+			add(MenuItem(MenuItem::EnvironmentSeparatorsContext));
+			break;
+
+		case md_switchquotes:
+			add(MenuItem(MenuItem::SwitchQuotes));
+			break;
+
 		case md_optsubmenu:
 		case md_submenu: {
 			lex.next(true);
@@ -687,7 +711,7 @@ void MenuDefinition::read(Lexer & lex)
 bool MenuDefinition::hasFunc(FuncRequest const & func) const
 {
 	for (const_iterator it = begin(), et = end(); it != et; ++it)
-		if (it->func() == func)
+		if (*it->func() == func)
 			return true;
 	return false;
 }
@@ -737,7 +761,7 @@ bool MenuDefinition::searchMenu(FuncRequest const & func, docstring_list & names
 	const_iterator m = begin();
 	const_iterator m_end = end();
 	for (; m != m_end; ++m) {
-		if (m->kind() == MenuItem::Command && m->func() == func) {
+		if (m->kind() == MenuItem::Command && *m->func() == func) {
 			names.push_back(qstring_to_ucs4(m->label()));
 			return true;
 		}
@@ -974,6 +998,8 @@ void MenuDefinition::expandDocuments()
 		QString label = toqstr(b.fileName().displayName(20));
 		if (!b.isClean())
 			label += "*";
+		if (b.notifiesExternalModification())
+			label += QChar(0x26a0);
 		if (i < 10)
 			label = QString::number(i) + ". " + label + '|' + QString::number(i);
 		add(MenuItem(MenuItem::Command, label,
@@ -991,6 +1017,8 @@ void MenuDefinition::expandDocuments()
 			QString label = toqstr(b->fileName().displayName(20));
 			if (!b->isClean())
 				label += "*";
+			if (b->notifiesExternalModification())
+				label += QChar(0x26a0);
 			if (i < 10)
 				label = QString::number(i) + ". " + label + '|' + QString::number(i);
 			item.submenu().add(MenuItem(MenuItem::Command, label,
@@ -1066,14 +1094,12 @@ void MenuDefinition::expandFormats(MenuItem::Kind const kind, Buffer const * buf
 	MenuItem item(MenuItem::Submenu, smenue);
 	item.setSubmenu(MenuDefinition(smenue));
 
-	FormatList::const_iterator fit = formats.begin();
-	FormatList::const_iterator end = formats.end();
-	for (; fit != end ; ++fit) {
-		if ((*fit)->dummy())
+	for (Format const * f : formats) {
+		if (f->dummy())
 			continue;
 
-		docstring lab = from_utf8((*fit)->prettyname());
-		docstring const scut = from_utf8((*fit)->shortcut());
+		docstring lab = f->prettyname();
+		docstring const scut = from_utf8(f->shortcut());
 		docstring const tmplab = lab;
 
 		if (!scut.empty())
@@ -1090,7 +1116,7 @@ void MenuDefinition::expandFormats(MenuItem::Kind const kind, Buffer const * buf
 			break;
 		case MenuItem::ViewFormats:
 		case MenuItem::UpdateFormats:
-			if ((*fit)->name() == buf->params().getDefaultOutputFormat()) {
+			if (f->name() == buf->params().getDefaultOutputFormat()) {
 				docstring lbl = (kind == MenuItem::ViewFormats
 					? bformat(_("View [%1$s]|V"), label)
 					: bformat(_("Update [%1$s]|U"), label));
@@ -1099,7 +1125,7 @@ void MenuDefinition::expandFormats(MenuItem::Kind const kind, Buffer const * buf
 			}
 		// fall through
 		case MenuItem::ExportFormats:
-			if (!(*fit)->inExportMenu())
+			if (!f->inExportMenu())
 				continue;
 			break;
 		default:
@@ -1114,14 +1140,14 @@ void MenuDefinition::expandFormats(MenuItem::Kind const kind, Buffer const * buf
 			// note that at this point, we know that buf is not null
 			LATTEST(buf);
 			item.submenu().addWithStatusCheck(MenuItem(MenuItem::Command,
-				toqstr(label), FuncRequest(action, (*fit)->name())));
+				toqstr(label), FuncRequest(action, f->name())));
 		} else {
 			if (buf)
 				addWithStatusCheck(MenuItem(MenuItem::Command, toqstr(label),
-					FuncRequest(action, (*fit)->name())));
+					FuncRequest(action, f->name())));
 			else
 				add(MenuItem(MenuItem::Command, toqstr(label),
-					FuncRequest(action, (*fit)->name())));
+					FuncRequest(action, f->name())));
 		}
 	}
 	if (view_update)
@@ -1203,7 +1229,7 @@ void MenuDefinition::expandFlexInsert(
 		}
 	}
 	// FIXME This is a little clunky.
-	if (items_.empty() && type == InsetLayout::CUSTOM && !buf->isReadonly())
+	if (items_.empty() && type == InsetLayout::CUSTOM && !buf->hasReadonlyFlag())
 		add(MenuItem(MenuItem::Help, qt_("No Custom Insets Defined!")));
 }
 
@@ -1306,25 +1332,20 @@ void MenuDefinition::expandToc(Buffer const * buf)
 	}
 
 	MenuDefinition other_lists;
-	FloatList const & floatlist = buf->params().documentClass().floats();
-	TocList const & toc_list = buf->tocBackend().tocs();
-	TocList::const_iterator cit = toc_list.begin();
-	TocList::const_iterator end = toc_list.end();
-	for (; cit != end; ++cit) {
+	// In the navigation menu, only add tocs from this document
+	TocBackend const & backend = buf->tocBackend();
+	TocList const & toc_list = backend.tocs();
+	for (pair<string, shared_ptr<Toc>> const & toc : toc_list) {
 		// Handle table of contents later
-		if (cit->first == "tableofcontents" || cit->second->empty())
+		if (toc.first == "tableofcontents" || toc.second->empty())
 			continue;
 		MenuDefinition submenu;
-		// "Open outliner..." entry
-		FuncRequest f(LFUN_DIALOG_SHOW, "toc " + cit->first);
-		submenu.add(MenuItem(MenuItem::Command, qt_("Open Outliner..."), f));
-		submenu.add(MenuItem(MenuItem::Separator));
-		// add entries
-		submenu.expandToc2(*cit->second, 0, cit->second->size(), 0, cit->first);
-		MenuItem item(MenuItem::Submenu, guiName(cit->first, buf->params()));
+		submenu.expandTocSubmenu(toc.first, *toc.second);
+		docstring const toc_name = backend.outlinerName(toc.first);
+		MenuItem item(MenuItem::Submenu, toqstr(toc_name));
 		item.setSubmenu(submenu);
 		// deserves to be in the main menu?
-		if (floatlist.typeExist(cit->first) || cit->first == "child")
+		if (!TocBackend::isOther(toc.first))
 			add(item);
 		else
 			other_lists.add(item);
@@ -1336,8 +1357,8 @@ void MenuDefinition::expandToc(Buffer const * buf)
 	}
 	// Handle normal TOC
 	add(MenuItem(MenuItem::Separator));
-	cit = toc_list.find("tableofcontents");
-	if (cit == end)
+	TocList::const_iterator cit = toc_list.find("tableofcontents");
+	if (cit == toc_list.end())
 		LYXERR(Debug::GUI, "No table of contents.");
 	else {
 		if (!cit->second->empty())
@@ -1349,6 +1370,17 @@ void MenuDefinition::expandToc(Buffer const * buf)
 }
 
 
+void MenuDefinition::expandTocSubmenu(std::string const & type, Toc const & toc)
+{
+	// "Open outliner..." entry
+	FuncRequest f(LFUN_DIALOG_SHOW, "toc " + type);
+	add(MenuItem(MenuItem::Command, qt_("Open Outliner..."), f));
+	add(MenuItem(MenuItem::Separator));
+	// add entries
+	expandToc2(toc, 0, toc.size(), 0, type);
+}
+
+
 void MenuDefinition::expandPasteRecent(Buffer const * buf)
 {
 	docstring_list const sel = cap::availableSelections(buf);
@@ -1388,7 +1420,7 @@ void MenuDefinition::expandToolbars()
 
 void MenuDefinition::expandBranches(Buffer const * buf)
 {
-	if (!buf || buf->isReadonly())
+	if (!buf || buf->hasReadonlyFlag())
 		return;
 
 	BufferParams const & master_params = buf->masterBuffer()->params();
@@ -1525,10 +1557,11 @@ void MenuDefinition::expandCiteStyles(BufferView const * bv)
 				    FuncRequest(LFUN_NOACTION)));
 		return;
 	}
-	InsetCommand const * citinset =
-				static_cast<InsetCommand const *>(inset);
+	InsetCitation const * citinset =
+				static_cast<InsetCitation const *>(inset);
 
 	Buffer const * buf = &bv->buffer();
+	BufferParams const & bp = buf->masterParams();
 	string const cmd = citinset->params().getCmdName();
 
 	docstring const & key = citinset->getParam("key");
@@ -1539,33 +1572,85 @@ void MenuDefinition::expandCiteStyles(BufferView const * bv)
 		return;
 	}
 
-	docstring const & before = citinset->getParam("before");
-	docstring const & after = citinset->getParam("after");
-
 	size_t const n = cmd.size();
-	bool const force = cmd[0] == 'C';
-	bool const full = cmd[n] == '*';
+	bool const force = isUpperCase(cmd[0]);
+	bool const star = cmd[n] == '*';
 
 	vector<docstring> const keys = getVectorFromString(key);
 
-	vector<CitationStyle> const citeStyleList = buf->params().citeStyles();
-	static const size_t max_length = 40;
-	vector<docstring> citeStrings =
-		buf->masterBibInfo().getCiteStrings(keys, citeStyleList, bv->buffer(),
-		before, after, from_utf8("dialog"), max_length);
+	vector<CitationStyle> const citeStyleList = bp.citeStyles();
 
-	vector<docstring>::const_iterator cit = citeStrings.begin();
-	vector<docstring>::const_iterator end = citeStrings.end();
+	CitationStyle cs = citinset->getCitationStyle(bp, cmd, citeStyleList);
+	bool const qualified = cs.hasQualifiedList
+		&& (keys.size() > 1
+		    || !citinset->getParam("pretextlist").empty()
+		    || !citinset->getParam("posttextlist").empty());
+	std::map<docstring, docstring> pres =
+		citinset->getQualifiedLists(citinset->getParam("pretextlist"));
+	std::map<docstring, docstring> posts =
+		citinset->getQualifiedLists(citinset->getParam("posttextlist"));
+
+	CiteItem ci;
+	ci.textBefore = citinset->getParam("before");
+	ci.textAfter = citinset->getParam("after");
+	ci.forceUpperCase = force;
+	ci.Starred = star;
+	ci.context = CiteItem::Dialog;
+	ci.max_size = 40;
+	ci.isQualified = qualified;
+	ci.pretexts = pres;
+	ci.posttexts = posts;
+	BiblioInfo::CiteStringMap citeStrings =
+		buf->masterBibInfo().getCiteStrings(keys, citeStyleList, bv->buffer(), ci);
+
+	BiblioInfo::CiteStringMap::const_iterator cit = citeStrings.begin();
+	BiblioInfo::CiteStringMap::const_iterator end = citeStrings.end();
 
 	for (int ii = 1; cit != end; ++cit, ++ii) {
-		docstring label = *cit;
+		docstring label = cit->second;
 		CitationStyle cs = citeStyleList[ii - 1];
 		cs.forceUpperCase &= force;
-		cs.fullAuthorList &= full;
+		cs.hasStarredVersion &= star;
 		addWithStatusCheck(MenuItem(MenuItem::Command, toqstr(label),
 				    FuncRequest(LFUN_INSET_MODIFY,
 						"changetype " + from_utf8(citationStyleToString(cs)))));
 	}
+
+	if (cs.hasStarredVersion) {
+		docstring starred = _("All authors|h");
+		// Check if we have a custom string/tooltip for the starred version
+		if (!cs.stardesc.empty()) {
+			string val =
+				bp.documentClass().getCiteMacro(buf->params().citeEngineType(), cs.stardesc);
+			if (!val.empty())
+				starred = translateIfPossible(from_utf8(val));
+			// Transform qt-style accelerators to menu-style
+			int const amps = count_char(starred, '&');
+			if (amps > 0) {
+				if (amps > 1)
+					starred = subst(starred, from_ascii("&&"), from_ascii("<:amp:>"));
+				size_t n = starred.find('&');
+				char_type accel = char_type();
+				if (n != docstring::npos && n < starred.size() - 1)
+					accel = starred[n + 1];
+				starred = subst(starred, from_ascii("&"), from_ascii(""));
+				if (amps > 1)
+					starred = subst(starred, from_ascii("<:amp:>"), from_ascii("&&"));
+				if (accel != char_type())
+					starred = starred + '|' + accel;
+			}
+		}
+		add(MenuItem(MenuItem::Separator));
+		addWithStatusCheck(MenuItem(MenuItem::Command, toqstr(starred),
+				    FuncRequest(LFUN_INSET_MODIFY, "toggleparam star")));
+	}
+
+	if (cs.forceUpperCase) {
+		if (!cs.hasStarredVersion)
+			add(MenuItem(MenuItem::Separator));
+		addWithStatusCheck(MenuItem(MenuItem::Command, toqstr(_("Force upper case|u")),
+				    FuncRequest(LFUN_INSET_MODIFY, "toggleparam casing")));
+	}
 }
 
 
@@ -1608,13 +1693,10 @@ void MenuDefinition::expandCaptions(Buffer const * buf, bool switchcap)
 		return;
 
 	DocumentClass const & dc = buf->params().documentClass();
-	typedef vector< pair<docstring, FuncRequest> > Caps;
-	Caps caps;
-	TextClass::InsetLayouts::const_iterator lit = dc.insetLayouts().begin();
-	TextClass::InsetLayouts::const_iterator len = dc.insetLayouts().end();
-	for (; lit != len; ++lit) {
+	vector< pair<docstring, FuncRequest> > caps;
+	for (pair<docstring, InsetLayout> const & il : dc.insetLayouts()) {
 		docstring instype;
-		docstring const type = split(lit->first, instype, ':');
+		docstring const type = split(il.first, instype, ':');
 		if (instype == from_ascii("Caption")) {
 			// skip forbidden caption types
 			FuncRequest const cmd = switchcap
@@ -1633,16 +1715,14 @@ void MenuDefinition::expandCaptions(Buffer const * buf, bool switchcap)
 	}
 
 	MenuDefinition captions;
-	Caps::const_iterator cit = caps.begin();
-	Caps::const_iterator end = caps.end();
-	for ( ; cit != end; ++cit) {
-		docstring const type = cit->first;
+	for (pair<docstring, FuncRequest> const & cap : caps) {
+		docstring const type = cap.first;
 		docstring const trtype = translateIfPossible(type);
 		docstring const cmitem = bformat(_("Caption (%1$s)"), trtype);
 		if (switchcap)
-			add(MenuItem(MenuItem::Command, toqstr(cmitem), cit->second));
+			add(MenuItem(MenuItem::Command, toqstr(cmitem), cap.second));
 		else
-			captions.add(MenuItem(MenuItem::Command, toqstr(trtype), cit->second));
+			captions.add(MenuItem(MenuItem::Command, toqstr(trtype), cap.second));
 	}
 	if (!captions.empty()) {
 		MenuItem item(MenuItem::Submenu, qt_("Caption"));
@@ -1652,7 +1732,130 @@ void MenuDefinition::expandCaptions(Buffer const * buf, bool switchcap)
 }
 
 
-void MenuDefinition::expandEnvironmentSeparators(BufferView const * bv)
+void MenuDefinition::expandQuotes(BufferView const * bv)
+{
+	if (!bv)
+		return;
+
+	if (!bv->cursor().inTexted())
+		return;
+
+	Inset const * inset = bv->cursor().nextInset();
+	if (!inset || inset->lyxCode() != QUOTE_CODE) {
+		add(MenuItem(MenuItem::Command,
+				    qt_("No Quote in Scope!"),
+				    FuncRequest(LFUN_NOACTION)));
+		return;
+	}
+	InsetQuotes const * qinset =
+		static_cast<InsetQuotes const *>(inset);
+
+	map<string, docstring> styles = quoteparams.getTypes();
+	string const qtype = qinset->getType();
+
+	map<string, docstring>::const_iterator qq = styles.begin();
+	map<string, docstring>::const_iterator end = styles.end();
+
+	MenuDefinition aqs;
+
+	BufferParams const & bp = bv->buffer().masterBuffer()->params();
+
+	// The global setting
+	InsetQuotesParams::QuoteStyle globalqs = bp.quotes_style;
+	char const globalqsc = quoteparams.getStyleChar(globalqs);
+
+	// The current language's default
+	InsetQuotesParams::QuoteStyle langdefqs =
+		bp.getQuoteStyle(bv->cursor().current_font.language()->quoteStyle());
+	char const langqs = quoteparams.getStyleChar(langdefqs);
+
+	bool main_global_qs = false;
+	bool main_langdef_qs = false;
+	bool main_dynamic_qs = false;
+	docstring const subcmd = from_ascii("changetype ");
+	docstring const wildcards = from_ascii("..");
+	// Add the items
+	// First the top level menu (all glyphs of the current style) ...
+	// Begin with dynamic (if they are current style),
+	if (qtype[0] == 'x') {
+		FuncRequest cmd = FuncRequest(LFUN_INSET_MODIFY, subcmd + from_ascii("xld"));
+		docstring desc = bformat(_("%1$s (dynamic)"),
+			quoteparams.getShortGuiLabel(globalqsc + from_ascii("ld")));
+		add(MenuItem(MenuItem::Command, toqstr(desc), cmd));
+		cmd = FuncRequest(LFUN_INSET_MODIFY, subcmd + from_ascii("xls"));
+		desc = bformat(_("%1$s (dynamic)"),
+			quoteparams.getShortGuiLabel(globalqsc + from_ascii("ls")));
+		add(MenuItem(MenuItem::Command, toqstr(desc), cmd));
+		cmd = FuncRequest(LFUN_INSET_MODIFY, subcmd + from_ascii("xrd"));
+		desc = bformat(_("%1$s (dynamic)"),
+			quoteparams.getShortGuiLabel(globalqsc + from_ascii("rd")));
+		add(MenuItem(MenuItem::Command, toqstr(desc), cmd));
+		cmd = FuncRequest(LFUN_INSET_MODIFY, subcmd + from_ascii("xrs"));
+		desc = bformat(_("%1$s (dynamic)"),
+			quoteparams.getShortGuiLabel(globalqsc + from_ascii("rs")));
+		add(MenuItem(MenuItem::Command, toqstr(desc), cmd));
+		main_dynamic_qs = true;
+	}
+	// now traverse through the static styles ...
+	for (; qq != end; ++qq) {
+		docstring const style = from_ascii(qq->first);
+		bool langdef = (style[0] == langqs);
+		bool globaldef = (style[0] == globalqsc);
+
+		if (prefixIs(style, qtype[0])) {
+			FuncRequest cmd = FuncRequest(LFUN_INSET_MODIFY, subcmd + style);
+			docstring const desc = quoteparams.getShortGuiLabel(style);
+			add(MenuItem(MenuItem::Command, toqstr(desc), cmd));
+			main_global_qs = globaldef;
+			main_langdef_qs = langdef;
+		}
+		else if (!langdef && !globaldef && suffixIs(style, from_ascii("ld"))) {
+			docstring const desc =
+				quoteparams.getGuiLabel(quoteparams.getQuoteStyle(to_ascii(style)));
+			FuncRequest cmd = FuncRequest(LFUN_INSET_MODIFY, subcmd + style[0] + "..");
+			aqs.add(MenuItem(MenuItem::Command, toqstr(desc), cmd));
+		}
+	}
+
+	add(MenuItem(MenuItem::Separator));
+
+	bool display_static = false;
+	// ... then potentially items to reset to the defaults and to dynamic style ...
+	if (!main_dynamic_qs && globalqsc != 'x') {
+		FuncRequest cmd = FuncRequest(LFUN_INSET_MODIFY, subcmd + 'x' + wildcards);
+		docstring const desc = bformat(_("Use dynamic quotes (%1$s)|d"),
+						quoteparams.getGuiLabel(globalqs));
+		add(MenuItem(MenuItem::Command, toqstr(desc), cmd));
+		display_static = true;
+	}
+	if (!main_global_qs && langdefqs != globalqs) {
+		docstring const variant = main_dynamic_qs ? _("dynamic[[Quotes]]") : _("static[[Quotes]]");
+		FuncRequest cmd = FuncRequest(LFUN_INSET_MODIFY, subcmd + globalqsc + wildcards);
+		docstring const desc = bformat(_("Reset to document default (%1$s, %2$s)|o"),
+						quoteparams.getGuiLabel(globalqs), variant);
+		add(MenuItem(MenuItem::Command, toqstr(desc), cmd));
+	}
+	if (!main_langdef_qs) {
+		FuncRequest cmd = FuncRequest(LFUN_INSET_MODIFY, subcmd + globalqsc + wildcards);
+		docstring const desc = (main_dynamic_qs || display_static)
+					? bformat(_("Reset to language default (%1$s, %2$s)|l"),
+						  quoteparams.getGuiLabel(langdefqs), _("static[[Quotes]]"))
+					: bformat(_("Reset to language default (%1$s)|l"),
+						  quoteparams.getGuiLabel(langdefqs));
+		add(MenuItem(MenuItem::Command, toqstr(desc), cmd));
+	}
+
+	add(MenuItem(MenuItem::Separator));
+
+	// ... and a subitem with the rest
+	MenuItem item(MenuItem::Submenu, qt_("Change Style|y"));
+	item.setSubmenu(aqs);
+	add(item);
+}
+
+
+void MenuDefinition::expandEnvironmentSeparators(BufferView const * bv,
+						 bool contextmenu)
 {
 	if (!bv)
 		return;
@@ -1665,38 +1868,81 @@ void MenuDefinition::expandEnvironmentSeparators(BufferView const * bv)
 	Paragraph const & par = text->getPar(pit);
 	docstring const curlayout = par.layout().name();
 	docstring outerlayout;
+	docstring prevlayout;
 	depth_type current_depth = par.params().depth();
-	// check if we have an environment in our nesting hierarchy
+	// check if we have an environment in our scope
 	Paragraph cpar = par;
 	while (true) {
-		if (pit == 0 || cpar.params().depth() == 0)
+		if (pit == 0)
 			break;
 		--pit;
 		cpar = text->getPar(pit);
+		if (cpar.layout().isEnvironment() && prevlayout.empty()
+		    && cpar.params().depth() <= current_depth)
+				prevlayout = cpar.layout().name();
 		if (cpar.params().depth() < current_depth
 		    && cpar.layout().isEnvironment()) {
 				outerlayout = cpar.layout().name();
 				current_depth = cpar.params().depth();
 		}
+		if (cpar.params().depth() == 0)
+			break;
 	}
 	if (par.layout().isEnvironment()) {
-		docstring const label =
-			bformat(_("Start New Environment (%1$s)"),
-				translateIfPossible(curlayout));
+		docstring label = contextmenu ?
+					bformat(_("Insert Separated %1$s Above"),
+						translateIfPossible(curlayout)) :
+					bformat(_("Separated %1$s Above"),
+						translateIfPossible(curlayout));
 		add(MenuItem(MenuItem::Command, toqstr(label),
-			     FuncRequest(LFUN_ENVIRONMENT_SPLIT)));
+			     FuncRequest(LFUN_ENVIRONMENT_SPLIT,
+					 from_ascii("before"))));
+		label = contextmenu ?
+				bformat(_("Insert Separated %1$s Below"),
+					translateIfPossible(curlayout)):
+				bformat(_("Separated %1$s Below"),
+					translateIfPossible(curlayout));
+		// We use command-alternatives here since this is how the binding is defined
+		// (otherwise, the binding is not displayed in the menu)
+		if (getStatus(FuncRequest(LFUN_ENVIRONMENT_SPLIT)).enabled())
+			add(MenuItem(MenuItem::Command, toqstr(label),
+				     FuncRequest(LFUN_COMMAND_ALTERNATIVES,
+						 from_ascii("environment-split ; environment-split previous"))));
+	}
+	else if (!prevlayout.empty()) {
+		docstring const label = contextmenu ?
+			bformat(_("Insert Separated %1$s Below"),
+				translateIfPossible(prevlayout)) :
+			bformat(_("Separated %1$s Below"),
+				translateIfPossible(prevlayout));
+		// We use command-alternatives here since this is how the binding is defined
+		// (otherwise, the binding is not displayed in the menu)
+		add(MenuItem(MenuItem::Command, toqstr(label),
+			     FuncRequest(LFUN_COMMAND_ALTERNATIVES,
+					 from_ascii("environment-split ; environment-split previous"))));
 	}
 	if (!outerlayout.empty()) {
-		docstring const label =
-			bformat(_("Start New Parent Environment (%1$s)"),
-				translateIfPossible(outerlayout));
+		docstring label;
+		if (contextmenu) {
+			label = (outerlayout == curlayout) ?
+				bformat(_("Insert Separated Outer %1$s Below"),
+					translateIfPossible(outerlayout)) :
+				bformat(_("Insert Separated %1$s Below"),
+					translateIfPossible(outerlayout));
+		} else {
+			label = (outerlayout == curlayout) ?
+				bformat(_("Separated Outer %1$s Below"),
+					translateIfPossible(outerlayout)) :
+				bformat(_("Separated %1$s Below"),
+					translateIfPossible(outerlayout));
+		}
 		add(MenuItem(MenuItem::Command, toqstr(label),
 			     FuncRequest(LFUN_ENVIRONMENT_SPLIT,
 					 from_ascii("outer"))));
 	}
 }
 
-} // namespace anon
+} // namespace
 
 
 /////////////////////////////////////////////////////////////////////
@@ -1707,7 +1953,7 @@ struct Menu::Impl
 {
 	/// populates the menu or one of its submenu
 	/// This is used as a recursive function
-	void populate(QMenu & qMenu, MenuDefinition const & menu);
+	void populate(QMenu * qMenu, MenuDefinition const & menu);
 
 	/// Only needed for top level menus.
 	MenuDefinition * top_level_menu;
@@ -1740,7 +1986,7 @@ static QString label(MenuItem const & mi)
 	return label;
 }
 
-void Menu::Impl::populate(QMenu & qMenu, MenuDefinition const & menu)
+void Menu::Impl::populate(QMenu * qMenu, MenuDefinition const & menu)
 {
 	LYXERR(Debug::GUI, "populating menu " << menu.name());
 	if (menu.empty()) {
@@ -1748,21 +1994,26 @@ void Menu::Impl::populate(QMenu & qMenu, MenuDefinition const & menu)
 		return;
 	}
 	LYXERR(Debug::GUI, " *****  menu entries " << menu.size());
-	MenuDefinition::const_iterator m = menu.begin();
-	MenuDefinition::const_iterator end = menu.end();
-	for (; m != end; ++m) {
-		if (m->kind() == MenuItem::Separator)
-			qMenu.addSeparator();
-		else if (m->kind() == MenuItem::Submenu) {
-			QMenu * subMenu = qMenu.addMenu(label(*m));
-			populate(*subMenu, m->submenu());
+	for (MenuItem const & m : menu)
+		switch (m.kind()) {
+		case MenuItem::Separator:
+			qMenu->addSeparator();
+			break;
+		case MenuItem::Submenu: {
+			QMenu * subMenu = qMenu->addMenu(label(m));
+			populate(subMenu, m.submenu());
 			subMenu->setEnabled(!subMenu->isEmpty());
-		} else {
-			// we have a MenuItem::Command
-			qMenu.addAction(new Action(QIcon(), label(*m),
-				m->func(), m->tooltip(), &qMenu));
+			break;
+		}
+		case MenuItem::Command:
+		default:
+			// FIXME: A previous comment assured that MenuItem::Command was the
+			// only possible case in practice, but this is wrong.  It would be
+			// good to document which cases are actually treated here.
+			qMenu->addAction(new Action(m.func(), QIcon(), label(m),
+			                            m.tooltip(), qMenu));
+			break;
 		}
-	}
 }
 
 #if (defined(Q_OS_WIN) || defined(Q_CYGWIN_WIN)) && (QT_VERSION >= 0x040600)
@@ -1920,7 +2171,7 @@ void Menus::Impl::macxMenuBarInit(QMenuBar * qmb)
 		QAction::MenuRole role;
 	};
 
-	static MacMenuEntry entries[] = {
+	static const MacMenuEntry entries[] = {
 		{LFUN_DIALOG_SHOW, "aboutlyx", "About LyX",
 		 QAction::AboutRole},
 		{LFUN_DIALOG_SHOW, "prefs", "Preferences",
@@ -1949,13 +2200,13 @@ void Menus::Impl::macxMenuBarInit(QMenuBar * qmb)
 	// add the entries to a QMenu that will eventually be empty
 	// and therefore invisible.
 	QMenu * qMenu = qmb->addMenu("special");
-	MenuDefinition::const_iterator cit = mac_special_menu_.begin();
-	MenuDefinition::const_iterator end = mac_special_menu_.end();
-	for (size_t i = 0 ; cit != end ; ++cit, ++i) {
-		Action * action = new Action(QIcon(), cit->label(),
-			cit->func(), QString(), qMenu);
+	size_t i = 0;
+	for (MenuItem const & m : mac_special_menu_) {
+		Action * action = new Action(m.func(), QIcon(), m.label(),
+		                             QString(), qMenu);
 		action->setMenuRole(entries[i].role);
 		qMenu->addAction(action);
+		++i;
 	}
 }
 
@@ -1989,6 +2240,19 @@ void Menus::Impl::expand(MenuDefinition const & frommenu,
 			tomenu.expandFormats(cit->kind(), buf);
 			break;
 
+		case MenuItem::ExportFormat: {
+			if (!buf)
+				break;
+			string const format = buf->params().getDefaultOutputFormat();
+			Format const * f = theFormats().getFormat(format);
+			docstring const name = f ? f->prettyname() : from_utf8(format);
+			docstring const label = bformat(_("Export [%1$s]|E"), name);
+			MenuItem item(MenuItem::Command, toqstr(label),
+			              FuncRequest(LFUN_BUFFER_EXPORT));
+			tomenu.addWithStatusCheck(item);
+			break;
+		}
+
 		case MenuItem::CharStyles:
 			tomenu.expandFlexInsert(buf, InsetLayout::CHARSTYLE);
 			break;
@@ -2077,6 +2341,14 @@ void Menus::Impl::expand(MenuDefinition const & frommenu,
 			tomenu.expandEnvironmentSeparators(bv);
 			break;
 
+		case MenuItem::EnvironmentSeparatorsContext:
+			tomenu.expandEnvironmentSeparators(bv, true);
+			break;
+
+		case MenuItem::SwitchQuotes:
+			tomenu.expandQuotes(bv);
+			break;
+
 		case MenuItem::Submenu: {
 			MenuItem item(*cit);
 			item.setSubmenu(MenuDefinition(cit->submenuname()));
@@ -2092,7 +2364,7 @@ void Menus::Impl::expand(MenuDefinition const & frommenu,
 			break;
 
 		case MenuItem::Command:
-			if (!mac_special_menu_.hasFunc(cit->func()))
+			if (!mac_special_menu_.hasFunc(*cit->func()))
 				tomenu.addWithStatusCheck(*cit);
 		}
 	}
@@ -2331,7 +2603,7 @@ void Menus::updateMenu(Menu * qmenu)
 	if (qmenu->d->view)
 		bv = qmenu->d->view->currentBufferView();
 	d->expand(fromLyxMenu, *qmenu->d->top_level_menu, bv);
-	qmenu->d->populate(*qmenu, *qmenu->d->top_level_menu);
+	qmenu->d->populate(qmenu, *qmenu->d->top_level_menu);
 }
 
 
