@@ -23,8 +23,6 @@
 #include "support/gettext.h"
 #include "support/lstrings.h"
 
-#include "support/bind.h"
-
 #include <algorithm>
 
 using namespace std;
@@ -40,14 +38,16 @@ namespace frontend {
 //
 /////////////////////////////////////////////////////////////////////////
 
-ToolbarItem::ToolbarItem(Type type, FuncRequest const & func, docstring const & label)
-	: type_(type), func_(func), label_(label)
+ToolbarItem::ToolbarItem(Type type, FuncRequest const & func,
+                         docstring const & label)
+	: type_(type), func_(make_shared<FuncRequest>(func)), label_(label)
 {
 }
 
 
-ToolbarItem::ToolbarItem(Type type, string const & name, docstring const & label)
-	: type_(type), label_(label), name_(name)
+ToolbarItem::ToolbarItem(Type type, string const & name,
+                         docstring const & label)
+	: type_(type), func_(make_shared<FuncRequest>()), label_(label), name_(name)
 {
 }
 
@@ -55,7 +55,7 @@ ToolbarItem::ToolbarItem(Type type, string const & name, docstring const & label
 void ToolbarInfo::add(ToolbarItem const & item)
 {
 	items.push_back(item);
-	items.back().func_.setOrigin(FuncRequest::TOOLBAR);
+	items.back().func_->setOrigin(FuncRequest::TOOLBAR);
 }
 
 
@@ -196,19 +196,16 @@ ToolbarInfo & ToolbarInfo::read(Lexer & lex)
 		case TO_VIEWFORMATS: {
 			FormatList formats = (code == TO_IMPORTFORMATS) ?
 				theConverters().importableFormats() :
-				theConverters().exportableFormats(code != TO_EXPORTFORMATS);
+				theConverters().exportableFormats(true);
 			sort(formats.begin(), formats.end());
-			vector<Format const *>::const_iterator fit = formats.begin();
-			vector<Format const *>::const_iterator end = formats.end();
-			for (; fit != end ; ++fit) {
-				if ((*fit)->dummy())
+			for (Format const * f : formats) {
+				if (f->dummy())
 					continue;
 				if (code != TO_IMPORTFORMATS &&
-				    !(*fit)->documentFormat())
+				    !f->documentFormat())
 					continue;
 
-				docstring const prettyname =
-					from_utf8((*fit)->prettyname());
+				docstring const prettyname = f->prettyname();
 				docstring tooltip;
 				FuncCode lfun = LFUN_NOACTION;
 				switch (code) {
@@ -229,7 +226,7 @@ ToolbarInfo & ToolbarInfo::read(Lexer & lex)
 					tooltip = _("View %1$s");
 					break;
 				}
-				FuncRequest func(lfun, (*fit)->name(),
+				FuncRequest func(lfun, f->name(),
 						FuncRequest::TOOLBAR);
 				add(ToolbarItem(ToolbarItem::COMMAND, func,
 						bformat(tooltip, translateIfPossible(prettyname))));
