@@ -21,9 +21,11 @@
 #include "DispatchResult.h"
 #include "FuncCode.h"
 #include "FuncRequest.h"
+#include "FuncStatus.h"
 #include "LaTeXFeatures.h"
 #include "output_xhtml.h"
 #include "ParIterator.h"
+#include "texstream.h"
 #include "TocBackend.h"
 
 #include "support/debug.h"
@@ -53,73 +55,151 @@ InsetCitation::InsetCitation(Buffer * buf, InsetCommandParams const & p)
 InsetCitation::~InsetCitation()
 {
 	if (isBufferLoaded())
-		buffer().removeBiblioTempFiles();
+		/* We do not use buffer() because Coverity believes that this
+		 * may throw an exception. Actually this code path is not
+		 * taken when buffer_ == 0 */
+		buffer_->removeBiblioTempFiles();
 }
 
 
 ParamInfo const & InsetCitation::findInfo(string const & /* cmdName */)
 {
-	// standard cite does only take one argument if jurabib is
-	// not used, but jurabib extends this to two arguments, so
+	// standard cite does only take one argument, but biblatex, jurabib
+	// and natbib extend this to two arguments, so
 	// we have to allow both here. InsetCitation takes care that
 	// LaTeX output is nevertheless correct.
 	if (param_info_.empty()) {
-		param_info_.add("after", ParamInfo::LATEX_OPTIONAL);
-		param_info_.add("before", ParamInfo::LATEX_OPTIONAL);
+		param_info_.add("after", ParamInfo::LATEX_OPTIONAL,
+				ParamInfo::HANDLING_LATEXIFY);
+		param_info_.add("before", ParamInfo::LATEX_OPTIONAL,
+				ParamInfo::HANDLING_LATEXIFY);
 		param_info_.add("key", ParamInfo::LATEX_REQUIRED);
+		param_info_.add("pretextlist", ParamInfo::LATEX_OPTIONAL,
+				ParamInfo::HANDLING_LATEXIFY);
+		param_info_.add("posttextlist", ParamInfo::LATEX_OPTIONAL,
+				ParamInfo::HANDLING_LATEXIFY);
+		param_info_.add("literal", ParamInfo::LYX_INTERNAL);
 	}
 	return param_info_;
 }
 
 
-namespace {
-
-vector<string> const init_possible_cite_commands()
+// We allow any command here, since we fall back to cite
+// anyway if a command is not allowed by a style
+bool InsetCitation::isCompatibleCommand(string const &)
 {
-	char const * const possible[] = {
-		"cite", "nocite", "citet", "citep", "citealt", "citealp",
-		"citeauthor", "citeyear", "citeyearpar",
-		"citet*", "citep*", "citealt*", "citealp*", "citeauthor*",
-		"Citet",  "Citep",  "Citealt",  "Citealp",  "Citeauthor",
-		"Citet*", "Citep*", "Citealt*", "Citealp*", "Citeauthor*",
-		"fullcite",
-		"footcite", "footcitet", "footcitep", "footcitealt",
-		"footcitealp", "footciteauthor", "footciteyear", "footciteyearpar",
-		"citefield", "citetitle", "cite*"
-	};
-	size_t const size_possible = sizeof(possible) / sizeof(possible[0]);
-
-	return vector<string>(possible, possible + size_possible);
+	return true;
 }
 
 
-vector<string> const & possibleCiteCommands()
+CitationStyle InsetCitation::getCitationStyle(BufferParams const & bp, string const & input,
+				  vector<CitationStyle> const & valid_styles) const
 {
-	static vector<string> const possible = init_possible_cite_commands();
-	return possible;
-}
+	CitationStyle cs = valid_styles[0];
+	cs.forceUpperCase = false;
+	cs.hasStarredVersion = false;
 
+	string normalized_input = input;
+	string::size_type const n = input.size() - 1;
+	if (isUpperCase(input[0]))
+		normalized_input[0] = lowercase(input[0]);
+	if (input[n] == '*')
+		normalized_input = normalized_input.substr(0, n);
 
-} // anon namespace
+	string const alias = bp.getCiteAlias(normalized_input);
+	if (!alias.empty())
+		normalized_input = alias;
 
+	vector<CitationStyle>::const_iterator it  = valid_styles.begin();
+	vector<CitationStyle>::const_iterator end = valid_styles.end();
+	for (; it != end; ++it) {
+		CitationStyle this_cs = *it;
+		if (this_cs.name == normalized_input) {
+			cs = *it;
+			break;
+		}
+	}
 
-// FIXME: use the citeCommands provided by the TextClass
-// instead of possibleCiteCommands defined in this file.
-bool InsetCitation::isCompatibleCommand(string const & cmd)
-{
-	vector<string> const & possibles = possibleCiteCommands();
-	vector<string>::const_iterator const end = possibles.end();
-	return find(possibles.begin(), end, cmd) != end;
+	return cs;
 }
 
 
 void InsetCitation::doDispatch(Cursor & cur, FuncRequest & cmd)
 {
-	if (cmd.action() == LFUN_INSET_MODIFY) {
+	switch (cmd.action()) {
+	case LFUN_INSET_MODIFY: {
 		buffer().removeBiblioTempFiles();
 		cache.recalculate = true;
+		if (cmd.getArg(0) == "toggleparam") {
+			string cmdname = getCmdName();
+			string const alias =
+				buffer().masterParams().getCiteAlias(cmdname);
+			if (!alias.empty())
+				cmdname = alias;
+			string const par = cmd.getArg(1);
+			string newcmdname = cmdname;
+			if (par == "star") {
+				if (suffixIs(cmdname, "*"))
+					newcmdname = rtrim(cmdname, "*");
+				else
+					newcmdname = cmdname + "*";
+			} else if (par == "casing") {
+				if (isUpperCase(cmdname[0]))
+					newcmdname[0] = lowercase(cmdname[0]);
+				else
+					newcmdname[0] = uppercase(newcmdname[0]);
+			}
+			cmd = FuncRequest(LFUN_INSET_MODIFY, "changetype " + newcmdname);
+		}
+	}
+		// fall through
+	default:
+		InsetCommand::doDispatch(cur, cmd);
+	}
+}
+
+
+bool InsetCitation::getStatus(Cursor & cur, FuncRequest const & cmd,
+	FuncStatus & status) const
+{
+	switch (cmd.action()) {
+	// Handle the alias case
+	case LFUN_INSET_MODIFY:
+		if (cmd.getArg(0) == "changetype") {
+			string cmdname = getCmdName();
+			string const alias =
+				buffer().masterParams().getCiteAlias(cmdname);
+			if (!alias.empty())
+				cmdname = alias;
+			if (suffixIs(cmdname, "*"))
+				cmdname = rtrim(cmdname, "*");
+			string const newtype = cmd.getArg(1);
+			status.setEnabled(isCompatibleCommand(newtype));
+			status.setOnOff(newtype == cmdname);
+		}
+		if (cmd.getArg(0) == "toggleparam") {
+			string cmdname = getCmdName();
+			string const alias =
+				buffer().masterParams().getCiteAlias(cmdname);
+			if (!alias.empty())
+				cmdname = alias;
+			vector<CitationStyle> citation_styles =
+				buffer().masterParams().citeStyles();
+			CitationStyle cs = getCitationStyle(buffer().masterParams(),
+							    cmdname, citation_styles);
+			if (cmd.getArg(1) == "star") {
+				status.setEnabled(cs.hasStarredVersion);
+				status.setOnOff(suffixIs(cmdname, "*"));
+			}
+			else if (cmd.getArg(1) == "casing") {
+				status.setEnabled(cs.forceUpperCase);
+				status.setOnOff(isUpperCase(cmdname[0]));
+			}
+		}
+		return true;
+	default:
+		return InsetCommand::getStatus(cur, cmd, status);
 	}
-	InsetCommand::doDispatch(cur, cmd);
 }
 
 
@@ -164,52 +244,67 @@ docstring InsetCitation::toolTip(BufferView const & bv, int, int) const
 	if (key.empty())
 		return _("No citations selected!");
 
+	CiteItem ci;
+	ci.richtext = true;
 	vector<docstring> keys = getVectorFromString(key);
-	vector<docstring>::const_iterator it = keys.begin();
-	vector<docstring>::const_iterator en = keys.end();
+	if (keys.size() == 1)
+		return bi.getInfo(keys[0], buffer(), ci);
+
 	docstring tip;
-	for (; it != en; ++it) {
-		docstring const key_info = bi.getInfo(*it, buffer());
+	tip += "<ol>";
+	int count = 0;
+	for (docstring const & key : keys) {
+		docstring const key_info = bi.getInfo(key, buffer(), ci);
+		// limit to reasonable size.
+		if (count > 9 && keys.size() > 11) {
+			tip.push_back(0x2026);// HORIZONTAL ELLIPSIS
+			tip += "<p>"
+				+ bformat(_("+ %1$d more entries."), int(keys.size() - count))
+				+ "</p>";
+			break;
+		}
 		if (key_info.empty())
 			continue;
-		if (!tip.empty())
-			tip += "\n";
-		tip += wrap(key_info, -4);
+		tip += "<li>" + key_info + "</li>";
+		++count;
 	}
+	tip += "</ol>";
 	return tip;
 }
 
 
 namespace {
 
-
-CitationStyle asValidLatexCommand(string const & input, vector<CitationStyle> const & valid_styles)
+CitationStyle asValidLatexCommand(BufferParams const & bp, string const & input,
+				  vector<CitationStyle> const & valid_styles)
 {
 	CitationStyle cs = valid_styles[0];
 	cs.forceUpperCase = false;
-	cs.fullAuthorList = false;
-	if (!InsetCitation::isCompatibleCommand(input))
-		return cs;
+	cs.hasStarredVersion = false;
 
 	string normalized_input = input;
 	string::size_type const n = input.size() - 1;
-	if (input[0] == 'C')
-		normalized_input[0] = 'c';
+	if (isUpperCase(input[0]))
+		normalized_input[0] = lowercase(input[0]);
 	if (input[n] == '*')
 		normalized_input = normalized_input.substr(0, n);
 
+	string const alias = bp.getCiteAlias(normalized_input);
+	if (!alias.empty())
+		normalized_input = alias;
+
 	vector<CitationStyle>::const_iterator it  = valid_styles.begin();
 	vector<CitationStyle>::const_iterator end = valid_styles.end();
 	for (; it != end; ++it) {
 		CitationStyle this_cs = *it;
-		if (this_cs.cmd == normalized_input) {
+		if (this_cs.name == normalized_input) {
 			cs = *it;
 			break;
 		}
 	}
 
-	cs.forceUpperCase &= input[0] == 'C';
-	cs.fullAuthorList &= input[n] == '*';
+	cs.forceUpperCase &= input[0] == uppercase(input[0]);
+	cs.hasStarredVersion &= input[n] == '*';
 
 	return cs;
 }
@@ -226,17 +321,22 @@ inline docstring wrapCitation(docstring const & key,
 			html::htmlize(content, XHTMLStream::ESCAPE_ALL) + "</a>";
 }
 
-
-docstring protectArgument(docstring & arg)
-{
-	if (contains(arg, '[') || contains(arg, ']'))
-		// protect brackets
-		arg = '{' + arg + '}';
-	return arg;
-}
-
 } // anonymous namespace
 
+
+map<docstring, docstring> InsetCitation::getQualifiedLists(docstring const p) const
+{
+	vector<docstring> ps =
+		getVectorFromString(p, from_ascii("\t"));
+	std::map<docstring, docstring> res;
+	for (docstring const & s: ps) {
+		docstring key;
+		docstring val = split(s, key, ' ');
+		res[key] = val;
+	}
+	return res;
+}
+
 docstring InsetCitation::generateLabel(bool for_xhtml) const
 {
 	docstring label;
@@ -265,17 +365,18 @@ docstring InsetCitation::complexLabel(bool for_xhtml) const
 	if (key.empty())
 		return _("No citations selected!");
 
-	// We don't currently use the full or forceUCase fields.
 	string cite_type = getCmdName();
-	if (cite_type[0] == 'C')
-		// If we were going to use them, this would mean ForceUCase
-		cite_type = string(1, 'c') + cite_type.substr(1);
-	if (cite_type[cite_type.size() - 1] == '*')
-		// and this would mean FULL
+	bool const uppercase = isUpperCase(cite_type[0]);
+	if (uppercase)
+		cite_type[0] = lowercase(cite_type[0]);
+	bool const starred = (cite_type[cite_type.size() - 1] == '*');
+	if (starred)
 		cite_type = cite_type.substr(0, cite_type.size() - 1);
 
-	docstring const & before = getParam("before");
-	docstring const & after = getParam("after");
+	// handle alias
+	string const alias = buf.masterParams().getCiteAlias(cite_type);
+	if (!alias.empty())
+		cite_type = alias;
 
 	// FIXME: allow to add cite macros
 	/*
@@ -284,7 +385,30 @@ docstring InsetCitation::complexLabel(bool for_xhtml) const
 	*/
 	docstring label;
 	vector<docstring> keys = getVectorFromString(key);
-	label = biblist.getLabel(keys, buffer(), cite_type, for_xhtml, UINT_MAX, before, after);
+	CitationStyle cs = getCitationStyle(buffer().masterParams(),
+					    cite_type, buffer().masterParams().citeStyles());
+	bool const qualified = cs.hasQualifiedList
+		&& (keys.size() > 1
+		    || !getParam("pretextlist").empty()
+		    || !getParam("posttextlist").empty());
+	map<docstring, docstring> pres = getQualifiedLists(getParam("pretextlist"));
+	map<docstring, docstring> posts = getQualifiedLists(getParam("posttextlist"));
+
+	CiteItem ci;
+	ci.textBefore = getParam("before");
+	ci.textAfter = getParam("after");
+	ci.forceUpperCase = uppercase;
+	ci.Starred = starred;
+	ci.max_size = UINT_MAX;
+	ci.isQualified = qualified;
+	ci.pretexts = pres;
+	ci.posttexts = posts;
+	if (for_xhtml) {
+		ci.max_key_size = UINT_MAX;
+		ci.context = CiteItem::Export;
+	}
+	ci.richtext = for_xhtml;
+	label = biblist.getLabel(keys, buffer(), cite_type, ci);
 	return label;
 }
 
@@ -333,16 +457,17 @@ void InsetCitation::updateBuffer(ParIterator const &, UpdateType)
 
 
 void InsetCitation::addToToc(DocIterator const & cpit, bool output_active,
-							 UpdateType) const
+							 UpdateType, TocBackend & backend) const
 {
 	// NOTE
-	// BiblioInfo::collectCitedEntries() uses the TOC to collect the citations 
+	// BiblioInfo::collectCitedEntries() uses the TOC to collect the citations
 	// from the document. It is used indirectly, via BiblioInfo::makeCitationLables,
 	// by both XHTML and plaintext output. So, if we change what goes into the TOC,
 	// then we will also need to change that routine.
 	docstring const tocitem = getParam("key");
-	shared_ptr<Toc> toc = buffer().tocBackend().toc("citation");
-	toc->push_back(TocItem(cpit, 0, tocitem, output_active));
+	TocBuilder & b = backend.builder("citation");
+	b.pushItem(cpit, tocitem, output_active);
+	b.pop();
 }
 
 
@@ -420,30 +545,78 @@ void InsetCitation::forOutliner(docstring & os, size_t const, bool const) const
 // engine, e.g. \cite[]{} for the basic engine.
 void InsetCitation::latex(otexstream & os, OutputParams const & runparams) const
 {
-	vector<CitationStyle> citation_styles = buffer().params().citeStyles();
-	CitationStyle cs = asValidLatexCommand(getCmdName(), citation_styles);
 	BiblioInfo const & bi = buffer().masterBibInfo();
+	docstring const key = getParam("key");
+	// "keyonly" command: output the plain key and stop.
+	if (getCmdName() == "keyonly") {
+		// Special command to only return the key
+		if (!bi.isBibtex(getParam("key")))
+			// escape chars with bibitems
+			os << escape(cleanupWhitespace(key));
+		else
+			os << cleanupWhitespace(key);
+		return;
+	}
+	vector<CitationStyle> citation_styles = buffer().masterParams().citeStyles();
+	CitationStyle cs = asValidLatexCommand(buffer().masterParams(),
+					       getCmdName(), citation_styles);
 	// FIXME UNICODE
-	docstring const cite_str = from_utf8(citationStyleToString(cs));
+	docstring const cite_str = from_utf8(citationStyleToString(cs, true));
+
+	// check if we have to do a qualified list
+	vector<docstring> keys = getVectorFromString(cleanupWhitespace(key));
+	bool const qualified = cs.hasQualifiedList
+		&& (!getParam("pretextlist").empty()
+		    || !getParam("posttextlist").empty());
 
 	if (runparams.inulemcmd > 0)
 		os << "\\mbox{";
 
 	os << "\\" << cite_str;
 
-	docstring before = getParam("before");
-	docstring after  = getParam("after");
-	if (!before.empty() && cs.textBefore)
-		os << '[' << protectArgument(before)
-		   << "][" << protectArgument(after) << ']';
-	else if (!after.empty() && cs.textAfter)
-		os << '[' << protectArgument(after) << ']';
+	if (qualified)
+		os << "s";
 
-	if (!bi.isBibtex(getParam("key")))
+	docstring before = params().prepareCommand(runparams, getParam("before"),
+						   param_info_["before"].handling());
+	docstring after = params().prepareCommand(runparams, getParam("after"),
+						   param_info_["after"].handling());
+	if (!before.empty() && cs.textBefore) {
+		if (qualified)
+			os << '(' << protectArgument(before, '(', ')')
+			   << ")(" << protectArgument(after, '(', ')') << ')';
+		else
+			os << '[' << protectArgument(before) << "]["
+			   << protectArgument(after) << ']';
+	} else if (!after.empty() && cs.textAfter) {
+		if (qualified)
+			os << '(' << protectArgument(after, '(', ')') << ')';
+		else
+			os << '[' << protectArgument(after) << ']';
+	}
+
+	if (!bi.isBibtex(key))
 		// escape chars with bibitems
-		os << '{' << escape(cleanupWhitespace(getParam("key"))) << '}';
-	else
-		os << '{' << cleanupWhitespace(getParam("key")) << '}';
+		os << '{' << escape(cleanupWhitespace(key)) << '}';
+	else {
+		if (qualified) {
+			map<docstring, docstring> pres = getQualifiedLists(getParam("pretextlist"));
+			map<docstring, docstring> posts = getQualifiedLists(getParam("posttextlist"));
+			for (docstring const & k: keys) {
+				docstring bef = params().prepareCommand(runparams, pres[k],
+									param_info_["pretextlist"].handling());
+				docstring aft  = params().prepareCommand(runparams, posts[k],
+									 param_info_["posttextlist"].handling());
+				if (!bef.empty())
+					os << '[' << protectArgument(bef)
+					   << "][" << protectArgument(aft) << ']';
+				else if (!aft.empty())
+					os << '[' << protectArgument(aft) << ']';
+				os << '{' << k << '}';
+			}
+		} else
+			os << '{' << cleanupWhitespace(key) << '}';
+	}
 
 	if (runparams.inulemcmd)
 		os << "}";
