@@ -14,6 +14,7 @@
 #define INSET_CITATION_H
 
 #include "InsetCommand.h"
+#include "Citation.h"
 
 namespace lyx {
 
@@ -46,6 +47,8 @@ public:
 	///
 	void doDispatch(Cursor & cur, FuncRequest & cmd);
 	///
+	bool getStatus(Cursor & cur, FuncRequest const & cmd, FuncStatus &) const;
+	///
 	InsetCode lyxCode() const { return CITE_CODE; }
 	///
 	void latex(otexstream &, OutputParams const &) const;
@@ -61,12 +64,10 @@ public:
 	///
 	void forOutliner(docstring &, size_t const, bool const) const;
 	///
-	void validate(LaTeXFeatures &) const {}
-	///
 	void updateBuffer(ParIterator const & it, UpdateType);
 	///
 	void addToToc(DocIterator const & di, bool output_active,
-				  UpdateType utype) const;
+				  UpdateType utype, TocBackend & backend) const;
 	///
 	std::string contextMenuName() const;
 	//@}
@@ -78,8 +79,13 @@ public:
 	///
 	static std::string defaultCommand() { return "cite"; }
 	///
-	static bool isCompatibleCommand(std::string const & cmd);
+	static bool isCompatibleCommand(std::string const &);
 	//@}
+	///
+	CitationStyle getCitationStyle(BufferParams const & bp, std::string const & input,
+				       std::vector<CitationStyle> const & valid_styles) const;
+	///
+	std::map<docstring, docstring> getQualifiedLists(docstring const p) const;
 
 private:
 	/// tries to make a pretty label and makes a basic one if not
