@@ -17,6 +17,7 @@
 #include "BufferParams.h"
 #include "Counters.h"
 #include "Language.h"
+#include "LaTeXFeatures.h"
 #include "Layout.h"
 #include "OutputParams.h"
 #include "ParIterator.h"
@@ -33,13 +34,17 @@ using namespace std;
 namespace lyx {
 
 InsetFoot::InsetFoot(Buffer * buf)
-	: InsetFootlike(buf), intitle_(false)
+	: InsetFootlike(buf), intitle_(false), infloattable_(false)
 {}
 
 
 docstring InsetFoot::layoutName() const
 {
-	return intitle_ ? from_ascii("Foot:InTitle") : from_ascii("Foot");
+	if (intitle_)
+		return from_ascii("Foot:InTitle");
+	else if (infloattable_)
+		return from_ascii("Foot:InFloatTable");
+	return from_ascii("Foot");
 }
 
 
@@ -53,7 +58,13 @@ void InsetFoot::updateBuffer(ParIterator const & it, UpdateType utype)
 	}
 
 	intitle_ = false;
-	for (size_type sl = 0 ; sl < it.depth() ; ++ sl) {
+	infloattable_ = false;
+	bool intable = false;
+	if (it.innerInsetOfType(TABULAR_CODE) != 0)
+		intable = true;
+	if (it.innerInsetOfType(FLOAT_CODE) != 0)
+		infloattable_ = intable;
+	for (size_type sl = 0 ; sl < it.depth() ; ++sl) {
 		if (it[sl].text() && it[sl].paragraph().layout().intitle) {
 			intitle_ = true;
 			break;
@@ -69,36 +80,17 @@ void InsetFoot::updateBuffer(ParIterator const & it, UpdateType utype)
 	custom_label_ += ' ' + cnts.theCounter(count, lang->code());
 	setLabel(custom_label_);
 
-	InsetCollapsable::updateBuffer(it, utype);
+	InsetCollapsible::updateBuffer(it, utype);
 	if (utype == OutputUpdate)
 		cnts.restoreLastCounter();
 }
 
 
-void InsetFoot::addToToc(DocIterator const & cpit, bool output_active,
-						 UpdateType utype) const
-{
-	DocIterator pit = cpit;
-	pit.push_back(CursorSlice(const_cast<InsetFoot &>(*this)));
-	
-	docstring tooltip;
-	text().forOutliner(tooltip, TOC_ENTRY_LENGTH);
-	docstring str = custom_label_ + ": " + tooltip;
-	tooltip = support::wrapParas(tooltip, 0, 60, 2);
-	
-	shared_ptr<Toc> toc = buffer().tocBackend().toc("footnote");
-	toc->push_back(TocItem(pit, 0, str, output_active, tooltip));
-	
-	// Proceed with the rest of the inset.
-	InsetFootlike::addToToc(cpit, output_active, utype);
-}
-
-
 docstring InsetFoot::toolTip(BufferView const & bv, int x, int y) const
 {
 	if (isOpen(bv))
 		// this will give us something useful if there is no button
-		return InsetCollapsable::toolTip(bv, x, y);
+		return InsetCollapsible::toolTip(bv, x, y);
 	return toolTipText(custom_label_+ ": ");
 }
 
@@ -123,4 +115,21 @@ int InsetFoot::docbook(odocstream & os, OutputParams const & runparams) const
 	return i;
 }
 
+
+void InsetFoot::validate(LaTeXFeatures & features) const
+{
+	// Use footnote package to provide footnotes in tables
+	// unless an alternative approach is built in the class.
+	if (!features.saveNoteEnv().empty()
+	    && !features.isProvided("footnote-alternative")) {
+		features.require("footnote");
+		features.addPreambleSnippet(
+			from_ascii("\\makesavenoteenv{"
+				   + features.saveNoteEnv()
+				   + "}\n"));
+	}
+
+	InsetText::validate(features);
+}
+
 } // namespace lyx
