@@ -68,6 +68,7 @@ TODO
 #include "OutputParams.h"
 #include "output_xhtml.h"
 #include "sgml.h"
+#include "texstream.h"
 #include "TocBackend.h"
 
 #include "frontends/alert.h"
@@ -84,10 +85,9 @@ TODO
 #include "support/os.h"
 #include "support/Systemcall.h"
 
-#include <boost/tuple/tuple.hpp>
-
 #include <algorithm>
 #include <sstream>
+#include <tuple>
 
 using namespace std;
 using namespace lyx::support;
@@ -107,7 +107,7 @@ string findTargetFormat(string const & format, OutputParams const & runparams)
 	    || runparams.flavor == OutputParams::XETEX
 	    || runparams.flavor == OutputParams::LUATEX) {
 		LYXERR(Debug::GRAPHICS, "findTargetFormat: PDF mode");
-		Format const * const f = formats.getFormat(format);
+		Format const * const f = theFormats().getFormat(format);
 		// Convert vector graphics to pdf
 		if (f && f->vectorFormat())
 			return "pdf6";
@@ -119,7 +119,7 @@ string findTargetFormat(string const & format, OutputParams const & runparams)
 	}
 	// for HTML, we leave the known formats and otherwise convert to png
 	if (runparams.flavor == OutputParams::HTML) {
-		Format const * const f = formats.getFormat(format);
+		Format const * const f = theFormats().getFormat(format);
 		// Convert vector graphics to svg
 		if (f && f->vectorFormat() && theConverters().isReachable(format, "svg"))
 			return "svg";
@@ -165,7 +165,7 @@ void readInsetGraphics(Lexer & lex, Buffer const & buf, bool allowOrigin,
 	}
 }
 
-} // namespace anon
+} // namespace
 
 
 InsetGraphics::InsetGraphics(Buffer * buf)
@@ -226,7 +226,7 @@ void InsetGraphics::doDispatch(Cursor & cur, FuncRequest & cmd)
 		// if the inset is part of a graphics group, all the
 		// other members should be updated too.
 		if (!params_.groupId.empty())
-			graphics::unifyGraphicsGroups(buffer(), 
+			graphics::unifyGraphicsGroups(buffer(),
 						      to_utf8(cmd.argument()));
 		break;
 	}
@@ -415,6 +415,7 @@ docstring InsetGraphics::toDocbookLength(Length const & len) const
 		case Length::PLW: // Percent of LineWidth
 		case Length::PTH: // Percent of TextHeight
 		case Length::PPH: // Percent of PaperHeight
+		case Length::BLS: // Percent of BaselineSkip
 			// Sigh, this will go wrong.
 			result << len.value() << "%";
 			break;
@@ -485,7 +486,7 @@ copyFileIfNeeded(FileName const & file_in, FileName const & file_out)
 		// Nothing to do...
 		return make_pair(IDENTICAL_CONTENTS, file_out);
 
-	Mover const & mover = getMover(formats.getFormatFromFile(file_in));
+	Mover const & mover = getMover(theFormats().getFormatFromFile(file_in));
 	bool const success = mover.copy(file_in, file_out);
 	if (!success) {
 		// FIXME UNICODE
@@ -509,7 +510,7 @@ copyToDirIfNeeded(DocFileName const & file, string const & dir)
 		return make_pair(IDENTICAL_PATHS, FileName(file_in));
 
 	string mangled = file.mangledFileName();
-	if (formats.isZippedFile(file)) {
+	if (theFormats().isZippedFile(file)) {
 		// We need to change _eps.gz to .eps.gz. The mangled name is
 		// still unique because of the counter in mangledFileName().
 		// We can't just call mangledFileName() with the zip
@@ -550,7 +551,7 @@ string const stripExtensionIfPossible(string const & file, string const & to, bo
 {
 	// No conversion is needed. LaTeX can handle the graphic file as is.
 	// This is true even if the orig_file is compressed.
-	string const to_format = formats.getFormat(to)->extension();
+	string const to_format = theFormats().getFormat(to)->extension();
 	string const file_format = getExtension(file);
 	// for latex .ps == .eps
 	if (to_format == file_format ||
@@ -560,7 +561,7 @@ string const stripExtensionIfPossible(string const & file, string const & to, bo
 	return latex_path(file, EXCLUDE_EXTENSION);
 }
 
-} // namespace anon
+} // namespace
 
 
 string InsetGraphics::prepareFile(OutputParams const & runparams) const
@@ -597,8 +598,7 @@ string InsetGraphics::prepareFile(OutputParams const & runparams) const
 	// we move it to a temp dir or uncompress it.
 	FileName temp_file;
 	GraphicsCopyStatus status;
-	boost::tie(status, temp_file) =
-			copyToDirIfNeeded(params().filename, temp_path);
+	tie(status, temp_file) = copyToDirIfNeeded(params().filename, temp_path);
 
 	if (status == FAILURE)
 		return orig_file;
@@ -632,7 +632,7 @@ string InsetGraphics::prepareFile(OutputParams const & runparams) const
 	// determine the export format
 	string const tex_format = flavor2format(runparams.flavor);
 
-	if (formats.isZippedFile(params().filename)) {
+	if (theFormats().isZippedFile(params().filename)) {
 		FileName const unzipped_temp_file =
 			FileName(unzippedFileName(temp_file.absFileName()));
 		output_file = unzippedFileName(output_file);
@@ -649,12 +649,12 @@ string InsetGraphics::prepareFile(OutputParams const & runparams) const
 		}
 	}
 
-	string const from = formats.getFormatFromFile(temp_file);
+	string const from = theFormats().getFormatFromFile(temp_file);
 	if (from.empty())
 		LYXERR(Debug::GRAPHICS, "\tCould not get file format.");
 
 	string const to   = findTargetFormat(from, runparams);
-	string const ext  = formats.extension(to);
+	string const ext  = theFormats().extension(to);
 	LYXERR(Debug::GRAPHICS, "\t we have: from " << from << " to " << to);
 
 	// We're going to be running the exported buffer through the LaTeX
@@ -669,12 +669,12 @@ string InsetGraphics::prepareFile(OutputParams const & runparams) const
 			// The LaTeX compiler will not be able to determine
 			// the file format from the extension, so we must
 			// change it.
-			FileName const new_file = 
+			FileName const new_file =
 				FileName(changeExtension(temp_file.absFileName(), ext));
 			if (temp_file.moveTo(new_file)) {
 				temp_file = new_file;
 				output_file = changeExtension(output_file, ext);
-				source_file = 
+				source_file =
 					FileName(changeExtension(source_file.absFileName(), ext));
 			} else {
 				LYXERR(Debug::GRAPHICS, "Could not rename file `"
@@ -893,20 +893,19 @@ string InsetGraphics::prepareHTMLFile(OutputParams const & runparams) const
 	// Copy to temporary directory.
 	FileName temp_file;
 	GraphicsCopyStatus status;
-	boost::tie(status, temp_file) =
-			copyToDirIfNeeded(params().filename, temp_path);
+	tie(status, temp_file) = copyToDirIfNeeded(params().filename, temp_path);
 
 	if (status == FAILURE)
 		return string();
 
-	string const from = formats.getFormatFromFile(temp_file);
+	string const from = theFormats().getFormatFromFile(temp_file);
 	if (from.empty()) {
 		LYXERR(Debug::GRAPHICS, "\tCould not get file format.");
 		return string();
 	}
 
 	string const to   = findTargetFormat(from, runparams);
-	string const ext  = formats.extension(to);
+	string const ext  = theFormats().extension(to);
 	string const orig_file = params().filename.absFileName();
 	string output_file = onlyFileName(temp_file.absFileName());
 	LYXERR(Debug::GRAPHICS, "\t we have: from " << from << " to " << to);
@@ -940,10 +939,10 @@ string InsetGraphics::prepareHTMLFile(OutputParams const & runparams) const
 
 	// FIXME (Abdel 12/08/06): Is there a need to show these errors?
 	ErrorList el;
-	bool const success = 
+	bool const success =
 		theConverters().convert(&buffer(), temp_file, to_file, params().filename,
 			from, to, el, Converters::try_default | Converters::try_cache);
-	if (!success)	
+	if (!success)
 		return string();
 	runparams.exportdata->addExternalFile("xhtml", to_file, output_to_file);
 	return output_to_file;
@@ -952,18 +951,18 @@ string InsetGraphics::prepareHTMLFile(OutputParams const & runparams) const
 
 docstring InsetGraphics::xhtml(XHTMLStream & xs, OutputParams const & op) const
 {
-	string const output_file = prepareHTMLFile(op);
+	string const output_file = op.dryrun ? string() : prepareHTMLFile(op);
 
-	if (output_file.empty()) {
-		LYXERR0("InsetGraphics::xhtml: Unable to prepare file `" 
+	if (output_file.empty() && !op.dryrun) {
+		LYXERR0("InsetGraphics::xhtml: Unable to prepare file `"
 		        << params().filename << "' for output. File missing?");
-		string const attr = "src='" + params().filename.absFileName() 
+		string const attr = "src='" + params().filename.absFileName()
 		                    + "' alt='image: " + output_file + "'";
 		xs << html::CompTag("img", attr);
 		return docstring();
 	}
 
-	// FIXME XHTML 
+	// FIXME XHTML
 	// We aren't doing anything with the crop and rotate parameters, and it would
 	// really be better to do width and height conversion, rather than to output
 	// these parameters here.
@@ -985,7 +984,7 @@ docstring InsetGraphics::xhtml(XHTMLStream & xs, OutputParams const & op) const
 	if (!imgstyle.empty())
 		imgstyle = "style='" + imgstyle + "' ";
 
-	string const attr = imgstyle + "src='" + output_file + "' alt='image: " 
+	string const attr = imgstyle + "src='" + output_file + "' alt='image: "
 	                    + output_file + "'";
 	xs << html::CompTag("img", attr);
 	return docstring();
@@ -1036,17 +1035,19 @@ InsetGraphicsParams const & InsetGraphics::params() const
 
 void InsetGraphics::editGraphics(InsetGraphicsParams const & p) const
 {
-	formats.edit(buffer(), p.filename,
-		     formats.getFormatFromFile(p.filename));
+	theFormats().edit(buffer(), p.filename,
+		     theFormats().getFormatFromFile(p.filename));
 }
 
 
 void InsetGraphics::addToToc(DocIterator const & cpit, bool output_active,
-							 UpdateType) const
+							 UpdateType, TocBackend & backend) const
 {
 	//FIXME UNICODE
 	docstring const str = from_utf8(params_.filename.onlyFileName());
-	buffer().tocBackend().toc("graphics")->push_back(TocItem(cpit, 0, str, output_active));
+	TocBuilder & b = backend.builder("graphics");
+	b.pushItem(cpit, str, output_active);
+	b.pop();
 }
 
 
