@@ -54,6 +54,8 @@ public:
 	InsetGraphicsParams getParams() const { return params_;}
 	///
 	bool clickable(BufferView const &, int, int) const { return true; }
+	///
+	bool canPaintChange(BufferView const &) const { return true; };
 
 private:
 	///
@@ -100,7 +102,7 @@ private:
 	bool getStatus(Cursor &, FuncRequest const &, FuncStatus &) const;
 	///
 	void addToToc(DocIterator const & di, bool output_active,
-				  UpdateType utype) const;
+				  UpdateType utype, TocBackend & backend) const;
 	///
 	std::string contextMenuName() const;
 	/// Force inset into LTR environment if surroundings are RTL
@@ -153,7 +155,7 @@ namespace graphics {
 	void unifyGraphicsGroups(Buffer &, std::string const &);
 	InsetGraphics * getCurrentGraphicsInset(Cursor const &);
 
-}
+} // namespace graphics
 
 } // namespace lyx
 
