@@ -36,6 +36,7 @@
 #include "output_plaintext.h"
 #include "output_xhtml.h"
 #include "OutputParams.h"
+#include "texstream.h"
 #include "TextClass.h"
 #include "TocBackend.h"
 
@@ -61,6 +62,7 @@
 #include "support/lstrings.h" // contains
 #include "support/lyxalgo.h"
 #include "support/mutex.h"
+#include "support/ExceptionMessage.h"
 
 #include "support/bind.h"
 
@@ -97,7 +99,7 @@ Types type(string const & s)
 		return VERB;
 	if (s == "verbatiminput*")
 		return VERBAST;
-	if (s == "lstinputlisting")
+	if (s == "lstinputlisting" || s == "inputminted")
 		return LISTINGS;
 	if (s == "include")
 		return INCLUDE;
@@ -163,15 +165,15 @@ InsetLabel * createLabel(Buffer * buf, docstring const & label_str)
 	return new InsetLabel(buf, icp);
 }
 
-} // namespace anon
+} // namespace
 
 
 InsetInclude::InsetInclude(Buffer * buf, InsetCommandParams const & p)
 	: InsetCommand(buf, p), include_label(uniqueID()),
-	  preview_(new RenderMonitoredPreview(this)), failedtoload_(false),
+	  preview_(make_unique<RenderMonitoredPreview>(this)), failedtoload_(false),
 	  set_label_(false), label_(0), child_buffer_(0)
 {
-	preview_->fileChanged(bind(&InsetInclude::fileChanged, this));
+	preview_->connect([=](){ fileChanged(); });
 
 	if (isListings(params())) {
 		InsetListingsParams listing_params(to_utf8(p["lstparams"]));
@@ -183,10 +185,10 @@ InsetInclude::InsetInclude(Buffer * buf, InsetCommandParams const & p)
 
 InsetInclude::InsetInclude(InsetInclude const & other)
 	: InsetCommand(other), include_label(other.include_label),
-	  preview_(new RenderMonitoredPreview(this)), failedtoload_(false),
+	  preview_(make_unique<RenderMonitoredPreview>(this)), failedtoload_(false),
 	  set_label_(false), label_(0), child_buffer_(0)
 {
-	preview_->fileChanged(bind(&InsetInclude::fileChanged, this));
+	preview_->connect([=](){ fileChanged(); });
 
 	if (other.label_)
 		label_ = new InsetLabel(*other.label_);
@@ -196,7 +198,10 @@ InsetInclude::InsetInclude(InsetInclude const & other)
 InsetInclude::~InsetInclude()
 {
 	if (isBufferLoaded())
-		buffer().invalidateBibfileCache();
+		/* We do not use buffer() because Coverity believes that this
+		 * may throw an exception. Actually this code path is not
+		 * taken when buffer_ == 0 */
+		buffer_->invalidateBibfileCache();
 	delete label_;
 }
 
@@ -260,24 +265,24 @@ void InsetInclude::doDispatch(Cursor & cur, FuncRequest & cmd)
 				InsetListingsParams new_params(to_utf8(p["lstparams"]));
 				docstring const new_label =
 					from_utf8(new_params.getParamValue("label"));
-				
+
 				if (new_label.empty()) {
 					delete label_;
 					label_ = 0;
 				} else {
 					docstring old_label;
-					if (label_) 
+					if (label_)
 						old_label = label_->getParam("name");
 					else {
 						label_ = createLabel(buffer_, new_label);
 						label_->setBuffer(buffer());
-					}					
+					}
 
 					if (new_label != old_label) {
 						label_->updateLabelAndRefs(new_label, &cur);
 						// the label might have been adapted (duplicate)
 						if (new_label != label_->getParam("name")) {
-							new_params.addParam("label", "{" + 
+							new_params.addParam("label", "{" +
 								to_utf8(label_->getParam("name")) + "}", true);
 							p["lstparams"] = from_utf8(new_params.params());
 						}
@@ -308,7 +313,7 @@ void InsetInclude::editIncluded(string const & file)
 		lyx::dispatch(fr);
 	} else
 		// tex file or other text file in verbatim mode
-		formats.edit(buffer(),
+		theFormats().edit(buffer(),
 			support::makeAbsPath(file, support::onlyPath(buffer().absFileName())),
 			"text");
 }
@@ -407,7 +412,7 @@ docstring InsetInclude::screenLabel() const
 
 Buffer * InsetInclude::getChildBuffer() const
 {
-	Buffer * childBuffer = loadIfNeeded(); 
+	Buffer * childBuffer = loadIfNeeded();
 
 	// FIXME RECURSIVE INCLUDE
 	// This isn't sufficient, as the inclusion could be downstream.
@@ -422,7 +427,7 @@ Buffer * InsetInclude::loadIfNeeded() const
 	// try to load the cloned child document again.
 	if (buffer().isClone())
 		return child_buffer_;
-	
+
 	// Don't try to load it again if we failed before.
 	if (failedtoload_ || isVerbatim(params()) || isListings(params()))
 		return 0;
@@ -431,7 +436,7 @@ Buffer * InsetInclude::loadIfNeeded() const
 	// Use cached Buffer if possible.
 	if (child_buffer_ != 0) {
 		if (theBufferList().isLoaded(child_buffer_)
-    		// additional sanity check: make sure the Buffer really is
+		    // additional sanity check: make sure the Buffer really is
 		    // associated with the file we want.
 		    && child_buffer_ == theBufferList().getBuffer(included_file))
 			return child_buffer_;
@@ -593,15 +598,87 @@ void InsetInclude::latex(otexstream & os, OutputParams const & runparams) const
 		break;
 	}
 	case LISTINGS: {
+		// Here, listings and minted have sligthly different behaviors.
+		// Using listings, it is always possible to have a caption,
+		// even for non-floats. Using minted, only floats can have a
+		// caption. So, with minted we use the following strategy.
+		// If a caption was specified but the float parameter was not,
+		// we ourselves add a caption above the listing (because the
+		// listing comes from a file and might span several pages).
+		// Otherwise, if float was specified, the floating listing
+		// environment provided by minted is used. In either case, the
+		// label parameter is taken as the label by which the float
+		// can be referenced, otherwise it will have the meaning
+		// intended by minted. In this last case, the label will
+		// serve as a sort of caption that, however, will be shown
+		// by minted only if the frame parameter is also specified.
+		bool const use_minted = buffer().params().use_minted;
 		runparams.exportdata->addExternalFile(tex_format, writefile,
 						      exportfile);
-		os << '\\' << from_ascii(params().getCmdName());
 		string const opt = to_utf8(params()["lstparams"]);
 		// opt is set in QInclude dialog and should have passed validation.
-		InsetListingsParams params(opt);
-		if (!params.params().empty())
-			os << "[" << from_utf8(params.params()) << "]";
-		os << '{'  << from_utf8(incfile) << '}';
+		InsetListingsParams lstparams(opt);
+		string parameters = lstparams.params();
+		string language;
+		string caption;
+		string label;
+		string placement;
+		bool isfloat = lstparams.isFloat();
+		if (use_minted) {
+			// Get float placement, language, caption, and
+			// label, then remove the relative options.
+			vector<string> opts =
+				getVectorFromString(parameters, ",", false);
+			for (size_t i = 0; i < opts.size(); ++i) {
+				if (prefixIs(opts[i], "float")) {
+					if (prefixIs(opts[i], "float="))
+						placement = opts[i].substr(6);
+					opts.erase(opts.begin() + i--);
+				} else if (prefixIs(opts[i], "language=")) {
+					language = opts[i].substr(9);
+					opts.erase(opts.begin() + i--);
+				} else if (prefixIs(opts[i], "caption=")) {
+					caption = opts[i].substr(8);
+					opts.erase(opts.begin() + i--);
+				} else if (prefixIs(opts[i], "label=")) {
+					label = opts[i].substr(6);
+					opts.erase(opts.begin() + i--);
+				}
+			}
+			if (!label.empty()) {
+				if (isfloat || !caption.empty())
+					label = trim(label, "{}");
+				else
+					opts.push_back("label=" + label);
+			}
+			parameters = getStringFromVector(opts, ",");
+		}
+		if (language.empty())
+			language = "TeX";
+		if (use_minted && isfloat) {
+			os << breakln << "\\begin{listing}";
+			if (!placement.empty())
+				os << '[' << placement << "]";
+			os << breakln;
+		} else if (use_minted && !caption.empty()) {
+			os << breakln << "\\lyxmintcaption[t]{" << caption;
+			if (!label.empty())
+				os << "\\label{" << label << "}";
+			os << "}\n";
+		}
+		os << (use_minted ? "\\inputminted" : "\\lstinputlisting");
+		if (!parameters.empty())
+			os << "[" << parameters << "]";
+		if (use_minted)
+			os << '{'  << ascii_lowercase(language) << '}';
+		os << '{'  << incfile << '}';
+		if (use_minted && isfloat) {
+			if (!caption.empty())
+				os << breakln << "\\caption{" << caption << "}";
+			if (!label.empty())
+				os << breakln << "\\label{" << label << "}";
+			os << breakln << "\\end{listing}\n";
+		}
 		break;
 	}
 	case INCLUDE: {
@@ -638,7 +715,8 @@ void InsetInclude::latex(otexstream & os, OutputParams const & runparams) const
 					"file\n`%1$s'\n"
 					"Please, check whether it actually exists."),
 					included_file.displayName());
-				Alert::warning(_("Missing included file"), text);
+				throw ExceptionMessage(ErrorException, _("Error: "),
+						       text);
 			}
 			return;
 		}
@@ -655,6 +733,18 @@ void InsetInclude::latex(otexstream & os, OutputParams const & runparams) const
 				Alert::warning(_("Different textclasses"), text, true);
 			}
 
+			string const child_tf = tmp->params().useNonTeXFonts ? "true" : "false";
+			string const master_tf = masterBuffer->params().useNonTeXFonts ? "true" : "false";
+			if (tmp->params().useNonTeXFonts != masterBuffer->params().useNonTeXFonts) {
+				docstring text = bformat(_("Included file `%1$s'\n"
+					"has use-non-TeX-fonts set to `%2$s'\n"
+					"while parent file has use-non-TeX-fonts set to `%3$s'."),
+					included_file.displayName(),
+					from_utf8(child_tf),
+					from_utf8(master_tf));
+				Alert::warning(_("Different use-non-TeX-fonts settings"), text, true);
+			}
+
 			// Make sure modules used in child are all included in master
 			// FIXME It might be worth loading the children's modules into the master
 			// over in BufferParams rather than doing this check.
@@ -681,7 +771,7 @@ void InsetInclude::latex(otexstream & os, OutputParams const & runparams) const
 		// Don't assume the child's format is latex
 		string const inc_format = tmp->params().bufferFormat();
 		FileName const tmpwritefile(changeExtension(writefile.absFileName(),
-			formats.extension(inc_format)));
+			theFormats().extension(inc_format)));
 
 		// FIXME: handle non existing files
 		// The included file might be written in a different encoding
@@ -701,7 +791,7 @@ void InsetInclude::latex(otexstream & os, OutputParams const & runparams) const
 				onlyPath().absFileName(), runparams, Buffer::OnlyBody)) {
 			if (!runparams.silent) {
 				docstring msg = bformat(_("Included file `%1$s' "
-					"was not exported correctly.\nWarning: "
+					"was not exported correctly.\n "
 					"LaTeX export is probably incomplete."),
 					included_file.displayName());
 				ErrorList const & el = tmp->errorList("Export");
@@ -709,7 +799,8 @@ void InsetInclude::latex(otexstream & os, OutputParams const & runparams) const
 					msg = bformat(from_ascii("%1$s\n\n%2$s\n\n%3$s"),
 						msg, el.begin()->error,
 						el.begin()->description);
-				Alert::warning(_("Export failure"), msg);
+				throw ExceptionMessage(ErrorException, _("Error: "),
+						       msg);
 			}
 		}
 		runparams.encoding = oldEnc;
@@ -726,14 +817,15 @@ void InsetInclude::latex(otexstream & os, OutputParams const & runparams) const
 
 			if (!success && !runparams.silent) {
 				docstring msg = bformat(_("Included file `%1$s' "
-						"was not exported correctly.\nWarning: "
+						"was not exported correctly.\n "
 						"LaTeX export is probably incomplete."),
 						included_file.displayName());
 				if (!el.empty())
 					msg = bformat(from_ascii("%1$s\n\n%2$s\n\n%3$s"),
 							msg, el.begin()->error,
 							el.begin()->description);
-				Alert::warning(_("Export failure"), msg);
+				throw ExceptionMessage(ErrorException, _("Error: "),
+						       msg);
 			}
 		}
 	} else {
@@ -806,20 +898,20 @@ docstring InsetInclude::xhtml(XHTMLStream & xs, OutputParams const & rp) const
 		return docstring();
 
 	// are we generating only some paragraphs, or all of them?
-	bool const all_pars = !rp.dryrun || 
-			(rp.par_begin == 0 && 
+	bool const all_pars = !rp.dryrun ||
+			(rp.par_begin == 0 &&
 			 rp.par_end == (int)buffer().text().paragraphs().size());
-	
+
 	OutputParams op = rp;
 	if (all_pars) {
 		op.par_begin = 0;
 		op.par_end = 0;
 		ibuf->writeLyXHTMLSource(xs.os(), op, Buffer::IncludedFile);
 	} else
-		xs << XHTMLStream::ESCAPE_NONE 
-		   << "<!-- Included file: " 
-		   << from_utf8(included_file.absFileName()) 
-		   << XHTMLStream::ESCAPE_NONE 
+		xs << XHTMLStream::ESCAPE_NONE
+		   << "<!-- Included file: "
+		   << from_utf8(included_file.absFileName())
+		   << XHTMLStream::ESCAPE_NONE
 			 << " -->";
 	return docstring();
 }
@@ -942,8 +1034,16 @@ void InsetInclude::validate(LaTeXFeatures & features) const
 	features.useInsetLayout(getLayout());
 	if (isVerbatim(params()))
 		features.require("verbatim");
-	else if (isListings(params()))
-		features.require("listings");
+	else if (isListings(params())) {
+		if (buffer().params().use_minted) {
+			features.require("minted");
+			string const opts = to_utf8(params()["lstparams"]);
+			InsetListingsParams lstpars(opts);
+			if (!lstpars.isFloat() && contains(opts, "caption="))
+				features.require("lyxmintcaption");
+		} else
+			features.require("listings");
+	}
 
 	// Here we must do the fun stuff...
 	// Load the file in the include if it needs
@@ -972,7 +1072,7 @@ void InsetInclude::validate(LaTeXFeatures & features) const
 }
 
 
-void InsetInclude::collectBibKeys(InsetIterator const & /*di*/) const
+void InsetInclude::collectBibKeys(InsetIterator const & /*di*/, FileNameList & checkedFiles) const
 {
 	Buffer * child = loadIfNeeded();
 	if (!child)
@@ -982,7 +1082,7 @@ void InsetInclude::collectBibKeys(InsetIterator const & /*di*/) const
 	// But it'll have to do for now.
 	if (child == &buffer())
 		return;
-	child->collectBibKeys();
+	child->collectBibKeys(checkedFiles);
 }
 
 
@@ -1002,7 +1102,7 @@ void InsetInclude::metrics(MetricsInfo & mi, Dimension & dim) const
 	} else {
 		if (!set_label_) {
 			set_label_ = true;
-			button_.update(screenLabel(), true);
+			button_.update(screenLabel(), true, false);
 		}
 		button_.metrics(mi, dim);
 	}
@@ -1067,7 +1167,7 @@ void InsetInclude::fileChanged() const
 		return;
 
 	preview_->removePreview(*buffer);
-	add_preview(*preview_.get(), *this, *buffer);
+	add_preview(*preview_, *this, *buffer);
 	preview_->startLoading(*buffer);
 }
 
@@ -1085,9 +1185,8 @@ bool preview_wanted(InsetCommandParams const & params, Buffer const & buffer)
 
 docstring latexString(InsetInclude const & inset)
 {
-	TexRow texrow;
 	odocstringstream ods;
-	otexstream os(ods, texrow);
+	otexstream os(ods);
 	// We don't need to set runparams.encoding since this will be done
 	// by latex() anyway.
 	OutputParams runparams(0);
@@ -1110,7 +1209,7 @@ void add_preview(RenderMonitoredPreview & renderer, InsetInclude const & inset,
 	}
 }
 
-} // namespace anon
+} // namespace
 
 
 void InsetInclude::addPreview(DocIterator const & /*inset_pos*/,
@@ -1126,40 +1225,40 @@ void InsetInclude::addPreview(DocIterator const & /*inset_pos*/,
 
 
 void InsetInclude::addToToc(DocIterator const & cpit, bool output_active,
-							UpdateType utype) const
+                            UpdateType utype, TocBackend & backend) const
 {
-	TocBackend & backend = buffer().tocBackend();
-
 	if (isListings(params())) {
 		if (label_)
-			label_->addToToc(cpit, output_active, utype);
-
+			label_->addToToc(cpit, output_active, utype, backend);
+		TocBuilder & b = backend.builder("listing");
+		b.pushItem(cpit, screenLabel(), output_active);
 		InsetListingsParams p(to_utf8(params()["lstparams"]));
-		string caption = p.getParamValue("caption");
-		if (caption.empty())
-			return;
-		shared_ptr<Toc> toc = backend.toc("listing");
-		docstring str = convert<docstring>(toc->size() + 1)
-			+ ". " +  from_utf8(caption);
-		DocIterator pit = cpit;
-		toc->push_back(TocItem(pit, 0, str, output_active));
+		b.argumentItem(from_utf8(p.getParamValue("caption")));
+		b.pop();
 	} else {
 		Buffer const * const childbuffer = getChildBuffer();
+
+		TocBuilder & b = backend.builder("child");
+		docstring str = childbuffer ? childbuffer->fileName().displayName()
+			: from_ascii("?");
+		b.pushItem(cpit, str, output_active);
+		b.pop();
+
 		if (!childbuffer)
 			return;
 
-		shared_ptr<Toc> toc = backend.toc("child");
-		docstring str = childbuffer->fileName().displayName();
-		toc->push_back(TocItem(cpit, 0, str, output_active));
-
+		// Update the child's tocBackend. The outliner uses the master's, but
+		// the navigation menu uses the child's.
 		childbuffer->tocBackend().update(output_active, utype);
-		TocList const & childtoclist = childbuffer->tocBackend().tocs();
-		TocList::const_iterator it = childtoclist.begin();
-		TocList::const_iterator const end = childtoclist.end();
-		for(; it != end; ++it) {
-			shared_ptr<Toc> toc = backend.toc(it->first);
-			toc->insert(toc->end(), it->second->begin(), it->second->end());
-		}
+		// Include Tocs from children
+		childbuffer->inset().addToToc(DocIterator(), output_active, utype,
+		                              backend);
+		//Copy missing outliner names (though the user has been warned against
+		//having different document class and module selection between master
+		//and child).
+		for (pair<string, docstring> const & name
+			     : childbuffer->params().documentClass().outlinerNames())
+			backend.addName(name.first, translateIfPossible(name.second));
 	}
 }
 
@@ -1182,13 +1281,13 @@ void InsetInclude::updateCommand()
 	InsetListingsParams par(to_utf8(params()["lstparams"]));
 	par.addParam("label", "{" + to_utf8(new_label) + "}", true);
 	p["lstparams"] = from_utf8(par.params());
-	setParams(p);	
+	setParams(p);
 }
 
 
 void InsetInclude::updateBuffer(ParIterator const & it, UpdateType utype)
 {
-	button_.update(screenLabel(), true);
+	button_.update(screenLabel(), true, false);
 
 	Buffer const * const childbuffer = getChildBuffer();
 	if (childbuffer) {
