@@ -12,7 +12,7 @@
 #ifndef INSET_INFO_H
 #define INSET_INFO_H
 
-#include "InsetCollapsable.h"
+#include "InsetCollapsible.h"
 
 namespace lyx {
 
@@ -61,8 +61,8 @@ icon: argument is the name of the LFUN such as "paste". The syntax is the same
 
 buffer: argument can be one of "name", "path", "class". This inset output the
     filename, path, and textclass of this buffer.
-		
-lyxinfo: argument must (presently) be "version". This inset outputs information 
+
+lyxinfo: argument must (presently) be "version". This inset outputs information
 		about the version of LyX currently in use.
 
 There is currently no GUI, no menu entry for this inset. A user can define a
@@ -77,7 +77,7 @@ the command buffer (view->Toolbar->Command Buffer).
 
 */
 
-class InsetInfo : public InsetCollapsable {
+class InsetInfo : public InsetCollapsible {
 public:
 	enum info_type {
 		UNKNOWN_INFO,   // Invalid type
@@ -100,6 +100,11 @@ public:
 	docstring layoutName() const;
 	///
 	Inset * editXY(Cursor & cur, int x, int y);
+	/** FIXME: we would like to do that, but then InsetText::updateBuffer breaks
+	 * on info insets. Do we need to run this method on InsetInfo contents?
+	 * Having a InsetInfo that hides an InsetText is really annoying, actually.
+	 */
+	///bool isActive() const { return false; }
 	///
 	bool editable() const { return false; }
 	///
@@ -130,7 +135,7 @@ public:
 	std::string contextMenu(BufferView const &, int, int) const;
 	///
 	std::string contextMenuName() const;
-	/// should paragraph indendation be ommitted in any case?
+	/// should paragraph indendation be omitted in any case?
 	bool neverIndent() const { return true; }
 
 private:
@@ -141,7 +146,7 @@ private:
 	///
 	void setText(docstring const & str);
 	// make sure that the other version of setText is still available.
-	using InsetCollapsable::setText;
+	using InsetCollapsible::setText;
 	///
 	info_type type_;
 	///
