@@ -156,7 +156,7 @@ void InsetLabel::updateBuffer(ParIterator const & par, UpdateType utype)
 
 	if (utype == OutputUpdate) {
 		// save info on the active counter
-		Counters const & cnts = 
+		Counters const & cnts =
 			buffer().masterBuffer()->params().documentClass().counters();
 		active_counter_ = cnts.currentCounter();
 		Language const * lang = par->getParLanguage(buffer().params());
@@ -172,10 +172,10 @@ void InsetLabel::updateBuffer(ParIterator const & par, UpdateType utype)
 
 
 void InsetLabel::addToToc(DocIterator const & cpit, bool output_active,
-						  UpdateType) const
+						  UpdateType, TocBackend & backend) const
 {
 	docstring const & label = getParam("name");
-	shared_ptr<Toc> toc = buffer().tocBackend().toc("label");
+	shared_ptr<Toc> toc = backend.toc("label");
 	if (buffer().insetLabel(label) != this) {
 		toc->push_back(TocItem(cpit, 0, screen_label_, output_active));
 	} else {
@@ -296,9 +296,9 @@ docstring InsetLabel::xhtml(XHTMLStream & xs, OutputParams const &) const
 {
 	// FIXME XHTML
 	// Unfortunately, the name attribute has been deprecated, so we have to use
-	// id here to get the document to validate as XHTML 1.1. This will cause a 
+	// id here to get the document to validate as XHTML 1.1. This will cause a
 	// problem with some browsers, though, I'm sure. (Guess which!) So we will
-	// have to figure out what to do about this later. 
+	// have to figure out what to do about this later.
 	docstring const attr = "id=\"" + html::cleanAttr(getParam("name")) + '"';
 	xs << html::CompTag("a", to_utf8(attr));
 	return docstring();
