@@ -30,6 +30,8 @@
 #include "output_xhtml.h"
 #include "OutputParams.h"
 #include "TextClass.h"
+#include "TexRow.h"
+#include "texstream.h"
 
 #include "support/debug.h"
 #include "support/docstream.h"
@@ -53,6 +55,7 @@ namespace lyx {
 InsetListings::InsetListings(Buffer * buf, InsetListingsParams const & par)
 	: InsetCaptionable(buf,"listing")
 {
+	params_.setMinted(buffer().params().use_minted);
 	status_ = par.status();
 }
 
@@ -69,6 +72,15 @@ Inset::DisplayType InsetListings::display() const
 }
 
 
+docstring InsetListings::layoutName() const
+{
+	if (buffer().params().use_minted)
+		return from_ascii("MintedListings");
+	else
+		return from_ascii("Listings");
+}
+
+
 void InsetListings::write(ostream & os) const
 {
 	os << "listings" << "\n";
@@ -107,12 +119,58 @@ void InsetListings::read(Lexer & lex)
 }
 
 
+Encoding const * InsetListings::forcedEncoding(Encoding const * inner_enc,
+											   Encoding const * outer_enc) const
+{
+	// The listings package cannot deal with multi-byte-encoded
+	// glyphs, except if full-unicode aware backends
+	// such as XeTeX or LuaTeX are used, and with pLaTeX.
+	// Minted can deal with all encodings.
+	if (buffer().params().use_minted
+		|| (buffer().params().encoding().package() == Encoding::japanese
+			&& inner_enc->package() == Encoding::japanese)
+		|| inner_enc->hasFixedWidth())
+		return 0;
+
+	// We try if there's a singlebyte encoding for the outer
+	// language; if not, fall back to latin1.
+	return (outer_enc->hasFixedWidth()) ?
+			outer_enc : encodings.fromLyXName("iso8859-1");
+}
+
+
 void InsetListings::latex(otexstream & os, OutputParams const & runparams) const
 {
 	string param_string = params().params();
 	// NOTE: I use {} to quote text, which is an experimental feature
 	// of the listings package (see page 25 of the manual)
 	bool const isInline = params().isInline();
+	bool const use_minted = buffer().params().use_minted;
+	string minted_language;
+	string float_placement;
+	bool const isfloat = params().isFloat();
+	if (use_minted && (isfloat || contains(param_string, "language="))) {
+		// Get float placement and/or language of the code,
+		// then remove the relative options.
+		vector<string> opts =
+			getVectorFromString(param_string, ",", false);
+		for (size_t i = 0; i < opts.size(); ++i) {
+			if (prefixIs(opts[i], "float")) {
+				if (prefixIs(opts[i], "float="))
+					float_placement = opts[i].substr(6);
+				opts.erase(opts.begin() + i--);
+			}
+			else if (prefixIs(opts[i], "language=")) {
+				minted_language = opts[i].substr(9);
+				opts.erase(opts.begin() + i--);
+			}
+		}
+		param_string = getStringFromVector(opts, ",");
+	}
+	// Minted needs a language specification
+	if (minted_language.empty())
+		minted_language = "TeX";
+
 	// get the paragraphs. We can not output them directly to given odocstream
 	// because we can not yet determine the delimiter character of \lstinline
 	docstring code;
@@ -122,33 +180,25 @@ void InsetListings::latex(otexstream & os, OutputParams const & runparams) const
 
 	bool encoding_switched = false;
 	Encoding const * const save_enc = runparams.encoding;
-	// The listings package cannot deal with multi-byte-encoded
-	// glyphs, except if full-unicode aware backends
-	// such as XeTeX or LuaTeX are used, and with pLaTeX.
-	bool const multibyte_possible =	runparams.isFullUnicode()
-	    || (buffer().params().bufferFormat() == "platex"
-	        && runparams.encoding->package() == Encoding::japanese);
 
-	if (!multibyte_possible && !runparams.encoding->hasFixedWidth()) {
+	Encoding const * const outer_encoding =
+		(runparams.local_font != 0) ?
+			runparams.local_font->language()->encoding()
+			: buffer().params().language->encoding();
+	Encoding const * fixedlstenc = forcedEncoding(runparams.encoding, outer_encoding);
+	if (fixedlstenc) {
 		// We need to switch to a singlebyte encoding, due to
 		// the restrictions of the listings package (see above).
 		// This needs to be consistent with
 		// LaTeXFeatures::getTClassI18nPreamble().
-		Language const * const outer_language =
-			(runparams.local_font != 0) ?
-				runparams.local_font->language()
-				: buffer().params().language;
-		// We try if there's a singlebyte encoding for the current
-		// language; if not, fall back to latin1.
-		Encoding const * const lstenc =
-			(outer_language->encoding()->hasFixedWidth()) ?
-				outer_language->encoding()
-				: encodings.fromLyXName("iso8859-1");
-		switchEncoding(os.os(), buffer().params(), runparams, *lstenc, true);
-		runparams.encoding = lstenc;
+		switchEncoding(os.os(), buffer().params(), runparams, *fixedlstenc, true);
+		runparams.encoding = fixedlstenc;
 		encoding_switched = true;
 	}
 
+	bool const captionfirst = !isfloat && par->isInset(0)
+				&& par->getInset(0)->lyxCode() == CAPTION_CODE;
+
 	while (par != end) {
 		pos_type siz = par->size();
 		bool captionline = false;
@@ -156,8 +206,20 @@ void InsetListings::latex(otexstream & os, OutputParams const & runparams) const
 			if (i == 0 && par->isInset(i) && i + 1 == siz)
 				captionline = true;
 			// ignore all struck out text and (caption) insets
-			if (par->isDeleted(i) || par->isInset(i))
+			if (par->isDeleted(i)
+			    || (par->isInset(i) && par->getInset(i)->lyxCode() == CAPTION_CODE))
 				continue;
+			if (par->isInset(i)) {
+				// Currently, this can only be a quote inset
+				// that is output as plain quote here, but
+				// we use more generic code anyway.
+				otexstringstream ots;
+				OutputParams rp = runparams;
+				rp.pass_thru = true;
+				par->getInset(i)->latex(ots, rp);
+				code += ots.str();
+				continue;
+			}
 			char_type c = par->getChar(i);
 			// we can only output characters covered by the current
 			// encoding!
@@ -211,23 +273,59 @@ void InsetListings::latex(otexstream & os, OutputParams const & runparams) const
 			}
 		}
 		docstring const delim(1, delimiters[pos]);
-		os << "\\lstinline";
-		if (!param_string.empty())
-			os << "[" << from_utf8(param_string) << "]";
-		else if (pos >= delimiters.find('Q'))
-			// We need to terminate the command before the delimiter
-			os << " ";
+		if (use_minted) {
+			os << "\\mintinline";
+			if (!param_string.empty())
+				os << "[" << from_utf8(param_string) << "]";
+			os << "{" << ascii_lowercase(minted_language) << "}";
+		} else {
+			os << "\\lstinline";
+			if (!param_string.empty())
+				os << "[" << from_utf8(param_string) << "]";
+			else if (pos >= delimiters.find('Q'))
+				// We need to terminate the command before
+				// the delimiter
+				os << " ";
+		}
 		os << delim << code << delim;
+	} else if (use_minted) {
+		OutputParams rp = runparams;
+		rp.moving_arg = true;
+		TexString caption = getCaption(rp);
+		if (isfloat) {
+			os << breakln << "\\begin{listing}";
+			if (!float_placement.empty())
+				os << '[' << float_placement << "]";
+		} else if (captionfirst && !caption.str.empty()) {
+			os << breakln << "\\lyxmintcaption[t]{"
+			   << move(caption) << "}\n";
+		}
+		os << breakln << "\\begin{minted}";
+		if (!param_string.empty())
+			os << "[" << param_string << "]";
+		os << "{" << ascii_lowercase(minted_language) << "}\n"
+		   << code << breakln << "\\end{minted}\n";
+		if (isfloat) {
+			if (!caption.str.empty())
+				os << "\\caption{" << move(caption) << "}\n";
+			os << "\\end{listing}\n";
+		} else if (!captionfirst && !caption.str.empty()) {
+			os << breakln << "\\lyxmintcaption[b]{"
+			   << move(caption) << "}";
+		}
 	} else {
 		OutputParams rp = runparams;
 		rp.moving_arg = true;
-		docstring const caption = getCaption(rp);
-		if (param_string.empty() && caption.empty())
-			os << breakln << "\\begin{lstlisting}\n";
+		TexString caption = getCaption(rp);
+		os << breakln << "\\begin{lstlisting}";
+		if (param_string.empty() && caption.str.empty())
+			os << "\n";
 		else {
-			os << breakln << "\\begin{lstlisting}[";
-			if (!caption.empty()) {
-				os << "caption={" << caption << '}';
+			if (!runparams.nice)
+				os << safebreakln;
+			os << "[";
+			if (!caption.str.empty()) {
+				os << "caption={" << move(caption) << '}';
 				if (!param_string.empty())
 					os << ',';
 			}
@@ -245,7 +343,7 @@ void InsetListings::latex(otexstream & os, OutputParams const & runparams) const
 	if (!uncodable.empty() && !runparams.silent) {
 		// issue a warning about omitted characters
 		// FIXME: should be passed to the error dialog
-		if (!multibyte_possible && !runparams.encoding->hasFixedWidth())
+		if (fixedlstenc)
 			frontend::Alert::warning(_("Uncodable characters in listings inset"),
 				bformat(_("The following characters in one of the program listings are\n"
 					  "not representable in the current encoding and have been omitted:\n%1$s.\n"
@@ -371,11 +469,18 @@ docstring const InsetListings::buttonLabel(BufferView const & bv) const
 
 void InsetListings::validate(LaTeXFeatures & features) const
 {
-	features.require("listings");
 	features.useInsetLayout(getLayout());
 	string param_string = params().params();
-	if (param_string.find("\\color") != string::npos)
-		features.require("color");
+	if (buffer().params().use_minted) {
+		features.require("minted");
+		OutputParams rp = features.runparams();
+		if (!params().isFloat() && !getCaption(rp).str.empty())
+			features.require("lyxmintcaption");
+	} else {
+		features.require("listings");
+		if (contains(param_string, "\\color"))
+			features.require("color");
+	}
 	InsetCaptionable::validate(features);
 }
 
@@ -388,18 +493,13 @@ bool InsetListings::showInsetDialog(BufferView * bv) const
 }
 
 
-docstring InsetListings::getCaption(OutputParams const & runparams) const
+TexString InsetListings::getCaption(OutputParams const & runparams) const
 {
-	if (paragraphs().empty())
-		return docstring();
-
 	InsetCaption const * ins = getCaptionInset();
 	if (ins == 0)
-		return docstring();
+		return TexString();
 
-	TexRow texrow;
-	odocstringstream ods;
-	otexstream os(ods, texrow);
+	otexstringstream os;
 	ins->getArgs(os, runparams);
 	ins->getArgument(os, runparams);
 
@@ -408,8 +508,9 @@ docstring InsetListings::getCaption(OutputParams const & runparams) const
 
 	// the caption may contain \label{} but the listings
 	// package prefer caption={}, label={}
-	docstring cap = ods.str();
-	if (!contains(to_utf8(cap), "\\label{"))
+	TexString cap = os.release();
+	if (buffer().params().use_minted
+	    || !contains(cap.str, from_ascii("\\label{")))
 		return cap;
 	// convert from
 	//     blah1\label{blah2} blah3
@@ -421,11 +522,15 @@ docstring InsetListings::getCaption(OutputParams const & runparams) const
 	// NOTE that } is not allowed in blah2.
 	regex const reg("(.*)\\\\label\\{(.*?)\\}(.*)");
 	string const new_cap("$1$3},label={$2");
+	// TexString validity: the substitution preserves the number of newlines.
+	// Moreover we assume that $2 does not contain newlines, so that the texrow
+	// information remains accurate.
 	// Replace '\n' with an improbable character from Private Use Area-A
 	// and then return to '\n' after the regex replacement.
-	string const capstr = to_utf8(subst(cap, char_type('\n'), 0xffffd));
-	return subst(from_utf8(regex_replace(capstr, reg, new_cap)),
-		     0xffffd, char_type('\n'));
+	docstring const capstr = subst(cap.str, char_type('\n'), 0xffffd);
+	cap.str = subst(from_utf8(regex_replace(to_utf8(capstr), reg, new_cap)),
+			0xffffd, char_type('\n'));
+	return cap;
 }
 
 
