@@ -81,7 +81,7 @@ ScriptTranslatorLoc const & scripttranslator_loc()
 	return translator;
 }
 
-} // anon
+} // namespace
 
 
 InsetScriptParams::InsetScriptParams()
@@ -160,7 +160,7 @@ Inset::DisplayType InsetScript::display() const
 void InsetScript::metrics(MetricsInfo & mi, Dimension & dim) const
 {
 	int const shift = params_.shift(mi.base.font);
-	ScriptChanger dummy(mi.base);
+	Changer dummy = mi.base.changeScript();
 	InsetText::metrics(mi, dim);
 	dim.asc -= shift;
 	dim.des += shift;
@@ -170,7 +170,7 @@ void InsetScript::metrics(MetricsInfo & mi, Dimension & dim) const
 void InsetScript::draw(PainterInfo & pi, int x, int y) const
 {
 	int const shift = params_.shift(pi.base.font);
-	ScriptChanger dummy(pi.base);
+	Changer dummy = pi.base.changeScript();
 	InsetText::draw(pi, x, y + shift);
 }
 
@@ -235,7 +235,7 @@ bool InsetScript::insetAllowed(InsetCode code) const
 	case BOX_CODE:
 	case BRANCH_CODE:
 	case CAPTION_CODE:
-	case COLLAPSABLE_CODE:
+	case COLLAPSIBLE_CODE:
 	case FLOAT_CODE:
 	case FLOAT_LIST_CODE:
 	case FOOT_CODE:
@@ -263,8 +263,6 @@ bool InsetScript::getStatus(Cursor & cur, FuncRequest const & cmd,
 		FuncStatus & flag) const
 {
 	switch (cmd.action()) {
-	case LFUN_LAYOUT:
-	case LFUN_LAYOUT_PARAGRAPH:
 	case LFUN_MATH_DISPLAY:
 	case LFUN_BOX_INSERT:
 	case LFUN_BRANCH_INSERT:
