@@ -27,7 +27,6 @@
 #include "ErrorList.h"
 #include "FuncRequest.h"
 #include "FuncStatus.h"
-#include "InsetCaption.h"
 #include "InsetList.h"
 #include "Intl.h"
 #include "Language.h"
@@ -48,6 +47,7 @@
 #include "Row.h"
 #include "sgml.h"
 #include "TexRow.h"
+#include "texstream.h"
 #include "TextClass.h"
 #include "Text.h"
 #include "TextMetrics.h"
@@ -56,13 +56,13 @@
 #include "frontends/alert.h"
 #include "frontends/Painter.h"
 
+#include "support/bind.h"
 #include "support/convert.h"
 #include "support/debug.h"
 #include "support/gettext.h"
-#include "support/lstrings.h"
-
-#include "support/bind.h"
 #include "support/lassert.h"
+#include "support/lstrings.h"
+#include "support/RefChanger.h"
 
 #include <algorithm>
 
@@ -137,7 +137,7 @@ void InsetText::clear()
 }
 
 
-Dimension const InsetText::dimension(BufferView const & bv) const
+Dimension const InsetText::dimensionHelper(BufferView const & bv) const
 {
 	TextMetrics const & tm = bv.textMetrics(&text_);
 	Dimension dim = tm.dimension();
@@ -195,7 +195,7 @@ void InsetText::metrics(MetricsInfo & mi, Dimension & dim) const
 	mi.base.textwidth -= 2 * TEXT_TO_INSET_OFFSET;
 
 	// This can happen when a layout has a left and right margin,
-	// and the view is made very narrow. We can't do better than 
+	// and the view is made very narrow. We can't do better than
 	// to draw it partly out of view (bug 5890).
 	if (mi.base.textwidth < 1)
 		mi.base.textwidth = 1;
@@ -215,24 +215,48 @@ void InsetText::draw(PainterInfo & pi, int x, int y) const
 {
 	TextMetrics & tm = pi.base.bv->textMetrics(&text_);
 
+	int const w = tm.width() + TEXT_TO_INSET_OFFSET;
+	int const yframe = y - TEXT_TO_INSET_OFFSET - tm.ascent();
+	int const h = tm.height() + 2 * TEXT_TO_INSET_OFFSET;
+	int const xframe = x + TEXT_TO_INSET_OFFSET / 2;
+	bool change_drawn = false;
 	if (drawFrame_ || pi.full_repaint) {
-		int const w = tm.width() + TEXT_TO_INSET_OFFSET;
-		int const yframe = y - TEXT_TO_INSET_OFFSET - tm.ascent();
-		int const h = tm.height() + 2 * TEXT_TO_INSET_OFFSET;
-		int const xframe = x + TEXT_TO_INSET_OFFSET / 2;
 		if (pi.full_repaint)
 			pi.pain.fillRectangle(xframe, yframe, w, h,
 				pi.backgroundColor(this));
 
+		// Change color of the frame in tracked changes, like for tabulars.
+		// Only do so if the color is not custom. But do so even if RowPainter
+		// handles the strike-through already.
+		Color c;
+		if (pi.change_.changed()
+		    // Originally, these are the colors with role Text, from role() in
+		    // ColorCache.cpp.  The code is duplicated to avoid depending on Qt
+		    // types, and also maybe it need not match in the future.
+		    && (frameColor() == Color_foreground
+		        || frameColor() == Color_cursor
+		        || frameColor() == Color_preview
+		        || frameColor() == Color_tabularline
+		        || frameColor() == Color_previewframe)) {
+			c = pi.change_.color();
+			change_drawn = true;
+		} else
+			c = frameColor();
 		if (drawFrame_)
-			pi.pain.rectangle(xframe, yframe, w, h, frameColor());
+			pi.pain.rectangle(xframe, yframe, w, h, c);
 	}
-	ColorCode const old_color = pi.background_color;
-	pi.background_color = pi.backgroundColor(this, false);
-
-	tm.draw(pi, x + TEXT_TO_INSET_OFFSET, y);
-
-	pi.background_color = old_color;
+	{
+		Changer dummy = make_change(pi.background_color,
+		                            pi.backgroundColor(this, false));
+		// The change tracking cue must not be inherited
+		Changer dummy2 = make_change(pi.change_, Change());
+		tm.draw(pi, x + TEXT_TO_INSET_OFFSET, y);
+	}
+	if (canPaintChange(*pi.base.bv) && (!change_drawn || pi.change_.deleted()))
+		// Do not draw the change tracking cue if already done by RowPainter and
+		// do not draw the cue for INSERTED if the information is already in the
+		// color of the frame
+		pi.change_.paintCue(pi, xframe, yframe, xframe + w, yframe + h);
 }
 
 
@@ -250,7 +274,7 @@ void InsetText::edit(Cursor & cur, bool front, EntryDirection entry_from)
 		pos = temp_cur.pos();
 	}
 
-	text_.setCursor(cur.top(), pit, pos);
+	cur.top().setPitPos(pit, pos);
 	cur.finishUndo();
 }
 
@@ -266,7 +290,7 @@ void InsetText::doDispatch(Cursor & cur, FuncRequest & cmd)
 	LYXERR(Debug::ACTION, "InsetText::doDispatch(): cmd: " << cmd);
 
 	// See bug #9042, for instance.
-	if (isPassThru() && lyxCode() != ARG_CODE) {
+	if (isPassThru()) {
 		// Force any new text to latex_language FIXME: This
 		// should only be necessary in constructor, but new
 		// paragraphs that are created by pressing enter at
@@ -290,8 +314,8 @@ void InsetText::doDispatch(Cursor & cur, FuncRequest & cmd)
 		break;
 
 	case LFUN_INSET_DISSOLVE: {
-		bool const main_inset = &buffer().inset() == this;
-		bool const target_inset = cmd.argument().empty() 
+		bool const main_inset = text_.isMainText();
+		bool const target_inset = cmd.argument().empty()
 			|| cmd.getArg(0) == insetName(lyxCode());
 		// cur.inset() is the tabular when this is a single cell (bug #9954)
 		bool const one_cell = cur.inset().nargs() == 1;
@@ -312,7 +336,7 @@ void InsetText::doDispatch(Cursor & cur, FuncRequest & cmd)
 	default:
 		text_.dispatch(cur, cmd);
 	}
-	
+
 	if (!cur.result().dispatched())
 		Inset::doDispatch(cur, cmd);
 }
@@ -323,8 +347,8 @@ bool InsetText::getStatus(Cursor & cur, FuncRequest const & cmd,
 {
 	switch (cmd.action()) {
 	case LFUN_INSET_DISSOLVE: {
-		bool const main_inset = &buffer().inset() == this;
-		bool const target_inset = cmd.argument().empty() 
+		bool const main_inset = text_.isMainText();
+		bool const target_inset = cmd.argument().empty()
 			|| cmd.getArg(0) == insetName(lyxCode());
 		// cur.inset() is the tabular when this is a single cell (bug #9954)
 		bool const one_cell = cur.inset().nargs() == 1;
@@ -340,29 +364,21 @@ bool InsetText::getStatus(Cursor & cur, FuncRequest const & cmd,
 			status.setEnabled(false);
 			return true;
 		}
-		if (&buffer().inset() == this || !cur.paragraph().layout().args().empty())
+		if (text_.isMainText() || !cur.paragraph().layout().args().empty())
 			return text_.getStatus(cur, cmd, status);
 
 		Layout::LaTeXArgMap args = getLayout().args();
 		Layout::LaTeXArgMap::const_iterator const lait = args.find(arg);
 		if (lait != args.end()) {
 			status.setEnabled(true);
-			ParagraphList::const_iterator pit = paragraphs().begin();
-			for (; pit != paragraphs().end(); ++pit) {
-				InsetList::const_iterator it = pit->insetList().begin();
-				InsetList::const_iterator end = pit->insetList().end();
-				for (; it != end; ++it) {
-					if (it->inset->lyxCode() == ARG_CODE) {
-						InsetArgument const * ins =
-							static_cast<InsetArgument const *>(it->inset);
+			for (Paragraph const & par : paragraphs())
+				for (auto const & table : par.insetList())
+					if (InsetArgument const * ins = table.inset->asInsetArgument())
 						if (ins->name() == arg) {
 							// we have this already
 							status.setEnabled(false);
 							return true;
 						}
-					}
-				}
-			}
 		} else
 			status.setEnabled(false);
 		return true;
@@ -374,7 +390,7 @@ bool InsetText::getStatus(Cursor & cur, FuncRequest const & cmd,
 		bool ret = false;
 		if (cur.text() == &text_)
 			ret = text_.getStatus(cur, cmd, status);
-		
+
 		if (!ret)
 			ret = Inset::getStatus(cur, cmd, status);
 		return ret;
@@ -423,8 +439,8 @@ void InsetText::rejectChanges()
 void InsetText::validate(LaTeXFeatures & features) const
 {
 	features.useInsetLayout(getLayout());
-	for_each(paragraphs().begin(), paragraphs().end(),
-		 bind(&Paragraph::validate, _1, ref(features)));
+	for (Paragraph const & p : paragraphs())
+		p.validate(features);
 }
 
 
@@ -432,7 +448,7 @@ void InsetText::latex(otexstream & os, OutputParams const & runparams) const
 {
 	// This implements the standard way of handling the LaTeX
 	// output of a text inset, either a command or an
-	// environment. Standard collapsable insets should not
+	// environment. Standard collapsible insets should not
 	// redefine this, non-standard ones may call this.
 	InsetLayout const & il = getLayout();
 	if (il.forceOwnlines())
@@ -575,7 +591,7 @@ docstring InsetText::xhtml(XHTMLStream & xs, OutputParams const & runparams) con
 // One solution would be to make the footnote <span>, but the problem is
 // completely general, and so we'd have to make absolutely everything into
 // span. What I think will work is to check if we're about to write "div" and,
-// if so, try to close fonts, etc. 
+// if so, try to close fonts, etc.
 // There are probably limits to how well we can do here, though, and we will
 // have to rely upon users not putting footnotes inside noun-type insets.
 docstring InsetText::insetAsXHTML(XHTMLStream & xs, OutputParams const & rp,
@@ -586,11 +602,11 @@ docstring InsetText::insetAsXHTML(XHTMLStream & xs, OutputParams const & rp,
 	OutputParams runparams = rp;
 	runparams.par_begin = 0;
 	runparams.par_end = text().paragraphs().size();
-	
+
 	if (undefined()) {
-		xs.startParagraph(false);
+		xs.startDivision(false);
 		xhtmlParagraphs(text_, buffer(), xs, runparams);
-		xs.endParagraph();
+		xs.endDivision();
 		return docstring();
 	}
 
@@ -604,7 +620,7 @@ docstring InsetText::insetAsXHTML(XHTMLStream & xs, OutputParams const & rp,
 		cntrs.step(il.counter(), OutputUpdate);
 		// FIXME: translate to paragraph language
 		if (!il.htmllabel().empty()) {
-			docstring const lbl = 
+			docstring const lbl =
 				cntrs.counterLabel(from_utf8(il.htmllabel()), bp.language->code());
 			// FIXME is this check necessary?
 			if (!lbl.empty()) {
@@ -624,9 +640,9 @@ docstring InsetText::insetAsXHTML(XHTMLStream & xs, OutputParams const & rp,
 	if (il.isPassThru())
 		runparams.pass_thru = true;
 
-	xs.startParagraph(false);
+	xs.startDivision(false);
 	xhtmlParagraphs(text_, buffer(), xs, runparams);
-	xs.endParagraph();
+	xs.endDivision();
 
 	if (opts & WriteInnerTag)
 		xs << html::EndTag(il.htmlinnertag());
@@ -637,6 +653,7 @@ docstring InsetText::insetAsXHTML(XHTMLStream & xs, OutputParams const & rp,
 	return docstring();
 }
 
+
 void InsetText::getArgs(otexstream & os, OutputParams const & runparams_in,
 			bool const post) const
 {
@@ -646,9 +663,11 @@ void InsetText::getArgs(otexstream & os, OutputParams const & runparams_in,
 	if (isPassThru())
 		runparams.pass_thru = true;
 	if (post)
-		latexArgInsets(paragraphs(), paragraphs().begin(), os, runparams, getLayout().postcommandargs(), "post:");
+		latexArgInsetsForParent(paragraphs(), os, runparams,
+		                        getLayout().postcommandargs(), "post:");
 	else
-		latexArgInsets(paragraphs(), paragraphs().begin(), os, runparams, getLayout().latexargs());
+		latexArgInsetsForParent(paragraphs(), os, runparams,
+		                        getLayout().latexargs());
 }
 
 
@@ -744,8 +763,9 @@ ParagraphList & InsetText::paragraphs()
 bool InsetText::insetAllowed(InsetCode code) const
 {
 	switch (code) {
-	// Arguments are also allowed in PassThru insets
+	// Arguments and (plain) quotes are also allowed in PassThru insets
 	case ARG_CODE:
+	case QUOTE_CODE:
 		return true;
 	default:
 		return !isPassThru();
@@ -805,20 +825,20 @@ void InsetText::forOutliner(docstring & os, size_t const maxlen,
 
 
 void InsetText::addToToc(DocIterator const & cdit, bool output_active,
-						 UpdateType utype) const
+						 UpdateType utype, TocBackend & backend) const
 {
 	DocIterator dit = cdit;
 	dit.push_back(CursorSlice(const_cast<InsetText &>(*this)));
-	iterateForToc(dit, output_active, utype);
+	iterateForToc(dit, output_active, utype, backend);
 }
 
 
 void InsetText::iterateForToc(DocIterator const & cdit, bool output_active,
-							  UpdateType utype) const
+							  UpdateType utype, TocBackend & backend) const
 {
 	DocIterator dit = cdit;
 	// This also ensures that any document has a table of contents
-	shared_ptr<Toc> toc = buffer().tocBackend().toc("tableofcontents");
+	shared_ptr<Toc> toc = backend.toc("tableofcontents");
 
 	BufferParams const & bufparams = buffer_->params();
 	int const min_toclevel = bufparams.documentClass().min_toclevel();
@@ -826,26 +846,50 @@ void InsetText::iterateForToc(DocIterator const & cdit, bool output_active,
 	// can't hurt too much to do it again
 	bool const doing_output = output_active && producesOutput();
 
-	// For each paragraph, traverse its insets and let them add
-	// their toc items
+	// For each paragraph,
+	// * Add a toc item for the paragraph if it is AddToToc--merging adjacent
+	//   paragraphs as needed.
+	// * Traverse its insets and let them add their toc items
+	// * Compute the main table of contents (this is hardcoded)
+	// * Add the list of changes
 	ParagraphList const & pars = paragraphs();
 	pit_type pend = paragraphs().size();
+	// Record pairs {start,end} of where a toc item was opened for a paragraph
+	// and where it must be closed
+	stack<pair<pit_type, pit_type>> addtotoc_stack;
+
 	for (pit_type pit = 0; pit != pend; ++pit) {
 		Paragraph const & par = pars[pit];
 		dit.pit() = pit;
-		// if we find an optarg, we'll save it for use later.
-		InsetText const * arginset = 0;
-		InsetList::const_iterator it  = par.insetList().begin();
-		InsetList::const_iterator end = par.insetList().end();
-		for (; it != end; ++it) {
-			Inset & inset = *it->inset;
-			dit.pos() = it->pos;
-			//lyxerr << (void*)&inset << " code: " << inset.lyxCode() << std::endl;
-			inset.addToToc(dit, doing_output, utype);
-			if (inset.lyxCode() == ARG_CODE)
-				arginset = inset.asInsetText();
+		dit.pos() = 0;
+
+		// Custom AddToToc in paragraph layouts (i.e. theorems)
+		if (par.layout().addToToc() && text().isFirstInSequence(pit)) {
+			pit_type end =
+				openAddToTocForParagraph(pit, dit, output_active, backend);
+			addtotoc_stack.push({pit, end});
 		}
-		// now the toc entry for the paragraph
+
+		// If we find an InsetArgument that is supposed to provide the TOC caption,
+		// we'll save it for use later.
+		InsetArgument const * arginset = nullptr;
+		for (auto const & table : par.insetList()) {
+			dit.pos() = table.pos;
+			table.inset->addToToc(dit, doing_output, utype, backend);
+			if (InsetArgument const * x = table.inset->asInsetArgument())
+				if (x->isTocCaption())
+					arginset = x;
+		}
+
+		// End custom AddToToc in paragraph layouts
+		while (!addtotoc_stack.empty() && addtotoc_stack.top().second == pit) {
+			// execute the closing function
+			closeAddToTocForParagraph(addtotoc_stack.top().first,
+			                          addtotoc_stack.top().second, backend);
+			addtotoc_stack.pop();
+		}
+
+		// now the toc entry for the paragraph in the main table of contents
 		int const toclevel = text().getTocLevel(pit);
 		if (toclevel != Layout::NOT_IN_TOC && toclevel >= min_toclevel) {
 			// insert this into the table of contents
@@ -861,20 +905,47 @@ void InsetText::iterateForToc(DocIterator const & cdit, bool output_active,
 				par.forOutliner(tocstring, length);
 			dit.pos() = 0;
 			toc->push_back(TocItem(dit, toclevel - min_toclevel,
-								  tocstring, doing_output, tocstring));
+			                       tocstring, doing_output));
 		}
-		
+
 		// And now the list of changes.
-		par.addChangesToToc(dit, buffer(), doing_output);
+		par.addChangesToToc(dit, buffer(), doing_output, backend);
 	}
 }
 
 
+pit_type InsetText::openAddToTocForParagraph(pit_type pit,
+                                             DocIterator const & dit,
+                                             bool output_active,
+                                             TocBackend & backend) const
+{
+	Paragraph const & par = paragraphs()[pit];
+	TocBuilder & b = backend.builder(par.layout().tocType());
+	docstring const label = par.labelString();
+	b.pushItem(dit, label + (label.empty() ? "" : " "), output_active);
+	return text().lastInSequence(pit);
+}
+
+
+void InsetText::closeAddToTocForParagraph(pit_type start, pit_type end,
+                                          TocBackend & backend) const
+{
+	Paragraph const & par = paragraphs()[start];
+	TocBuilder & b = backend.builder(par.layout().tocType());
+	if (par.layout().isTocCaption()) {
+		docstring str;
+		text().forOutliner(str, TOC_ENTRY_LENGTH, start, end);
+		b.argumentItem(str);
+	}
+	b.pop();
+}
+
+
 bool InsetText::notifyCursorLeaves(Cursor const & old, Cursor & cur)
 {
 	if (buffer().isClean())
 		return Inset::notifyCursorLeaves(old, cur);
-	
+
 	// find text inset in old cursor
 	Cursor insetCur = old;
 	int scriptSlice	= insetCur.find(this);
@@ -883,10 +954,10 @@ bool InsetText::notifyCursorLeaves(Cursor const & old, Cursor & cur)
 	LASSERT(scriptSlice != -1, return true);
 	insetCur.cutOff(scriptSlice);
 	LASSERT(&insetCur.inset() == this, return true);
-	
+
 	// update the old paragraph's words
 	insetCur.paragraph().updateWords();
-	
+
 	return Inset::notifyCursorLeaves(old, cur);
 }
 
@@ -946,7 +1017,7 @@ bool InsetText::insertCompletion(Cursor & cur, docstring const & s,
 }
 
 
-void InsetText::completionPosAndDim(Cursor const & cur, int & x, int & y, 
+void InsetText::completionPosAndDim(Cursor const & cur, int & x, int & y,
 	Dimension & dim) const
 {
 	TextMetrics const & tm = cur.bv().textMetrics(&text_);
@@ -958,7 +1029,7 @@ string InsetText::contextMenu(BufferView const &, int, int) const
 {
 	string context_menu = contextMenuName();
 	if (context_menu != InsetText::contextMenuName())
-		context_menu += ";" + InsetText::contextMenuName(); 
+		context_menu += ";" + InsetText::contextMenuName();
 	return context_menu;
 }
 
@@ -969,10 +1040,8 @@ string InsetText::contextMenuName() const
 }
 
 
-docstring InsetText::toolTipText(docstring prefix,
-		size_t numlines, size_t len) const
+docstring InsetText::toolTipText(docstring prefix, size_t const len) const
 {
-	size_t const max_length = numlines * len;
 	OutputParams rp(&buffer().params().encoding());
 	rp.for_tooltip = true;
 	odocstringstream oss;
@@ -982,63 +1051,17 @@ docstring InsetText::toolTipText(docstring prefix,
 	ParagraphList::const_iterator end = paragraphs().end();
 	ParagraphList::const_iterator it = beg;
 	bool ref_printed = false;
-	docstring str;
 
 	for (; it != end; ++it) {
 		if (it != beg)
 			oss << '\n';
-		writePlaintextParagraph(buffer(), *it, oss, rp, ref_printed, max_length);
-		str = oss.str();
-		if (str.length() >= max_length)
+		writePlaintextParagraph(buffer(), *it, oss, rp, ref_printed, len);
+		if (oss.tellp() >= 0 && size_t(oss.tellp()) > len)
 			break;
 	}
-	return support::wrapParas(str, 4, len, numlines);
-}
-
-
-InsetCaption const * InsetText::getCaptionInset() const
-{
-	ParagraphList::const_iterator pit = paragraphs().begin();
-	for (; pit != paragraphs().end(); ++pit) {
-		InsetList::const_iterator it = pit->insetList().begin();
-		for (; it != pit->insetList().end(); ++it) {
-			Inset & inset = *it->inset;
-			if (inset.lyxCode() == CAPTION_CODE) {
-				InsetCaption const * ins =
-					static_cast<InsetCaption const *>(it->inset);
-				return ins;
-			}
-		}
-	}
-	return 0;
-}
-
-
-docstring InsetText::getCaptionText(OutputParams const & runparams) const
-{
-	InsetCaption const * ins = getCaptionInset();
-	if (ins == 0)
-		return docstring();
-
-	odocstringstream ods;
-	ins->getCaptionAsPlaintext(ods, runparams);
-	return ods.str();
-}
-
-
-docstring InsetText::getCaptionHTML(OutputParams const & runparams) const
-{
-	InsetCaption const * ins = getCaptionInset();
-	if (ins == 0)
-		return docstring();
-
-	odocstringstream ods;
-	XHTMLStream xs(ods);
-	docstring def = ins->getCaptionAsHTML(xs, runparams);
-	if (!def.empty())
-		// should already have been escaped
-		xs << XHTMLStream::ESCAPE_NONE << def << '\n';
-	return ods.str();
+	docstring str = oss.str();
+	support::truncateWithEllipsis(str, len);
+	return str;
 }
 
 
