@@ -13,6 +13,7 @@
 #define MATH_NESTINSET_H
 
 #include "InsetMath.h"
+#include "MathData.h"
 
 #include <map>
 
@@ -32,24 +33,10 @@ public:
 	///
 	void setBuffer(Buffer &);
 
-	// The method below hides inset::metrics() intentionally!
-	// We have to tell clang not to be fussy about that.
-#ifdef __clang__
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Woverloaded-virtual"
-#endif
-	/// the size is usually some sort of convex hull of the cells
-	void metrics(MetricsInfo const & mi) const;
-#ifdef __clang__
-#pragma clang diagnostic pop
-#endif
+	/// Update the cells metrics
+	void cellsMetrics(MetricsInfo const & mi) const;
 	/// draw background if locked
 	void draw(PainterInfo & pi, int x, int y) const;
-	/// draw selection background
-	void drawSelection(PainterInfo & pi, int x, int y) const;
-	/// draw decorations.
-	void drawDecoration(PainterInfo & pi, int x, int y) const
-	{ drawMarkers(pi, x, y); }
 	///
 	void updateBuffer(ParIterator const &, UpdateType);
 	/// identifies NestInsets
@@ -60,7 +47,7 @@ public:
 	void cursorPos(BufferView const & bv, CursorSlice const & sl,
 		bool boundary, int & x, int & y) const;
 	///
-	void edit(Cursor & cur, bool front, 
+	void edit(Cursor & cur, bool front,
 		EntryDirection entry_from = ENTRY_DIRECTION_IGNORE);
 	///
 	Inset * editXY(Cursor & cur, int x, int y);
@@ -120,7 +107,7 @@ public:
 	///
 	bool setMouseHover(BufferView const * bv, bool mouse_hover) const;
 	///
-	bool mouseHovered(BufferView const * bv) const 
+	bool mouseHovered(BufferView const * bv) const
 		{ return mouse_hover_[bv]; }
 
 	///
@@ -142,6 +129,9 @@ public:
 	///
 	InsetCode lyxCode() const { return MATH_NEST_CODE; }
 
+	///
+	bool confirmDeletion() const { return nargs() > 0; }
+
 protected:
 	///
 	InsetMathNest(InsetMathNest const & inset);
@@ -190,9 +180,9 @@ private:
 	/// afterwards if found
 	bool findMacroToFoldUnfold(Cursor & searchCur, bool fold) const;
 	/// move cursor forward
-	bool cursorMathForward(Cursor & cur);
+	bool cursorMathForward(Cursor & cur, bool enter = true);
 	/// move cursor backwards
-	bool cursorMathBackward(Cursor & cur);
+	bool cursorMathBackward(Cursor & cur, bool enter = true);
 
 protected:
 	/// we store the cells in a vector
@@ -203,7 +193,7 @@ protected:
 	bool lock_;
 	///
 	mutable std::map<BufferView const *, bool> mouse_hover_;
-};	
+};
 
 
 
