@@ -21,6 +21,7 @@
 #include "FuncRequest.h"
 #include "FuncStatus.h"
 #include "LaTeXFeatures.h"
+#include "MetricsInfo.h"
 
 #include "insets/InsetSpace.h"
 
@@ -74,7 +75,7 @@ SpaceInfo space_info[] = {
 int const nSpace = sizeof(space_info)/sizeof(SpaceInfo);
 int const defaultSpace = 4;
 
-} // anon namespace
+} // namespace
 
 InsetMathSpace::InsetMathSpace()
 	: space_(defaultSpace)
@@ -120,6 +121,7 @@ Inset * InsetMathSpace::clone() const
 
 void InsetMathSpace::metrics(MetricsInfo & mi, Dimension & dim) const
 {
+	Changer dummy = mi.base.changeEnsureMath();
 	dim.asc = 4;
 	dim.des = 0;
 	if (space_info[space_].custom)
@@ -131,6 +133,7 @@ void InsetMathSpace::metrics(MetricsInfo & mi, Dimension & dim) const
 
 void InsetMathSpace::draw(PainterInfo & pi, int x, int y) const
 {
+	Changer dummy = pi.base.changeEnsureMath();
 	// Sadly, HP-UX CC can't handle that kind of initialization.
 	// XPoint p[4] = {{++x, y-3}, {x, y}, {x+width-2, y}, {x+width-2, y-3}};
 	if (!space_info[space_].visible)
@@ -205,14 +208,14 @@ void InsetMathSpace::mathmlize(MathStream & ms) const
 		ss << si.width;
 		l = ss.str() + "px";
 	}
-	
+
 	ms << "<mspace";
 	if (!l.empty())
 		ms << " width=\"" << from_ascii(l) << "\"";
 	ms << " />";
 }
 
-	
+
 void InsetMathSpace::htmlize(HtmlStream & ms) const
 {
 	SpaceInfo const & si = space_info[space_];
@@ -243,7 +246,7 @@ void InsetMathSpace::htmlize(HtmlStream & ms) const
 	case InsetSpaceParams::CUSTOM:
 	case InsetSpaceParams::CUSTOM_PROTECTED: {
 		string l = length_.asHTMLString();
-		ms << MTag("span", "width='" + l + "'") 
+		ms << MTag("span", "width='" + l + "'")
 		   << from_ascii("&nbsp;") << ETag("span");
 		break;
 	}
@@ -256,7 +259,7 @@ void InsetMathSpace::htmlize(HtmlStream & ms) const
 	}
 }
 
-	
+
 void InsetMathSpace::normalize(NormalStream & os) const
 {
 	os << "[space " << int(space_) << "] ";
@@ -265,7 +268,9 @@ void InsetMathSpace::normalize(NormalStream & os) const
 
 void InsetMathSpace::write(WriteStream & os) const
 {
-	// no MathEnsurer - all kinds work in text and math mode
+	// All kinds work in text and math mode, so simply suspend
+	// writing a possibly pending mode closing brace.
+	MathEnsurer ensurer(os, false);
 	if (space_info[space_].escape)
 		os << '\\';
 	os << space_info[space_].name.c_str();
@@ -300,8 +305,6 @@ bool InsetMathSpace::getStatus(Cursor & cur, FuncRequest const & cmd,
 	case LFUN_INSET_MODIFY:
 	case LFUN_INSET_DIALOG_UPDATE:
 	case LFUN_MOUSE_RELEASE:
-	case LFUN_MOUSE_PRESS:
-	case LFUN_MOUSE_MOTION:
 		status.setEnabled(true);
 		return true;
 	default:
@@ -327,18 +330,13 @@ void InsetMathSpace::doDispatch(Cursor & cur, FuncRequest & cmd)
 		break;
 
 	case LFUN_MOUSE_RELEASE:
-		if (cmd.button() == mouse_button::button1) {
+		if (cmd.button() == mouse_button::button1 && !cur.selection()) {
 			showInsetDialog(&cur.bv());
 			break;
 		}
 		cur.undispatched();
 		break;
 
-	case LFUN_MOUSE_PRESS:
-	case LFUN_MOUSE_MOTION:
-		// eat other mouse commands
-		break;
-
 	default:
 		InsetMath::doDispatch(cur, cmd);
 		break;
