@@ -12,10 +12,14 @@
 
 #include "InsetMathSqrt.h"
 
-#include "LaTeXFeatures.h"
 #include "MathData.h"
 #include "MathStream.h"
+#include "MathSupport.h"
+
+#include "LaTeXFeatures.h"
+#include "MetricsInfo.h"
 #include "TextPainter.h"
+
 #include "frontends/Painter.h"
 
 
@@ -34,29 +38,33 @@ Inset * InsetMathSqrt::clone() const
 
 void InsetMathSqrt::metrics(MetricsInfo & mi, Dimension & dim) const
 {
+	Changer dummy = mi.base.changeEnsureMath();
 	cell(0).metrics(mi, dim);
-	dim.asc += 4;
-	dim.des += 2;
-	dim.wid += 12;
-	metricsMarkers(dim);
+	// make sure that the dim is high enough for any character
+	Dimension fontDim;
+	math_font_max_dim(mi.base.font, fontDim.asc, fontDim.des);
+	dim += fontDim;
+	// Some room for the decoration
+	dim.asc += 1;
+	dim.wid += 7;
 }
 
 
 void InsetMathSqrt::draw(PainterInfo & pi, int x, int y) const
 {
-	cell(0).draw(pi, x + 10, y);
+	Changer dummy = pi.base.changeEnsureMath();
+	cell(0).draw(pi, x + 9, y);
 	Dimension const dim = dimension(*pi.base.bv);
 	int const a = dim.ascent();
 	int const d = dim.descent();
 	int xp[3];
 	int yp[3];
 	pi.pain.line(x + dim.width(), y - a + 1,
-		x + 8, y - a + 1, pi.base.font.color());
-	xp[0] = x + 8;            yp[0] = y - a + 1;
-	xp[1] = x + 5;            yp[1] = y + d - 1;
+		x + 7, y - a + 1, pi.base.font.color());
+	xp[0] = x + 7;            yp[0] = y - a + 1;
+	xp[1] = x + 4;            yp[1] = y + d - 1;
 	xp[2] = x;                yp[2] = y + (d - a)/2;
 	pi.pain.lines(xp, yp, 3, pi.base.font.color());
-	drawMarkers(pi, x, y);
 }
 
 
@@ -118,8 +126,8 @@ void InsetMathSqrt::mathmlize(MathStream & os) const
 void InsetMathSqrt::htmlize(HtmlStream & os) const
 {
 	os << MTag("span", "class='sqrt'")
-	   << from_ascii("&radic;") 
-	   << MTag("span", "class='sqrtof'")	<< cell(0) << ETag("span") 
+	   << from_ascii("&radic;")
+	   << MTag("span", "class='sqrtof'")	<< cell(0) << ETag("span")
 		 << ETag("span");
 }
 
