@@ -15,16 +15,23 @@
 
 #include "InsetMathFont.h"
 #include "InsetMathSymbol.h"
+#include "Length.h"
 #include "MathData.h"
+#include "MathFactory.h"
 #include "MathParser.h"
 #include "MathStream.h"
 
+#include "LaTeXFeatures.h"
+#include "MetricsInfo.h"
+
 #include "frontends/FontLoader.h"
 #include "frontends/FontMetrics.h"
 #include "frontends/Painter.h"
 
 #include "support/debug.h"
 #include "support/docstream.h"
+#include "support/lassert.h"
+#include "support/lyxlib.h"
 
 #include <map>
 #include <algorithm>
@@ -498,7 +505,7 @@ deco_struct const * search_deco(docstring const & name)
 }
 
 
-} // namespace anon
+} // namespace
 
 
 int mathed_font_em(FontInfo const & font)
@@ -506,6 +513,39 @@ int mathed_font_em(FontInfo const & font)
 	return theFontMetrics(font).em();
 }
 
+/* The math units. Quoting TeX by Topic, p.205:
+ *
+ * Spacing around mathematical objects is measured in mu units. A mu
+ * is 1/18th part of \fontdimen6 of the font in family 2 in the
+ * current style, the ‘quad’ value of the symbol font.
+ *
+ * A \thickmuskip (default value in plain TeX: 5mu plus 5mu) is
+ * inserted around (binary) relations, except where these are preceded
+ * or followed by other relations or punctuation, and except if they
+ * follow an open, or precede a close symbol.
+ *
+ * A \medmuskip (default value in plain TeX: 4mu plus 2mu minus 4mu)
+ * is put around binary operators.
+ *
+ * A \thinmuskip (default value in plain TeX: 3mu) follows after
+ * punctuation, and is put around inner objects, except where these
+ * are followed by a close or preceded by an open symbol, and except
+ * if the other object is a large operator or a binary relation.
+ *
+ * See the file MathClass.cpp for a formal implementation of the rules
+ * above.
+ */
+
+int mathed_mu(FontInfo const & font, double mu)
+{
+	MetricsBase mb(nullptr, font);
+	return Length(mu, Length::MU).inPixels(mb);
+}
+
+int mathed_thinmuskip(FontInfo const & font) { return mathed_mu(font, 3.0); }
+int mathed_medmuskip(FontInfo const & font) { return mathed_mu(font, 4.0); }
+int mathed_thickmuskip(FontInfo const & font) { return mathed_mu(font, 5.0); }
+
 
 int mathed_char_width(FontInfo const & font, char_type c)
 {
@@ -516,7 +556,7 @@ int mathed_char_width(FontInfo const & font, char_type c)
 int mathed_char_kerning(FontInfo const & font, char_type c)
 {
 	frontend::FontMetrics const & fm = theFontMetrics(font);
-	return fm.rbearing(c) - fm.width(c);
+	return max(0, fm.rbearing(c) - fm.width(c));
 }
 
 
@@ -613,10 +653,45 @@ void mathed_draw_deco(PainterInfo & pi, int x, int y, int w, int h,
 }
 
 
+void mathedSymbolDim(MetricsBase & mb, Dimension & dim, latexkeys const * sym)
+{
+	LASSERT((bool)sym, return);
+	//lyxerr << "metrics: symbol: '" << sym->name
+	//	<< "' in font: '" << sym->inset
+	//	<< "' drawn as: '" << sym->draw
+	//	<< "'" << endl;
+
+	bool const italic_upcase_greek = sym->inset == "cmr" &&
+		sym->extra == "mathalpha" &&
+		mb.fontname == "mathit";
+	std::string const font = italic_upcase_greek ? "cmm" : sym->inset;
+	Changer dummy = mb.changeFontSet(font);
+	mathed_string_dim(mb.font, sym->draw, dim);
+}
+
+
+void mathedSymbolDraw(PainterInfo & pi, int x, int y, latexkeys const * sym)
+{
+	LASSERT((bool)sym, return);
+	//lyxerr << "drawing: symbol: '" << sym->name
+	//	<< "' in font: '" << sym->inset
+	//	<< "' drawn as: '" << sym->draw
+	//	<< "'" << endl;
+
+	bool const italic_upcase_greek = sym->inset == "cmr" &&
+		sym->extra == "mathalpha" &&
+		pi.base.fontname == "mathit";
+	std::string const font = italic_upcase_greek ? "cmm" : sym->inset;
+
+	Changer dummy = pi.base.changeFontSet(font);
+	pi.draw(x, y, sym->draw);
+}
+
+
 void metricsStrRedBlack(MetricsInfo & mi, Dimension & dim, docstring const & str)
 {
 	FontInfo font = mi.base.font;
-	augmentFont(font, from_ascii("mathnormal"));
+	augmentFont(font, "mathnormal");
 	mathed_string_dim(font, str, dim);
 }
 
@@ -624,7 +699,7 @@ void metricsStrRedBlack(MetricsInfo & mi, Dimension & dim, docstring const & str
 void drawStrRed(PainterInfo & pi, int x, int y, docstring const & str)
 {
 	FontInfo f = pi.base.font;
-	augmentFont(f, from_ascii("mathnormal"));
+	augmentFont(f, "mathnormal");
 	f.setColor(Color_latex);
 	pi.pain.text(x, y, str, f);
 }
@@ -633,7 +708,7 @@ void drawStrRed(PainterInfo & pi, int x, int y, docstring const & str)
 void drawStrBlack(PainterInfo & pi, int x, int y, docstring const & str)
 {
 	FontInfo f = pi.base.font;
-	augmentFont(f, from_ascii("mathnormal"));
+	augmentFont(f, "mathnormal");
 	f.setColor(Color_foreground);
 	pi.pain.text(x, y, str, f);
 }
@@ -665,6 +740,7 @@ FontShape  const inh_shape  = INHERIT_SHAPE;
 // does not work
 fontinfo fontinfos[] = {
 	// math fonts
+	// Color_math determines which fonts are math (see isMathFont)
 	{"mathnormal",    ROMAN_FAMILY, MEDIUM_SERIES,
 			  ITALIC_SHAPE, Color_math},
 	{"mathbf",        inh_family, BOLD_SERIES,
@@ -763,11 +839,10 @@ fontinfo fontinfos[] = {
 };
 
 
-fontinfo * lookupFont(docstring const & name0)
+fontinfo * lookupFont(string const & name)
 {
 	//lyxerr << "searching font '" << name << "'" << endl;
 	int const n = sizeof(fontinfos) / sizeof(fontinfo);
-	string name = to_utf8(name0);
 	for (int i = 0; i < n; ++i)
 		if (fontinfos[i].cmd_ == name) {
 			//lyxerr << "found '" << i << "'" << endl;
@@ -777,7 +852,7 @@ fontinfo * lookupFont(docstring const & name0)
 }
 
 
-fontinfo * searchFont(docstring const & name)
+fontinfo * searchFont(string const & name)
 {
 	fontinfo * f = lookupFont(name);
 	return f ? f : fontinfos;
@@ -786,27 +861,27 @@ fontinfo * searchFont(docstring const & name)
 }
 
 
-bool isFontName(docstring const & name)
+bool isFontName(string const & name)
 {
 	return lookupFont(name);
 }
 
 
-bool isMathFont(docstring const & name)
+bool isMathFont(string const & name)
 {
 	fontinfo * f = lookupFont(name);
 	return f && f->color_ == Color_math;
 }
 
 
-bool isTextFont(docstring const & name)
+bool isTextFont(string const & name)
 {
 	fontinfo * f = lookupFont(name);
 	return f && f->color_ == Color_foreground;
 }
 
 
-FontInfo getFont(docstring const & name)
+FontInfo getFont(string const & name)
 {
 	FontInfo font;
 	augmentFont(font, name);
@@ -814,7 +889,7 @@ FontInfo getFont(docstring const & name)
 }
 
 
-void fakeFont(docstring const & orig, docstring const & fake)
+void fakeFont(string const & orig, string const & fake)
 {
 	fontinfo * forig = searchFont(orig);
 	fontinfo * ffake = searchFont(fake);
@@ -824,22 +899,22 @@ void fakeFont(docstring const & orig, docstring const & fake)
 		forig->shape_  = ffake->shape_;
 		forig->color_  = ffake->color_;
 	} else {
-		lyxerr << "Can't fake font '" << to_utf8(orig) << "' with '"
-		       << to_utf8(fake) << "'" << endl;
+		lyxerr << "Can't fake font '" << orig << "' with '"
+		       << fake << "'" << endl;
 	}
 }
 
 
-void augmentFont(FontInfo & font, docstring const & name)
+void augmentFont(FontInfo & font, string const & name)
 {
 	static bool initialized = false;
 	if (!initialized) {
 		initialized = true;
 		// fake fonts if necessary
-		if (!theFontLoader().available(getFont(from_ascii("mathfrak"))))
-			fakeFont(from_ascii("mathfrak"), from_ascii("lyxfakefrak"));
-		if (!theFontLoader().available(getFont(from_ascii("mathcal"))))
-			fakeFont(from_ascii("mathcal"), from_ascii("lyxfakecal"));
+		if (!theFontLoader().available(getFont("mathfrak")))
+			fakeFont("mathfrak", "lyxfakefrak");
+		if (!theFontLoader().available(getFont("mathcal")))
+			fakeFont("mathcal", "lyxfakecal");
 	}
 	fontinfo * info = searchFont(name);
 	if (info->family_ != inh_family)
@@ -877,8 +952,7 @@ bool isAlphaSymbol(MathAtom const & at)
 docstring asString(MathData const & ar)
 {
 	odocstringstream os;
-	TexRow texrow(false);
-	otexrowstream ots(os,texrow);
+	otexrowstream ots(os);
 	WriteStream ws(ots);
 	ws << ar;
 	return os.str();
@@ -887,8 +961,13 @@ docstring asString(MathData const & ar)
 
 void asArray(docstring const & str, MathData & ar, Parse::flags pf)
 {
+	// If the QUIET flag is set, we are going to parse for either
+	// a paste operation or a macro definition. We try to do the
+	// right thing in all cases.
+
 	bool quiet = pf & Parse::QUIET;
-	if ((str.size() == 1 && quiet) || (!mathed_parse_cell(ar, str, pf) && quiet))
+	bool macro = pf & Parse::MACRODEF;
+	if ((str.size() == 1 && quiet) || (!mathed_parse_cell(ar, str, pf) && quiet && !macro))
 		mathed_parse_cell(ar, str, pf | Parse::VERBATIM);
 }
 
@@ -896,8 +975,7 @@ void asArray(docstring const & str, MathData & ar, Parse::flags pf)
 docstring asString(InsetMath const & inset)
 {
 	odocstringstream os;
-	TexRow texrow(false);
-	otexrowstream ots(os,texrow);
+	otexrowstream ots(os);
 	WriteStream ws(ots);
 	inset.write(ws);
 	return os.str();
@@ -907,12 +985,30 @@ docstring asString(InsetMath const & inset)
 docstring asString(MathAtom const & at)
 {
 	odocstringstream os;
-	TexRow texrow(false);
-	otexrowstream ots(os,texrow);
+	otexrowstream ots(os);
 	WriteStream ws(ots);
 	at->write(ws);
 	return os.str();
 }
 
 
+int axis_height(MetricsBase & mb)
+{
+	Changer dummy = mb.changeFontSet("mathnormal");
+	return theFontMetrics(mb.font).ascent('-') - 1;
+}
+
+
+void validate_math_word(LaTeXFeatures & features, docstring const & word)
+{
+	MathWordList const & words = mathedWordList();
+	MathWordList::const_iterator it = words.find(word);
+	if (it != words.end()) {
+		string const req = it->second.requires;
+		if (!req.empty())
+			features.require(req);
+	}
+}
+
+
 } // namespace lyx
