@@ -18,6 +18,7 @@
 
 namespace lyx {
 
+class MetricsBase;
 class MetricsInfo;
 class PainterInfo;
 class FontInfo;
@@ -25,10 +26,20 @@ class Dimension;
 class MathData;
 class MathAtom;
 class InsetMath;
+class latexkeys;
+class LaTeXFeatures;
 
 
 int mathed_font_em(FontInfo const &);
 
+int mathed_mu(FontInfo const & font, double mu);
+
+int mathed_thinmuskip(FontInfo const & font);
+
+int mathed_medmuskip(FontInfo const & font);
+
+int mathed_thickmuskip(FontInfo const & font);
+
 int mathed_char_width(FontInfo const &, char_type c);
 
 int mathed_char_kerning(FontInfo const &, char_type c);
@@ -42,6 +53,10 @@ void mathed_string_dim(FontInfo const & font,
 
 int mathed_string_width(FontInfo const &, docstring const & s);
 
+void mathedSymbolDim(MetricsBase & mb, Dimension & dim, latexkeys const * sym);
+
+void mathedSymbolDraw(PainterInfo & pi, int x, int y, latexkeys const * sym);
+
 void metricsStrRedBlack(MetricsInfo & mi, Dimension & dim, docstring const & s);
 
 void drawStrRed(PainterInfo & pi, int x, int y, docstring const & s);
@@ -49,13 +64,13 @@ void drawStrBlack(PainterInfo & pi, int x, int y, docstring const & s);
 
 void math_font_max_dim(FontInfo const &, int & asc, int & desc);
 
-void augmentFont(FontInfo & f, docstring const & cmd);
+void augmentFont(FontInfo & f, std::string const & cmd);
 
-bool isFontName(docstring const & name);
+bool isFontName(std::string const & name);
 
-bool isMathFont(docstring const & name);
+bool isMathFont(std::string const & name);
 
-bool isTextFont(docstring const & name);
+bool isTextFont(std::string const & name);
 
 bool isAlphaSymbol(MathAtom const & at);
 
@@ -67,6 +82,11 @@ docstring asString(MathAtom const &);
 // converts string to single cell
 void asArray(docstring const &, MathData &, Parse::flags f = Parse::NORMAL);
 
+// simulate axis height (font dimension sigma 22)
+int axis_height(MetricsBase & mb);
+
+void validate_math_word(LaTeXFeatures & features, docstring const & word);
+
 } // namespace lyx
 
 #endif
