@@ -23,6 +23,7 @@
 #include "support/os.h"
 #include "support/ProgressInterface.h"
 
+#include "LyX.h"
 #include "LyXRC.h"
 
 #include <cstdlib>
@@ -39,7 +40,7 @@
 
 struct Sleep : QThread
 {
-	static void millisec(unsigned long ms) 
+	static void millisec(unsigned long ms)
 	{
 		QThread::usleep(ms * 1000);
 	}
@@ -54,7 +55,7 @@ namespace lyx {
 namespace support {
 
 
-class ProgressDummy : public ProgressInterface 
+class ProgressDummy : public ProgressInterface
 {
 public:
 	ProgressDummy() {}
@@ -230,8 +231,7 @@ string const parsecmd(string const & incmd, string & infile, string & outfile,
 	return trim(outcmd[0]);
 }
 
-} // namespace anon
-
+} // namespace
 
 
 int Systemcall::startscript(Starttype how, string const & what,
@@ -239,7 +239,10 @@ int Systemcall::startscript(Starttype how, string const & what,
 			    bool process_events)
 {
 	string const what_ss = commandPrep(what);
-	LYXERR(Debug::INFO,"Running: " << what_ss);
+	if (verbose)
+		lyxerr << "\nRunning: " << what_ss << endl;
+	else
+		LYXERR(Debug::INFO,"Running: " << what_ss);
 
 	string infile;
 	string outfile;
@@ -252,8 +255,8 @@ int Systemcall::startscript(Starttype how, string const & what,
 #ifdef Q_OS_WIN32
 	// QProcess::startDetached cannot provide environment variables. When the
 	// environment variables are set using the latexEnvCmdPrefix and the process
-	// is started with QProcess::startDetached, a console window is shown every 
-	// time a viewer is started. To avoid this, we fall back on Windows to the 
+	// is started with QProcess::startDetached, a console window is shown every
+	// time a viewer is started. To avoid this, we fall back on Windows to the
 	// original implementation that creates a QProcess object.
 	d.startProcess(cmd, path, lpath, false);
 	if (!d.waitWhile(SystemcallPrivate::Starting, process_events, -1)) {
@@ -295,16 +298,10 @@ int Systemcall::startscript(Starttype how, string const & what,
 }
 
 
-SystemcallPrivate::SystemcallPrivate(std::string const & sf,
-				     std::string const & of,
-				     std::string const & ef) :
-				process_(new QProcess), 
-				out_index_(0),
-				err_index_(0),
-				in_file_(sf),
-				out_file_(of), 
-				err_file_(ef), 
-				process_events_(false)
+SystemcallPrivate::SystemcallPrivate(std::string const & sf, std::string const & of,
+                                     std::string const & ef)
+	: state(Error), process_(new QProcess), out_index_(0), err_index_(0),
+	  in_file_(sf), out_file_(of), err_file_(ef), process_events_(false)
 {
 	if (!in_file_.empty())
 		process_->setStandardInputFile(QString::fromLocal8Bit(in_file_.c_str()));
@@ -412,7 +409,7 @@ bool queryStopCommand(QString const & cmd)
 			1, 1, _("&Stop it"), _("Let it &run")) == 0;
 }
 
-}
+} // namespace
 
 
 bool SystemcallPrivate::waitWhile(State waitwhile, bool process_events, int timeout)
@@ -455,7 +452,7 @@ bool SystemcallPrivate::waitWhile(State waitwhile, bool process_events, int time
 			waitAndProcessEvents();
 		}
 		return state != Error;
-	} 
+	}
 
 	// process events while waiting with timeout
 	QTime timer;
@@ -558,7 +555,7 @@ void SystemcallPrivate::processError(QProcess::ProcessError)
 }
 
 
-QString SystemcallPrivate::errorMessage() const 
+QString SystemcallPrivate::errorMessage() const
 {
 	if (!process_)
 		return "No QProcess available";
