@@ -21,6 +21,7 @@
 
 #include <config.h>
 
+#include "LyX.h"
 #include "LyXRC.h"
 
 #include "support/filetools.h"
@@ -51,6 +52,7 @@
 #endif
 
 #include <cerrno>
+#include <climits>
 #include <cstdlib>
 #include <cstdio>
 
@@ -643,35 +645,6 @@ string const onlyFileName(string const & fname)
 }
 
 
-// Create absolute path. If impossible, don't do anything
-// Supports ./ and ~/. Later we can add support for ~logname/. (Asger)
-string const expandPath(string const & path)
-{
-	// checks for already absolute path
-	string rTemp = replaceEnvironmentPath(path);
-	if (FileName::isAbsolute(rTemp))
-		return rTemp;
-
-	string temp;
-	string const copy = rTemp;
-
-	// Split by next /
-	rTemp = split(rTemp, temp, '/');
-
-	if (temp == ".")
-		return FileName::getcwd().absFileName() + '/' + rTemp;
-
-	if (temp == "~")
-		return Package::get_home_dir().absFileName() + '/' + rTemp;
-
-	if (temp == "..")
-		return makeAbsPath(copy).absFileName();
-
-	// Don't know how to handle this
-	return copy;
-}
-
-
 // Search the string for ${VAR} and $VAR and replace VAR using getenv.
 string const replaceEnvironmentPath(string const & path)
 {
@@ -1023,6 +996,11 @@ cmd_ret const runCommand(string const & cmd)
 	// pstream (process stream), with the
 	// variants ipstream, opstream
 
+	if (verbose)
+		lyxerr << "\nRunning: " << cmd << endl;
+	else
+		LYXERR(Debug::INFO,"Running: " << cmd);
+
 #if defined (_WIN32)
 	STARTUPINFO startup;
 	PROCESS_INFORMATION process;
@@ -1065,7 +1043,7 @@ cmd_ret const runCommand(string const & cmd)
 				0, 0, &startup, &process)) {
 
 			CloseHandle(process.hThread);
-			int fno = _open_osfhandle((long)in, _O_RDONLY);
+			int fno = _open_osfhandle((intptr_t)in, _O_RDONLY);
 			CloseHandle(out);
 			inf = _fdopen(fno, "r");
 		}
