@@ -79,7 +79,7 @@ inline QChar const ucs4_to_qchar(char_type const ucs4)
 
 /// Maximum valid UCS4 code point
 char_type const ucs4_max = 0x10ffff;
-} // anon namespace
+} // namespace
 
 
 bool isLetterChar(char_type c)
@@ -182,6 +182,18 @@ bool isASCII(char_type c)
 }
 
 
+bool isOpenPunctuation(char_type c)
+{
+	if (!is_utf16(c)) {
+		// assume that no non-utf16 character is an op
+		// c outside the UCS4 range is catched as well
+		return false;
+	}
+	QChar const qc = ucs4_to_qchar(c);
+	return qc.category() == QChar::Punctuation_Open;
+}
+
+
 namespace support {
 
 int compare_no_case(docstring const & s, docstring const & s2)
@@ -208,24 +220,7 @@ int compare_no_case(docstring const & s, docstring const & s2)
 
 int compare_locale(docstring const & s, docstring const & s2)
 {
-	// FIXME We have a report that this does not work on windows (bug 9030)
-	try
-	{
-		string const l = to_local8bit(s);
-		string const r = to_local8bit(s2);
-		return strcoll(l.c_str(), r.c_str());
-	}
-	catch (bad_cast & e)
-	{
-		// fall back to builtin sorting
-		LYXERR0("Could not compare using the current locale: "
-		        << e.what() << ", using fallback.");
-		if (s < s2)
-			return -1;
-		if (s > s2)
-			return 1;
-		return 0;
-	}
+	return QString::localeAwareCompare(toqstr(s), toqstr(s2));
 }
 
 
@@ -238,7 +233,7 @@ Char ascii_tolower(Char c) {
 	return c;
 }
 
-}
+} // namespace
 
 
 int compare_ascii_no_case(string const & s, string const & s2)
@@ -499,7 +494,7 @@ template<typename Char> struct local_ascii_lowercase {
 	Char operator()(Char c) const { return ascii_tolower(c); }
 };
 
-} // end of anon namespace
+} // namespace
 
 
 docstring const lowercase(docstring const & a)
@@ -734,6 +729,12 @@ bool containsOnly(string const & s, string const & cset)
 }
 
 
+bool containsOnly(docstring const & s, string const & cset)
+{
+	return s.find_first_not_of(from_ascii(cset)) == string::npos;
+}
+
+
 // ale970405+lasgoutt-970425
 // rewritten to use new string (Lgb)
 string const token(string const & a, char delim, int n)
@@ -891,7 +892,7 @@ docstring const subst_string(docstring const & a,
 	return lstr;
 }
 
-}
+} // namespace
 
 
 string const subst(string const & a, char oldchar, char newchar)
@@ -933,7 +934,7 @@ int count_char(string const & str, char chr)
 }
 
 
-/// Count all occurences of char \a chr inside \a str
+/// Count all occurrences of char \a chr inside \a str
 int count_char(docstring const & str, docstring::value_type chr)
 {
 	int count = 0;
@@ -1107,7 +1108,7 @@ docstring const doSplit(docstring const & a, docstring & piece, Char delim)
 	return tmp;
 }
 
-} // anon
+} // namespace
 
 
 string const split(string const & a, string & piece, char delim)
@@ -1202,6 +1203,16 @@ docstring const escape(docstring const & lab)
 }
 
 
+docstring const protectArgument(docstring & arg, char const l,
+			  char const r)
+{
+	if (contains(arg, l) || contains(arg, r))
+		// protect brackets
+		arg = '{' + arg + '}';
+	return arg;
+}
+
+
 bool truncateWithEllipsis(docstring & str, size_t const len)
 {
 	if (str.size() <= len)
@@ -1248,7 +1259,7 @@ vector<docstring> wrapToVec(docstring const & str, int ind,
 	return retval;
 }
 
-}
+} // namespace
 
 
 docstring wrap(docstring const & str, int const ind, size_t const width)
@@ -1356,7 +1367,7 @@ template<typename String> const String
 	return str;
 }
 
-} // namespace anon
+} // namespace
 
 
 vector<string> const getVectorFromString(string const & str,
@@ -1420,7 +1431,6 @@ std::string formatFPNumber(double x)
 }
 
 
-template<>
 docstring bformat(docstring const & fmt, int arg1)
 {
 	LATTEST(contains(fmt, from_ascii("%1$d")));
@@ -1429,7 +1439,6 @@ docstring bformat(docstring const & fmt, int arg1)
 }
 
 
-template<>
 docstring bformat(docstring const & fmt, long arg1)
 {
 	LATTEST(contains(fmt, from_ascii("%1$d")));
@@ -1438,7 +1447,16 @@ docstring bformat(docstring const & fmt, long arg1)
 }
 
 
-template<>
+#ifdef LYX_USE_LONG_LONG
+docstring bformat(docstring const & fmt, long long arg1)
+{
+	LATTEST(contains(fmt, from_ascii("%1$d")));
+	docstring const str = subst(fmt, from_ascii("%1$d"), convert<docstring>(arg1));
+	return subst(str, from_ascii("%%"), from_ascii("%"));
+}
+#endif
+
+
 docstring bformat(docstring const & fmt, unsigned int arg1)
 {
 	LATTEST(contains(fmt, from_ascii("%1$d")));
@@ -1447,8 +1465,7 @@ docstring bformat(docstring const & fmt, unsigned int arg1)
 }
 
 
-template<>
-docstring bformat(docstring const & fmt, docstring arg1)
+docstring bformat(docstring const & fmt, docstring const & arg1)
 {
 	LATTEST(contains(fmt, from_ascii("%1$s")));
 	docstring const str = subst(fmt, from_ascii("%1$s"), arg1);
@@ -1456,7 +1473,6 @@ docstring bformat(docstring const & fmt, docstring arg1)
 }
 
 
-template<>
 docstring bformat(docstring const & fmt, char * arg1)
 {
 	LATTEST(contains(fmt, from_ascii("%1$s")));
@@ -1465,8 +1481,7 @@ docstring bformat(docstring const & fmt, char * arg1)
 }
 
 
-template<>
-docstring bformat(docstring const & fmt, docstring arg1, docstring arg2)
+docstring bformat(docstring const & fmt, docstring const & arg1, docstring const & arg2)
 {
 	LATTEST(contains(fmt, from_ascii("%1$s")));
 	LATTEST(contains(fmt, from_ascii("%2$s")));
@@ -1476,8 +1491,7 @@ docstring bformat(docstring const & fmt, docstring arg1, docstring arg2)
 }
 
 
-template<>
-docstring bformat(docstring const & fmt, docstring arg1, int arg2)
+docstring bformat(docstring const & fmt, docstring const & arg1, int arg2)
 {
 	LATTEST(contains(fmt, from_ascii("%1$s")));
 	LATTEST(contains(fmt, from_ascii("%2$d")));
@@ -1487,8 +1501,7 @@ docstring bformat(docstring const & fmt, docstring arg1, int arg2)
 }
 
 
-template<>
-docstring bformat(docstring const & fmt, char const * arg1, docstring arg2)
+docstring bformat(docstring const & fmt, char const * arg1, docstring const & arg2)
 {
 	LATTEST(contains(fmt, from_ascii("%1$s")));
 	LATTEST(contains(fmt, from_ascii("%2$s")));
@@ -1498,7 +1511,6 @@ docstring bformat(docstring const & fmt, char const * arg1, docstring arg2)
 }
 
 
-template<>
 docstring bformat(docstring const & fmt, int arg1, int arg2)
 {
 	LATTEST(contains(fmt, from_ascii("%1$d")));
@@ -1509,8 +1521,7 @@ docstring bformat(docstring const & fmt, int arg1, int arg2)
 }
 
 
-template<>
-docstring bformat(docstring const & fmt, docstring arg1, docstring arg2, docstring arg3)
+docstring bformat(docstring const & fmt, docstring const & arg1, docstring const & arg2, docstring const & arg3)
 {
 	LATTEST(contains(fmt, from_ascii("%1$s")));
 	LATTEST(contains(fmt, from_ascii("%2$s")));
@@ -1522,9 +1533,8 @@ docstring bformat(docstring const & fmt, docstring arg1, docstring arg2, docstri
 }
 
 
-template<>
 docstring bformat(docstring const & fmt,
-	       docstring arg1, docstring arg2, docstring arg3, docstring arg4)
+	       docstring const & arg1, docstring const & arg2, docstring const & arg3, docstring const & arg4)
 {
 	LATTEST(contains(fmt, from_ascii("%1$s")));
 	LATTEST(contains(fmt, from_ascii("%2$s")));
