@@ -151,8 +151,11 @@ inline bool contains(docstring const & a, char_type b)
 	return a.find(b) != docstring::npos;
 }
 
-///
+/// Returns true if the first argument is made of ascii chars given in the
+/// second argument.
 bool containsOnly(std::string const &, std::string const &);
+///
+bool containsOnly(docstring const &, std::string const &);
 
 /** Extracts a token from this string at the nth delim.
     Doesn't modify the original string. Similar to strtok.
@@ -193,13 +196,13 @@ std::string const subst(std::string const & a,
 docstring const subst(docstring const & a,
 		docstring const & oldstr, docstring const & newstr);
 
-/// Count all occurences of char \a chr inside \a str
+/// Count all occurrences of char \a chr inside \a str
 int count_char(std::string const & str, char chr);
 
-/// Count all occurences of char \a chr inside \a str
+/// Count all occurrences of char \a chr inside \a str
 int count_char(docstring const & str, docstring::value_type chr);
 
-/** Count all occurences of binary chars inside \a str.
+/** Count all occurrences of binary chars inside \a str.
     It is assumed that \a str is utf-8 encoded and that a binary char
     belongs to the unicode class names Zl, Zp, Cc, Cf, Cs, Co, or Cn
     (excluding white space characters such as '\t', '\n', '\v', '\f', '\r').
@@ -238,7 +241,7 @@ docstring const rtrim(docstring const & a, char const * p = " ");
 std::string const ltrim(std::string const & a, char const * p = " ");
 docstring const ltrim(docstring const & a, char const * p = " ");
 
-/** Splits the string given in the first argument at the first occurence 
+/** Splits the string given in the first argument at the first occurence
     of the third argument, delim.
     What precedes delim is returned in the second argument, piece; this
     will be the whole of the string if no delimiter is found.
@@ -266,6 +269,10 @@ docstring const rsplit(docstring const & a, char_type delim);
 /// problems in latex labels.
 docstring const escape(docstring const & lab);
 
+/// Group contents of an argument if needed
+docstring const protectArgument(docstring & arg, char const l = '[',
+			  char const r = ']');
+
 /// Truncates a string with an ellipsis at the end.  Leaves str unchanged and
 /// returns false if it is shorter than len. Otherwise resizes str to len, with
 /// U+2026 HORIZONTAL ELLIPSIS at the end, and returns true.
@@ -286,15 +293,15 @@ docstring const escape(docstring const & lab);
 /// be displayed in worst-case scenarios.
 ///
 /// FIXME: apply those principles in the current code.
-/// 
+///
 bool truncateWithEllipsis(docstring & str, size_t const len);
 
 /// Word-wraps the provided docstring, returning a line-broken string
-/// of width no wider than width, with the string broken at spaces. 
-/// If the string cannot be broken appropriately, it returns something 
+/// of width no wider than width, with the string broken at spaces.
+/// If the string cannot be broken appropriately, it returns something
 /// with "..." at the end, again no wider than width.
 /// We assume here that str does not contain newlines.
-/// If indent is positive, then the first line is indented that many 
+/// If indent is positive, then the first line is indented that many
 /// spaces. If it is negative, then successive lines are indented, as
 /// if the first line were "outdented".
 ///
@@ -342,30 +349,20 @@ int findToken(char const * const str[], std::string const & search_token);
 std::string formatFPNumber(double);
 
 
-template <class Arg1>
-docstring bformat(docstring const & fmt, Arg1);
-
-template <class Arg1, class Arg2>
-docstring bformat(docstring const & fmt, Arg1, Arg2);
-
-template <class Arg1, class Arg2, class Arg3>
-docstring bformat(docstring const & fmt, Arg1, Arg2, Arg3);
-
-template <class Arg1, class Arg2, class Arg3, class Arg4>
-docstring bformat(docstring const & fmt, Arg1, Arg2, Arg3, Arg4);
-
-
-template<> docstring bformat(docstring const & fmt, int arg1);
-template<> docstring bformat(docstring const & fmt, long arg1);
-template<> docstring bformat(docstring const & fmt, unsigned int arg1);
-template<> docstring bformat(docstring const & fmt, docstring arg1);
-template<> docstring bformat(docstring const & fmt, char * arg1);
-template<> docstring bformat(docstring const & fmt, docstring arg1, docstring arg2);
-template<> docstring bformat(docstring const & fmt, docstring arg1, int arg2);
-template<> docstring bformat(docstring const & fmt, char const * arg1, docstring arg2);
-template<> docstring bformat(docstring const & fmt, int arg1, int arg2);
-template<> docstring bformat(docstring const & fmt, docstring arg1, docstring arg2, docstring arg3);
-template<> docstring bformat(docstring const & fmt, docstring arg1, docstring arg2, docstring arg3, docstring arg4);
+docstring bformat(docstring const & fmt, int arg1);
+docstring bformat(docstring const & fmt, long arg1);
+#ifdef LYX_USE_LONG_LONG
+docstring bformat(docstring const & fmt, long long arg1);
+#endif
+docstring bformat(docstring const & fmt, unsigned int arg1);
+docstring bformat(docstring const & fmt, docstring const & arg1);
+docstring bformat(docstring const & fmt, char * arg1);
+docstring bformat(docstring const & fmt, docstring const & arg1, docstring const & arg2);
+docstring bformat(docstring const & fmt, docstring const & arg1, int arg2);
+docstring bformat(docstring const & fmt, char const * arg1, docstring const & arg2);
+docstring bformat(docstring const & fmt, int arg1, int arg2);
+docstring bformat(docstring const & fmt, docstring const & arg1, docstring const & arg2, docstring const & arg3);
+docstring bformat(docstring const & fmt, docstring const & arg1, docstring const & arg2, docstring const & arg3, docstring const & arg4);
 
 
 } // namespace support
