@@ -125,15 +125,9 @@ public:
 
 	iparserdocstream(idocstream & is) : is_(is) {}
 
-#ifdef LYX_USE_CXX11
 	/// Like std::istream::operator bool()
 	/// Do not convert is_ implicitly to bool, since that is forbidden in C++11.
 	explicit operator bool() const { return s_.empty() ? !is_.fail() : true; }
-#else
-	/// Like std::istream::operator void*()
-	operator void*() const { return (s_.empty() && is_.fail()) ?
-			0 : const_cast<iparserdocstream *>(this); }
-#endif
 
 	/// change the encoding of the input stream to \p e (iconv name)
 	void setEncoding(std::string const & e);
@@ -292,7 +286,7 @@ public:
 	 * found and the member \p second is the value. If \p
 	 * allow_linebreak is false, then the parsing is limited to one line
 	 */
-	Arg verbatimStuff(std::string const & end_string, 
+	Arg verbatimStuff(std::string const & end_string,
 			  bool allow_linebreak = true);
 	/*
 	 * \returns the contents of the environment \p name.
