@@ -52,17 +52,17 @@ namespace {
  */
 const char * const known_languages[] = {"acadian", "afrikaans", "albanian",
 "american", "arabic", "arabtex", "australian", "austrian", "bahasa", "bahasai",
-"bahasam", "basque", "belarusian", "brazil", "brazilian", "breton", "british",
+"bahasam", "basque", "belarusian", "bosnian", "brazil", "brazilian", "breton", "british",
 "bulgarian", "canadian", "canadien", "catalan", "croatian", "czech", "danish",
 "dutch", "english", "esperanto", "estonian", "farsi", "finnish", "francais",
-"french", "frenchb", "frenchle", "frenchpro", "galician", "german", "germanb",
+"french", "frenchb", "frenchle", "frenchpro", "friulan", "galician", "german", "germanb",
 "georgian", "greek", "hebrew", "hungarian", "icelandic", "indon", "indonesian",
 "interlingua", "irish", "italian", "japanese", "kazakh", "kurmanji", "latin",
-"latvian", "lithuanian", "lowersorbian", "lsorbian", "magyar", "malay", "meyalu",
+"latvian", "lithuanian", "lowersorbian", "lsorbian", "macedonian", "magyar", "malay", "meyalu",
 "mongolian", "naustrian", "newzealand", "ngerman", "ngermanb", "norsk", "nswissgerman",
-"nynorsk", "polutonikogreek", "polish", "portuges", "portuguese", "romanian", "russian",
-"russianb", "samin", "scottish", "serbian", "serbian-latin", "slovak",
-"slovene", "spanish", "swedish", "swissgerman", "thai", "turkish", "turkmen",
+"nynorsk", "piedmontese", "polutonikogreek", "polish", "portuges", "portuguese",
+"romanian", "romansh", "russian", "russianb", "samin", "scottish", "serbian", "serbian-latin",
+"slovak", "slovene", "spanish", "swedish", "swissgerman", "thai", "turkish", "turkmen",
 "ukraineb", "ukrainian", "uppersorbian", "UKenglish", "USenglish", "usorbian",
 "vietnam", "welsh",
 0};
@@ -73,17 +73,17 @@ const char * const known_languages[] = {"acadian", "afrikaans", "albanian",
  */
 const char * const known_coded_languages[] = {"french", "afrikaans", "albanian",
 "american", "arabic_arabi", "arabic_arabtex", "australian", "austrian", "bahasa", "bahasa",
-"bahasam", "basque", "belarusian", "brazilian", "brazilian", "breton", "british",
+"bahasam", "basque", "belarusian", "bosnian", "brazilian", "brazilian", "breton", "british",
 "bulgarian", "canadian", "canadien", "catalan", "croatian", "czech", "danish",
 "dutch", "english", "esperanto", "estonian", "farsi", "finnish", "french",
-"french", "french", "french", "french", "galician", "german", "german",
+"french", "french", "french", "french", "friulan", "galician", "german", "german",
 "georgian", "greek", "hebrew", "magyar", "icelandic", "bahasa", "bahasa",
 "interlingua", "irish", "italian", "japanese", "kazakh", "kurmanji", "latin",
-"latvian", "lithuanian", "lowersorbian", "lowersorbian", "magyar", "bahasam", "bahasam",
+"latvian", "lithuanian", "lowersorbian", "lowersorbian", "macedonian", "magyar", "bahasam", "bahasam",
 "mongolian", "naustrian", "newzealand", "ngerman", "ngerman", "norsk", "german-ch",
-"nynorsk", "polutonikogreek", "polish", "portuguese", "portuguese", "romanian", "russian",
-"russian", "samin", "scottish", "serbian", "serbian-latin", "slovak",
-"slovene", "spanish", "swedish", "german-ch-old", "thai", "turkish", "turkmen",
+"nynorsk", "piedmontese", "polutonikogreek", "polish", "portuguese", "portuguese",
+"romanian", "romansh", "russian", "russian", "samin", "scottish", "serbian", "serbian-latin",
+"slovak", "slovene", "spanish", "swedish", "german-ch-old", "thai", "turkish", "turkmen",
 "ukrainian", "ukrainian", "uppersorbian", "english", "english", "uppersorbian",
 "vietnamese", "welsh",
 0};
@@ -99,18 +99,19 @@ const char * const known_english_quotes_languages[] = {"american", "australian",
 
 /// languages with french quotes (.lyx names)
 const char * const known_french_quotes_languages[] = {"albanian",
-"arabic_arabi", "arabic_arabtex", "basque", "canadien", "catalan", "french",
-"galician", "greek", "italian", "norsk", "nynorsk", "polutonikogreek",
-"russian", "spanish", "spanish-mexico", "turkish", "turkmen", "ukrainian",
-"vietnamese", 0};
+"arabic_arabi", "arabic_arabtex", "asturian", "basque", "canadien", "catalan",
+"french", "friulan", "galician", "greek", "italian", "norsk", "nynorsk",
+"piedmontese", "polutonikogreek", "russian", "spanish", "spanish-mexico",
+"turkish", "turkmen", "ukrainian", "vietnamese", 0};
 
 /// languages with german quotes (.lyx names)
 const char * const known_german_quotes_languages[] = {"austrian", "bulgarian",
-"czech", "german", "georgian", "icelandic", "lithuanian", "lowersorbian", "naustrian",
-"ngerman", "serbian", "serbian-latin", "slovak", "slovene", "uppersorbian", 0};
+"czech", "german", "georgian", "icelandic", "lithuanian", "lowersorbian", "macedonian",
+"naustrian", "ngerman", "romansh", "serbian", "serbian-latin", "slovak", "slovene",
+"uppersorbian", 0};
 
 /// languages with polish quotes (.lyx names)
-const char * const known_polish_quotes_languages[] = {"afrikaans", "croatian",
+const char * const known_polish_quotes_languages[] = {"afrikaans", "bosnian", "croatian",
 "dutch", "estonian", "magyar", "polish", "romanian", 0};
 
 /// languages with swedish quotes (.lyx names)
@@ -124,17 +125,18 @@ const char * const known_old_language_packages[] = {"french", "frenchle",
 char const * const known_fontsizes[] = { "10pt", "11pt", "12pt", 0 };
 
 const char * const known_roman_fonts[] = { "ae", "beraserif", "bookman",
-"ccfonts", "chancery", "charter", "cmr", "fourier", "garamondx", "libertine",
-"libertine-type1", "lmodern", "mathdesign", "mathpazo", "mathptmx", "newcent",
-"tgbonum", "tgchorus", "tgpagella", "tgschola", "tgtermes", "utopia", 0};
+"ccfonts", "chancery", "charter", "cmr", "cochineal", "crimson", "fourier",
+"garamondx", "libertine", "libertine-type1", "lmodern", "mathdesign", "mathpazo",
+"mathptmx", "newcent", "NotoSerif-TLF", "tgbonum", "tgchorus", "tgpagella", "tgschola",
+"tgtermes", "utopia", 0};
 
 const char * const known_sans_fonts[] = { "avant", "berasans", "biolinum-type1",
 "cmbr", "cmss", "helvet", "iwona", "iwonac", "iwonal", "iwonalc", "kurier",
-"kurierc", "kurierl", "kurierlc", "lmss", "tgadventor", "tgheros", 0};
+"kurierc", "kurierl", "kurierlc", "lmss", "NotoSans-TLF", "tgadventor", "tgheros", 0};
 
 const char * const known_typewriter_fonts[] = { "beramono", "cmtl", "cmtt",
 "courier", "lmtt", "luximono", "fourier", "libertineMono-type1", "lmodern",
-"mathpazo", "mathptmx", "newcent", "tgcursor", "txtt", 0};
+"mathpazo", "mathptmx", "newcent", "NotoMono-TLF", "tgcursor", "txtt", 0};
 
 const char * const known_math_fonts[] = { "eulervm", "newtxmath", 0};
 
@@ -181,9 +183,10 @@ const char * const known_xetex_packages[] = {"arabxetex", "fixlatvian",
 const char * const known_lyx_packages[] = {"amsbsy", "amsmath", "amssymb",
 "amstext", "amsthm", "array", "babel", "booktabs", "calc", "CJK", "color",
 "float", "fontspec", "framed", "graphicx", "hhline", "ifthen", "longtable",
-"makeidx", "multirow", "nomencl", "pdfpages", "prettyref", "refstyle", "rotating",
-"rotfloat", "splitidx", "setspace", "subscript", "textcomp", "tipa", "tipx",
-"tone", "ulem", "url", "varioref", "verbatim", "wrapfig", "xcolor", "xunicode", 0};
+"makeidx", "minted", "multirow", "nomencl", "pdfpages", "prettyref", "refstyle",
+"rotating", "rotfloat", "splitidx", "setspace", "subscript", "textcomp", "tipa",
+"tipx", "tone", "ulem", "url", "varioref", "verbatim", "wrapfig", "xcolor",
+"xunicode", 0};
 
 // codes used to remove packages that are loaded automatically by LyX.
 // Syntax: package_beg_sep<name>package_mid_sep<package loading code>package_end_sep
@@ -296,36 +299,38 @@ string process_keyval_opt(vector<string> & options, string name)
  * FIXME: support spelling=old for german variants (german vs. ngerman LyX names etc)
  */
 const char * const Preamble::polyglossia_languages[] = {
-"albanian", "croatian", "hebrew", "norsk", "swedish", "amharic", "czech", "hindi",
-"nynorsk", "syriac", "arabic", "danish", "icelandic", "occitan", "tamil",
-"armenian", "divehi", "interlingua", "polish", "telugu", "asturian", "dutch",
-"irish", "portuges", "thai", "bahasai", "english", "italian", "romanian", "turkish",
-"bahasam", "esperanto", "lao", "russian", "turkmen", "basque", "estonian", "latin",
-"samin", "ukrainian", "bengali", "farsi", "latvian", "sanskrit", "tibetan", "urdu",
-"brazil", "brazilian", "finnish", "lithuanian", "scottish", "usorbian", "breton",
-"french", "lsorbian", "serbian", "vietnamese", "bulgarian", "galician", "magyar",
-"slovak", "welsh", "catalan", "german", "malayalam", "slovenian", "coptic", "greek",
-"marathi", "spanish", "austrian",
-"american", "ancient", "australian", "british", "monotonic", "newzealand",
-"polytonic", 0};
+"albanian", "american", "amharic", "ancient", "arabic", "armenian", "asturian", "australian",
+"bahasai", "bahasam", "basque", "bengali", "brazil", "brazilian", "breton", "british", "bulgarian",
+"catalan", "coptic", "croatian", "czech", "danish", "divehi", "dutch",
+"english", "esperanto", "estonian", "farsi", "finnish", "french", "friulan",
+"galician", "greek", "monotonic", "hebrew", "hindi",
+"icelandic", "interlingua", "irish", "italian", "kannada", "khmer",
+"lao", "latin", "latvian", "lithuanian", "lsorbian", "magyar", "malayalam", "marathi",
+"austrian", "newzealand", "german", "norsk", "nynorsk", "occitan",
+"piedmontese", "polish", "polytonic", "portuges", "romanian", "romansh", "russian",
+"samin", "sanskrit", "scottish", "serbian", "slovak", "slovenian", "spanish", "swedish", "syriac",
+"tamil", "telugu", "thai", "tibetan", "turkish", "turkmen",
+"ukrainian", "urdu", "usorbian", "vietnamese", "welsh", 0};
+// not yet supported by LyX: "korean", "nko"
 
 /**
  * the same as polyglossia_languages with .lyx names
  * please keep this in sync with polyglossia_languages line by line!
  */
 const char * const Preamble::coded_polyglossia_languages[] = {
-"albanian", "croatian", "hebrew", "norsk", "swedish", "amharic", "czech", "hindi",
-"nynorsk", "syriac", "arabic_arabi", "danish", "icelandic", "occitan", "tamil",
-"armenian", "divehi", "interlingua", "polish", "telugu", "asturian", "dutch",
-"irish", "portuges", "thai", "bahasa", "english", "italian", "romanian", "turkish",
-"bahasam", "esperanto", "lao", "russian", "turkmen", "basque", "estonian", "latin",
-"samin", "ukrainian", "bengali", "farsi", "latvian", "sanskrit", "tibetan", "urdu",
-"brazilian", "brazilian", "finnish", "lithuanian", "scottish", "uppersorbian", "breton",
-"french", "lowersorbian", "serbian", "vietnamese", "bulgarian", "galician", "magyar",
-"slovak", "welsh", "catalan", "ngerman", "malayalam", "slovene", "coptic", "greek",
-"marathi", "spanish", "naustrian",
-"american", "ancientgreek", "australian", "british", "greek", "newzealand",
-"polutonikogreek", 0};
+"albanian", "american", "amharic", "ancientgreek", "arabic_arabi", "armenian", "asturian", "australian",
+"bahasa", "bahasam", "basque", "bengali", "brazilian", "brazilian", "breton", "british", "bulgarian",
+"catalan", "coptic", "croatian", "czech", "danish", "divehi", "dutch",
+"english", "esperanto", "estonian", "farsi", "finnish", "french", "friulan",
+"galician", "greek", "greek", "hebrew", "hindi",
+"icelandic", "interlingua", "irish", "italian", "kannada", "khmer",
+"lao", "latin", "latvian", "lithuanian", "lowersorbian", "magyar", "malayalam", "marathi",
+"naustrian","newzealand", "ngerman", "norsk", "nynorsk", "occitan",
+"piedmontese", "polish", "polutonikogreek", "portuges", "romanian", "romansh", "russian",
+"samin", "sanskrit", "scottish", "serbian", "slovak", "slovene", "spanish", "swedish", "syriac",
+"tamil", "telugu", "thai", "tibetan", "turkish", "turkmen",
+"ukrainian", "urdu", "uppersorbian", "vietnamese", "welsh", 0};
+// not yet supported by LyX: "korean-polyglossia", "nko"
 
 
 bool Preamble::usePolyglossia() const
@@ -457,7 +462,8 @@ string remove_braces(string const & value)
 
 
 Preamble::Preamble() : one_language(true), explicit_babel(false),
-	title_layout_found(false), index_number(0), h_font_cjk_set(false)
+	title_layout_found(false), index_number(0), h_font_cjk_set(false),
+	h_use_microtype("false")
 {
 	//h_backgroundcolor;
 	//h_boxbgcolor;
@@ -467,6 +473,7 @@ Preamble::Preamble() : one_language(true), explicit_babel(false),
 	h_cite_engine_type        = "default";
 	h_color                   = "#008000";
 	h_defskip                 = "medskip";
+	h_dynamic_quotes          = false;
 	//h_float_placement;
 	//h_fontcolor;
 	h_fontencoding            = "default";
@@ -487,6 +494,8 @@ Preamble::Preamble() : one_language(true), explicit_babel(false),
 	h_font_tt_scale[0]        = "100";
 	h_font_tt_scale[1]        = "100";
 	//h_font_cjk
+	h_is_mathindent           = "0";
+	h_math_numbering_side     = "default";
 	h_graphics                = "default";
 	h_default_output_format   = "default";
 	h_html_be_strict          = "false";
@@ -529,7 +538,7 @@ Preamble::Preamble() : one_language(true), explicit_babel(false),
 	h_pdf_pdfusetitle         = "0";
 	//h_pdf_pagemode;
 	//h_pdf_quoted_options;
-	h_quotes_language         = "english";
+	h_quotes_style         = "english";
 	h_secnumdepth             = "3";
 	h_shortcut[0]             = "idx";
 	h_spacing                 = "single";
@@ -539,11 +548,14 @@ Preamble::Preamble() : one_language(true), explicit_babel(false),
 	h_tocdepth                = "3";
 	h_tracking_changes        = "false";
 	h_use_bibtopic            = "false";
+	h_use_dash_ligatures      = "true";
 	h_use_indices             = "false";
 	h_use_geometry            = "false";
 	h_use_default_options     = "false";
 	h_use_hyperref            = "false";
+	h_use_microtype	          = "false";
 	h_use_refstyle            = false;
+	h_use_minted              = false;
 	h_use_packages["amsmath"]    = "1";
 	h_use_packages["amssymb"]    = "0";
 	h_use_packages["cancel"]     = "0";
@@ -677,7 +689,6 @@ void Preamble::handle_package(Parser &p, string const & name,
 {
 	vector<string> options = split_options(opts);
 	add_package(name, options);
-	char const * const * where = 0;
 
 	if (is_known(name, known_xetex_packages)) {
 		xetex = true;
@@ -743,6 +754,36 @@ void Preamble::handle_package(Parser &p, string const & name,
 	else if (name == "mathptmx")
 		h_font_roman[0] = "times";
 
+	if (name == "crimson")
+		h_font_roman[0] = "cochineal";
+
+	if (name == "cochineal") {
+		h_font_roman[0] = "cochineal";
+		// cochineal can have several options, e.g. [proportional,osf]
+		string::size_type pos = opts.find("osf");
+		if (pos != string::npos)
+			h_font_osf = "true";
+	}
+
+	if (name == "noto") {
+		// noto can have several options
+		if (opts.empty())
+			h_font_roman[0] = "NotoSerif-TLF";
+		string::size_type pos = opts.find("rm");
+		if (pos != string::npos)
+			h_font_roman[0] = "NotoSerif-TLF";
+		pos = opts.find("sf");
+		if (pos != string::npos)
+			h_font_sans[0] = "NotoSans-TLF";
+		pos = opts.find("nott");
+		if (pos != string::npos) {
+			h_font_roman[0] = "NotoSerif-TLF";
+			h_font_sans[0] = "NotoSans-TLF";
+		}
+		// noto as typewriter is handled in handling of \ttdefault
+		// special cases are handled in handling of \rmdefault and \sfdefault
+	}
+
 	// sansserif fonts
 	if (is_known(name, known_sans_fonts)) {
 		h_font_sans[0] = name;
@@ -794,6 +835,8 @@ void Preamble::handle_package(Parser &p, string const & name,
 			h_font_math[0] = "libertine-ntxm";
 		else if (opts == "minion")
 			h_font_math[0] = "minion-ntxm";
+		else if (opts == "cochineal")
+			h_font_math[0] = "cochineal-ntxm";
 	}
 
 	if (name == "iwona")
@@ -809,7 +852,7 @@ void Preamble::handle_package(Parser &p, string const & name,
 	if (is_known(name, known_roman_fonts) || is_known(name, known_sans_fonts)
 		||	is_known(name, known_typewriter_fonts) || is_known(name, known_math_fonts))
 		;
-
+	//"On". See the enum Package in BufferParams.h if you thought that "2" should have been "42"
 	else if (name == "amsmath" || name == "amssymb" || name == "cancel" ||
 	         name == "esint" || name == "mhchem" || name == "mathdots" ||
 	         name == "mathtools" || name == "stackrel" ||
@@ -948,7 +991,9 @@ void Preamble::handle_package(Parser &p, string const & name,
 	else if (is_known(name, known_lyx_packages) && options.empty()) {
 		if (name == "splitidx")
 			h_use_indices = "true";
-		if (name == "refstyle")
+		else if (name == "minted")
+			h_use_minted = true;
+		else if (name == "refstyle")
 			h_use_refstyle = true;
 		else if (name == "prettyref")
 			h_use_refstyle = false;
@@ -970,7 +1015,7 @@ void Preamble::handle_package(Parser &p, string const & name,
 	else if (name == "subfig")
 		; // ignore this FIXME: Use the package separator mechanism instead
 
-	else if ((where = is_known(name, known_languages)))
+	else if (char const * const * where = is_known(name, known_languages))
 		h_language = known_coded_languages[where - known_languages];
 
 	else if (name == "natbib") {
@@ -1013,6 +1058,13 @@ void Preamble::handle_package(Parser &p, string const & name,
 				h_options += ',' + join(options, ",");
 		}
 	}
+	else if (name == "microtype") {
+		//we internally support only microtype without params
+		if (options.empty())
+			h_use_microtype = "true";
+		else
+			h_preamble << "\\usepackage[" << opts << "]{microtype}";
+	}
 
 	else if (!in_lyx_preamble) {
 		if (options.empty())
@@ -1061,28 +1113,28 @@ bool Preamble::writeLyXHeader(ostream & os, bool subdoc, string const & outfiled
 	// LyX only knows the following quotes languages:
 	// english, swedish, german, polish, french and danish
 	// (quotes for "japanese" and "chinese-traditional" are missing because
-	//  they wouldn't be useful: http://www.lyx.org/trac/ticket/6383)
+	//  they wouldn't be useful: https://www.lyx.org/trac/ticket/6383)
 	// conversion list taken from
-	// http://en.wikipedia.org/wiki/Quotation_mark,_non-English_usage
+	// https://en.wikipedia.org/wiki/Quotation_mark,_non-English_usage
 	// (quotes for kazakh and interlingua are unknown)
 	// danish
 	if (is_known(h_language, known_danish_quotes_languages))
-		h_quotes_language = "danish";
+		h_quotes_style = "danish";
 	// french
 	else if (is_known(h_language, known_french_quotes_languages))
-		h_quotes_language = "french";
+		h_quotes_style = "french";
 	// german
 	else if (is_known(h_language, known_german_quotes_languages))
-		h_quotes_language = "german";
+		h_quotes_style = "german";
 	// polish
 	else if (is_known(h_language, known_polish_quotes_languages))
-		h_quotes_language = "polish";
+		h_quotes_style = "polish";
 	// swedish
 	else if (is_known(h_language, known_swedish_quotes_languages))
-		h_quotes_language = "swedish";
+		h_quotes_style = "swedish";
 	//english
 	else if (is_known(h_language, known_english_quotes_languages))
-		h_quotes_language = "english";
+		h_quotes_style = "english";
 
 	if (contains(h_float_placement, "H"))
 		registerAutomaticallyLoadedPackage("float");
@@ -1162,7 +1214,9 @@ bool Preamble::writeLyXHeader(ostream & os, bool subdoc, string const & outfiled
 	   << ' ' << h_font_tt_scale[1] << '\n';
 	if (!h_font_cjk.empty())
 		os << "\\font_cjk " << h_font_cjk << '\n';
-	os << "\\graphics " << h_graphics << '\n'
+	os << "\\use_microtype " << h_use_microtype << '\n'
+	   << "\\use_dash_ligatures " << h_use_dash_ligatures << '\n'
+	   << "\\graphics " << h_graphics << '\n'
 	   << "\\default_output_format " << h_default_output_format << "\n"
 	   << "\\output_sync " << h_output_sync << "\n";
 	if (h_output_sync == "1")
@@ -1210,7 +1264,8 @@ bool Preamble::writeLyXHeader(ostream & os, bool subdoc, string const & outfiled
 	   << "\\paperorientation " << h_paperorientation << '\n'
 	   << "\\suppress_date " << h_suppress_date << '\n'
 	   << "\\justification " << h_justification << '\n'
-	   << "\\use_refstyle " << h_use_refstyle << '\n';
+	   << "\\use_refstyle " << h_use_refstyle << '\n'
+	   << "\\use_minted " << h_use_minted << '\n';
 	if (!h_fontcolor.empty())
 		os << "\\fontcolor " << h_fontcolor << '\n';
 	if (!h_notefontcolor.empty())
@@ -1240,7 +1295,12 @@ bool Preamble::writeLyXHeader(ostream & os, bool subdoc, string const & outfiled
 		os << "\\defskip " << h_defskip << "\n";
 	else
 		os << "\\paragraph_indentation " << h_paragraph_indentation << "\n";
-	os << "\\quotes_language " << h_quotes_language << "\n"
+	os << "\\is_math_indent " << h_is_mathindent << "\n";
+	if (!h_mathindentation.empty())
+		os << "\\math_indentation " << h_mathindentation << "\n";
+	os << "\\math_numbering_side " << h_math_numbering_side << "\n";
+	os << "\\quotes_style " << h_quotes_style << "\n"
+	   << "\\dynamic_quotes " << h_dynamic_quotes << "\n"
 	   << "\\papercolumns " << h_papercolumns << "\n"
 	   << "\\papersides " << h_papersides << "\n"
 	   << "\\paperpagestyle " << h_paperpagestyle << "\n";
@@ -1627,6 +1687,24 @@ void Preamble::parse(Parser & p, string const & forceclass,
 			handle_opt(opts, known_languages, h_language);
 			delete_opt(opts, known_languages);
 
+			// math indentation
+			if ((it = find(opts.begin(), opts.end(), "fleqn"))
+				 != opts.end()) {
+				h_is_mathindent = "1";
+				opts.erase(it);
+			}
+			// formula numbering side
+			if ((it = find(opts.begin(), opts.end(), "leqno"))
+				 != opts.end()) {
+				h_math_numbering_side = "left";
+				opts.erase(it);
+			}
+			else if ((it = find(opts.begin(), opts.end(), "reqno"))
+				 != opts.end()) {
+				h_math_numbering_side = "right";
+				opts.erase(it);
+			}
+
 			// paper orientation
 			if ((it = find(opts.begin(), opts.end(), "landscape")) != opts.end()) {
 				h_paperorientation = "landscape";
@@ -1790,6 +1868,8 @@ void Preamble::parse(Parser & p, string const & forceclass,
 					h_defskip = "bigskip";
 				else
 					h_defskip = translate_len(content);
+			} else if (name == "\\mathindent") {
+				h_mathindentation = translate_len(content);
 			} else
 				h_preamble << "\\setlength{" << name << "}{" << content << "}";
 		}
