@@ -48,6 +48,8 @@ public:
 	std::string fontCJK() const { return h_font_cjk; }
 	///
 	void fontCJK(std::string const & f) { h_font_cjk_set = true; h_font_cjk = f; }
+	///
+	bool minted() const { return h_use_minted; }
 	/// The document language
 	std::string docLanguage() const { return h_language; }
 	/// The language of text which is not explicitly marked
@@ -136,6 +138,7 @@ private:
 	std::string h_cite_engine_type;
 	std::string h_color;
 	std::string h_defskip;
+	bool h_dynamic_quotes;
 	std::string h_float_placement;
 	std::string h_fontcolor;
 	std::string h_fontencoding;
@@ -151,6 +154,10 @@ private:
 	std::string h_font_tt_scale[2];
 	bool h_font_cjk_set;
 	std::string h_font_cjk;
+	std::string h_use_microtype;
+	std::string h_is_mathindent;
+	std::string h_math_numbering_side;
+	std::string h_mathindentation;
 	std::string h_graphics;
 	std::string h_default_output_format;
 	std::string h_html_be_strict;
@@ -194,7 +201,7 @@ private:
 	std::string h_pdf_pdfusetitle;
 	std::string h_pdf_pagemode;
 	std::string h_pdf_quoted_options;
-	std::string h_quotes_language;
+	std::string h_quotes_style;
 	std::string h_secnumdepth;
 	std::string h_shortcut[99];
 	std::string h_spacing;
@@ -204,12 +211,14 @@ private:
 	std::string h_tocdepth;
 	std::string h_tracking_changes;
 	std::string h_use_bibtopic;
+	std::string h_use_dash_ligatures;
 	std::string h_use_indices;
 	std::string h_use_geometry;
 	std::map<std::string, std::string> h_use_packages;
 	std::string h_use_default_options;
 	std::string h_use_hyperref;
 	bool h_use_refstyle;
+	bool h_use_minted;
 
 	/*!
 	 * Add package \p name with options \p options to used_packages.
