@@ -282,10 +282,11 @@ void initModules()
 		for (; it != end; ++it) {
 			string const module = it->getID();
 			LayoutModuleList m;
+			LayoutModuleList c;
 			vector<string> v;
 			if (!addModule(module, baseClass, m, v))
 				continue;
-			modules[module] = getDocumentClass(baseClass, m);
+			modules[module] = getDocumentClass(baseClass, m, c);
 		}
 		init = false;
 	}
@@ -311,7 +312,7 @@ bool addModule(string const & module)
 	return false;
 }
 
-}
+} // namespace
 
 
 bool checkModule(string const & name, bool command)
@@ -1059,7 +1060,9 @@ int TeX2LyXApp::run()
 		if (configFileNeedsUpdate("lyxrc.defaults") ||
 		    configFileNeedsUpdate("lyxmodules.lst") ||
 		    configFileNeedsUpdate("textclass.lst") ||
-		    configFileNeedsUpdate("packages.lst"))
+		    configFileNeedsUpdate("packages.lst") ||
+		    configFileNeedsUpdate("lyxciteengines.lst") ||
+		    configFileNeedsUpdate("xtemplates.lst"))
 			package().reconfigureUserLyXDir("");
 		fileUnlock(fd, lock_file.c_str());
 	} else
